/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 * arch/arc/kernel/entry.S
 *
 * Copyright (C)
 *
 * This program is free software you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Author(s) : Amit Bhor, Rahul Trivedi
 */


/*
      ------------------------------------------------------------------
      Function                            OLD GCC ABI     Metaware ABI
      ------------------------------------------------------------------
      Arguments                           r0 - r7         r0 - r7
   *  Caller Saved Registers              r0 - r15        r0 - r12
                                          r24, r25
   *  Callee Saved Registers              r16 - r23       r13- r25
   *  Stack chain pointer                 r24             r11
   *  Register used for temporary         r25             r12
      calculation               
      Global Pointer (gp)                 r26             r26
      Frame Pointer (fp)                  r27             r27
      Stack Pointer (sp)                  r28             r28
      Interrupt link register (ilink1)    r29             r29
      Interrupt link register (ilink2)    r30             r30
      Branch link register (blink)        r31             r31
      ------------------------------------------------------------------
GCC for A5, ARC600 and ARC700 now confirms to the Metware High C ABI

*/
	.cpu A7
	
#include <linux/config.h>

	.section .vector, "ax",@progbits
	.align 4

/* Each entry in the vector table must occupy 2 words. Since it is a jump
 * across sections (.vector to .text) we are gauranteed that 'j somewhere'
 * will use the 'j limm' form of the intrsuction as long as somewhere is in
 * a section other than .vector. 
 */
	
;j	res_service		; 0x0,	restart vector
;j	mem_service		; 0x8,	memory exception
;j	instr_service		; 0x10,	instruction error
.rept	29			
;j	handle_interrupt	; top level interrupt handler
.endr
;j	EV_MachineCheck		; 0x100, fatal Machine check exception
;j	EV_TLBMissI		; 0x108, Intruction TLB miss
;j	EV_TLBMissD		; 0x110, Data TLB miss
;j	EV_TLBProtV		; 0x118, Protection Violation
;j	EV_PrivilegeV		; 0x120, Privilege Violation
;j	EV_Trap			; 0x128, Trap exception
;j	EV_Extension		; 0x130, Extension intruction exception
.rept	24
j	reserved		; Reserved Exceptions
.endr




#include <linux/sys.h>		 /* contains the declaration of NR_syscalls */
#include <linux/linkage.h>	 /* contains declarations of SYMBOL_NAME, etc */
#include <asm/entry.h>		 /* For the SAVE_* macros */
#include <asm/ptrace.h>		 /* For the PT_* offsets */
#include <asm/unistd.h>		 /* For SYSCALL_IRQ  */
#include <asm/errno.h>		 
#include <asm/system.h>
#include <asm/arcregs.h>
#include <asm/arcdefs.h>

; the bit of ECR which should be set if EV_TRAP was caused beacuse a break point was hit
#define TRAP_S_BRKPT	1	

/* Simulator fails to disable interrupts in exceptions */
#ifdef	CONFIG_ARC700_SIM_RUN	
//#define SIM_EXCEPTION_BUG
#endif

; the ENTRY macro in linux/linkage.h does not work for us ';' is treated as a comment
; and I could not find a way to put a new line in a #define

.macro ARC_ENTRY name
  .globl SYMBOL_NAME(\name) 
  .align 4		
  SYMBOL_NAME_LABEL(\name)
.endm  
  
#ifdef CONFIG_ARC_TRACE_UNIT

#ifndef CONFIG_ARC_TRACE_DEPTH
#define CONFIG_ARC_TRACE_DEPTH 16
#endif

.macro STORE_TRACE_INDEX index
	/* get entry defined by index */
	mov %r0,0x00000000 | (\index << 2) /* source */
	sr %r0,[0xad] 
	lr %r0,[0xae]
	st %r0,[%r1]
	add %r1,%r1,4
	mov %r0,0x00000001 | (\index << 2) /* dest */
	sr %r0,[0xad] 
	lr %r0,[0xae]
	st %r0,[%r1]
	add %r1,%r1,4
	mov %r0,0x00000002 | (\index << 2) /* other */
	sr %r0,[0xad] 
	lr %r0,[0xae]
	st %r0,[%r1]
	add %r1,%r1,4
.endm

.macro FILL_TRACE from=0,to=CONFIG_ARC_TRACE_DEPTH
	STORE_TRACE_INDEX(\from)
.if \to-\from
	FILL_TRACE "(\from+1)",\to
.endif
.endm

.macro STORE_TRACE 
	sub %r0,%r0,%r0
	sub %r1,%r1,%r1
	mov %r1,trace_buffer
	FILL_TRACE
.endm
#endif


; ---------------------------------------------
	.section .text, "ax",@progbits

SYMBOL_NAME_LABEL(res_service)	; processor restart
	flag	0x1		; not implemented
	nop
	nop

SYMBOL_NAME_LABEL(reserved)	; processor restart
		rtie       	; jump to processor initializations

#if 0
	
#if 0
ARC_ENTRY handle_interrupt
	st	r9, [SYMBOL_NAME(int1_saved_reg)] ;  save r9
	st	r9, [SYMBOL_NAME(int2_saved_reg)] 
	lr	r9, [status32]
	and.f	0, r9, STATUS_A1_MASK ;  find out what level interrupt occurred
	nop			
	bnz	handle_interrupt_level1	;  level 1 interrupt
	; fall through for a level 2 interrupt
	
SYMBOL_NAME_LABEL(handle_interrupt_level2)
	lr	r9, [status32_l2]
#if 0
	and.f	0, r9, STATUS_AE_MASK
	bz	1f
	flag	0x1	; Interrupt taken in exception, insane !!! halt
	nop
	nop
1:
#endif
	; find out in which mode was the system running when the interrupt 
	; occurred
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	bz	1f		
	; interrupt occurred in user mode hence load sp kernel stack pointer 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, (0x2000 - 4)
1:
	SAVE_ALL_INT2

	mov	r8, SYMBOL_NAME(irq_stat)	 
	ld	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]
	add	r9, r9, 1
	st	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]

	lr	r0, [icause2]
	and	r0, r0, 0x1f
	
	mov	r1, sp

	bl	SYMBOL_NAME(process_interrupt)

	b	ret_from_interrupt
	
	
ARC_ENTRY handle_interrupt_level1
	lr	r9, [status32_l1]
#if 0
	and.f	0, r9, STATUS_AE_MASK
	bz	1f
	flag	0x1	; Interrupt taken in exception, insane !!! halt
	nop
	nop
1:
#endif
	; find out in which mode was the system running when the interrupt 
	; occurred
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	bz	1f		
	; interrupt occurred in user mode hence load sp kernel stack pointer 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, (0x2000 - 4)
1:
	SAVE_ALL_INT1

	mov	r8, SYMBOL_NAME(irq_stat)	 
	ld	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]
	add	r9, r9, 1
	st	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]

	lr	r0, [icause1]
	and	r0, r0, 0x1f
		
	mov	r1, sp

	bl	SYMBOL_NAME(process_interrupt)

	b	ret_from_interrupt
#endif

/* Instruction Error Exception Handler */	
ARC_ENTRY instr_service
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
1:
	SAVE_ALL_SYS

#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif
	
; We dont want exceptions to be disabled while the fault is handled
; Now that we have saved the context we return from exception hence re-enable
; exceptions and remain in kernel mode. Interrupts remain disabled.
; Also this is required because the current process is going die, and this status32 will be inherited by the next process
; that will be scheduled.
	ld	r9, [sp, PT_status32]	;read erstatus so we return to the same state of interrupt enables.  
	; set mode we are returning to as kernel mode
	and	r9, r9, ~(STATUS_U_MASK|STATUS_L_MASK|STATUS_DE_MASK)
	sr	r9, [erstatus]
	mov	r9, 2f
	sr	r9, [eret]
	rtie
2:
	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
	lr	r1, [efa]       /* faulting address */
	mov	r2, sp 
	bl	instruction_error
	j	ret_from_exception

/* Memory Error Exception Handler */	
ARC_ENTRY mem_service
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
1:
	SAVE_ALL_SYS

#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif

	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
	lr	r1, [efa]       /* faulting address */
	mov	r2, sp 
	bl	memory_error
	j	ret_from_exception

/* Protection Violation Exception Handler */	
ARC_ENTRY EV_MachineCheck
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
	
1:
	SAVE_ALL_SYS
#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif


	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
	lr	r1, [efa]       /* faulting address */
	mov	r2, sp 
	bl	machine_check
	j	ret_from_exception
	
/* Protection Violation Exception Handler */	
ARC_ENTRY EV_TLBProtV
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
#ifdef SIM_EXCEPTION_BUG
	cli_level_1	r9
#endif
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
	
1:
	SAVE_ALL_SYS
#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif

; We dont want exceptions to be disabled while the fault is handled
; Now that we have saved the context we return from exception hence re-enable
; exceptions and remain in kernel mode. Interrupts remain disabled.
	ld	r9, [sp, PT_status32]	;read erstatus so we return to the same state of interrupt enables.  
	; set mode we are returning to as kernel mode
	and	r9, r9, ~(STATUS_U_MASK|STATUS_L_MASK|STATUS_DE_MASK)
	sr	r9, [erstatus]
	mov	r9, 2f
	sr	r9, [eret]
	rtie
2:
	lr	r0, [ecr]	/* load ecr in order to get the cause code and parameter */
	cmp	r0, 0x230400	/* Misaligned data acces ?*/
	bne	3f
	lr	r1, [efa]
	mov	r2, sp
	bl	protection_fault
3:
	mov	r1, 0x0
	breq	r3, 0x230100, 4f	; is it a LD exception ? write = 0
	mov	r1, 0x1		; else write = 1
4:	lr	r2, [efa]       /* faulting address */
	mov	r0, sp
	bl	do_page_fault
	j	ret_from_exception

/* Privilege Violation Exception Handler */	
ARC_ENTRY EV_PrivilegeV
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
#ifdef SIM_EXCEPTION_BUG
	cli_level_1	r9
#endif
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
	
1:
	SAVE_ALL_SYS
#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif

; We dont want exceptions to be disabled while the fault is handled
; Now that we have saved the context we return from exception hence re-enable
; exceptions and remain in kernel mode. Interrupts remain disabled.
	ld	r9, [sp, PT_status32]	;read erstatus so we return to the same state of interrupt enables.  
	; set mode we are returning to as kernel mode
	and	r9, r9, (~STATUS_U_MASK)
	sr	r9, [erstatus]
	mov	r9, 2f
	sr	r9, [eret]
	rtie
2:
	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
	lr	r1, [efa]       /* faulting address */
	mov	r2, sp 
	bl	privilege_fault
	j	ret_from_exception
	
/* Extension Instruction Exception Handler */	
ARC_ENTRY EV_Extension
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
	
1:
	SAVE_ALL_SYS
#ifdef CONFIG_ARC_TRACE_UNIT
	STORE_TRACE
#endif


	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
	lr	r1, [efa]       /* faulting address */
	mov	r2, sp 
	bl	extension_fault
	j	ret_from_exception
	
	
	
; System call handler. parameters in r0-r7. r0 has the system call number
; EV_Trap caused by TRAP_S and TRAP0 instructions.
; We manipulate ERET to return from exception and stay in kernel mode and
; proceed.

ARC_ENTRY EV_Trap
	st	r9, [SYMBOL_NAME(ex_saved_reg1)]
#ifdef SIM_EXCEPTION_BUG
	cli_level_1	r9
#endif
	lr	r9, [erstatus]
	and.f	0, r9, STATUS_U_MASK
	mov	r9, sp	
	; r9 has the user stack pointer or the kernel stack pointer 
	; in case we came here from the kernel ( kernel threads ) 
	
	; using flags set by instruction "and.f	" above.
	bz	1f	; dont switch to kernel stack 
	ld	sp, [SYMBOL_NAME(_current_task)]
	add	sp, sp, ( 0x2000 - 4 )	; 0x2000 = THREAD_SIZE, the size of 
					; task_struct
1:
	SAVE_ALL_SYS_TRAP	; save all registers required for system call handling

; We dont want exceptions to be disabled while system calls are handled. 
; Now that we have saved the context we return from exception hence re-enable
; exceptions and remain in kernel mode.
	ld	r9, [sp, PT_status32]	;read erstatus so we return to the same state of interrupt enables.  
	; set mode we are returning to as kernel mode
	and	r9, r9, (~STATUS_U_MASK)
	sr	r9, [erstatus]
	mov	r9, sys_trap_ret
	sr	r9, [eret]
	rtie
	 
sys_trap_ret:	

	; check if a brk pt caused the exception
	lr	r10, [ecr]
	and.f	0,r10,TRAP_S_BRKPT
;	jz	not_traps_brkpt
	
	;make sure orig_r8 is a positive value
	add	r10, sp, PT_orig_r8
	st	NR_syscalls + 1, [r10] 
	lr	r0, [ecr]    /* load ecr in order to get the cause code and parameter */
	lr	r1, [efa]       /* faulting address */
	mov	r2, sp 

	ld	r9, [SYMBOL_NAME(_current_task)]	; current task in r9	
	add	r10, r9, (TASK_THREAD + THREAD_USP)
	st	sp, [r10]
	
	;Save callee regs so that the tracer gets the correct value.
	SAVE_CALLEE_SAVED
	add	r10, r9, (TASK_THREAD + THREAD_PC)
	st	sp, [r10]
	bl	trap_s_brkpt
	
	;sp should point to pt_regs 
	ld	r9, [SYMBOL_NAME(_current_task)]	; current task in r9	
	add	r10, r9, (TASK_THREAD + THREAD_USP)
	ld	sp,[r10]
	j	ret_from_exception	 

	
ARC_ENTRY ret_from_system_call
	; put the return value of system call 
	st	r0, [sp, PT_r0]
	jal	ret_from_exception

#if 0
ARC_ENTRY ret_from_interrupt
	; 	 check if we need to do software interrupts 
	; FIXME :: this needs protection for level 2 interrupts
	mov	r8, SYMBOL_NAME(irq_stat)	 
	ld	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]
	sub	r9, r9, 1
	st	r9, [r8, CPUSTAT_LOCAL_IRQ_COUNT]

	ld	r9, [r8, CPUSTAT_SOFTIRQ_PENDING]
	mov.f	0, r9
	nop 
	bz	ret_from_exception

;	jl	SYMBOL_NAME(do_softirq)				
	;  fall through
#endif
	
ARC_ENTRY ret_from_exception
	; check if we are returning to kernel mode
	ld	r8, [sp, PT_status32]	; sp points to pt_regs structure on stack
	and.f	0, r8, STATUS_U_MASK	; test
	nop
	bz	skip_check
	; else returning to user mode
	; load current task
	ld	r8, [SYMBOL_NAME(_current_task)]
	; check for rescheduling
	ld	r9, [r8, TASK_NEED_RESCHED]
	mov.f	0, r9		; test
	nop
	bnz	reschedule	; uses flags set by the mov.f 0, r9 intruction 
	; check for signals
	ld	r9, [r8, TASK_SIGPENDING]
	mov.f	0, r9
	nop
	bz	skip_check
	;  there are pending signals 
	mov	r0, sp		;  pass pointer tp the pt_regs structure
	mov	r1, 0           ;  pass offset = NULL

	ld	r9, [SYMBOL_NAME(_current_task)]	; current task in r9	
	add	r10, r9, (TASK_THREAD + THREAD_USP)
	st	sp,[r10]
	
	; save callee regs so that incase of a core dump, the correct regs are available
	SAVE_CALLEE_SAVED
	;FIXME - dont use KSP, use PC instead
	add	r9, r8, (TASK_THREAD + THREAD_PC) 
	st	sp, [r9]
	
	;jl	SYMBOL_NAME(do_signal)

	;sp pointing back to pt_regs
	ld	r9, [SYMBOL_NAME(_current_task)]	; current task in r9	
	add	r10, r9, (TASK_THREAD + THREAD_USP)
	ld	sp,[r10]
	
	; fall through 	
skip_check : 	
	; Exceptions cannot be disabled. We work with the understanding that
	; from this point on till the rtie at the end, no exceptions will occur.
	; We are always restoring from kernel stack which will be in
	; untranslated region
	; Restore the saved sys context.
	lr	r10, [status32]
	and.f	0, r10, STATUS_AE_MASK
	nop
	bz	not_exception
	; Disable interrupts, do_softirq would have enabled them !
	cli_level_12 r9
	RESTORE_ALL_SYS
	rtie
not_exception:	
	and.f	0, r10, STATUS_A1_MASK
	nop
	bz	not_level1_interrupt
	; Disable level 2 interrupts, do_softirq would have enabled them !
	cli_level_12 r9
	RESTORE_ALL_INT1
	rtie
not_level1_interrupt:
	and.f	0, r10, STATUS_A2_MASK
	nop
	bz	not_level2_interrupt
	; Disable interrupts, do_softirq would have enabled them !
	cli_level_12 r9
	RESTORE_ALL_INT2
	rtie
not_level2_interrupt:	
	; Disable interrupts, do_softirq would have enabled them !
	cli_level_12 r9
	;this case is for syscalls for which the exception active bit is not
	;set
	RESTORE_ALL_SYS
	; FIXME :: might not be required, waiting for further info
	; Ensure all pending memory operations have physically completed before
	; returning.
	; sync 
	; return to interrupted pc
	rtie
	
reschedule :
	;jl	schedule
	b	ret_from_exception

badsys :
	st	-ENOSYS, [sp, PT_r0]
	jal	ret_from_system_call


ARC_ENTRY __switch_to
	/* save return address onto stack */
	st.a	blink, [sp, -4]
	/* save frame pointer stack */
	st.a	fp, [sp, -4]
	SAVE_CALLEE_SAVED
	/* save kernel stack pointer */
	add	r9, r0, (TASK_THREAD + THREAD_KSP) 
	st	sp, [r9]
	/* return last task in r0 ( return registor) */
	ld	r0, [SYMBOL_NAME(_current_task)]
	
	/* switch to new task , second parameter to this function , ie r1*/
	st	r1, [SYMBOL_NAME(_current_task)]
	/* load kernel stack pointer from the new task struct */
	add	r9, r1, (TASK_THREAD + THREAD_KSP) 
	ld	sp, [r9]
	/* restore the registers for the new task */
	RESTORE_CALLEE_SAVED
	/* restore the frame pointer */
	ld.ab	fp, [sp, 4]
	/* restore blink (return address) */
	ld.ab	blink, [sp, 4]
	/* return to the point where this thread "slept" */
	j	[blink]
	/* FIXME : unused delay slot */
	

; The syscall table starts here...
#ifndef NR_syscalls
#define NR_syscalls 256
#endif
.data
ARC_ENTRY sys_call_table
__syscall_start:
__syscall_end:

		.rept	NR_syscalls - (__syscall_end - __syscall_start) / 4
/*				.long	SYMBOL_NAME(sys_ni_syscall)
*/
		.endr

#endif
