/*
 *  linux/init/main.c
 *
 *  Copyright (C) 1991, 1992  Linus Torvalds
 *
 *  GK 2/5/95  -  Changed to support mounting root fs via NFS
 *  Added initrd & change_root: Werner Almesberger & Hans Lermen, Feb '96
 *  Moan early if gcc is old, avoiding bogus kernels - Paul Gortmaker, May '96
 *  Simplified starting of init:  Michael A. Griffith <grif@acm.org> 
 */

#define __KERNEL_SYSCALLS__

#include <linux/config.h>
#include <linux/proc_fs.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/unistd.h>
#include <linux/string.h>
#include <linux/ctype.h>
#include <linux/delay.h>
#include <linux/utsname.h>
#include <linux/ioport.h>
#include <linux/init.h>
#include <linux/smp_lock.h>
#include <linux/blk.h>
#include <linux/hdreg.h>
#include <linux/iobuf.h>
#include <linux/bootmem.h>
#include <linux/file.h>
#include <linux/tty.h>
#include <linux/interrupt.h>

#include <linux/autoconf.h>

#include <asm/io.h>
//#include <asm/bugs.h>
#include <asm/serial.h>
//#include "../ramdsize.h"
#include "arc_pgu.h"
#include "arc_tcon.h"


#define VECTOR_TABLE_SIZE 0x2000

unsigned int OUTPUT_OFFSET=0x0;
//Flowbee@960228 unsigned int FBBaseAddr=0x83000000;
#ifdef CONFIG_RESERVE_FB_MEM
unsigned int FBBaseAddr = END_MEM - CONFIG_RESERVE_FB_MEM_SIZE;
#else
unsigned int FBBaseAddr = END_MEM - 0x1000000;
#endif

#ifdef CONFIG_KERNEL_ATAG
extern unsigned long atag_head;
#endif
extern unsigned int _bss_start, _bss_end;
extern unsigned int logo_output;
extern unsigned int audio_output;
extern char *linux_banner;
extern int _text;
extern int _etext;
extern int _end;
extern int _start_cp_text;
extern char logo_start[], logo_end[];
extern char audio_start[], audio_end[];
extern char ramdisk2_start[], ramdisk2_end[];
extern unsigned int jpeg_size;
extern unsigned int get_total_len(void);
extern void decompress_kernel(void);
extern void decompress_logo(void);
extern int ramd_start;
extern int arc_pgufb_init(void);
extern int hdjpeg(int);
extern int PGU_Enable(void);
extern int set_rgb_pointer(void);
extern int set_yuv_pointer(void);

// jinn 080514 - this array map to skyconfig.cfg
extern unsigned char skycfg_start[], skycfg_end[];

extern void* memset(void* s, int c, size_t n);


/* A7 does not autoswitch stacks on interrupt or exception ,so we do it manually.
 * But we need one register for each interrupt level to play with which is saved 
 * and restored from the following variable.*/
volatile unsigned long int1_saved_reg, int2_saved_reg, ex_saved_reg1, ex_saved_reg2;
#if 0 //wschen 2008-07-16
volatile unsigned long icse1, icse2, ilnk1, ilnk2, blnk, stat32;
#endif


struct task_struct *_current_task;	/* currently active task */
irq_cpustat_t irq_stat[NR_CPUS] ____cacheline_aligned;
#ifdef CONFIG_ARC700_SERIAL
asmlinkage void __init arcserial_put_char(char ch)
{
	aa3_uart *uart = (aa3_uart *)UART_BASEADDR;
    //int i=0;

	/* check if TXEMPTY (bit 7) in the status reg. is 1. Else, we
	 * aren't supposed to write right now. Wait for some time.
	 */

	while(!(uart->status & UART_TXEMPTY))
	{
//	  if (i++ > 50000)
//	    break;
	};

	/* actually write the character */
	uart->data = ch;
}

asmlinkage int __init __strlen(const char *s)
{
	int i = 0;

	while (*s++)
		i++;
	return i;
}

asmlinkage void __init __puts(const char *s)
{
	int len;
	len = __strlen(s);
	while(len-- > 0)
	{
		if (*s == '\n')
			arcserial_put_char('\r');
		arcserial_put_char(*s++);
	}

}

asmlinkage void __init
__print_hex(unsigned long val)
{
        int i, nibbles = sizeof(val)*2;
        char buf[sizeof(val)*2+1];

        for (i = nibbles-1;  i >= 0;  i--) {
                buf[i] = (val & 0xf) + '0';
                if (buf[i] > '9')
                    buf[i] += ('a'-'0'-10);
                val >>= 4;
        }
        buf[nibbles] = '\0';
	__puts(buf);
}

asmlinkage void __init
__printn_hex(unsigned long val)
{
	__print_hex(val);
	__puts("\n");
}
#else
#define arcserial_put_char(...)
#define __strlen(...)
#define __puts(...)
#define __print_hex(...)
#define __printn_hex(...)
#endif

asmlinkage void* __init __memcpy(void* __dest, __const void* __src,
			    size_t __n)
{
	int i;
	char *d = (char *)__dest, *s = (char *)__src;

	for (i=0;i<__n;i++)
	{
		 d[i] = s[i];
	}
	return __dest;
}

asmlinkage void __init flush_all_dcache(void)
{
	unsigned long value;
	write_new_aux_reg(ARC_REG_DC_FLSH, 1);

	/* poll till the flush is over */
	do {
		value = read_new_aux_reg(ARC_REG_DC_CTRL);
	} while(value & 0x100); /* Check with the FS bit */
}

asmlinkage void __init relocate_vm(void)
{
	unsigned int cplen = get_total_len();
#ifdef CONFIG_KERNEL_ATAG
	void (*kernel)();
	kernel = (void (*)())( CONFIG_CODE_OFFSET_TEXTADDR );
#endif

	if (OUTPUT_OFFSET != 0)
	{
		__memcpy( (void *)((unsigned int )CONFIG_CODE_OFFSET_TEXTBASE), 
			(void *)((unsigned long)(CONFIG_CODE_OFFSET_TEXTBASE+OUTPUT_OFFSET)), 
			VECTOR_TABLE_SIZE);

		__memcpy( (void *)((unsigned int )CONFIG_CODE_OFFSET_TEXTADDR), 
        		(void *)((unsigned long)(CONFIG_CODE_OFFSET_TEXTADDR+OUTPUT_OFFSET)), 
        		cplen);
	}
	//write_aux_reg(int_vector_base, CONFIG_CODE_OFFSET_TEXTBASE);

     // jinn - 20080514 - copy skycfg_start[] to CONFIG_DATA_BASE
     #ifdef CONFIG_DATA_BASE
          if ( ( skycfg_end - skycfg_start ) > 0 ) {
               __puts("Copy skyconfig data to CONFIG_DATA_BASE\n");
               __puts("DLIOC size = 0x");
               __print_hex( (unsigned long)(skycfg_end - skycfg_start) );
               __puts("\n");
               __memcpy( (void *)((unsigned int)CONFIG_DATA_BASE),
     			     (void *)(skycfg_start),
     			     (512 + 7168) );
     	}
     #endif

#ifdef CONFIG_KERNEL_ATAG
	if (atag_head != 0)
	{
		__asm__ __volatile__(   
		        "mov r0,0\n\t"
		        "lr  r1,[identity]\n\t"
		        "ld  r2,%0\n\t"        
		        "ld  r3,%1\n\t"
		        "jl  [r3]\n\t"
		        ::"m"(atag_head),"m"(kernel)
//		        ::"m"(atag_head)
		        :"r0","r1","r2");
	}
	flush_all_dcache();
#endif

	(((void (*)(void)) CONFIG_CODE_OFFSET_TEXTADDR )());
}

/*
 *	Activate the first processor.
 */
//-- Flowbee 2008-04-16
static volatile unsigned long vtval; 	
asmlinkage void __init start_kernel(void)
{
	unsigned int v_i=0;
	unsigned char *magic;

	//-- Flowbee 2008-04-16
	#if CONFIG_TERAWINS_TCON
	SCL_OUTPUT;
	SDA_OUTPUT;	
	SCL_HI;	
	SDA_HI;	
	#endif

	//-- Jimmy 2008-08-01 turn off backlight here and turn on backlight at PGU_Enable()
	#if CONFIG_SKY_IO_QMI_COMPANY_V2
	QMI_BL_OUTPUT;
	QMI_BL_LO;
	E5V_OUTPUT;
	E5V_HI;
	#endif //CONFIG_SKY_IO_QMI_COMPANY_V2	
     
	/* Reset the USB register. */
#ifdef CONFIG_SKY_ASIC_VER_A	
	*((volatile unsigned char*)0xc0f04a93) = 0x00;
#endif	
	write_aux_reg(int_vector_base, 0x80600000);

	/* UART initial */
	*((volatile unsigned int *)0xc0fc1000) = 0x00000001;
	*((volatile unsigned int *)0xc0fc1004) = 0x000000fe;
	*((volatile unsigned int *)0xc0fc1008) = 0x00000002;
	*((volatile unsigned int *)0xc0fc100c) = 0x00000000;
	//*((unsigned int *)0xc0fc1010) = 0x00000000;
	*((volatile unsigned int *)0xc0fc1014) = 0x00000024;
//	*((unsigned int *)0xc0fc1018) = 0x0000002e;	//CLK=70MHz & BAUD=57600
//	*((unsigned int *)0xc0fc101c) = 0x00000001;	//CLK=70MHz & BAUD=57600
#ifdef CONFIG_ARC700_SERIAL
	*((volatile unsigned int *)0xc0fc1018) = (((CONFIG_ARC700_CLK/(CONFIG_ARC700_SERIAL_BAUD*4)) - 1) & 0xff);
	*((volatile unsigned int *)0xc0fc101c) = (((CONFIG_ARC700_CLK/(CONFIG_ARC700_SERIAL_BAUD*4)) - 1) & 0xff00) >> 8;
#else
	*((unsigned int *)0xc0fc1018) = (((162000000/(57600*4)) - 1) & 0xff);
	*((unsigned int *)0xc0fc101c) = (((162000000/(57600*4)) - 1) & 0xff00) >> 8;
#endif

	//__puts("identity="); __printn_hex(read_aux_reg(identity));
	//__puts("int_vector_base="); __printn_hex(read_aux_reg(int_vector_base));
        //write_aux_reg(int_vector_base, 0x80600000);
	//__puts("new int_vector_base="); __printn_hex(read_aux_reg(int_vector_base));
/*
__puts("new 0xc0fc1800="); __printn_hex(*((unsigned int *)0xc0fc1800));
__puts("new 0xc0fc1818="); __printn_hex(*((unsigned int *)0xc0fc1818));
__puts("new 0xc0fc1814="); __printn_hex(*((unsigned int *)0xc0fc1814));
__puts("new 0xc0fc700c="); __printn_hex(*((unsigned int *)0xc0fc700c));
__puts("new 0xc0fc7004="); __printn_hex(*((unsigned int *)0xc0fc7004));
__puts("new 0xc0fc7008="); __printn_hex(*((unsigned int *)0xc0fc7008));
__puts("new 0xc0fca000="); __printn_hex(*((unsigned int *)0xc0fca000));
__puts("new 0xc0fca004="); __printn_hex(*((unsigned int *)0xc0fca004));
__puts("new 0xc0fca008="); __printn_hex(*((unsigned int *)0xc0fca008));
__puts("new 0xc0fca00c="); __printn_hex(*((unsigned int *)0xc0fca00c));
*/

	for (v_i = &_bss_start; v_i < &_bss_end; v_i += 4)
	{
		 *(volatile unsigned int *)(v_i) = 0;
	}

//save ramdisk2 information
	*((volatile unsigned int *)0xc0f00004) = &ramdisk2_start[0];
	*((volatile unsigned int *)0xc0f00008) = &ramdisk2_end[0];

//	write_aux_reg(int_vector_base, 0x80600000);


//__puts("new AUX 0x21="); __printn_hex(read_new_aux_reg(0x21));
//__puts("new AUX 0x22="); __printn_hex(read_new_aux_reg(0x22));
//__puts("new AUX 0x23="); __printn_hex(read_new_aux_reg(0x23));
//write_new_aux_reg(0x22, 0x00000000);
//__puts("update AUX 0x22="); __printn_hex(read_new_aux_reg(0x22));

//Setup StartUP Audio
#ifdef CONFIG_SOUND_SKY_AUDIO
	if ((&audio_end[0] - &audio_start[0]) > 0x100)
	{
		decompress_audio();
    		magic = (unsigned char *)audio_output;
    		v_i = get_total_len();
    		magic[v_i+1] = 0;
    		magic[v_i+2] = 0;
    		magic[v_i+3] = 0;
    		magic[v_i+4] = 0;
    		v_i = ((get_total_len() + 3)>>2)<<2;
//		i2s_init(&audio_start[0]);
		i2s_init(magic, v_i);
	}
#endif


/* setup LCD */
#if 1
	//__puts("\nStart Terawins Init !!\n");
	if(arc_pgufb_init() == -1){
		__puts("\nTerawins init fail\n");
	}

	if ((&logo_end[0] - &logo_start[0]) >0)
	{
#ifdef CONFIG_RESERVE_FB_MEM
		FBBaseAddr = END_MEM - CONFIG_RESERVE_FB_MEM_SIZE;
#else
		FBBaseAddr = END_MEM - 0x01000000;	//16MB ==> 0x83000000
#endif
		/* Flowbee@960310 */
		//arc_pgufb_init();
		decompress_logo();
    		magic = (unsigned char *)logo_output;

		if ((magic[0] == 0xff && magic[1] == 0xd8) || (magic[6] == 'J' && magic[7] == 'F' &&magic[8] == 'I' &&magic[9] == 'F'))
		{
			//JPEG
			__puts("BG Format : JPEG\n");
			jpeg_size = get_total_len();

			flush_all_dcache();
			hdjpeg(1);
		}
		else
		{
			//RGB Raw
			__puts("BG Format : RGB Raw\n");
			__memcpy( (void *)(FBBaseAddr), (void *)logo_output, 614400);
			flush_all_dcache();

//			set_rgb_pointer();
			set_yuv_pointer();
			PGU_Enable();
		}
		__puts("LCD Setup done!\n");
	}
	else
	{
		//arc_pgufb_init();
		memset((void *)(FBBaseAddr), 0x00, 614400);
		flush_all_dcache();
		set_rgb_pointer();
		PGU_Enable();
	}
#endif
#if 0	// alexlu move the ramdisk size setting to ramset.c for easy customization
	/*
	 * move ramdisk.image.gz from ramd_start to 10M (0x80a10000)
	 * It's used for arch/arc/kernel/setup.c initrd_start.
	 */
	if (_RAMDISK_SIZE > 0)
	{
#if CONFIG_SRAM_SIZE >= 0x04000000
		__memcpy( (void *)0x82800000, 
			(void *)((unsigned long)&ramd_start), 
			_RAMDISK_SIZE);
#elif CONFIG_SRAM_SIZE >= 0x02000000
		__memcpy( (void *)0x81400000, 
			(void *)((unsigned long)&ramd_start), 
			_RAMDISK_SIZE);
#else
		__memcpy( (void *)0x80a10000, 
			(void *)((unsigned long)&ramd_start), 
			_RAMDISK_SIZE);
#endif



	}
#if 1 //wschen 2007-06-18 put ramdisk size in SIMD SRAM
	*((volatile unsigned int *)0xc0f00000) = _RAMDISK_SIZE;
#endif
#else
	{
		extern void ramdisk_set(void*, void*);
#if CONFIG_SRAM_SIZE >= 0x04000000
	ramdisk_set((void *)0x82800000, (void *)&ramd_start);
#elif CONFIG_SRAM_SIZE >= 0x02000000
	ramdisk_set((void *)0x81400000, (void *)&ramd_start);
#else
	ramdisk_set((void *)0x80a10000, (void *)&ramd_start);
#endif
	}
#endif

	decompress_kernel();

	relocate_vm();
}

void force_reset(void)
{
	int i;
#ifdef CONFIG_ARC700_SERIAL
	__puts("....BUG! force reset...\n");
#endif
	/* add some delay */
	for (i=0; i<10000; i++)
		;
	while(1)
	{
		*((volatile unsigned int *)0xc0fc1110) = 0x00000000;
	}
}

#if 0
/* handle the irq */
void process_interrupt(unsigned int irq, struct pt_regs *fp)
{
__puts("process_interrupt\n");

	return;
}


asmlinkage void do_page_fault(struct pt_regs *regs, int write,
							  unsigned long address)
{
	return;
}
#endif

