/*
 * arch/sh/boot/compressed/misc.c
 * 
 * This is a collection of several routines from gzip-1.0.3 
 * adapted for Linux.
 *
 * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
 *
 */

#include <linux/config.h>
#include <asm/uaccess.h>
#include <asm/serial.h>

/*
 * gzip declarations
 */

#define OF(args)  args
#define STATIC static

#undef memset
#undef memcpy
#undef memzero
#define memzero(s, n)     memset ((s), 0, (n))

typedef unsigned char  uch;
typedef unsigned short ush;
typedef unsigned long  ulg;

#define WSIZE 0x8000		/* Window size must be at least 32k, */
				/* and a power of two */

static uch *inbuf;	     /* input buffer */
//static uch window[WSIZE];    /* Sliding window buffer */
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
static unsigned char *window=NULL;
static unsigned insize = 0;  /* valid bytes in inbuf */
static unsigned inptr = 0;   /* index of next byte to be processed in inbuf */
static unsigned outcnt = 0;  /* bytes in output buffer */
#endif

#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
/* gzip flag byte */
#define ASCII_FLAG   0x01 /* bit 0 set: file probably ASCII text */
#define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
#define EXTRA_FIELD  0x04 /* bit 2 set: extra field present */
#define ORIG_NAME    0x08 /* bit 3 set: original file name present */
#define COMMENT      0x10 /* bit 4 set: file comment present */
#define ENCRYPTED    0x20 /* bit 5 set: file is encrypted */
#define RESERVED     0xC0 /* bit 6,7:   reserved */

#define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
#endif
		
/* Diagnostic functions */
#ifdef DEBUG
#  define verbose 2
#  define Assert(cond,msg) {if(!(cond)) error(msg);}
#  define Trace(x) fprintf x
#  define Tracev(x) {if (verbose) fprintf x ;}
#  define Tracevv(x) {if (verbose>1) fprintf x ;}
#  define Tracec(c,x) {if (verbose && (c)) fprintf x ;}
#  define Tracecv(c,x) {if (verbose>1 && (c)) fprintf x ;}
#else
#  define Assert(cond,msg)
#  define Trace(x)
#  define Tracev(x)
#  define Tracevv(x)
#  define Tracec(c,x)
#  define Tracecv(c,x)
#endif

static int  fill_inbuf(void);
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
static void flush_window(void);
#endif
static void error(char *m);
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
static void gzip_mark(void **);
static void gzip_release(void **);
#endif
  
extern char input_data[];
extern char input_data_end[];
extern char logo_start[];
extern char logo_end[];
extern char audio_start[];
extern char audio_end[];
extern void relocate_vm(void);
//extern int input_len;
unsigned int decompress_target=0;
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
static long bytes_out = 0;
#else
static unsigned int bytes_out = 0;
#endif
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
static uch *output_data;
#endif
static unsigned long output_ptr = 0;

unsigned int logo_output = 0x80c00000;
unsigned int audio_output = 0x80b00000;
void *malloc(int size);
static void free(void *where);
static void error(char *m);
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
static void gzip_mark(void **);
static void gzip_release(void **);
#endif
#ifdef CONFIG_ARC700_SERIAL
extern void __puts(const char *); 
#else
#define __puts(...)
#endif
//void puts(const char *);
extern unsigned int OUTPUT_OFFSET;
extern int _text;		/* Defined in vmlinux.lds.S */
extern int _end;
static unsigned long free_mem_ptr;
static unsigned long free_mem_end_ptr;

//#define HEAP_SIZE             0x10000
#define HEAP_SIZE             0x20000
//#include "../../../../lib/inflate.c"

#include "inflate.c"

void *malloc(int size)
{
	void *p;

	if (size <0) error("Malloc error\n");
	if (free_mem_ptr == 0) error("Memory error\n");

	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */

	p = (void *)free_mem_ptr;
	free_mem_ptr += size;

	if (free_mem_ptr >= free_mem_end_ptr)
		error("\nOut of memory\n");

	return p;
}

static void free(void *where)
{	/* Don't care */
}
static void gzip_mark(void **ptr)
{
	*ptr = (void *) free_mem_ptr;
}

static void gzip_release(void **ptr)
{
	free_mem_ptr = (long) *ptr;
}

void* memset(void* s, int c, size_t n)
{
	int i;
	char *ss = (char*)s;

	for (i=0;i<n;i++) ss[i] = c;
	return s;
}

void* memcpy(void* __dest, __const void* __src,
			    size_t __n)
{
	int i;
	char *d = (char *)__dest, *s = (char *)__src;

	for (i=0;i<__n;i++) d[i] = s[i];
	return __dest;
}

/* ===========================================================================
 * Fill the input buffer. This is called only when the buffer is empty
 * and at least one byte is really needed.
 */
static int fill_inbuf(void)
{
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
	if (insize != 0) {
		error("ran out of input data\n");
	}
#endif

	if (decompress_target == 0)	//kernel
	{
 	  inbuf = input_data;
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
 	  insize = &input_data_end[0] - &input_data[0];
#endif
        }
        else if (decompress_target == 1)	//logo
        {
 	  inbuf = logo_start;
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
 	  insize = &logo_end[0] - &logo_start[0];
#endif
        }
        else if (decompress_target == 2)	//audio
        {
 	  inbuf = audio_start;
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //robert 2007-10-29 
 	  insize = &audio_end[0] - &audio_start[0];
#endif
        }
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
	inptr = 1;
#endif
	return inbuf[0];
}

#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
/* ===========================================================================
 * Write the output window window[0..outcnt-1] and update crc and bytes_out.
 * (Used for the decompressed data only.)
 */
static void flush_window(void)
{
    ulg c = crc;         /* temporary variable */
    unsigned n;
    uch *in, *out, ch;
    
    in = window;
    out = &output_data[output_ptr]; 
    for (n = 0; n < outcnt; n++) {
	    ch = *out++ = *in++;
	    c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
    }
    crc = c;
    bytes_out += (ulg)outcnt;
    output_ptr += (ulg)outcnt;
    outcnt = 0;
}
#endif

static void error(char *x)
{
	__puts("\n\n");
	__puts(x);
	__puts("\n\n -- System halted");

	while(1);	/* Halt */
}

#ifndef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
#include "lzma.h"

#define MyMin(a,b) a < b ? a : b
#define MyMax(a,b) a > b ? a : b

#define NULL   0

#ifndef E_INVALIDARG
#define E_INVALIDARG    -1;
#endif

#define RETURN_IF_NOT_S_OK(x) { int __result_ = (x); if(__result_ != 0) return __result_; }

/**********************************	Dictionary Size	***********************************/
#define DEF_DICTIONARY_SIZE               0x200000
#define DEF_LITERAL_POS_STATE_BITS        0
#define DEF_LITERAL_CONTEXT_BITS          3
#define DEF_POS_STATE_BITS                2





/********************************  RangeCoder	********************************************/


inline void CRangeDecoder_Normalize(CRangeDecoder *rDecoder)
{

  while (rDecoder->Range < kTopValue)
  {
    rDecoder->Code = (rDecoder->Code << 8) | *(rDecoder->local_ptr++);
    rDecoder->Range <<= 8;
  }
}


inline void CRangeDecoder_Init(CRangeDecoder *rDecoder, unsigned char *stream)
{
  int i=0;

  rDecoder->local_ptr = stream;
  rDecoder->Code = 0;
  rDecoder->Range = (unsigned int)-1;

  for(i = 0; i < 5; i++)
  {
    rDecoder->Code = (rDecoder->Code << 8) | *(rDecoder->local_ptr++);
  }
}

#if 0 //wschen 2007-05-10 
void CRangeDecoder_ReleaseStream(CRangeDecoder *rDecoder)
{
  /* Stream.ReleaseStream(); */
}
#endif


inline UINT32 CRangeDecoder_GetThreshold(CRangeDecoder *rDecoder, UINT32 total)
{

  return (rDecoder->Code) / ((rDecoder->Range) /= total);
}


inline void CRangeDecoder_Decode(CRangeDecoder *rDecoder, UINT32 start, UINT32 size, UINT32 total)
{

  rDecoder->Code -= start * (rDecoder->Range);
  rDecoder->Range *= size;

  while (rDecoder->Range < kTopValue)
  {
    rDecoder->Code = (rDecoder->Code << 8) | *(rDecoder->local_ptr++);
    rDecoder->Range <<= 8;
  }
}


inline UINT32 CRangeDecoder_DecodeDirectBits(CRangeDecoder *rDecoder, UINT32 numTotalBits)
{
  UINT32 t,i = 0;
  UINT32 range = rDecoder->Range;
  UINT32 code = rDecoder->Code;
  UINT32 result = 0;

  for (i = numTotalBits; i > 0; i--)
  {
    range >>= 1;
    t = (code - range) >> 31;
    code -= range & (t - 1);
    result = (result << 1) | (1 - t);

    if (range < kTopValue)
    {
      code = (code << 8) | *(rDecoder->local_ptr++);
      range <<= 8;
    }
  }
  rDecoder->Range = range;
  rDecoder->Code = code;

  return result;
}


inline UINT32 CRangeDecoder_DecodeBit(CRangeDecoder *rDecoder, UINT32 size0, UINT32 numTotalBits)
{
  UINT32 newBound = ((rDecoder->Range) >> numTotalBits) * size0;
  UINT32 symbol;

  if (rDecoder->Code < newBound)
  {
    symbol = 0;
    rDecoder->Range = newBound;
  }
  else
  {
    symbol = 1;
    rDecoder->Code -= newBound;
    rDecoder->Range -= newBound;
  }

  while (rDecoder->Range < kTopValue)
  {
    rDecoder->Code = (rDecoder->Code << 8) | *(rDecoder->local_ptr++);
    rDecoder->Range <<= 8;
  }

  return symbol;
}

#if 0 //wschen 2007-05-10 
UINT32 CRangeDecoder_GetProcessedSize(CRangeDecoder *rDecoder)
{

  return 0;
}
#endif


/********************************  AriBitCoder	********************************************/


inline void CMyBitDecoder_Init(CMyBitDecoder *bitDecoder)
{

  bitDecoder->Probability = kBitModelTotal/2;
}


inline UINT32 CMyBitDecoder_Decode(CMyBitDecoder *bitDecoder, CRangeDecoder *rangeDecoder)
{
  UINT32 newBound = (rangeDecoder->Range >> kNumBitModelTotalBits) * (bitDecoder->Probability);
  if (rangeDecoder->Code < newBound)
  {
    rangeDecoder->Range = newBound;
    bitDecoder->Probability += (kBitModelTotal - bitDecoder->Probability) >> (bitDecoder->aNumMoveBits);
    if (rangeDecoder->Range < kTopValue)
    {
      rangeDecoder->Code = (rangeDecoder->Code << 8) | *(rangeDecoder->local_ptr++);;
      rangeDecoder->Range <<= 8;
    }
    return 0;
  }
  else
  {
    rangeDecoder->Range -= newBound;
    rangeDecoder->Code -= newBound;
    bitDecoder->Probability -= (bitDecoder->Probability) >> (bitDecoder->aNumMoveBits);
    if (rangeDecoder->Range < kTopValue)
    {
      rangeDecoder->Code = (rangeDecoder->Code << 8) | *(rangeDecoder->local_ptr++);
      rangeDecoder->Range <<= 8;
    }
    return 1;
  }
}


/********************************  BitTreeCoder	********************************************/


inline void CBitTreeDecoder_Init(CBitTreeDecoder *btDecoder)
{
  UINT32 i;

  for(i = 1; i < (1 << btDecoder->NumBitLevels); i++)
    CMyBitDecoder_Init(&(btDecoder->Models[i]));
}


inline UINT32 CBitTreeDecoder_Decode(CBitTreeDecoder *btDecoder, CMyRangeDecoder *rangeDecoder)
{
  UINT32 bitIndex, modelIndex = 1;
  RC_INIT_VAR

    for(bitIndex = btDecoder->NumBitLevels; bitIndex > 0; bitIndex--)
    {
      RC_GETBIT(btDecoder->numMoveBits, btDecoder->Models[modelIndex].Probability, modelIndex)
    }
  RC_FLUSH_VAR

    return modelIndex - (1 << btDecoder->NumBitLevels);
}


inline UINT8 CReverseBitTreeDecoder2_Create(CReverseBitTreeDecoder2 *rbtDecoder2, UINT32 numBitLevels)
{

  rbtDecoder2->NumBitLevels = numBitLevels;
  rbtDecoder2->Models = (CMyBitDecoder *)malloc(sizeof(CMyBitDecoder) * (1 << numBitLevels));
  if (rbtDecoder2->Models != NULL)
    return (1);
  else
    return (-1);
}


inline void CReverseBitTreeDecoder2_Init(CReverseBitTreeDecoder2 *rbtDecoder2)
{
  UINT32 i;
  UINT32 numModels = 1 << (rbtDecoder2->NumBitLevels);

  for(i = 1; i < numModels; i++)
    CMyBitDecoder_Init(&(rbtDecoder2->Models[i]));
}


inline UINT32 CReverseBitTreeDecoder2_Decode(CReverseBitTreeDecoder2 *rbtDecoder2, CMyRangeDecoder *rangeDecoder)
{
  UINT32 modelIndex = 1;
  UINT32 symbol = 0;
  UINT32 bitIndex;
  RC_INIT_VAR

    for(bitIndex = 0; bitIndex < rbtDecoder2->NumBitLevels; bitIndex++)
    {
      RC_GETBIT2(rbtDecoder2->numMoveBits, rbtDecoder2->Models[modelIndex].Probability, modelIndex, ; , symbol |= (1 << bitIndex))
    }
  RC_FLUSH_VAR
    return symbol;
}


inline void CReverseBitTreeDecoder_Init(CReverseBitTreeDecoder *rbtDecoder)
{
  UINT32 i;

  for(i = 1; i < (1 << (rbtDecoder->NumBitLevels)); i++)
    CMyBitDecoder_Init(&(rbtDecoder->Models[i]));
}


inline UINT32 CReverseBitTreeDecoder_Decode(CReverseBitTreeDecoder *rbtDecoder, CMyRangeDecoder *rangeDecoder)
{
  UINT32 modelIndex = 1;
  UINT32 symbol = 0;
  UINT32 bitIndex;
  RC_INIT_VAR

    for(bitIndex = 0; bitIndex < (rbtDecoder->NumBitLevels); bitIndex++)
    {
      RC_GETBIT2(rbtDecoder->numMoveBits, rbtDecoder->Models[modelIndex].Probability, modelIndex, ; , symbol |= (1 << bitIndex))
    }
  RC_FLUSH_VAR

    return symbol;
}



/********************************  LenCoder	********************************************/


inline void CDecoderLength_Initialize(CDecoderLength *decoderLength)
{
  int i;

  decoderLength->_choice.aNumMoveBits = kNumMoveBits;
  decoderLength->_choice2.aNumMoveBits = kNumMoveBits;

  for(i=0; i < kNumPosStatesMax; i++)
  {
    decoderLength->_lowCoder[i].numMoveBits 	= kNumMoveBits;
    decoderLength->_lowCoder[i].NumBitLevels 	= kNumLenBits;
    decoderLength->_lowCoder[i].Models	     	= (CMyBitDecoder *)malloc(sizeof(CMyBitDecoder) * (1 << kNumLenBits));

    decoderLength->_midCoder[i].numMoveBits 	= kNumMoveBits;
    decoderLength->_midCoder[i].NumBitLevels 	= kNumMidBits;
    decoderLength->_midCoder[i].Models	     	= (CMyBitDecoder *)malloc(sizeof(CMyBitDecoder) * (1 << kNumMidBits));
  }

  decoderLength->_highCoder.numMoveBits 	= kNumMoveBits;
  decoderLength->_highCoder.NumBitLevels 	= kNumHighBits;
  decoderLength->_highCoder.Models		= (CMyBitDecoder *)malloc(sizeof(CMyBitDecoder) * (1 << kNumHighBits));
}


inline void CDecoderLength_Create(CDecoderLength *decoderLength, UINT32 numPosStates)
{

  decoderLength->_numPosStates = numPosStates;
}


inline void CDecoderLength_Init(CDecoderLength *decoderLength)
{
  UINT32 posState;

  //Ramesh ???
  CDecoderLength_Initialize(decoderLength);

  CMyBitDecoder_Init(&(decoderLength->_choice));

  for (posState = 0; posState < decoderLength->_numPosStates; posState++)
  {
    CBitTreeDecoder_Init(&(decoderLength->_lowCoder[posState]));
    CBitTreeDecoder_Init(&(decoderLength->_midCoder[posState]));
  }

  CMyBitDecoder_Init(&(decoderLength->_choice2));
  CBitTreeDecoder_Init(&(decoderLength->_highCoder));
}


inline UINT32 CDecoderLength_Decode(CDecoderLength *decoderLength, CMyRangeDecoder *rangeDecoder, UINT32 posState)
{

  if (CMyBitDecoder_Decode(&(decoderLength->_choice),rangeDecoder) == 0)
  {
    return (CBitTreeDecoder_Decode(&(decoderLength->_lowCoder[posState]),rangeDecoder));
  }
  if (CMyBitDecoder_Decode(&(decoderLength->_choice2),rangeDecoder) == 0)
  {
    return (kNumLowSymbols + CBitTreeDecoder_Decode(&(decoderLength->_midCoder[posState]),rangeDecoder));
  }
  return (kNumLowSymbols + kNumMidSymbols + CBitTreeDecoder_Decode(&(decoderLength->_highCoder),rangeDecoder));
}



/********************************  LiteralCoder	********************************************/


inline void CDecoder2_Init(CDecoder2 *decoder2)
{
  int i,j;

  for (i = 0; i < 3; i++)
  {
    for (j = 1; j < (1 << 8); j++)
    {
      CMyBitDecoder_Init(&(decoder2->_decoders[i][j]));
      decoder2->_decoders[i][j].aNumMoveBits = kNumMoveBits;
    }
  }
}


inline BYTE CDecoder2_DecodeNormal(CDecoder2 *decoder2,CMyRangeDecoder *rangeDecoder)
{
  UINT32 symbol = 1;
  RC_INIT_VAR

    do
    {
      RC_GETBIT(kNumMoveBits, decoder2->_decoders[0][symbol].Probability, symbol);
    }
    while (symbol < 0x100);
  RC_FLUSH_VAR

    return symbol;
}


inline BYTE CDecoder2_DecodeWithMatchByte(CDecoder2 *decoder2,CMyRangeDecoder *rangeDecoder, BYTE matchByte)
{
  UINT32 symbol = 1;
  RC_INIT_VAR

    do
    {
      UINT32 bit;
      UINT32 matchBit = (matchByte >> 7) & 1;
      matchByte <<= 1;

      RC_GETBIT2(kNumMoveBits, decoder2->_decoders[1 + matchBit][symbol].Probability, symbol, bit = 0, bit = 1)
	if (matchBit != bit)
	{
	  while (symbol < 0x100)
	  {
	    RC_GETBIT(kNumMoveBits, decoder2->_decoders[0][symbol].Probability, symbol)
	  }
	  break;
	}
    }
    while (symbol < 0x100);
  RC_FLUSH_VAR

    return symbol;
}


inline void CDecoderLiteral_Create(CDecoderLiteral *decoderLiteral,UINT32 numPosBits, UINT32 numPrevBits)
{
  UINT32 numStates;

  decoderLiteral->_numPosBits = numPosBits;
  decoderLiteral->_posMask = (1 << numPosBits) - 1;
  decoderLiteral->_numPrevBits = numPrevBits;
  numStates = 1 << (decoderLiteral->_numPrevBits + decoderLiteral->_numPosBits);
  decoderLiteral->_coders = (CDecoder2 *)malloc(sizeof(CDecoder2) * numStates);
}


inline void CDecoderLiteral_Init(CDecoderLiteral *decoderLiteral)
{
  UINT32 i;
  UINT32 numStates = 1 << (decoderLiteral->_numPrevBits + decoderLiteral->_numPosBits);

  for (i = 0; i < numStates; i++)
  {
    CDecoder2_Init(&(decoderLiteral->_coders[i]));
  }
}


inline UINT32 CDecoderLiteral_GetState(CDecoderLiteral *decoderLiteral,UINT32 pos, BYTE prevByte)
{

  return ((pos & decoderLiteral->_posMask) << decoderLiteral->_numPrevBits) + (prevByte >> (8 - decoderLiteral->_numPrevBits));
}


inline BYTE CDecoderLiteral_DecodeNormal(CDecoderLiteral *decoderLiteral,CMyRangeDecoder *rangeDecoder, UINT32 pos, BYTE prevByte)
{
  UINT32 state;

  state = CDecoderLiteral_GetState(decoderLiteral,pos,prevByte);
  return CDecoder2_DecodeNormal(&(decoderLiteral->_coders[state]),rangeDecoder);
}


inline BYTE CDecoderLiteral_DecodeWithMatchByte(CDecoderLiteral *decoderLiteral,CMyRangeDecoder *rangeDecoder, UINT32 pos, BYTE prevByte, BYTE matchByte)
{
  UINT32 state;

  state = CDecoderLiteral_GetState(decoderLiteral,pos,prevByte);
  return CDecoder2_DecodeWithMatchByte(&(decoderLiteral->_coders[state]),rangeDecoder,matchByte);
}




/********************************  LZMA	********************************************/



inline void CState_Init(CState *state)
{

  state->Index = 0;
}


inline void UpdateChar(CState *state)
{

  state->Index = kLiteralNextStates[state->Index];
}


inline void UpdateMatch(CState *state)
{

  state->Index = kMatchNextStates[state->Index];
}


inline void UpdateRep(CState *state)
{
  state->Index = kRepNextStates[state->Index];
}


inline void UpdateShortRep(CState *state)
{

  state->Index = kShortRepNextStates[state->Index];
}


inline void CBaseCoder_Init(CBaseCoder *baseCoder)
{
  int i;

  CState_Init(&(baseCoder->_state));
  baseCoder->_previousByte = 0;
  for(i = 0 ; i < kNumRepDistances; i++)
    baseCoder->_repDistances[i] = 0;
}



/********************************  Decoder	********************************************/


inline void RareOutputStream_OutStreamInit(RareOutputStream *routStream, unsigned char *ptr)
{

  routStream->_buffer = ptr;
  routStream->_pos = 0;
}


inline void RareOutputStream_PutOneByte(RareOutputStream *routStream, BYTE b)
{

  routStream->_buffer[routStream->_pos++] = b;
}


inline unsigned char RareOutputStream_GetOneByte(RareOutputStream *routStream, unsigned int index)
{
  unsigned int pos = routStream->_pos + index;

  return routStream->_buffer[pos];
}


inline void RareOutputStream_CopyBackBlock(RareOutputStream *routStream, unsigned int distance, unsigned int len)
{
  unsigned int pos = routStream->_pos - distance - 1;

  for(; len > 0; len--)
  {
    routStream->_buffer[routStream->_pos++] = routStream->_buffer[pos++];
  }
}


inline void CDecoder_CDecoder(CDecoder *decoder)
{

  init_RangeCoder_Variables();
  init_AriBitCoder_Variables();
  init_LenCoder_Variables();
  init_LZMA_Variables();
  Create(decoder);
}


int Create(CDecoder *decoder)
{
  int i;

  for(i = 0; i < kNumPosModels; i++)
  {
    CReverseBitTreeDecoder2_Create(&(decoder->_posDecoders[i]), (((kStartPosModelIndex + i) >> 1) - 1));
  }

  return(0);
}


inline int Init(CDecoder *decoder, unsigned char *InPtr, unsigned char *OutPtr)
{
  int i,j;

  CRangeDecoder_Init(&(decoder->_rangeDecoder),InPtr);
  RareOutputStream_OutStreamInit(&(decoder->_outWindowStream), OutPtr);

  for(i = 0; i < kNumStates; i++)
  {
    for (j = 0; j <= decoder->_posStateMask; j++)
    {
      CMyBitDecoder_Init(&(decoder->_mainChoiceDecoders[i][j]));
      CMyBitDecoder_Init(&(decoder->_matchRepShortChoiceDecoders[i][j]));

      decoder->_mainChoiceDecoders[i][j].aNumMoveBits = kNumMoveBitsForMainChoice;
      decoder->_matchRepShortChoiceDecoders[i][j].aNumMoveBits = kNumMoveBitsForMainChoice;
    }

    CMyBitDecoder_Init(&(decoder->_matchChoiceDecoders[i]));
    CMyBitDecoder_Init(&(decoder->_matchRepChoiceDecoders[i]));
    CMyBitDecoder_Init(&(decoder->_matchRep1ChoiceDecoders[i]));
    CMyBitDecoder_Init(&(decoder->_matchRep2ChoiceDecoders[i]));

    decoder->_matchChoiceDecoders[i].aNumMoveBits = kNumMoveBitsForMainChoice;
    decoder->_matchRepChoiceDecoders[i].aNumMoveBits = kNumMoveBitsForMainChoice;
    decoder->_matchRep1ChoiceDecoders[i].aNumMoveBits = kNumMoveBitsForMainChoice;
    decoder->_matchRep2ChoiceDecoders[i].aNumMoveBits = kNumMoveBitsForMainChoice;
  }

  CDecoderLiteral_Init(&(decoder->_literalDecoder));

  for (i = 0; i < kNumLenToPosStates; i++)
  {
    decoder->_posSlotDecoder[i].numMoveBits = kNumMoveBitsForPosSlotCoder;
    decoder->_posSlotDecoder[i].NumBitLevels = kNumPosSlotBits;
    decoder->_posSlotDecoder[i].Models = (CMyBitDecoder *)malloc(sizeof(CMyBitDecoder) * (1 << kNumPosSlotBits));
    CBitTreeDecoder_Init(&(decoder->_posSlotDecoder[i]));
  }

  for(i = 0; i < kNumPosModels; i++)
  {
    decoder->_posDecoders[i].numMoveBits = kNumMoveBitsForPosCoders;
    CReverseBitTreeDecoder2_Init(&(decoder->_posDecoders[i]));
  }

  CDecoderLength_Init(&(decoder->_lenDecoder));
  CDecoderLength_Init(&(decoder->_repMatchLenDecoder));

  decoder->_posAlignDecoder.numMoveBits = kNumMoveBitsForAlignCoders;
  decoder->_posAlignDecoder.NumBitLevels = kNumAlignBits;
  decoder->_posAlignDecoder.Models = (CMyBitDecoder *)malloc(sizeof(CMyBitDecoder) * (1 << kNumAlignBits));
  CReverseBitTreeDecoder_Init(&(decoder->_posAlignDecoder));

  return(0);
}


inline int CodeReal(CDecoder *decoder, unsigned char *InPtr,unsigned char *OutPtr, const UINT32 *inSize, const UINT32 *outSize, unsigned int *progress)
{
  CState state;
  int i;
  unsigned int nowPos32 = 0;
  unsigned int size = (outSize == NULL) ? (unsigned int)(int)(-1) : *outSize;
  int peviousIsMatch = -1;
  BYTE previousByte = 0;
  unsigned int repDistances[kNumRepDistances];


  RareOutputStream *stream 	= &(decoder->_outWindowStream);
  CMyRangeDecoder  *rDecoder 	= &(decoder->_rangeDecoder);
  CReverseBitTreeDecoder *rbtDecoder = &(decoder->_posAlignDecoder);
  CDecoderLength  *lenDecoder  = &(decoder->_lenDecoder);
  CDecoderLength  *repMatchLenDecoder  = &(decoder->_repMatchLenDecoder);
  CDecoderLiteral *literalDecoder = &(decoder->_literalDecoder);

  Init(decoder,InPtr,OutPtr);

  CState_Init(&state);

  for(i = 0 ; i < kNumRepDistances; i++)
    repDistances[i] = 0;

  while(nowPos32 < size)
  {
    unsigned int nextPos = MyMin(nowPos32 + (1 << 18), size);
    while(nowPos32 < nextPos)
    {
      unsigned int posState = (unsigned int)(nowPos32) & decoder->_posStateMask;


      if (CMyBitDecoder_Decode(&(decoder->_mainChoiceDecoders[state.Index][posState]), rDecoder) == kMainChoiceLiteralIndex)
      {
	UpdateChar(&state);
	if(peviousIsMatch > 0)
	{
	  BYTE matchByte = RareOutputStream_GetOneByte(stream, (0 - repDistances[0] - 1));
	  previousByte = CDecoderLiteral_DecodeWithMatchByte(literalDecoder,rDecoder,(unsigned int)(nowPos32),previousByte,matchByte);
	  peviousIsMatch = -1;
	}
	else
	{
	  previousByte = CDecoderLiteral_DecodeNormal(literalDecoder,rDecoder, (unsigned int)(nowPos32), previousByte);
	}
	RareOutputStream_PutOneByte(stream, previousByte);
	nowPos32++;
      }
      else
      {
	unsigned int distance, len;
	peviousIsMatch = 1;

	if(CMyBitDecoder_Decode(&(decoder->_matchChoiceDecoders[state.Index]), rDecoder) == kMatchChoiceRepetitionIndex)
	{
	  if(CMyBitDecoder_Decode(&(decoder->_matchRepChoiceDecoders[state.Index]), rDecoder) == 0)
	  {
	    if(CMyBitDecoder_Decode(&(decoder->_matchRepShortChoiceDecoders[state.Index][posState]), rDecoder) == 0)
	    {
	      UpdateShortRep(&state);
	      previousByte = RareOutputStream_GetOneByte(stream,(0 - repDistances[0] - 1));
	      RareOutputStream_PutOneByte(stream, previousByte);
	      nowPos32++;
	      continue;
	    }
	    distance = repDistances[0];
	  }
	  else
	  {
	    if(CMyBitDecoder_Decode(&(decoder->_matchRep1ChoiceDecoders[state.Index]), rDecoder) == 0)
	      distance = repDistances[1];
	    else
	    {
	      if(CMyBitDecoder_Decode(&(decoder->_matchRep2ChoiceDecoders[state.Index]), rDecoder) == 0)
		distance = repDistances[2];
	      else
	      {
		distance = repDistances[3];
		repDistances[3] = repDistances[2];
	      }
	      repDistances[2] = repDistances[1];
	    }
	    repDistances[1] = repDistances[0];
	    repDistances[0] = distance;
	  }
	  len = CDecoderLength_Decode(repMatchLenDecoder, rDecoder, posState) + kMatchMinLen;

	  UpdateRep(&state);
	}
	else
	{
	  unsigned int posSlot;
	  len = CDecoderLength_Decode(lenDecoder, rDecoder, posState) + kMatchMinLen;
	  UpdateMatch(&state);
	  posSlot = CBitTreeDecoder_Decode(&(decoder->_posSlotDecoder[GetLenToPosState(len)]), rDecoder);
	  if (posSlot >= kStartPosModelIndex)
	  {
	    unsigned int numDirectBits = (posSlot >> 1) - 1;
	    distance = ((2 | (posSlot & 1)) << numDirectBits);

	    if (posSlot < kEndPosModelIndex)
	      distance += CReverseBitTreeDecoder2_Decode(&(decoder->_posDecoders[posSlot - kStartPosModelIndex]), rDecoder);
	    else
	    {
	      distance += (CRangeDecoder_DecodeDirectBits(rDecoder,(numDirectBits - kNumAlignBits)) << kNumAlignBits);
	      distance += CReverseBitTreeDecoder_Decode(rbtDecoder, rDecoder);
	    }
	  }
	  else
	    distance = posSlot;

	  repDistances[3] = repDistances[2];
	  repDistances[2] = repDistances[1];
	  repDistances[1] = repDistances[0];
	  repDistances[0] = distance;
	}
	if (distance >= nowPos32 || distance >= decoder->_dictionarySizeCheck)
	{
	  if (distance == (unsigned int)(-1) && size == (unsigned int)(int)(-1))
	  {
	    return 0;
	  }
	}

	RareOutputStream_CopyBackBlock(stream,distance, len);
	nowPos32 += len;
	previousByte = RareOutputStream_GetOneByte(stream,(0 - 1));
      }
    }

  }

  return(0);
}


inline int SetDictionarySize(CDecoder *decoder, UINT32 dictionarySize)
{

  if (decoder->_dictionarySize != dictionarySize)
  {
    unsigned int blockSize;
    decoder->_dictionarySize = dictionarySize;
    decoder->_dictionarySizeCheck = MyMax(decoder->_dictionarySize, (unsigned int)(1));
    blockSize = MyMax(decoder->_dictionarySizeCheck, (unsigned int)(1 << 12));
  }

  return(0);
}


inline int SetLiteralProperties(CDecoder *decoder, UINT32 numLiteralPosStateBits, UINT32 numLiteralContextBits)
{

  if (numLiteralPosStateBits > 8)
    return E_INVALIDARG;
  if (numLiteralContextBits > 8)
    return E_INVALIDARG;

  CDecoderLiteral_Create(&(decoder->_literalDecoder),numLiteralPosStateBits, numLiteralContextBits);

  return(0);
}


inline int SetPosBitsProperties(CDecoder *decoder, UINT32 numPosStateBits)
{
  unsigned int numPosStates;

  if (numPosStateBits > kNumPosStatesBitsMax)
    return E_INVALIDARG;

  numPosStates = 1 << numPosStateBits;
  CDecoderLength_Create(&(decoder->_lenDecoder), numPosStates);
  CDecoderLength_Create(&(decoder->_repMatchLenDecoder), numPosStates);
  decoder->_posStateMask = numPosStates - 1;

  return(0);
}



int cm_hwDecodeLZMA(unsigned char *pbDest, unsigned int uiDecompressedLength, unsigned char *pbSource, unsigned int uiCompressedLength, void *fptr)
{
  CDecoder MyDecoder;
  //wschen
  int res;
  int i;

  CDecoder_CDecoder(&MyDecoder);
  SetDictionarySize(&MyDecoder,DEF_DICTIONARY_SIZE);
  SetLiteralProperties(&MyDecoder, DEF_LITERAL_POS_STATE_BITS, DEF_LITERAL_CONTEXT_BITS);
  SetPosBitsProperties(&MyDecoder, DEF_POS_STATE_BITS);

  /*======================================================
    Decode
    =======================================================*/
  res = CodeReal(&MyDecoder, pbSource + 0x20, pbDest, &uiCompressedLength, &uiDecompressedLength, 0);

  if (MyDecoder._posAlignDecoder.Models) {
    free(MyDecoder._posAlignDecoder.Models);
  }

  for (i = 0; i < kNumLenToPosStates; i++) {
    if (MyDecoder._posSlotDecoder[i].Models) {
      free(MyDecoder._posSlotDecoder[i].Models);
    }
  }

  if (MyDecoder._literalDecoder._coders) {
    free(MyDecoder._literalDecoder._coders);
  }

  for (i = 0; i < kNumPosStatesMax; i++) {
    if (MyDecoder._lenDecoder._lowCoder[i].Models) {
      free(MyDecoder._lenDecoder._lowCoder[i].Models);
    }
    if (MyDecoder._lenDecoder._midCoder[i].Models) {
      free(MyDecoder._lenDecoder._midCoder[i].Models);
    }
  }

  if (MyDecoder._lenDecoder._highCoder.Models) {
    free(MyDecoder._lenDecoder._highCoder.Models);
  }

  for (i = 0; i < kNumPosStatesMax; i++) {
    if (MyDecoder._repMatchLenDecoder._lowCoder[i].Models) {
      free(MyDecoder._repMatchLenDecoder._lowCoder[i].Models);
    }
    if (MyDecoder._repMatchLenDecoder._midCoder[i].Models) {
      free(MyDecoder._repMatchLenDecoder._midCoder[i].Models);
    }
  }

  if (MyDecoder._repMatchLenDecoder._highCoder.Models) {
    free(MyDecoder._repMatchLenDecoder._highCoder.Models);
  }


  for (i = 0; i < kNumPosModels; i++) {
    if (MyDecoder._posDecoders[i].Models) {
      free(MyDecoder._posDecoders[i].Models);
    }
  }

  return res;
}

#endif

void decompress_kernel(void)
{
        decompress_target = 0;
        bytes_out = 0;
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
        insize = 0;
        inptr = 0;
        outcnt = 0;
	output_data = 0;
#else
	unsigned char *ptr;
	unsigned int uncompr_size;
#endif
	output_ptr = (unsigned long)(CONFIG_CODE_OFFSET_TEXTBASE + OUTPUT_OFFSET);
/* set free_mem_ptr at memory 15M + 0x1000 */
#if CONFIG_SRAM_SIZE >= 0x04000000
	free_mem_ptr = (unsigned long)(0x83f01000 );
#elif CONFIG_SRAM_SIZE >= 0x02000000
	free_mem_ptr = (unsigned long)(0x81f01000 );
#else
	free_mem_ptr = (unsigned long)(0x80f01000 );
#endif
	free_mem_end_ptr = free_mem_ptr + HEAP_SIZE;

#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
	window = malloc(WSIZE);

	makecrc();
#endif
	__puts("Uncompressing Linux... ");

#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
	gunzip();
#else
	fill_inbuf();
	ptr = inbuf;

	uncompr_size = *((unsigned int *)(ptr+20));

	if ((ptr[0] == '7') && (ptr[1] == 'z')) {
	  if (cm_hwDecodeLZMA((void *)output_ptr, uncompr_size, ptr, 0, 0)) {
	    __puts("Kernel compress error\n");
	  }
	} else {
	  __puts("Invalid kernel compressed format\n");
	}
	bytes_out = uncompr_size;
#endif
	__puts("Ok, booting the kernel.\n");
 
}

void decompress_logo(void)
{
        decompress_target = 1;
        bytes_out = 0;
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
        insize = 0;
        inptr = 0;
        outcnt = 0;
	output_data = 0;
#else
	unsigned char *ptr;
	unsigned int uncompr_size;
#endif

/* set free_mem_ptr at memory 15M + 0x1000 */
#if CONFIG_SRAM_SIZE >= 0x04000000
        logo_output = 0x83c00000;
	free_mem_ptr = (unsigned long)(0x83f01000 );
#elif CONFIG_SRAM_SIZE >= 0x02000000
        logo_output = 0x81c00000;
	free_mem_ptr = (unsigned long)(0x81f01000 );
#else
        logo_output = 0x80c00000;
	free_mem_ptr = (unsigned long)(0x80f01000 );
#endif
	free_mem_end_ptr = free_mem_ptr + HEAP_SIZE;

	output_ptr = (unsigned long)(logo_output);

#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
	window = malloc(WSIZE);

	makecrc();
#endif
	__puts("Uncompressing Logo... \n");

#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
	gunzip();
#else
	fill_inbuf();
	ptr = inbuf;

	uncompr_size = *((unsigned int *)(ptr+20));


	if ((ptr[0] == '7') && (ptr[1] == 'z')) {
	  if (cm_hwDecodeLZMA((void *)output_ptr, uncompr_size, ptr, 0, 0)) {
	    __puts("Logo compress error\n");
	  }
	} else {
	  __puts("Invalid logo compressed format\n");
	}

	bytes_out = uncompr_size;
#endif
	__puts("Setup Logo...\n");
 
}

void decompress_audio(void)
{
        decompress_target = 2;
        bytes_out = 0;
#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
        insize = 0;
        inptr = 0;
        outcnt = 0;
	output_data = 0;
#else
	unsigned char *ptr;
	unsigned int uncompr_size;
#endif

/* set free_mem_ptr at memory 15M + 0x1000 */
#if CONFIG_SRAM_SIZE >= 0x04000000
        audio_output = 0x83b00000;
	free_mem_ptr = (unsigned long)(0x83f01000 );
#elif CONFIG_SRAM_SIZE >= 0x02000000
        audio_output = 0x81b00000;
	free_mem_ptr = (unsigned long)(0x81f01000 );
#else
        audio_output = 0x80b00000;
	free_mem_ptr = (unsigned long)(0x80f01000 );
#endif
	free_mem_end_ptr = free_mem_ptr + HEAP_SIZE;

	output_ptr = (unsigned long)(audio_output);

#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
	window = malloc(WSIZE);

	makecrc();
#endif
	__puts("Uncompressing Audio... \n");

#ifdef CONFIG_BOOT_KERNEL_USE_GZIP //wschen 2007-05-07 
	gunzip();
#else
	fill_inbuf();
	ptr = inbuf;

	uncompr_size = *((unsigned int *)(ptr+20));


	if ((ptr[0] == '7') && (ptr[1] == 'z')) {
	  if (cm_hwDecodeLZMA((void *)output_ptr, uncompr_size, ptr, 0, 0)) {
	    __puts("Audio compress error\n");
	  }
	} else {
	  __puts("Invalid audio compressed format\n");
	}

	bytes_out = uncompr_size;
#endif
	__puts("Setup Audio...\n");
 
}


unsigned int get_total_len(void)
{
  return bytes_out;
}
