/*
 * memtester version 4
 *
 * Very simple but very effective user-space memory tester.
 * Originally by Simon Kirby <sim@stormix.com> <sim@neato.org>
 * Version 2 by Charles Cazabon <memtest@discworld.dyndns.org>
 * Version 3 not publicly released.
 * Version 4 rewrite:
 * Copyright (C) 2005 Charles Cazabon <memtest@discworld.dyndns.org>
 * Licensed under the terms of the GNU General Public License version 2 (only).
 * See the file COPYING for details.
 *
 */

#define __version__ "4.0.5"
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/miscdevice.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/arc_mem.h>
#include <asm/page.h>
#include <asm/cache.h>
#include <linux/vmalloc.h>

#include <asm/serial.h>
#include <asm/arcregs.h>
/*
#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <unistd.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
*/
#include "types.h"
#include "sizes.h"
#include "tests.h"

#define EXIT_FAIL_NONSTARTER    0x01
#define EXIT_FAIL_ADDRESSLINES  0x02
#define EXIT_FAIL_OTHERTEST     0x04

void select_memtester_item(void);
extern unsigned long arc700_clkfreq;

int select_test_item=-1;
unsigned int mem_userdefined_startaddr=0;
unsigned int mem_userdefined_pattern=0;
unsigned int mem_userdefined_size=0;
unsigned int mem_userdefined_loops=0;

struct test tests[] = {
    { "Random Value", test_random_value },
    { "Compare XOR", test_xor_comparison },
    { "Compare SUB", test_sub_comparison },
    { "Compare MUL", test_mul_comparison },
    { "Compare DIV",test_div_comparison },
    { "Compare OR", test_or_comparison },
    { "Compare AND", test_and_comparison },
    { "Sequential Increment", test_seqinc_comparison },
    { "Solid Bits", test_solidbits_comparison },
    { "Block Sequential", test_blockseq_comparison },
    { "Checkerboard", test_checkerboard_comparison },
    { "Bit Spread", test_bitspread_comparison },
    { "Bit Flip", test_bitflip_comparison },
    { "Walking Ones", test_walkbits1_comparison },
    { "Walking Zeroes", test_walkbits0_comparison },
    { "Butterfly", test_randomseq_comparison },
    { 0, 0 }
};

#ifdef _SC_VERSION
void check_posix_system(void) {
    if (sysconf(_SC_VERSION) < 198808L) {
        printk("A POSIX system is required.  Don't be surprised if "
            "this craps out.\n");
        printk("_SC_VERSION is %lu\n", sysconf(_SC_VERSION));
    }
}
#else
#define check_posix_system()
#endif

#ifdef _SC_PAGE_SIZE
int memtester_pagesize(void) {
    int pagesize = sysconf(_SC_PAGE_SIZE);
    if (pagesize == -1) {
        printk("get page size failed");
//        exit(EXIT_FAIL_NONSTARTER);
       return 0;
    }
    printk("pagesize is %ld\n", pagesize);
    return pagesize;
}
#else
int memtester_pagesize(void) {
    printk("sysconf(_SC_PAGE_SIZE) not supported; using pagesize of 8192\n");
    return 8192;
}
#endif

//int main(int argc, char **argv) {
int do_memtest(int wantmb, ul loops) {
//    ul loops, loop, i;
    ul loop, i;
//    size_t pagesize, wantmb, wantbytes, wantbytes_orig, bufsize, halflen, count;
    int pagesize, wantbytes, wantbytes_orig, bufsize, halflen, count;
    ptrdiff_t pagesizemask;
    void volatile *buf, *aligned;
    ulv *bufa, *bufb;
//    int do_mlock = 1, done_mem = 0;
    int do_mlock = 0, done_mem = 0;
    int exit_code = 0;
    int MemDev;
    unsigned long mem_addr=0;
    unsigned long start_jiffies=0, last_jiffies=0;
//disable UART interrupt (irq 5)
printk("read_aux_reg(0x40c)=0x%x\n", read_aux_reg(0x40c));
write_aux_reg( 0x40c, read_aux_reg(0x40c) & (~ (1<<5)));
printk("read_aux_reg(0x40c)=0x%x Disable UART Interrupt(irq 5)\n", read_aux_reg(0x40c));

//cli()
    printk("memtester version " __version__ " (%d-bit)\n", UL_LEN);
    printk("Copyright (C) 2005 Charles Cazabon.\n");
    printk("Licensed under the GNU General Public License version 2 (only).\n");
    printk("\n");
    check_posix_system();
    pagesize = memtester_pagesize();
    pagesizemask = (ptrdiff_t) ~(pagesize - 1);
    printk("pagesizemask is 0x%tx\n", pagesizemask);

//    if (argc < 2) {
//        printk("need memory argument, in MB\n");
//        exit(EXIT_FAIL_NONSTARTER);
//    }
//    wantmb = (int) strtoul(argv[1], 0, 0);
    wantbytes_orig = wantbytes = (int) (wantmb << 20);
    if (wantbytes < pagesize) {
        printk("bytes < pagesize -- memory argument too large?\n");
//        exit(EXIT_FAIL_NONSTARTER);
        return 0;
    }

//    if (argc < 3) {
//        loops = 0;
//    } else {
//        loops = strtoul(argv[2], 0, 0);
//    }

    printk("want %lluMB (%llu bytes)\n", (ull) wantmb, (ull) wantbytes);
    buf = 0;

    while (!done_mem) {
/**
#if 1
        while (!buf && wantbytes) {
            buf = (void volatile *) __get_free_pages(GFP_ATOMIC, get_order(wantbytes));
            if (!buf) wantbytes -= pagesize;
        }
#else
//        MemDev = open ("/dev/mem", O_RDWR);
        MemDev = open ("/dev/fb0", O_RDWR);
        buf = (void volatile *) mmap (0, wantbytes, PROT_READ | PROT_WRITE, MAP_SHARED, MemDev, 0);
#endif
**/
//        mem_addr = __get_free_pages(GFP_ATOMIC, get_order(wantbytes));
        {
          unsigned long page;
          unsigned int size = wantbytes;
          mem_addr = END_MEM - size;
//          for (page = mem_addr; page < mem_addr + size; page += PAGE_SIZE)
//                  SetPageReserved(virt_to_page(page));            
        }
        if (mem_addr != 0)
        {
          printk("Got mem_addr=0x%x\n", mem_addr);
          buf = (void volatile *)mem_addr;
        }
        bufsize = wantbytes;
        printk("got  %lluMB (%llu bytes)", (ull) wantbytes >> 20,
            (ull) wantbytes);
//        fflush(stdout);

        if (do_mlock) {
#if 0
            printk(", trying mlock ...");
//            fflush(stdout);
            if ((int) buf % pagesize) {
                /* printk("aligning to page\n"); */
                aligned = (void volatile *) ((int) buf & pagesizemask);
                bufsize -= ((int) aligned - (int) buf);
            } else {
                aligned = buf;
            }
            /* Try memlock */

            if (mlock((void *) aligned, bufsize) < 0) {
                switch(errno) {
                    case ENOMEM:
                        printk("too many pages, reducing...\n");
                        free((void *) buf);
                        buf = 0;
                        wantbytes -= pagesize;
                        break;
                    case EPERM:
                        printk("insufficient permission.\n");
                        printk("Trying again, unlocked:\n");
                        do_mlock = 0;
                        free((void *) buf);
                        buf = 0;
                        wantbytes = wantbytes_orig;
                        break;
                    default:
                        printk("failed for unknown reason.\n");
                        do_mlock = 0;
                        done_mem = 1;
                }
            } else {
                printk("locked.\n");
                done_mem = 1;
            }
#endif
        } else { 
            done_mem = 1;
            printk("\n"); 

            if ((int) buf % pagesize) {
                /* printk("aligning to page\n"); */
                aligned = (void volatile *) ((int) buf & pagesizemask);
                bufsize -= ((int) aligned - (int) buf);
            } else {
                aligned = buf;
            }


        }
    }

    if (!do_mlock) printk("Continuing with unlocked memory; testing "
        "will be slower and less reliable.\n");

    select_memtester_item();

    if (select_test_item == -3)
    {
      bufsize = mem_userdefined_size;
      aligned = (void volatile *) ((unsigned int) mem_userdefined_startaddr);
      loops = mem_userdefined_loops;
      printk("mem_userdefined_startaddr=0x%x\n", mem_userdefined_startaddr);
      printk("mem_userdefined_pattern=0x%x\n", mem_userdefined_pattern);
      printk("mem_userdefined_size=0x%x\n", mem_userdefined_size);
      printk("loops=0x%x\n", loops);

    }

    halflen = bufsize / 2;
    count = halflen / sizeof(ul);
    bufa = (ulv *) aligned;
    bufb = (ulv *) ((int) aligned + halflen);
    
    start_jiffies = jiffies;
    last_jiffies = jiffies;

printk("Reset memory area to 0x00 (addr:0x%x, size:0x%x)\n", aligned, bufsize);
memset(aligned, 0x00, bufsize);
printk("Reset memory area to 0x00 done!!\n");

    printk("Start time ==> 00:00.00\n");

    for(loop=1; ((!loops) || loop <= loops); loop++) {

/*
*((unsigned int volatile *)0xc0fc180c)= 0xc000100b;
printk("*((unsigned int volatile *)0xc0fc180c)=0x%x\n", *((unsigned int volatile *)0xc0fc180c));
{
for (int k=0; k<10000;k++);
}
*((unsigned int volatile *)0xc0fc180c)= 0xc000100a;
*/
//printk("jiffies=%lu\n", jiffies);
        printk("Current time ==> %.2d:%.2d:%.2d\n", ((jiffies-start_jiffies)/100)/3600, (((jiffies-start_jiffies)/100)%3600)/60, ((jiffies-start_jiffies)/100)%60  );
        last_jiffies = jiffies;

        printk("Loop %lu", loop);
        if (loops) {
            printk("/%lu", loops);
        }
        printk(":\n");
        if (select_test_item == -2 || select_test_item == -1)
        {
          printk("  %-20s: ================", "Stuck Address");

          if (!test_stuck_address(aligned, bufsize / sizeof(ul))) {
               printk("ok\n");
          } else {
              exit_code |= EXIT_FAIL_ADDRESSLINES;
          }

          if (select_test_item == -2)
          {
              printk("\n");
              continue;
          }    
        }

        if (select_test_item == -1)
        {
          for (i=0;;i++) {
            printk("Current time ==> %.2d:%.2d:%.2d\n", ((jiffies-start_jiffies)/100)/3600, (((jiffies-start_jiffies)/100)%3600)/60, ((jiffies-start_jiffies)/100)%60  );
            last_jiffies = jiffies;
//if (i<13)continue;
            if (!tests[i].name) break;
            printk("  %-20s: ", tests[i].name);
            if (!tests[i].fp(bufa, bufb, count)) {
                printk("ok\n");
            } else {
                exit_code |= EXIT_FAIL_OTHERTEST;
            }
//            fflush(stdout);
          }
        }
        else if (select_test_item == -3)
        {
            printk("Current time ==> %.2d:%.2d:%.2d\n", ((jiffies-start_jiffies)/100)/3600, (((jiffies-start_jiffies)/100)%3600)/60, ((jiffies-start_jiffies)/100)%60  );
            last_jiffies = jiffies;
            printk("  %-20s: ", "UserDefined");

            if (!test_userdefined_comparison(bufa, bufb, count)) {
                printk("ok\n");
            } else {
                exit_code |= EXIT_FAIL_OTHERTEST;
            }
        }
        else
        {
            printk("Current time ==> %.2d:%.2d:%.2d\n", ((jiffies-start_jiffies)/100)/3600, (((jiffies-start_jiffies)/100)%3600)/60, ((jiffies-start_jiffies)/100)%60  );
            last_jiffies = jiffies;
            if (select_test_item == 1)
            {
                for (i=1; i<=6; i++)
                {
                   printk("  %-20s: ", tests[i].name);
                   if (!tests[i].fp(bufa, bufb, count)) {
                      printk("ok\n");
                   } else {
                      exit_code |= EXIT_FAIL_OTHERTEST;
                   }
                }
            }
            else
            {
                printk("  %-20s: ", tests[select_test_item].name);
                if (!tests[select_test_item].fp(bufa, bufb, count)) {
                    printk("ok\n");
                } else {
                    exit_code |= EXIT_FAIL_OTHERTEST;
                }
            }
        }
        printk("\n");
//        fflush(stdout);
    }
    if (do_mlock) munlock((void *) aligned, bufsize);
    printk("Done.\n");
//    fflush(stdout);
//    exit(exit_code);

  //Enable UART Interrupt(irq 5)
    write_aux_reg( 0x40c, read_aux_reg(0x40c) | ( (1<<5)));

//  sti();
  return exit_code;
}


#ifdef CONFIG_ARC_MEMTESTER
asmlinkage void mem_arcserial_put_char(char ch)
{
	aa3_uart *uart = (aa3_uart *)UART_BASEADDR;
        int i=0;

	/* check if TXEMPTY (bit 7) in the status reg. is 1. Else, we
	 * aren't supposed to write right now. Wait for some time.
	 */

	while(!(uart->status & UART_TXEMPTY))
	{
//	  if (i++ > 50000)
//	    break;
	};

	/* actually write the character */
	uart->data = ch;
}

asmlinkage int  mem_strlen(const char *s)
{
	int i = 0;

	while (*s++)
		i++;
	return i;
}

asmlinkage void  mem_puts(const char *s)
{
	int len;
	len = mem_strlen(s);
	while(len-- > 0)
	{
		if (*s == '\n')
			mem_arcserial_put_char('\r');
		mem_arcserial_put_char(*s++);
	}

}

asmlinkage void mem_print_hex(unsigned long val)
{
        int i, nibbles = sizeof(val)*2;
        char buf[sizeof(val)*2+1];

        for (i = nibbles-1;  i >= 0;  i--) {
                buf[i] = (val & 0xf) + '0';
                if (buf[i] > '9')
                    buf[i] += ('a'-'0'-10);
                val >>= 4;
        }
        buf[nibbles] = '\0';
	mem_puts(buf);
}

asmlinkage void mem_printn_hex(unsigned long val)
{
	mem_print_hex(val);
	mem_puts("\n");
}

void select_memtester_item(void)
{
volatile aa3_uart *mem_uart = (aa3_uart *)UART_BASEADDR;
int loop, i;
int got_choice = 0;

//cli();

mem_puts("\n#####################################################\n");
mem_puts("Please select test item:\n");
mem_puts(" 0) Stuck Address\n");
mem_puts(" 1) Random Value\n");
mem_puts(" 2) Compare XOR/SUB/MUL/DIV/OR/AND\n");
mem_puts(" 3) Sequential Increment\n");
mem_puts(" 4) Solid Bits\n");
mem_puts(" 5) Block Sequential\n");
mem_puts(" 6) Checkerboard\n");
mem_puts(" 7) Bit Spread\n");
mem_puts(" 8) Bit Flip\n");
mem_puts(" 9) Walking Ones\n");
mem_puts(" b) Butterfly\n");
mem_puts(" a) Do All Test\n");
mem_puts(" ===================================\n");
mem_puts(" u) User Defined Pattern\n");

    for (;;)
    {
      {unsigned int iii;for (iii=0;iii<100000;iii++);}

      got_choice = mem_uart->data;
      if (got_choice != 0)
      {
          mem_puts("got_choice = 0x");mem_printn_hex(got_choice);
          if (got_choice >= '0' && got_choice <= '9' || got_choice == 'a' || got_choice == 'b' || got_choice == 'u')
              break;
          got_choice = 0;
      }
    }

    switch(got_choice)
    {
      case '1':	//Random Value
        select_test_item = 0;
        break;
      case '2':	//Compare XOR/SUB/MUL/DIV/OR/AND
        select_test_item = 1;
        break;
      case '3':	//Sequential Increment
        select_test_item = 7;
        break;
      case '4':	//Solid Bits
        select_test_item = 8;
        break;
      case '5':	//Block Sequential
        select_test_item = 9;
        break;
      case '6':	//Checkerboard
        select_test_item = 10;
        break;
      case '7':	//Bit Spread
        select_test_item = 11;
        break;
      case '8':	//Bit Flip
        select_test_item = 12;
        break;
      case '9':	//Walking Ones
        select_test_item = 13;
        break;
      case 'b':	//Butterfly
        select_test_item = 15;
        break;
      case '0':	//Stuck Address
        select_test_item = -2;
        break;
      case 'u':	//User Defined Pattern
        select_test_item = -3;
        mem_puts("\n");
        mem_puts("Please input starting address: (From 0x80100000)\n");
        mem_puts("==> 0x");

        for (;;)
        {
          {unsigned int iii;for (iii=0;iii<100000;iii++);}

          got_choice = mem_uart->data;
          if (got_choice != 0)
          {
            mem_arcserial_put_char(got_choice);
            if (got_choice >= '0' && got_choice <= '9')
            {
              mem_userdefined_startaddr = mem_userdefined_startaddr << 4;
              mem_userdefined_startaddr += got_choice - '0';
            }
            if (got_choice >= 'a' && got_choice <= 'f')
            {
              mem_userdefined_startaddr = mem_userdefined_startaddr << 4;
              mem_userdefined_startaddr += got_choice - 'a'+10;
            }
            if (got_choice >= 'A' && got_choice <= 'F')
            {
              mem_userdefined_startaddr = mem_userdefined_startaddr << 4;
              mem_userdefined_startaddr += got_choice - 'A'+10;
            }
            if (got_choice == '\n' || got_choice == '\r')
              break;
            got_choice = 0;
          }
        }

        mem_puts("Please input test pattern: (Example: 0x5a5a5a5a)\n");
        mem_puts("==> 0x");

        for (;;)
        {
          {unsigned int iii;for (iii=0;iii<100000;iii++);}

          got_choice = mem_uart->data;
          if (got_choice != 0)
          {
            mem_arcserial_put_char(got_choice);
            if (got_choice >= '0' && got_choice <= '9')
            {
              mem_userdefined_pattern = mem_userdefined_pattern << 4;
              mem_userdefined_pattern += got_choice - '0';
            }
            if (got_choice >= 'a' && got_choice <= 'f')
            {
              mem_userdefined_pattern = mem_userdefined_pattern << 4;
              mem_userdefined_pattern += got_choice - 'a'+10;
            }
            if (got_choice >= 'A' && got_choice <= 'F')
            {
              mem_userdefined_pattern = mem_userdefined_pattern << 4;
              mem_userdefined_pattern += got_choice - 'A' +10;
            }
            if (got_choice == '\n' || got_choice == '\r')
              break;
            got_choice = 0;
          }
        }

        mem_puts("Please input memory range(size): (Example: 1M = 0x100000)\n");
        mem_puts("==> 0x");
        for (;;)
        {
          {unsigned int iii;for (iii=0;iii<100000;iii++);}

          got_choice = mem_uart->data;
          if (got_choice != 0)
          {
            mem_arcserial_put_char(got_choice); 
            if (got_choice >= '0' && got_choice <= '9')
            {
              mem_userdefined_size = mem_userdefined_size << 4;
              mem_userdefined_size += got_choice - '0';
            }
            if (got_choice >= 'a' && got_choice <= 'f')
            {
              mem_userdefined_size = mem_userdefined_size << 4;
              mem_userdefined_size += got_choice - 'a' +10;
            }
            if (got_choice >= 'A' && got_choice <= 'F')
            {
              mem_userdefined_size = mem_userdefined_size << 4;
              mem_userdefined_size += got_choice - 'A' +10;
            }
            if (got_choice == '\n' || got_choice == '\r')
              break;
            got_choice = 0;
          }
        }
        mem_puts("Please input loop count: (0 or not 0)\n");
        mem_puts("==> 0x");
        for (;;)
        {
          {unsigned int iii;for (iii=0;iii<100000;iii++);}

          got_choice = mem_uart->data;
          if (got_choice != 0)
          {
            mem_arcserial_put_char(got_choice);
            if (got_choice >= '0' && got_choice <= '9')
            {
              mem_userdefined_loops = mem_userdefined_loops << 4;
              mem_userdefined_loops += got_choice - '0';
            }
            if (got_choice >= 'a' && got_choice <= 'f')
            {
              mem_userdefined_loops = mem_userdefined_loops << 4;
              mem_userdefined_loops += got_choice - 'a' +10;
            }
            if (got_choice >= 'A' && got_choice <= 'F')
            {
              mem_userdefined_loops = mem_userdefined_loops << 4;
              mem_userdefined_loops += got_choice - 'A' +10;
            }
            if (got_choice == '\n' || got_choice == '\r')
              break;
            got_choice = 0;
          }
        }

        break;
      case 'a':	//Default ==> All
      default:
        select_test_item = -1;
        break;
    }
}

void change_memtester_setting(void)
{
volatile aa3_uart *mem_uart = (aa3_uart *)UART_BASEADDR;
int loop, i;
int got_choice = 0;
unsigned int temp;

temp = read_new_aux_reg(ARC_REG_IC_CTRL);
write_new_aux_reg(ARC_REG_IC_CTRL, (temp|0x1));

//cli();
write_new_aux_reg(ARC_REG_DC_IVDC, 0x1);		
/* Disable D cache */
write_new_aux_reg(ARC_REG_DC_CTRL, 0x1);
mem_puts("\n#####################################################\n");
mem_puts("\n Disable D-Cache & I-Cache\n");
mem_puts("\n#####################################################\n");
mem_puts("Please select CPU Clock & Memory Clock\n");
mem_puts(" 1) CPU 162MHz,    SDRAM 162MHz\n");
mem_puts(" 2) CPU 162MHz,    SDRAM  81MHz\n");
mem_puts(" 3) CPU 148.5MHz,  SDRAM 148.5MHz\n");
mem_puts(" 4) CPU 148.5MHz,  SDRAM  74.25MHz\n");
mem_puts(" 5) CPU 135MHz,    SDRAM 135MHz\n");
mem_puts(" 6) CPU 135MHz,    SDRAM  67.5MHz\n");
mem_puts(" =======================================\n");
mem_puts(" 7) CPU 162MHz,    DDR  16Bit, 32MByte\n");
mem_puts(" 8) CPU 162MHz,    DDR  32Bit, 32MByte x2, CL=3\n");
mem_puts(" 9) CPU 162MHz,    DDR  32Bit, 32MByte x2, CL=2.5\n");
mem_puts(" 0) default\n");

    for (;;)
    {
      {unsigned int iii;for (iii=0;iii<100000;iii++);}

      got_choice = mem_uart->data;
      if (got_choice != 0)
      {
//          mem_puts("got_choice = 0x");mem_printn_hex(got_choice);
          if ((got_choice >= '1' && got_choice <= '9') || got_choice == '0')
          {
              mem_arcserial_put_char(got_choice);
              mem_puts("\n");
//              {unsigned int iii;for (iii=0;iii<100000;iii++);}
              break;
          }
      }
    }

    switch (got_choice)
    {
      case '1':		//162MHz  /1
        __asm__ __volatile__ (
//            "mov     r9, 0xc000100a \n\t"
            "mov    r5, 0x00000100 \n\t"
//            "mov    r6, 0x00000561 \n\t"
//            "mov    r7, 0x05053737 \n\t"
//            "mov    r10, 0x00040101 \n\t"
            "mov    r6, 0x00000161 \n\t"
            "mov    r7, 0x01013131 \n\t"
            "mov    r10, 0x00040100 \n\t"
            "mov    r8, 0x00000001 \n\t"
            "mov    r9, 0x70a309e3 \n\t"
//            "st.di   r9, [0xc0fc180c] \n\t"
//            "st.di   r9, [0xc0fc180c] \n\t"
            "st.di  r6, [0xc0fc1818] \n\t"
            "st.di  r7, [0xc0fc1814] \n\t"
            "__memdelay:	\n\t"
            "sub.f r5, r5, 1	\n\t"
            "jnz __memdelay	\n\t"
            "mov    r5, 0x00000100	\n\t"
            "st.di  r9, [0xc0fc7004]	\n\t"
            "st.di  r10, [0xc0fc7008]	\n\t"
            "st.di  r8, [0xc0fc700c]	\n\t"
            "__memdelay2:	\n\t"
            "sub.f  r5, r5, 1	\n\t"
            "jnz __memdelay2	\n\t"
        );
        break;
      case '2':		//162MHz /2	default
        __asm__ __volatile__ (
            "mov    r5, 0x00000100 \n\t"
            "mov    r6, 0x00000161 \n\t"
            "mov    r7, 0x01013131 \n\t"
            "mov    r9, 0x70a309e3 \n\t"
            "st.di  r6, [0xc0fc1818] \n\t"
            "st.di  r7, [0xc0fc1814] \n\t"
            "__memdelay21:	\n\t"
            "sub.f r5, r5, 1	\n\t"
            "jnz __memdelay21	\n\t"
            "st.di r9, [0xc0fc7004]	\n\t"
            "mov r5, 0x00000100	\n\t"
            "__memdelay22:	\n\t"
            "sub.f r5, r5, 1	\n\t"
            "jnz __memdelay22	\n\t"
        );

        break;
      case '3':		//148.5MHz  /1
        __asm__ __volatile__ (
            "mov     r11, 0xc000100b \n\t"
            "mov    r5, 0x00000100 \n\t"
//            "mov    r6, 0x00000561 \n\t"
//            "mov    r7, 0x05053737 \n\t"
//            "mov    r10, 0x00040101 \n\t"
            "mov    r6, 0x00000161 \n\t"
            "mov    r7, 0x01013131 \n\t"
            "mov    r10, 0x00040100 \n\t"
            "mov    r8, 0x00000001 \n\t"
            "mov    r9, 0x70a309e3 \n\t"
            "st.di   r11, [0xc0fc180c] \n\t"
            "st.di   r11, [0xc0fc180c] \n\t"
            "__memdelay31:	\n\t"
            "sub.f r5, r5, 1	\n\t"
            "jnz __memdelay31	\n\t"
            "mov r5, 0x00000100	\n\t"
            "st.di  r6, [0xc0fc1818] \n\t"
            "st.di  r7, [0xc0fc1814] \n\t"
            "__memdelay32:	\n\t"
            "sub.f r5, r5, 1	\n\t"
            "jnz __memdelay32	\n\t"
            "st.di r9, [0xc0fc7004]	\n\t"
            "st.di r10, [0xc0fc7008]	\n\t"
            "st.di r8, [0xc0fc700c]	\n\t"
            "mov r5, 0x00000100	\n\t"
            "__memdelay33:	\n\t"
            "sub.f r5, r5, 1	\n\t"
            "jnz __memdelay33	\n\t"
        );
        arc700_clkfreq = 148500000;
	*((unsigned int *)0xc0fc1018) = (((148500000/(57600*4)) - 1) & 0xff);
	*((unsigned int *)0xc0fc101c) = (((148500000/(57600*4)) - 1) & 0xff00) >> 8;
 
       break;
      case '4':		//148.5MHz  /2
        __asm__ __volatile__ (
            "mov     r11, 0xc000100b \n\t"
            "mov    r5, 0x00000100 \n\t"
            "mov    r6, 0x00000161 \n\t"
            "mov    r7, 0x01013131 \n\t"
            "mov    r9, 0x70a309e3 \n\t"
            "st.di   r11, [0xc0fc180c] \n\t"
            "st.di   r11, [0xc0fc180c] \n\t"
            "__memdelay41:	\n\t"
            "sub.f r5, r5, 1	\n\t"
            "jnz __memdelay41	\n\t"
            "mov r5, 0x00000100	\n\t"
            "st.di  r6, [0xc0fc1818] \n\t"
            "st.di  r7, [0xc0fc1814] \n\t"
            "__memdelay42:	\n\t"
            "sub.f r5, r5, 1	\n\t"
            "jnz __memdelay42	\n\t"
            "st.di r9, [0xc0fc7004]	\n\t"
            "mov r5, 0x00000100	\n\t"
            "__memdelay43:	\n\t"
            "sub.f r5, r5, 1	\n\t"
            "jnz __memdelay43	\n\t"
        );
        arc700_clkfreq = 148500000;
	*((unsigned int *)0xc0fc1018) = (((148500000/(57600*4)) - 1) & 0xff);
	*((unsigned int *)0xc0fc101c) = (((148500000/(57600*4)) - 1) & 0xff00) >> 8;

        break;
      case '5':		//135MHz	/1
        __asm__ __volatile__ (
//            "mov      r5,  0x00130000 \n\t"
            "mov      r5,  0x00000100 \n\t"
            "mov      r11, 0xc000100a \n\t"
            "mov      r6,  0x00000161 \n\t"
            "mov      r7,  0x01013131 \n\t"
            "mov     r10,  0x00040100 \n\t"
            "mov      r8,  0x00000001 \n\t"
            "mov      r9,  0x70a309e3 \n\t"
            "st.di   r11, [0xc0fc180c] \n\t"
            "st.di   r11, [0xc0fc180c] \n\t"
            "__memdelay51:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay51	\n\t"
//            "mov r5, 0x01130000	\n\t"
            "mov r5, 0x00000100	\n\t"
            "st.di    r6, [0xc0fc1818] \n\t"
            "st.di    r7, [0xc0fc1814] \n\t"
            "__memdelay52:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay52	\n\t"
            "st.di    r9, [0xc0fc7004]	\n\t"
            "st.di   r10, [0xc0fc7008]	\n\t"
            "st.di    r8, [0xc0fc700c]	\n\t"
//            "mov r5, 0x00130000	\n\t"
            "mov r5, 0x00000100	\n\t"
            "__memdelay53:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay53	\n\t"
        );
        arc700_clkfreq = 135000000;
	*((unsigned int *)0xc0fc1018) = (((135000000/(57600*4)) - 1) & 0xff);
	*((unsigned int *)0xc0fc101c) = (((135000000/(57600*4)) - 1) & 0xff00) >> 8;

        break;
      case '6':		//135MHz   /2
        __asm__ __volatile__ (
            "mov      r5,  0x00000100 \n\t"
            "mov      r11, 0xc000100a \n\t"
            "mov      r6,  0x00000161 \n\t"
            "mov      r7,  0x01013131 \n\t"
            "mov      r9,  0x70a309e3 \n\t"
            "st.di   r11, [0xc0fc180c] \n\t"
            "st.di   r11, [0xc0fc180c] \n\t"
            "__memdelay61:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay61	\n\t"
            "mov r5, 0x00000100	\n\t"
            "st.di    r6, [0xc0fc1818] \n\t"
            "st.di    r7, [0xc0fc1814] \n\t"
            "__memdelay62:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay62	\n\t"
            "st.di    r9, [0xc0fc7004]	\n\t"
            "mov r5, 0x00000100	\n\t"
            "__memdelay63:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay63	\n\t"
        );
        arc700_clkfreq = 135000000;
	*((unsigned int *)0xc0fc1018) = (((135000000/(57600*4)) - 1) & 0xff);
	*((unsigned int *)0xc0fc101c) = (((135000000/(57600*4)) - 1) & 0xff00) >> 8;

        break;
      case '7':		//162MHz DDR 16Bit, 32MByte
        __asm__ __volatile__ (
            "mov      r5,  0x00000100 \n\t"
//            "mov      r11, 0xc000100a \n\t"
            "mov      r6,  0x00000121 \n\t"
            "mov      r7,  0x01013131 \n\t"
            "mov      r8,  0x002a1200 \n\t"
            "mov      r9,  0x00000801 \n\t"
            "mov     r10,  0x124e6665 \n\t"
//           "st.di   r11, [0xc0fc180c] \n\t"
//            "st.di   r11, [0xc0fc180c] \n\t"
//            "__memdelay71:	\n\t"
//            "sub.f    r5,  r5, 1	\n\t"
//            "jnz __memdelay71	\n\t"
//            "mov r5, 0x00000100	\n\t"
            "st.di    r6, [0xc0fc1818] \n\t"
            "st.di    r7, [0xc0fc1814] \n\t"
            "__memdelay72:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay72	\n\t"
            "st.di    r8, [0xc0fca00c]	\n\t"
            "st.di    r9, [0xc0fca008]	\n\t"
            "st.di   r10, [0xc0fca000]	\n\t"
//            "mov r5, 0x00130000	\n\t"
            "mov r5, 0x00000100	\n\t"
            "__memdelay73:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay73	\n\t"
        );
//        arc700_clkfreq = 162000000;
//	*((unsigned int *)0xc0fc1018) = (((162000000/(57600*4)) - 1) & 0xff);
//	*((unsigned int *)0xc0fc101c) = (((162000000/(57600*4)) - 1) & 0xff00) >> 8;

        break;
      case '8':		//162MHz DDR 32Bit, 32MByte x2 CL=3
        __asm__ __volatile__ (
            "mov      r5,  0x00000100 \n\t"
//            "mov      r11, 0xc000100a \n\t"
            "mov      r6,  0x00000121 \n\t"
            "mov      r7,  0x01013131 \n\t"
            "mov      r8,  0x002a0200 \n\t"
            "mov      r9,  0x00000801 \n\t"
            "mov     r10,  0x124f7777 \n\t"
//            "mov     r11,  0x88a40480 \n\t"
//           "st.di   r11, [0xc0fc180c] \n\t"
//            "st.di   r11, [0xc0fc180c] \n\t"
//            "__memdelay71:	\n\t"
//            "sub.f    r5,  r5, 1	\n\t"
//            "jnz __memdelay71	\n\t"
//            "mov r5, 0x00000100	\n\t"
            "st.di    r6, [0xc0fc1818] \n\t"
            "st.di    r7, [0xc0fc1814] \n\t"
            "__memdelay82:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay82	\n\t"
//            "st.di   r11, [0xc0fca004]	\n\t"
            "st.di    r8, [0xc0fca00c]	\n\t"
            "st.di    r9, [0xc0fca008]	\n\t"
            "st.di   r10, [0xc0fca000]	\n\t"
//            "mov r5, 0x00130000	\n\t"
            "mov r5, 0x00000100	\n\t"
            "__memdelay83:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay83	\n\t"
        );
//        arc700_clkfreq = 162000000;
//	*((unsigned int *)0xc0fc1018) = (((162000000/(57600*4)) - 1) & 0xff);
//	*((unsigned int *)0xc0fc101c) = (((162000000/(57600*4)) - 1) & 0xff00) >> 8;

        break;
      case '9':		//162MHz DDR 32Bit, 32MByte x2 CL=2.5
        __asm__ __volatile__ (
            "mov      r5,  0x00000100 \n\t"
//            "mov      r11, 0xc000100a \n\t"
            "mov      r6,  0x00000121 \n\t"
            "mov      r7,  0x01013131 \n\t"
            "mov      r8,  0x002a0200 \n\t"
            "mov      r9,  0x00000810 \n\t"
            "mov     r10,  0x124eeeee \n\t"
            "mov     r11,  0x91460480 \n\t"
//           "st.di   r11, [0xc0fc180c] \n\t"
//            "st.di   r11, [0xc0fc180c] \n\t"
//            "__memdelay71:	\n\t"
//            "sub.f    r5,  r5, 1	\n\t"
//            "jnz __memdelay71	\n\t"
//            "mov r5, 0x00000100	\n\t"
            "st.di    r6, [0xc0fc1818] \n\t"
            "st.di    r7, [0xc0fc1814] \n\t"
            "__memdelay92:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay92	\n\t"
            "st.di   r11, [0xc0fca004]	\n\t"
            "st.di    r8, [0xc0fca00c]	\n\t"
            "st.di    r9, [0xc0fca008]	\n\t"
            "st.di   r10, [0xc0fca000]	\n\t"
//            "mov r5, 0x00130000	\n\t"
            "mov r5, 0x00000100	\n\t"
            "__memdelay93:	\n\t"
            "sub.f    r5,  r5, 1	\n\t"
            "jnz __memdelay93	\n\t"
        );
//        arc700_clkfreq = 162000000;
//	*((unsigned int *)0xc0fc1018) = (((162000000/(57600*4)) - 1) & 0xff);
//	*((unsigned int *)0xc0fc101c) = (((162000000/(57600*4)) - 1) & 0xff00) >> 8;

        break;

      default:
        break;
    }

temp = read_new_aux_reg(ARC_REG_IC_CTRL);
write_new_aux_reg(ARC_REG_IC_CTRL, (temp&(~0x1)));

    mem_puts("\n#####################################################\n");
    mem_puts("\nnew 0xc0fc1800=0x");mem_printn_hex( *((unsigned int *)0xc0fc1800) );
    mem_puts("new 0xc0fc180c=0x");mem_printn_hex( *((unsigned int *)0xc0fc180c));
    mem_puts("new 0xc0fc1818=0x");mem_printn_hex( *((unsigned int *)0xc0fc1818));
    mem_puts("new 0xc0fc1814=0x");mem_printn_hex( *((unsigned int *)0xc0fc1814));
    mem_puts("new 0xc0fc700c=0x");mem_printn_hex( *((unsigned int *)0xc0fc700c));
    mem_puts("new 0xc0fc7004=0x");mem_printn_hex( *((unsigned int *)0xc0fc7004));
    mem_puts("new 0xc0fc7008=0x");mem_printn_hex( *((unsigned int *)0xc0fc7008));
    mem_puts("new 0xc0fca00c=0x");mem_printn_hex( *((unsigned int *)0xc0fca00c));
    mem_puts("new 0xc0fca008=0x");mem_printn_hex( *((unsigned int *)0xc0fca008));
    mem_puts("new 0xc0fca004=0x");mem_printn_hex( *((unsigned int *)0xc0fca004));
    mem_puts("new 0xc0fca000=0x");mem_printn_hex( *((unsigned int *)0xc0fca000));
mem_puts("\n#####################################################\n");
mem_puts("\n Enable I-Cache\n");
mem_puts("\n#####################################################\n");

//sti();
}
#endif


