/*****************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/mmzone.h>
#include <linux/bootmem.h>
#include <linux/console.h>
#include <linux/ctype.h>
#include <asm/pgtable.h>
#include <asm/serial.h>
#include <asm/arcregs.h>
#include <asm/tlb.h>
#include <asm/irq.h>
#ifdef CONFIG_BLK_DEV_INITRD
#include <linux/blk.h>
#endif
#include <linux/seq_file.h>
#include <linux/delay.h>

#include <asm/atag.h>

#define COMMAND_LINE_SIZE 256

extern int  _text, _etext, _edata, _end;
extern unsigned long end_kernel;
extern unsigned long ramd_start,ramd_end;
extern int root_mountflags;


#if 0 //wschen 2007-06-18 
#include "../boot/ramdsize.h"
#else
volatile unsigned int *rs = 0xc0f00000;
unsigned int _RAMDISK_SIZE;
unsigned int ROOTFS_ON_NAND_START = 0;
unsigned int ROOTFS_ON_NAND_END = 0;
#endif

void __init reset_vuart(void);

unsigned int processor_id;
unsigned int processor_version;

char command_line[COMMAND_LINE_SIZE];
char saved_command_line[COMMAND_LINE_SIZE];

unsigned long arc700_clkfreq = CONFIG_ARC700_CLK;

#if 1 //wschen 2007-05-24 
volatile unsigned int *pll = 0xc0fc180c;
#endif

#if 0 //wschen 2008-02-19
struct task_struct *_current_task;	/* currently active task */
#else
volatile struct task_struct *_current_task;	/* currently active task */
#endif


/* A7 does not autoswitch stacks on interrupt or exception ,so we do it manually.
 * But we need one register for each interrupt level to play with which is saved 
 * and restored from the following variable.*/
volatile unsigned long int1_saved_reg, int2_saved_reg, ex_saved_reg1, ex_saved_reg2;
volatile unsigned long icse1, icse2, ilnk1, ilnk2, blnk, stat32;
#ifdef CONFIG_ARC700_SSRAM
void __init setup_ssram_controller(void);
#endif


void parse_cmdline_early (char *p)
{
	for (; p && *p; p++)
	{
		if (isspace(*p))
			continue;
		if (!strncmp(p, "arcclk=", 7)) {
			int t = 0;

			p += 7;
			while (isdigit(*p))
				t = 10*t + *p++ - '0';
			if (t)
				arc700_clkfreq = t * 1000000;
		}
	}
}

void setup_processor(void)
{
  unsigned long idreg;
	/* Initialize and setup the processor core */
	idreg = read_new_aux_reg(AUX_IDENTITY);
	processor_id = 0;
	processor_version = idreg & 0xff;

	/* Print processor info */
	printk("Detected Processor Number: 0x%2x, Version: 0x%2x ",
				processor_id, processor_version);

	if (processor_version >= 0x0 && processor_version <=0x0f)
		printk("(%s) \n", "ARCtangent A4 processor family");
	else if (processor_version >= 0x10 && processor_version <=0x1f)
		printk("(%s) \n", "ARCtangent A5 processor family");
	else if (processor_version >= 0x20 && processor_version <=0x2f)
		printk("(%s) \n", "ARC 600 processor family");
	else if (processor_version >= 0x30 && processor_version <=0x3f)
		printk("(%s) \n", "ARC 700 processor family");
	else 
		printk("(%s) \n", "ARC Unkown processor family");

	printk("Linux running at %u.%02u Mhz, default is %u.%02u Mhz\n",
	       (arc700_clkfreq/1000000), (arc700_clkfreq/10000) % 100,
	       (CONFIG_ARC700_CLK/1000000), (CONFIG_ARC700_CLK/10000) % 100);
}		

//====== ATAG parse =======
#ifdef CONFIG_KERNEL_ATAG
int parse_tag_clk_speed(struct tag *tag)
{
  printk("ATAG_CLK_SPEED: clock-speed = %d\n", tag->u.clk_speed.clk_speed_hz);
//  clk_speed = tag->u.clk_speed.clk_speed_hz;
  return 0;
}

__tagtable(ATAG_CLK_SPEED, parse_tag_clk_speed);
static struct tagtable __tagtable_test __tag = { ATAG_CLK_SPEED, parse_tag_clk_speed };

/* not yet useful... */
int parse_tag_ramdisk(struct tag *tag)
{
  printk("ATAG_RAMDISK: Flags for ramdisk = 0x%x\n", tag->u.ramdisk.flags);
  printk("ATAG_RAMDISK: ramdisk size = 0x%x\n", tag->u.ramdisk.size);
  printk("ATAG_RAMDISK: ramdisk start address = 0x%x\n", tag->u.ramdisk.start);
  return 0;

}

__tagtable(ATAG_RAMDISK, parse_tag_ramdisk);

/* not yet useful.... */
int parse_tag_initrd2(struct tag *tag)
{
  printk("ATAG_INITRD2: initrd start = 0x%x\n", tag->u.initrd.start);
  printk("ATAG_INITRD2: initrd size = 0x%x\n", tag->u.initrd.size);
  return 0;

}

__tagtable(ATAG_INITRD2, parse_tag_initrd2);

int parse_tag_cache(struct tag *tag)
{
  printk("ATAG_CACHE: ICACHE status = 0x%x\n", tag->u.cache.icache);
  printk("ATAG_CACHE: DCACHE status = 0x%x\n", tag->u.cache.dcache);
//  icache = tag->u.cache.icache;
//  dcache = tag->u.cache.dcache;
  return 0;

}

__tagtable(ATAG_CACHE, parse_tag_cache); 
	  


int parse_tag_serial(struct tag *tag)
{
  printk("ATAG_SERIAL: serial_nr = %d", tag->u.serial.serial_nr);
  printk("ATAG_SERIAL: serial baudrate = %d", tag->u.serial.baudrate);
//  serial_baudrate = tag->u.serial.baudrate; /* when we have multiple uart's serial_nr should also be processed */
  return 0;

}

__tagtable(ATAG_SERIAL, parse_tag_serial);

int parse_tag_vmac(struct tag *tag)
{
  int i;
  printk("ATAG_VMAC: vmac address = %d:%d:%d:%d:%d:%d\n", tag->u.vmac.addr[0],tag->u.vmac.addr[1],tag->u.vmac.addr[2],tag->u.vmac.addr[3],tag->u.vmac.addr[4],tag->u.vmac.addr[5]);
  
//  mac_addr.sa_family = 0;


//  for( i=0; i<6 ; i++)
//     mac_addr.sa_data[i] = tag->u.vmac.addr[i];

  return 0;

}

__tagtable(ATAG_VMAC, parse_tag_vmac);

int parse_tag_cmdline(struct tag *tag)
{
  printk("ATAG_CMDLINE: command line = %s\n", tag->u.cmdline.cmdline);
  strcpy(command_line,tag->u.cmdline.cmdline);
  return 0;

}
__tagtable(ATAG_CMDLINE, parse_tag_cmdline);


int parse_tag(struct tag *tag)
{
        extern struct tagtable __tagtable_begin, __tagtable_end;
	struct tagtable *t;

	for (t = &__tagtable_begin; t < &__tagtable_end; t++)
	{
printk("tag->hdr.tag=0x%x t->tag=0x%x\n", tag->hdr.tag, t->tag);
		if (tag->hdr.tag == t->tag) {
			t->parse(tag);
			break;
		}
	}

	return t < &__tagtable_end;
}



void parse_tags(struct tag *tags)
{
printk("in parse_tags, tags->hdr.tag=0x%x tags->hdr.size=%d (0x%x)\n", tags->hdr.tag, tags->hdr.size, tags->hdr.size);
  while(tags->hdr.tag != ATAG_NONE)
    {

      if(!parse_tag(tags))
	printk("Ignoring unknown tag type\n");
      
      tags = tag_next(tags);
    }
  
}

static struct init_tags {
	struct tag_header hdr1;
	struct tag_core   core;
#ifdef CONFIG_ARC700_SSRAMB0  
        struct tag_header hdr2;
	struct tag_mem32  mem0;
#endif
#ifdef CONFIG_ARC700_SSRAMB1  

        struct tag_header hdr3;
        struct tag_mem32  mem1;
#endif

#ifdef CONFIG_ARC700_SSRAMB2  
        struct tag_header hdr4; 
        struct tag_mem32  mem2;
     
#endif
        struct tag_header hdr5;
        struct tag_clk_speed clk_speed;
        struct tag_header hdr6;
        struct tag_cache cache;
#ifdef CONFIG_ARC700_SERIAL
        struct tag_header hdr7;
        struct tag_serial serial;
#endif
#ifdef CONFIG_ARCTANGENT_EMAC
        struct tag_header hdr8;
        struct tag_vmac vmac;
#endif
        struct tag_header hdr9; 
  
} init_tags __initdata = {
  
	{ tag_size(tag_core), ATAG_CORE },
	{},

#ifdef CONFIG_ARC700_SSRAMB0  
	{ tag_size(tag_mem32), ATAG_MEM},
	{  0x400000},
#endif

#ifdef CONFIG_ARC700_SSRAMB1  
	{ tag_size(tag_mem32), ATAG_MEM},
	{ 0x800000},
#endif

#ifdef CONFIG_ARC700_SSRAMB2 
	{ tag_size(tag_mem32), ATAG_MEM},
	{ 0xc00000},
#endif
	{ tag_size(tag_clk_speed), ATAG_CLK_SPEED },
	{ CONFIG_ARC700_CLK },

	{ tag_size(tag_cache), ATAG_CACHE },

	
#ifdef CONFIG_ARC700_USE_ICACHE	
	{  1 ,
#else
	{  0 ,

#endif
#ifdef CONFIG_ARC700_USE_DCACHE	
	  1 },
#else
	  0 },

#endif

#ifdef CONFIG_ARC700_SERIAL
        { tag_size(tag_serial), ATAG_SERIAL },
        { 0, CONFIG_ARC700_SERIAL_BAUD },
#endif

#ifdef CONFIG_ARCTANGENT_EMAC
        { tag_size(tag_vmac), ATAG_VMAC },
        {0,1,2,3,4,5}, /* Never set the mac address starting with 1 */
#endif
	
	{ 0, ATAG_NONE }
};
#endif

void setup_arch(char **cmdline_p)
{
	int bootmap_size;
	unsigned long start_pfn, max_pfn, size, end_kernel_addr;
	unsigned long reg;
#ifdef CONFIG_KERNEL_ATAG
	extern unsigned long atag_head;
	struct tag *tags = (struct tag *)&init_tags;
#endif

#if 1 //wschen 2007-06-18 
	_RAMDISK_SIZE = *rs;
#endif

	ROOTFS_ON_NAND_START = *(volatile unsigned int *)0xc0f00010;
	ROOTFS_ON_NAND_END = *(volatile unsigned int *)0xc0f00014;


#if 1 //wschen 2007-05-24 
	switch (*pll) {
	  case 0x0e:
	    arc700_clkfreq = 189000000;
	    break;
	  case 0x0c:
	    arc700_clkfreq = 162000000;
	    break;
	  case 0x0b:
	    arc700_clkfreq = 148500000;
	    break;
	  case 0x0a:
	    arc700_clkfreq = 135000000;
	    break;
	  default:
	    arc700_clkfreq = 162000000;
	    break;
	}
#endif

	/* If parameters passed by u-boot, override compile-time parameters */
#ifdef CONFIG_KERNEL_ATAG
	printk("atag_head = 0x%x\n", atag_head);
        if(atag_head)
        {
		tags = (struct tag *)atag_head;

		if(tags->hdr.tag == ATAG_CORE)
		{

			printk("Parsing ATAG parameters\n");
			parse_tags(tags);
		}
		else
			printk("ATAG parameters not provided by boot-loader\n");
	}
#endif

	setup_processor();
	tlb_init();
	a7_cache_init();

#ifdef CONFIG_ARC700_SERIAL	
	/* Disable UART0/UART1 Interrupt mask */
	reg = read_new_aux_reg(AUX_IENABLE);
	reg &= ~((1<<VUART_INT)|(1<<VUART1_INT));
	write_new_aux_reg(AUX_IENABLE, reg);
	/* FIXME: ARC : temporarily done to reset the vuart */
	reset_vuart(); 
#else
	/* Disable UART0/UART1 Interrupt mask */
	reg = read_new_aux_reg(AUX_IENABLE);
	reg &= ~((1<<VUART_INT)|(1<<VUART1_INT));
	write_new_aux_reg(AUX_IENABLE, reg);
#endif	/* CONFIG_ARC700_SERIAL */

	init_mm.start_code = (unsigned long) &_text;
	init_mm.end_code   = (unsigned long) &_etext;
	init_mm.end_data   = (unsigned long) &_edata;
	init_mm.brk	   = (unsigned long) &_end;

#if 0 //wschen 2007-05-24 
	arc700_clkfreq = CONFIG_ARC700_CLK;
#endif

//Robert 2008-08-04 Set root=/dev/skext1
	if (ROOTFS_ON_NAND_START != 0)
	{
		printk("ROOTFS_ON_NAND_START=0x%x\n", ROOTFS_ON_NAND_START);
		printk("ROOTFS_ON_NAND_END=0x%x\n", ROOTFS_ON_NAND_END);
		sprintf(command_line, "root=/dev/skext1");
	}

	*cmdline_p = command_line;
	/* 
	 *  save a copy of he unparsed command line for the
	 *  /proc/cmdline interface 
	 */
	//	memset(command_line,0,COMMAND_LINE_SIZE);
	memcpy(saved_command_line, command_line, sizeof(command_line));
	saved_command_line[COMMAND_LINE_SIZE-1] = '\0';
	printk("Cmdline %s, saved cmdline %s\n",command_line,saved_command_line);
	_current_task	= &init_task;

	parse_cmdline_early(command_line);

#define PFN_UP(x)	(((x) + PAGE_SIZE - 1) >> PAGE_SHIFT)
#define PFN_DOWN(x)	((x) >> PAGE_SHIFT)
#define PFN_PHYS(x)	((x) << PAGE_SHIFT)

#ifdef CONFIG_ARC_TRACE_UNIT
	printk("ARC Trace Unit enabled\n");
#endif

#ifdef CONFIG_BLK_DEV_INITRD
	if (_RAMDISK_SIZE > 0)
	{
		/* append ramdisk.image.gz to end_kernel, do ALIGN(8192*2) */
		end_kernel_addr = ((unsigned int )&end_kernel) + _RAMDISK_SIZE;
	}
	else
	{
		end_kernel_addr = ((unsigned int )&end_kernel);
	}
	start_pfn = PFN_UP(__pa(end_kernel_addr));

#else
	//	start_pfn = PFN_UP(__pa(&end));
	start_pfn = PFN_UP(__pa(&end_kernel));
#endif

#ifdef CONFIG_ARC700_SSRAM
	setup_ssram_controller();
	min_low_pfn = PAGE_OFFSET >> PAGE_SHIFT;	
	max_low_pfn = max_pfn = (PFN_DOWN(END_MEM));

	bootmap_size = init_bootmem_node(NODE_DATA(0), start_pfn,
					min_low_pfn,  max_low_pfn);
	size = max_pfn - start_pfn;
	free_bootmem(PFN_PHYS(start_pfn), PFN_PHYS(size));
	reserve_bootmem(PFN_PHYS(start_pfn), bootmap_size);

#else	/* !CONFIG_ARC700_SSRAM */
	min_low_pfn = PAGE_OFFSET >> PAGE_SHIFT;

	max_low_pfn = max_pfn = (PFN_DOWN(END_MEM));
	
	bootmap_size = init_bootmem_node(NODE_DATA(0), start_pfn,
					min_low_pfn,  max_low_pfn);
	size = max_pfn - start_pfn;
	free_bootmem(PFN_PHYS(start_pfn), PFN_PHYS(size));
	reserve_bootmem(PFN_PHYS(start_pfn), bootmap_size);

	// reserve for framebuffer	 (16MB)
#ifdef CONFIG_RESERVE_FB_MEM	
	reserve_bootmem(PFN_PHYS(max_pfn) - CONFIG_RESERVE_FB_MEM_SIZE, CONFIG_RESERVE_FB_MEM_SIZE);
#endif

#endif

#ifdef CONFIG_ARCIDE_IF
	ROOT_DEV = MKDEV(3,1);
#endif

#ifdef CONFIG_BLK_DEV_INITRD
	ROOT_DEV = MKDEV(RAMDISK_MAJOR,0);
	initrd_start = (unsigned long) &ramd_start;
	if (_RAMDISK_SIZE == 0)	//make zImage
	{
		initrd_end = (unsigned long) &ramd_end;
	}
	else	//make bzImage && arch/arc/boot/ramdisk.image.gz exist
	{
		initrd_end = initrd_start + _RAMDISK_SIZE;
#if CONFIG_SRAM_SIZE >= 0x04000000
		memcpy((unsigned char *)initrd_start, (void *)((unsigned int )0x82800000), _RAMDISK_SIZE);
#elif CONFIG_SRAM_SIZE >= 0x02000000
		memcpy((unsigned char *)initrd_start, (void *)((unsigned int )0x81400000), _RAMDISK_SIZE);
#else
		memcpy((unsigned char *)initrd_start, (void *)((unsigned int )0x80a10000), _RAMDISK_SIZE);
#endif
		printk("_RAMDISK_SIZE = 0x%.8x(%d)\n", _RAMDISK_SIZE, _RAMDISK_SIZE);
	}
	initrd_below_start_ok = 1;
	printk("initrd_start = 0x%lx , inird_end = 0x%lx\n",initrd_start, initrd_end);
#endif
	/* FIXME: arc : this flag setting should depend on 
	   boot parameters passed */
	root_mountflags &= ~MS_RDONLY;

#ifdef CONFIG_ARC_PGU_CONSOLE
	conswitchp = &dummy_con;
#endif
	paging_init();

}


#ifdef CONFIG_ARC700_SSRAM
void __init setup_ssram_controller(void)
{
	volatile unsigned int *reg;
	reg = (unsigned int *) SSRAM_CTRL_ID;
	if ( *(reg) != 0x0001e01f) 
		panic ("Unknown Memory controller\n"); 

}
#endif /* CONFIG_ARC700_SSRAM */
#ifdef CONFIG_ARC700_SERIAL	
/* This function  resets the vuart, this is done here to solve the 
   problem that when we restart the debugger the old values remain 
   in the VUART status register. Hence we reset the VUART here 
*/

void __init reset_vuart(void)
{
  volatile long *status_addr = (long *)ARC_UART_CREG_STATUS;
  volatile long *data_addr = (long *)ARC_UART_CREG_DATATX;
  int temp, status;
  
  do {
    temp = (int) *(data_addr);
    status = (int) *(status_addr);
    status &= 0x20; 
  } while(!status);
  
  *(status_addr) = 0xA0;
}
#endif	/* CONFIG_ARC700_SERIAL	 */

/*
 *	Get CPU information for use by the procfs.
 */

static int show_cpuinfo(struct seq_file *m, void *v)
{

	seq_printf(m, "Processor number	  : 0x%02x\nProcessor version	: 0x%2x ",
				processor_id, processor_version);

	if (processor_version >= 0x0 && processor_version <=0x0f)
		seq_printf(m, "(%s) \n", "ARCtangent A4 processor family");
	else if (processor_version >= 0x10 && processor_version <=0x1f)
		seq_printf(m, "(%s) \n", "ARCtangent A5 processor family");
	else if (processor_version >= 0x20 && processor_version <=0x2f)
		seq_printf(m, "(%s) \n", "ARC 600 processor family");
	else if (processor_version >= 0x30 && processor_version <=0x3f)
		seq_printf(m, "(%s) \n", "ARC 700 processor family");
	else 
		seq_printf(m, "(%s) \n", "ARC Unkown processor family");

	seq_printf(m, "CPU speed	  : %u.%02u MHz\n", (arc700_clkfreq/1000000), 
				(arc700_clkfreq/10000) % 100);
#if 0 //wschen 2007-05-10 
	seq_printf(m, "Bogo MIPS	  : %lu.%02lu\n",
		     loops_per_jiffy / (500000/HZ),
		     (loops_per_jiffy / (5000/HZ)) % 100);
#endif

	return 0;
}

static void *c_start(struct seq_file *m, loff_t *pos)
{
	return *pos < NR_CPUS ? ((void *) 0x12345678) : NULL;
}

static void *c_next(struct seq_file *m, void *v, loff_t *pos)
{
	++*pos;
	return c_start(m, pos);
}

static void c_stop(struct seq_file *m, void *v)
{
}

struct seq_operations cpuinfo_op = {
	start:	c_start,
	next:	c_next,
	stop:	c_stop,
	show:	show_cpuinfo,
};

