/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) ARC International
 */
#include <linux/config.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/types.h>
#include <linux/ptrace.h>
#include <linux/mman.h>
#include <linux/mm.h>
#include <linux/version.h>
#include <linux/console.h>

#include <asm/hardirq.h>
#include <asm/pgalloc.h>
#include <asm/mmu_context.h>
#include <asm/softirq.h>
#include <asm/system.h>
#include <asm/uaccess.h>

typedef struct tag_trace {
  unsigned int source;
  unsigned int dest;
  unsigned int asid:8;
  unsigned int mode:1;
  unsigned int repeat:1;
  unsigned int nothing:21;
  unsigned int valid:1;
} trace_element;

#ifdef CONFIG_ARC_TRACE_UNIT  //wschen 2008-02-25 
extern trace_element trace_buffer [16];
#endif

/*
 * Unlock any spinlocks which will prevent us from getting the
 * message out (timerlist_lock is acquired through the
 * console unblank code)
 */
void bust_spinlocks(int yes)
{
	spin_lock_init(&timerlist_lock);
	if (yes) {
		oops_in_progress = 1;
	} else {
		int loglevel_save = console_loglevel;
		oops_in_progress = 0;
		console_unblank();
		/*
		 * OK, the message is on the console.  Now we call printk()
		 * without oops_in_progress set so that printk will give klogd
		 * a poke.  Hold onto your hats...
		 */
		console_loglevel = 15;
		printk(" ");
		console_loglevel = loglevel_save;
	}
}

asmlinkage void do_page_fault(struct pt_regs *regs, int write, unsigned long address)
{
	struct vm_area_struct *vma;
	struct task_struct *tsk = current;
	struct mm_struct *mm = tsk->mm;
	unsigned long fixup;
	siginfo_t info;

	/*
	 * We fault-in kernel-space virtual memory on-demand. The
	 * 'reference' page table is init_mm.pgd.
	 *
	 * NOTE! We MUST NOT take any locks for this case. We may
	 * be in an interrupt or a critical region, and should
	 * only copy the information from the master page table,
	 * nothing more.
	 */
	if (address >= VMALLOC_START)
		goto vmalloc_fault;

	info.si_code = SEGV_MAPERR;
	/*
	 * If we're in an interrupt or have no user
	 * context, we must not take the fault..
	 */
#ifndef CONFIG_ARC_FAULTS_DBG
	if (in_interrupt() || !mm) {
	  goto no_context;
	}
#else
	if (in_interrupt() ) {
		printk ("Page fault in interrupt\n");
		goto no_context;
	}
	
	if( !mm) {
		printk ("Page fault for task without a mm\n");
		goto no_context;
	}
#endif
#if 0 //wschen 2008-02-14
	down_read(&mm->mmap_sem);
#else
#if 0 //wschen 2008-03-03


	if (1) {
	  struct task_struct *t;
	  printk("%s:%d tsk=%p %s mm=%p mmap=%x addr=%x regs=%p ret=%x write=%d\n", __func__, __LINE__, tsk, tsk->comm, mm, &mm->mmap_sem, address, regs, regs->ret, write);
//	  for_each_task(t) {
//	    printk("%p %s %d\n", t, t->comm, t->pid);
//	  }
	}
#endif

	if (!down_read_trylock(&mm->mmap_sem)) {
	  if (!user_mode(regs) && !search_exception_table(regs->ret)) {
	    goto no_context;
	  }
	  down_read(&mm->mmap_sem);
	}
#endif

	vma = find_vma(mm, address);
	if (!vma)
		goto bad_area;
	if (vma->vm_start <= address)
		goto good_area;
	if (!(vma->vm_flags & VM_GROWSDOWN))
		goto bad_area;
	if (expand_stack(vma, address))
		goto bad_area;
/*
 * Ok, we have a good vm_area for this memory access, so
 * we can handle it..
 */
good_area:
	info.si_code = SEGV_ACCERR;

	if (write) {
		if (!(vma->vm_flags & VM_WRITE))
			goto bad_area;
	} else {
		if (!(vma->vm_flags & (VM_READ | VM_EXEC)))
			goto bad_area;
	}

survive:
	/*
	 * If for any reason at all we couldn't handle the fault,
	 * make sure we exit gracefully rather than endlessly redo
	 * the fault.
	 */
	switch (handle_mm_fault(mm, vma, address, write)) {
	case 1:
		tsk->min_flt++;
		break;
	case 2:
		tsk->maj_flt++;
		break;
	case 0:
		goto do_sigbus;
	default:
		goto out_of_memory;
	}

	up_read(&mm->mmap_sem);
	return;

/*
 * Something tried to access memory that isn't in our memory map..
 * Fix it, but check if it's kernel or user first..
 */
bad_area:
	up_read(&mm->mmap_sem);

bad_area_nosemaphore:
	/* User mode accesses just cause a SIGSEGV */
	if (user_mode(regs)) {
	  //	  __builtin_arc_brk();
#ifdef CONFIG_ARC_TRACE_UNIT
		//PB
		int i;
		unsigned int value;
                unsigned int setting;
                /*
		for (i=0; i<16;i++)
		{
		  setting = 0x80000000 | (i << 2); 
		  __builtin_arc_sr(setting,0xad);		  
		  trace_buffer[i].source = __builtin_arc_lr(0xae);		 
		  __builtin_arc_sr(setting | 0x01,0xad);		  
		  trace_buffer[i].dest = __builtin_arc_lr(0xae);
		  __builtin_arc_sr(setting | 0x02,0xad);		  
		  value = __builtin_arc_lr(0xae);
		  trace_buffer[i].asid = (unsigned char)value;
		  trace_buffer[i].mode = (unsigned char)(value >> 8) & 0x01;
		  trace_buffer[i].asid = (unsigned char)(value >> 9) & 0x01;
		  trace_buffer[i].valid = (unsigned char)(value >> 31) & 0x01;
		}
                */
		//!PB
#endif
		tsk->thread.fault_address = address;
		tsk->thread.error_code = write;
#ifdef CONFIG_ARC_FAULTS_DBG
		printk("do_page_fault(): sending SIGSEGV to %s for illegal %s %08lx (eret = %08lx)\n", tsk->comm,
		       write ? "write access to" : "read access from",
		       address, regs->ret);

		//PS
		printk (KERN_ERR"\nstack_place_holder 0x%08x\t bta 0x%08x\t lp_start 0x%08x\t lp_end 0x%08x\n", regs->stack_place_holder, regs->bta, regs->lp_start, regs->lp_end);
		printk (KERN_ERR"lp_count 0x%08x\t status32 0x%08x\t ret 0x%08x\t blink 0x%08x\t\n", regs->lp_count, regs->status32, regs->ret, regs->blink);
		printk (KERN_ERR"fp 0x%08x\t r26 0x%08x\t r12 0x%08x\t r11 0x%08x\t r10 0x%08x\t\n", regs->fp, regs->r26, regs->r12, regs->r11, regs->r10);
		printk (KERN_ERR"r9 0x%08x\t r8 0x%08x\t r7 0x%08x\t r6 0x%08x\t\n", regs->r9, regs->r8, regs->r7, regs->r6);
		printk (KERN_ERR"r5 0x%08x\t r4 0x%08x\t r3 0x%08x\t r2 0x%08x\t \n", regs->r5, regs->r4, regs->r3, regs->r2);
		printk (KERN_ERR"r1 0x%08x\t r0 0x%08x\t orig_r0 0x%08x\t orig_r8 0x%08x\t\n", regs->r1, regs->r0, regs->orig_r0, regs->orig_r8);
		printk (KERN_ERR"sp 0x%08x\t\n", regs->sp);
		// !PS
#endif
#ifdef CONFIG_ARC_TRACE_UNIT
		//PB
		for (i=0; i<16; i++)
                {
		  printk (KERN_ERR"\ntrace[%u] src: 0x%.8X, dest: 0x%.8X, asid: 0x%.2X, mode: %c, repeat %c, valid %c",i,trace_buffer[i].source,trace_buffer[i].dest,trace_buffer[i].asid,(trace_buffer[i].mode==0x01)?'U':'K',(trace_buffer[i].repeat==0x01)?'Y':'N',(trace_buffer[i].valid==0x01)?'Y':'N');
		}
  		//!PB
#endif
		info.si_signo = SIGSEGV;
		info.si_errno = 0;
		/* info.si_code has been set above */
		info.si_addr = (void *) address;
		force_sig_info(SIGSEGV, &info, tsk);
		return;
	}

no_context:
#if 0 //wschen 2008-03-03
printk("%s:%d tsk=%p %s mm=%p mmap=%x addr=%x regs=%p ret=%x write=%d\n", __func__, __LINE__, tsk, tsk->comm, mm, &mm->mmap_sem, address, regs, regs->ret, write);

	if (1) {
	  struct task_struct *t;
	  for_each_task(t) {
	    printk("%p %s %d\n", t, t->comm, t->pid);
	  }
	}
#endif
	/* Are we prepared to handle this kernel fault? 
	 *
	 * (The kernel has valid exception-points in the source 
	 *  when it acesses user-memory. When it fails in one
	 *  of those points, we find it in a table and do a jump
	 *  to some fixup code that loads an appropriate error
	 *  code)
	 */
	if((fixup = search_exception_table(regs->ret)) != 0) {
		regs->ret = fixup;
		return;
	}
	
	/*
	 * Oops. The kernel tried to access some bad page. We'll have to
	 * terminate things with extreme prejudice.
	 */

	printk(KERN_ALERT "Unable to handle kernel paging request at virtual address %08lx, eret == %08lx\n", address, regs->ret);
	die("Oops", regs, 0);
	/* Game over.  */

/*
 * We ran out of memory, or some other thing happened to us that made
 * us unable to handle the page fault gracefully.
 */
out_of_memory:
#if 1 //wschen 2008-02-14
	up_read(&mm->mmap_sem);
#endif

	if (tsk->pid == 1) {
		yield();
#if 1 //wschen 2008-02-14
		down_read(&mm->mmap_sem);
#endif
		goto survive;
	}
#if 0 //wschen 2008-02-14
	up_read(&mm->mmap_sem);
#endif

	printk(KERN_NOTICE "VM: killing process %s\n", tsk->comm);
	if (user_mode(regs))
		do_exit(SIGKILL);
	goto no_context;

do_sigbus:
	up_read(&mm->mmap_sem);

	/*
	 * Send a sigbus, regardless of whether we were in kernel
	 * or user mode.
	 */
	tsk->thread.fault_address = address;
	info.si_signo = SIGBUS;
	info.si_errno = 0;
	info.si_code = BUS_ADRERR;
	info.si_addr = (void *) address;
	force_sig_info(SIGBUS, &info, tsk);

	/* Kernel mode? Handle exceptions or die */
	if (!user_mode(regs))
		goto no_context;
	
	return;

vmalloc_fault:
	{
		/*
		 * Synchronize this task's top level page-table
		 * with the 'reference' page table.
		 */
		int offset = pgd_index(address);
		pgd_t *pgd, *pgd_k;
		pmd_t *pmd, *pmd_k;

		pgd = tsk->active_mm->pgd + offset;
		pgd_k = init_mm.pgd + offset;

		if (!pgd_present(*pgd)) {
			if (!pgd_present(*pgd_k))
				goto bad_area_nosemaphore;
			set_pgd(pgd, *pgd_k);
			return;
		}

		pmd = pmd_offset(pgd, address);
		pmd_k = pmd_offset(pgd_k, address);

		if (pmd_present(*pmd) || !pmd_present(*pmd_k))
			goto bad_area_nosemaphore;
		set_pmd(pmd, *pmd_k);
	}
}
