/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 * arch/arc/mm/tlb.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authors: Rahul Trivedi
 *
 * a700 specific mmu/tlb code. 
 */

#include <linux/config.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/mm.h>

#include <asm/page.h>
#include <asm/arcregs.h>
#include <asm/pgtable.h>
#include <asm/mmu_context.h>
#include <asm/system.h>
#include <asm/tlb.h>

/* A copy of the ASID from the PID reg is kept in asid_cache 
 */
#if 0 //wschen 2008-07-18
int asid_cache;
#else
volatile int asid_cache;
#endif

/*
 * Dummy functions
 */
void _mwmutex_create () { }
void _mwmutex_delete () { }

/* ASID to mm struct mapping. We have one extra entry corresponding to
 * NO_ASID to save us a compare when clearing the mm entry for old asid
 * see get_new_mmu_context (asm-arc/mmu_context.h) 
 */
struct mm_struct *asid_mm_map[ NUM_ASID + 1 ];

/* Needed to avoid Cache aliasing */
unsigned int ARC_shmlba;

void local_flush_tlb_all(void)
{
	unsigned long flags;
	int entry;

#ifdef CONFIG_ARC_TLB_DBG_V
	printk("[tlball]\n");
#endif

#if 0 //wschen 2008-07-18
	save_and_cli(flags);
#else
	local_irq_save(flags);
#endif


	write_new_aux_reg(ARC_REG_TLBPD1, 0);
	write_new_aux_reg(ARC_REG_TLBPD0, 0);

	for(entry = 0; entry < TLB_SIZE ; entry++) {
		write_new_aux_reg(ARC_REG_TLBINDEX, entry);
		/* write this entry to the TLB */
		write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBWrite);
	}
#if 0 //wschen 2008-07-18
	restore_flags(flags);
#else
	local_irq_restore(flags);
#endif

}

void local_flush_tlb_mm(struct mm_struct *mm)
{
#if 0 //wschen 2008-07-18
	if(mm->context.asid != NO_ASID) {
		unsigned long flags;
    
#ifdef CONFIG_ARC_TLB_DBG_V
		printk("[tlbmm<%lu>]", (unsigned long) mm->context.asid);
#endif

		save_and_cli(flags);
		get_new_mmu_context(mm);
		write_new_aux_reg(ARC_REG_PID,(mm->context.asid & 0xff) | PAGE_OFFSET);
		restore_flags(flags);
	}
#else
	get_new_mmu_context(mm, 1);
#endif
}

void local_flush_tlb_range(struct mm_struct *mm, unsigned long start,
						   unsigned long end)
{
	unsigned long flags, size;
  
	if(mm->context.asid != NO_ASID) {
#ifdef CONFIG_ARC_TLB_DBG_V
		printk("[tlbrange<%lu,0x%08lx,0x%08lx>]",
			   (mm->context.asid & 0xff), start, end);
#endif

#if 0 //wschen 2008-07-18
		save_and_cli(flags);
#else
		local_irq_save(flags);
#endif
		size = (end - start + (PAGE_SIZE - 1)) >> PAGE_SHIFT;
		if(size < TLB_SIZE) {
			int newpid = (mm->context.asid & 0xff);

			start &= PAGE_MASK;
			end += PAGE_SIZE -1;
			end &= PAGE_MASK;
			while(start < end) {
				int idx;
	
				write_new_aux_reg(ARC_REG_TLBPD0, (start|newpid));
				start += PAGE_SIZE;
				/* find the index in the tlb for this virtual address and asid */
				write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBProbe);
				idx = read_new_aux_reg(ARC_REG_TLBINDEX);
				write_new_aux_reg(ARC_REG_TLBPD1, 0);
				write_new_aux_reg(ARC_REG_TLBPD0, 0);
				if(idx < 0)
					continue;
				write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBWrite);
			}
		}
		else {
#if 0 //wschen 2008-07-18
			get_new_mmu_context(mm);
			write_new_aux_reg(ARC_REG_PID,(mm->context.asid & 0xff) | PAGE_OFFSET);
#else

			get_new_mmu_context(mm, 1);
#endif
		}
#if 0 //wschen 2008-07-18
		restore_flags(flags);
#else
		local_irq_restore(flags);
#endif
	}
}

void local_flush_tlb_page(struct vm_area_struct *vma, unsigned long page)
{
	if(!vma || vma->vm_mm->context.asid != NO_ASID) {
		unsigned long flags;
		int newpid, idx;
    
#ifdef CONFIG_ARC_TLB_DBG_V
		printk("[tlbpage<%lu,0x%08lx>]", vma->vm_mm->context.asid, page);
#endif
    
		newpid = (vma->vm_mm->context.asid & 0xff);
		page &= PAGE_MASK;
		save_and_cli(flags);
		/* get the index of the page to be flushed */
		write_new_aux_reg(ARC_REG_TLBPD0, (page | newpid));
		write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBProbe);
		idx = read_new_aux_reg(ARC_REG_TLBINDEX);
		/* set that TLB entry to 0 */
		write_new_aux_reg(ARC_REG_TLBPD1, 0);
		write_new_aux_reg(ARC_REG_TLBPD0, 0);
		if(idx >= 0)
			write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBWrite);
#if 0 //wschen 2008-07-18
		restore_flags(flags);
#else
		local_irq_restore(flags);
#endif
	}
}

#define GLV_MASK 0xe00
#if 1 //wschen 2008-07-18
#define pte_pfn(pte)		(pte_val(pte) >> PAGE_SHIFT)
#define pfn_valid(pfn)		(((pfn) - (PAGE_OFFSET >> PAGE_SHIFT)) < max_mapnr)
#endif

void update_mmu_cache(struct vm_area_struct *vma, unsigned long address,
					  pte_t pte)
{
	unsigned long flags;
	pgd_t *pgdp;
	pmd_t *pmdp;
	pte_t *ptep;
#if 0 //wschen 2008-07-18
	int idx, pid;
#else
	int pid;
#if 0 //wschen 2008-07-30
	unsigned long idx;
#else
	volatile unsigned long idx;
#endif
#endif
	unsigned long glv_bits;
#if 1 //wschen 2008-07-18
	unsigned long pfn = pte_pfn(pte);

	if(!pfn_valid(pfn))
		return;
#endif

	if(current->active_mm != vma->vm_mm)
		return;

#if 1 //wschen 2008-07-18
 	local_irq_save(flags); 
#endif

	pid = (read_new_aux_reg(ARC_REG_PID) & 0xff);
  
#ifdef CONFIG_ARC_TLB_DBG_V
	if ((pid != (vma->vm_mm->context.asid & 0xff)) || (vma->vm_mm->context.asid == NO_ASID)) {
		printk("update_mmu_cache: Wheee, bogus tlbpid mmpid=%lu tlbpid=%d\n",
			   (vma->vm_mm->context.asid & 0xff), pid);
	}
#endif
  
#if 0 //wschen 2008-07-18
	save_and_cli(flags);
#endif
	address &= PAGE_MASK;
	write_new_aux_reg(ARC_REG_TLBPD0, (address | pid));
	pgdp = pgd_offset(vma->vm_mm, address);
	write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBProbe);
	pmdp = pmd_offset(pgdp, address);
	idx = read_new_aux_reg(ARC_REG_TLBINDEX);
	ptep = pte_offset(pmdp, address);

	/* rtee: set valid and accessed bits : verify start */

	pte_val(*ptep) |= (_PAGE_VALID | _PAGE_ACCESSED);

	/* verify end */	

	/* Extract the Global,Valid and Locked bits from the PTE,
	 * these will be stored into the upper 32 bits of the TLB entry i.e
	 * in TLBPD0.
	 * The remaining bits from the PTE will be written as is to the the lower
	 * 32 bits of the TLB entry i.e. TLBPD1 
	 */

	glv_bits = ((pte_val(*ptep) & GLV_MASK) >> 1);
	write_new_aux_reg(ARC_REG_TLBPD0, (address | glv_bits | pid));
	write_new_aux_reg(ARC_REG_TLBPD1, (pte_val(*ptep) & TLBPD1_MASK));
	if (/* wschen 2008-07-30 idx < 0 */ idx == 0x80000000) {
		/* The entry is not present in the TLB, hence we need a new
		 * entry. Hardware using its replacement algorithm will provide an 
		 * index to write the new entry 
		 */
		write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBGetIndex);
		idx = read_new_aux_reg(ARC_REG_TLBINDEX);
		if (/* wschen 2008-05-19 idx < 0 */ idx & 0x80000000) {
			/* TODO rtee: This condition will arise if all the possible places where
			 * this entry can be stored are locked, in our case it will arise if _both_
			 * the ways are locked. 
			 * For now i assume that this condition wont arise, while locking of entries
			 * itself care will be taken that the other way is kept free
			 */
			printk("oops !! update_mmu_cache: no index to load the TLB entry\n");
			return;
		}
	}
	write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBWrite);
#if 0 //wschen 2008-07-18
	restore_flags(flags);
#else
	local_irq_restore(flags);
#endif

#if 1 //wschen 2008-02-12
	flush_cache_page(vma, address);
#endif

}

void __init tlb_init(void)
{
	int i;

	asid_cache = FIRST_ASID;

	/* clear asid to mm map table */	
	for ( i=0; i<MAX_ASID; i++ ) {
		asid_mm_map[i] = (struct mm_struct *) NULL;
	}		
		
	local_flush_tlb_all();

	/* enable the TLB, set 31st bit in pid register */
	write_new_aux_reg(ARC_REG_PID,0x80000000);

	/* swapper_pg_dir is the pgd for the kernel, used by vmalloc */
	write_new_aux_reg(ARC_REG_SCRATCH_DATA0, swapper_pg_dir);
}

#if 0 //wschen 2008-07-31
unsigned long tlb_find_phy_from_virt(unsigned long virt_addr)
{
        unsigned int idx, pd1; 

        write_new_aux_reg(ARC_REG_TLBPD0, 
                    ((virt_addr & PAGE_MASK)| (current->mm->context.asid)));

        write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBGetIndex);
        idx = read_new_aux_reg(ARC_REG_TLBINDEX);
        if (idx < 0) { 
            return 0;
        }
        else {
            write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBRead);
            pd1 = read_new_aux_reg(ARC_REG_TLBPD1);
        }    

        return (unsigned long)pd1;
}
#endif

#ifdef CONFIG_ARC_TLB_DBG //wschen 2008-07-31
void tlb_find_asid(unsigned int asid)
{
        unsigned int pd0, i; 

        for ( i = 0; i < 256; i++) {

        write_new_aux_reg(ARC_REG_TLBINDEX, i);

        write_new_aux_reg(ARC_REG_TLBCOMMAND, TLBRead);
        pd0 = read_new_aux_reg(ARC_REG_TLBPD0);

        if ( (pd0 & 0x400) && ((pd0 & 0xFF ) == asid ) )
            printk("Entry for ASID %d exists at set #%d, way #%d\n",
                        asid, i/128, i%128);
        }    
}
#endif

