/* ld script to make ARC Linux kernel
 * taken from the armnommu/ version by Martin Mares
 *
 * Copyright ARC International
 *
 */

OUTPUT_ARCH(arc)
ENTRY(stext)


SECTIONS
{
	/* . = 0x80000000;  */
	. = 0x80008000;
	.vector : {
		*(.vector)
	}
	/* . = 0x80002000;  */
	. = 0x8000a000;
	.init : {			/* Init code and data		*/
		_stext = .;
		_init_begin = .;
			*(.text.init)
			*(.data.init)
		. = ALIGN(16);
		__setup_start = .;
			*(.setup.init)
		__setup_end = .;
		__initcall_start = .;
			*(.initcall.init)
		__initcall_end = .;
		__tagtable_begin = .;
			*(.taglist)
		__tagtable_end = .;
		. = ALIGN(8192); 
		_init_end = .;
	}

	/DISCARD/ : {			/* Exit code and data		*/
		*(.text.exit)
		*(.data.exit)
		*(.exitcall.exit)
	}

	.text : {			/* Real text segment		*/
		_text = .;		/* Text and read-only data	*/
			*(.text)
			*(.fixup)
			*(.gnu.warning)
			*(.text.lock)	/* out-of-line lock text */
			*(.rodata)
			*(.rodata.*)
			*(.glue_7)
			*(.glue_7t)
			*(.kstrtab)
		. = ALIGN(16);
		_start___ex_table = .;	/* Exception table		*/
			*(__ex_table)
		_stop___ex_table = .;

		_start___ksymtab = .;	/* Kernel symbol table		*/
			*(__ksymtab)
		_stop___ksymtab = .;

		_start___kallsyms = .; /* All kernel symbols           */
			*(__kallsyms)
		_stop___kallsyms = .;

		*(.got)			/* Global offset table		*/

		_etext = .;		/* End of text section		*/
	}

	. = ALIGN(8192);

	.data : {
		/*
		 * first, the init task union, aligned
		 * to an 8192 byte boundary.
		 */
		*(.init.task)

		/*
		 * then the cacheline aligned data
		 */
		. = ALIGN(32);
		*(.data.cacheline_aligned)

		/*
		 * and the usual data section
		 */
		*(.data)

		CONSTRUCTORS
		_edata = .;
	}
        
	.bss : {
		_bss_start = .;	/* BSS				*/
		*(.bss)
		_bss_end = .;
		*(COMMON)
		_end = . ;
	}
	
	. = ALIGN(8192);
	ramd_start = .;	
	.data.ramdisk : { *(.data.ramdisk) } 
	ramd_end = .;

	. = ALIGN(8192);
	end_kernel = .;

	/* Stabs debugging sections.	*/
	.stab 0 : { *(.stab) }
	.stabstr 0 : { *(.stabstr) }
	.stab.excl 0 : { *(.stab.excl) }
	.stab.exclstr 0 : { *(.stab.exclstr) }
	.stab.index 0 : { *(.stab.index) }
	.stab.indexstr 0 : { *(.stab.indexstr) }
	.comment 0 : { *(.comment) }
	.debug_aranges 0 : { *(.debug_aranges) }
	.debug_pubnames 0 : { *(.debug_pubnames) }
	.debug_info 0 : { *(.debug_info) }
	.debug_abbrev 0 : { *(.debug_abbrev) }
	.debug_line 0 : { *(.debug_line) }
	.debug_frame 0 : { *(.debug_frame) }
	.debug_str 0 : { *(.debug_str) }
	.debug_loc 0 : { *(.debug_loc) }
	.debug_macinfo 0 : { *(.debug_macinfo) }
}
