/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  linux/arch/arc/kernel/process.c
 *  Based on from the m68k port	
 *
 *  Copyright (C)
 *
 *  Authors : Amit Bhor
 */

/*
 * This file handles the architecture-dependent parts of process handling..
 */
#include <linux/config.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>
#include <linux/stddef.h>
#include <linux/unistd.h>
#include <linux/ptrace.h>
#include <linux/slab.h>
#include <linux/a.out.h>
#include <linux/reboot.h>
#include <linux/sys.h> // for NR_SYSCALS
#include <asm/uaccess.h>
#include <asm/system.h>
#include <asm/setup.h>
#include <asm/pgtable.h>
#include <asm/system.h>
#include <asm/processor.h>
#include <asm/elf.h>
#include <asm/ptrace.h>
#include <linux/delay.h>

/*
 * The idle thread. There's no useful work to be
 * done, so just try to conserve power and have a
 * low exit latency (ie sit in a loop waiting for
 * somebody to say that they'd like to reschedule)
 */
void cpu_idle(void)
{
	/* endless idle loop with no priority at all */
	init_idle();
	current->nice = 20;
	current->counter = -100;
	while (1) {
		
		/* call the scheduler to check there is any other
		 * task to be run 
		 */
#if 1 //wschen 2008-07-16
		while (!need_resched()) {
                  __asm__ __volatile__ ("sleep \n"); // lower power than looping.
		}
#endif
		schedule();
		check_pgt_cache();
	}		

}

#define ROMReg          (*(volatile unsigned int   *)(0xC0FCF004))                 
void ROMtoRAM(void)
{
    // 1: ROM   0: RAM
        ROMReg = 0;
    //memset(0,0,0x30000);
}

void machine_restart(char * __unused)
{
#if 0 //wschen 2008-07-31
	unsigned int _temp;
	int i;
#endif
	
#if 1 //wschen 2008-07-30
	cli();
#endif
	/* 
	 * Soft reset : jump to reset vector 
	 * Reset First 20K to ROM Area
	 * ROM<->RAM Register : 0xC0FCF004
	 * // 1: ROM   0: RAM
	 */
	(*(volatile unsigned int   *)(0xC0FCF004)) = 1;
	printk("Reset to ROM Code\n");

	/*
	 * Use UART2 Tx to trigger external reset
	 */
	/*rywu20080609
	while(1)
	{
		(*(volatile unsigned int   *)(0xC0FC1110)) = 0;
	}*/

#if 0
    printk("stablize CPU clock...\n");
    _temp = (*(volatile unsigned int   *)(0xC0FC1800));
    printk("   R 0xC0FC1800(0x%8.8x)\n", _temp);
    
    //write pattern
    _temp = (*(volatile unsigned int   *)(0xC0FC180C));
    _temp |= 0xC0000000;
    *((volatile unsigned int   *)0xC0FC180C) = _temp;
    //set wait_cpu_clock_stable
    _temp = (*(volatile unsigned int   *)(0xC0FC180C));
    printk("   W 0xC0FC180C(0x%8.8x) ->", _temp);
    _temp |= 0xC0001000;//bit 12
    printk(" 0x%8.8x\n\n", _temp);
    *((volatile unsigned int   *)0xC0FC180C) = _temp;
    for (i=0; i<20; i++)
    {
        __udelay(1000);
        *((volatile unsigned int   *)0xC0FC180C) = _temp;
    }
    printk("\n");
    
    __udelay(1000);
    _temp = (*(volatile unsigned int   *)(0xC0FC1808));
    _temp &= 0x0ffffffe;
    _temp |= 0xC0000000;
    for(;;) {
        *((volatile unsigned int   *)0xC0FC1808L) = _temp;
    }
#else
	(*(volatile unsigned int   *)(0xC0FC1808)) = (*(volatile unsigned int   *)(0xC0FC1808))|0xc0000000;
	(*(volatile unsigned int   *)(0xC0FC1808)) = (*(volatile unsigned int   *)(0xC0FC1808))&0xfffffffe;
	for(;;)
	{
		;
	}
#endif	
//	__asm__ __volatile__ ("jal	0x0" ::);	
}

void machine_halt(void)
{
	/* Halt the processor */
	__asm__ __volatile__ ("flag	%0" ::"i" (STATUS_H_MASK));	
}

void machine_power_off(void)
{
	/* FIXME ::  power off ??? */

	/* Halt the processor */
	__asm__ __volatile__ ("flag	%0" ::"i" (STATUS_H_MASK));	
}

/*
 * Create a kernel thread
 */
int arch_kernel_thread(int (*fn)(void *), void * arg, unsigned long flags)
{
	long retval;
	long clone_arg = flags | CLONE_VM;
	mm_segment_t fs;

	fs = get_fs();
	set_fs (KERNEL_DS);

	__asm__ __volatile__ (
		"mov	r1, sp \n\t"	
		"mov	r0, %5 \n\t"
		"mov	r8, %1 \n\t"
		"trap0   	; call clone system call \n\t"
		"nop \n\t"
		"nop \n\t"
		"; check return value. 0 - child , otherwise parent \n\t"
		"mov.f	%0, r0 \n\t"
		"jnz	1f	; jump if parent \n\t" 	
		"mov	r0, %3 	; move parameters for child \n\t"
		"jl.nd	[%4] 	; call child function \n\t"
		"mov	r8, %2 	; call exit system call \n\t"
		"trap0 \n\t"
		"nop \n\t"
		"nop \n\t"
		"1: "
		: "=&r" ( retval )
		: "i" ( __NR_clone ), 
		  "i" ( __NR_exit ),
		  "r" ( arg),
		  "r" ( fn ),
		  "r" ( clone_arg )
		: "cc", "r0", "r1", "r8"  
	);

	set_fs (fs);
	return retval;
}

asmlinkage int sys_fork(struct pt_regs *regs)
{
  return do_fork(SIGCHLD, regs->sp , regs, 0);

}

asmlinkage int sys_vfork(struct pt_regs *regs)
{
  return do_fork(CLONE_VFORK | CLONE_VM | SIGCHLD, regs->sp, regs, 0);
}

asmlinkage int sys_clone(unsigned long clone_flags,
		unsigned long newsp, struct pt_regs *regs)
{
  if (!newsp)
    newsp = regs->sp;
  return do_fork(clone_flags, newsp, regs, 0);
}

asmlinkage void ret_from_fork(void);

int copy_thread(int nr, unsigned long clone_flags,
		unsigned long usp, unsigned long topstk,
		struct task_struct * p, struct pt_regs * regs)
{
	struct pt_regs	*childregs;
	struct callee_regs *childcregs, *cregs;
	unsigned long	stack_offset;
	unsigned long	*childksp;
	

	stack_offset = KERNEL_STACK_SIZE - sizeof(struct pt_regs);
	/* leave entries at the end of stack for the child function */ 
	stack_offset -= (sizeof (long)) * 2;
	childregs = (struct pt_regs *) ((unsigned long) p + stack_offset);
	
	/* Copy parents context */
	*childregs = *regs;
	/* fork return value 0 for the child */
	childregs->r0 = 0;

	/* IF the parent is a kernel thread then we change the stack pointer
	 * of the child its kernel stack
	 */
	if (!(regs->status32 & STATUS_U_MASK)) {
		/* Arc gcc stack adjustment */ 
		childregs->sp = (unsigned long)p + ( KERNEL_STACK_SIZE - 8 );
	}
	else
		childregs->sp = usp;

	/* push frame pointer and return address (blink) as expected by
	 * __switch_to function */
	childksp = (unsigned long *) childregs;
	
	*(--childksp) = (unsigned long) ret_from_fork; /* push blink*/
	*(--childksp) = 0;	/* push fp*/


	/* copy remaining context */
	cregs = ((struct callee_regs *) regs) -1;
	childcregs = ((struct callee_regs *) childksp) -1;
	*childcregs = *cregs;

	/* Set kernel stack pointer for the child.
	 * ksp points to next free element on stack i.e. the long word 
	 * before childregs. */
	 
	p->thread.ksp = (unsigned long) childcregs ;
	p->thread.usp = usp;
	
	return 0;
}

asmlinkage int sys_execve(char *filenamei, char **argv, char **envp, struct pt_regs *regs)
{
	int error;
	char * filename;

	filename = getname(filenamei);
	error = PTR_ERR(filename);
	if (IS_ERR(filename))
		goto out;
	error = do_execve(filename, argv, envp, regs);
	putname(filename);
out:
	return error;
}

unsigned int get_wchan(struct task_struct *p)
{
	/* DUMMY: just a dummy function to remove the undefined references */
	return 0;
}

void release_thread(struct task_struct *dead_task)
{
	  /* DUMMY: just a dummy function to remove the undefined references */
}

void flush_thread(void)
{
	  /* DUMMY: just a dummy function to remove the undefined references */
}

void exit_thread(void)
{
	  /* DUMMY: just a dummy function to remove the undefined references */
}

int dump_fpu(struct pt_regs *regs, elf_fpregset_t *fpu)
{
	return 0;
}

/*
 * fill in the user structure for a aout core dump..
 */
void dump_thread(struct pt_regs * regs, struct user * dump)
{
	struct task_struct *tsk = current;
	struct callee_regs *calleeregs;
	/*
	 * Dont use KSP as kernel uses it during a context switch
	 */
	calleeregs= (struct callee_regs *)current->thread.pc;


	dump->magic = CMAGIC;
	dump->start_code = tsk->mm->start_code;
	dump->start_data = tsk->mm->start_data;
	
	//Why dont we take this from the task struct?? This may not be actual stack start.
	
	dump->start_stack = regs->sp & ~(PAGE_SIZE - 1);

	dump->u_tsize = (tsk->mm->end_code - tsk->mm->start_code) >> PAGE_SHIFT;
	dump->u_dsize = (tsk->mm->brk - tsk->mm->start_data + PAGE_SIZE - 1) >> PAGE_SHIFT;
	dump->u_ssize = 0;

	if (dump->start_stack < TASK_SIZE)
		dump->u_ssize = (TASK_SIZE - dump->start_stack) >> PAGE_SHIFT;

	
	/* dump->regs = *regs; */

	/* copy pt_regs */
	memcpy(&dump->regs, regs, sizeof(struct pt_regs));
	/* copy callee_regs */
	memcpy(((char *)&dump->regs) + sizeof(*regs), calleeregs, sizeof(struct callee_regs));

#ifdef CORE_DEBUG
	printk("--- core dumped ---\n");
#endif
}

/*
 * fill in the user structure for an elf core dump
 * NOTE: Before making any additions to the number of registers dumped,   
 * please change the size of the array elf_gregset_t, defined in include/asm/elf.h
 */
void arc_elf_core_copy_regs(elf_gregset_t *pr_reg,struct pt_regs *regs)
{
	struct callee_regs *calleeregs;
	/*
	 * Dont use KSP as kernel uses it during a context switch
	 */
	calleeregs = (struct callee_regs *)current->thread.pc;

	/* copy pt_regs */
	memcpy(((char * )pr_reg), regs, sizeof(struct pt_regs));
	/* copy callee_regs */
	memcpy(((char *)pr_reg) + sizeof(*regs), calleeregs, sizeof(struct callee_regs));

	if(regs->orig_r8 > NR_syscalls + 1)
	{
#ifdef CORE_DEBUG
		printk ("Writing stop_pc(efa) %08lx\n",current->thread.fault_address);
#endif
		memcpy(((char *)pr_reg) + (sizeof(*regs)+sizeof(*calleeregs)), &current->thread.fault_address, sizeof(unsigned long));
	}
	/* orig_r8 -1 for int1 and -2 for int2 and NR_SYSCALLS + 1
	   is for a non-trap_s exception*/
	else
	{
#ifdef CORE_DEBUG
	        printk ("Writing stop_pc(ret) %08lx\n",regs->ret);	
#endif
	  	memcpy(((char *)pr_reg) + sizeof(*regs)+sizeof(*calleeregs), &regs->ret, sizeof(unsigned long));

	}

#ifdef CORE_DEBUG 
	printk("%s done\n", __FUNCTION__);
#endif
}

