#ifndef _SHA2_H
#define _SHA2_H

//#include <stdlib.h>

#define IS_LITTLE_ENDIAN        1234 /* byte 0 is least significant (i386) */
#define IS_BIG_ENDIAN           4321 /* byte 0 is most significant (mc68k) */
#ifdef BIG_ENDIAN
#define PLATFORM_BYTE_ORDER     IS_BIG_ENDIAN
#else
#define PLATFORM_BYTE_ORDER     IS_LITTLE_ENDIAN
#endif


typedef unsigned char       uint_8t;
typedef unsigned short      uint_16t;
typedef unsigned int        uint_32t;
//typedef unsigned long       uint_32t;
//typedef unsigned long long  uint_64t;
typedef ULONGLONG uint_64t;

#define void_ret  void
#define int_ret   int

#undef	EXIT_SUCCESS
#undef	EXIT_FAILURE
#define	EXIT_SUCCESS	0x00
#define	EXIT_FAILURE	0x01		

/* define the hash functions that you need  */
#define SHA_2   /* for dynamic hash length  */
#define SHA_256


/* Note that the following function prototypes are the same */
/* for both the bit and byte oriented implementations.  But */
/* the length fields are in bytes or bits as is appropriate */
/* for the version used.  Bit sequences are arrays of bytes */
/* in which bit sequence indexes increase from the most to  */
/* the least significant end of each byte                   */
#define SHA256_DIGEST_SIZE  32
#define SHA256_BLOCK_SIZE   64

/* type to hold the SHA256 (and SHA224) context */

typedef struct
{   uint_32t count[2];
    uint_32t hash[8];
    uint_32t wbuf[16];
} sha256_ctx;


void_ret sha256_compile(sha256_ctx ctx[1]);
void_ret sha256_begin(sha256_ctx ctx[1]);
void_ret sha256_hash(const unsigned char data[], unsigned long len, sha256_ctx ctx[1]);
void_ret sha256_end(unsigned char hval[], sha256_ctx ctx[1]);
void_ret sha256(unsigned char hval[], const unsigned char data[], unsigned long len);
 

#ifndef SHA_64BIT

typedef struct
{   union
    { sha256_ctx  ctx256[1];
    } uu[1];
    uint_32t    sha2_len;
} sha2_ctx;

#else

/* type to hold the SHA384 (and SHA512) context */
typedef struct
{   uint_64t count[2];
    uint_64t hash[8];
    uint_64t wbuf[16];
} sha512_ctx;
//typedef sha512_ctx  sha384_ctx;

typedef struct
{   union
    { sha256_ctx  ctx256[1];
      sha512_ctx  ctx512[1];
    } uu[1];
    uint_32t    sha2_len;
} sha2_ctx;

int_ret  sha2_begin(unsigned long size, sha2_ctx ctx[1]);
void_ret sha2_hash(const unsigned char data[], unsigned long len, sha2_ctx ctx[1]);
void_ret sha2_end(unsigned char hval[], sha2_ctx ctx[1]);
int_ret  sha2(unsigned char hval[], unsigned long size, const unsigned char data[], unsigned long len);

#endif
#endif	/* _SHA2_H */
