/*
 *
 *
 *
 */ 

#include <linux/config.h>
#include <linux/sched.h>
#include <linux/signal.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/mii.h>
#include <linux/ethtool.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/usb.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <linux/crc32.h>

#define deverr(usbnet, fmt, arg...) \
	printk("%s: " fmt "\n" , (usbnet)->net->name , ## arg)
//	printk(KERN_ERR "%s: " fmt "\n" , (usbnet)->net->name , ## arg)
#define devwarn(usbnet, fmt, arg...) \
	printk("%s: " fmt "\n" , (usbnet)->net->name , ## arg)
//	printk(KERN_WARNING "%s: " fmt "\n" , (usbnet)->net->name , ## arg)
#define devinfo(usbnet, fmt, arg...) \
	printk("%s: " fmt "\n" , (usbnet)->net->name , ## arg);
//	printk(KERN_INFO "%s: " fmt "\n" , (usbnet)->net->name , ## arg);

// for vendor-specific control operations
/* msec */
#define CONTROL_TIMEOUT_MS	(500)
#define CONTROL_TIMEOUT_JIFFIES ((CONTROL_TIMEOUT_MS * HZ)/1000)

#define NEED_MII

#define AX_CMD_SET_SW_MII			0x06
#define AX_CMD_READ_MII_REG			0x07
#define AX_CMD_WRITE_MII_REG		0x08
#define AX_CMD_SET_HW_MII			0x0a
#define AX_CMD_READ_EEPROM			0x0b
#define AX_CMD_WRITE_EEPROM			0x0c
#define AX_CMD_WRITE_RX_CTL			0x10
#define AX_CMD_READ_IPG012			0x11
#define AX_CMD_WRITE_IPG0			0x12
#define AX_CMD_WRITE_IPG1			0x13
#define AX_CMD_WRITE_IPG2			0x14
#define AX_CMD_WRITE_MULTI_FILTER	0x16
#define AX_CMD_READ_NODE_ID			0x17
#define AX_CMD_READ_PHY_ID			0x19
#define AX_CMD_WRITE_MEDIUM_MODE	0x1b
#define AX_CMD_READ_MONITOR_MODE	0x1c
#define AX_CMD_WRITE_MONITOR_MODE	0x1d
#define AX_CMD_WRITE_GPIOS			0x1f
#define AX_CMD_SW_RESET 			0x20
#define AX_CMD_SW_PHY_STATUS		0x21
#define AX_CMD_SW_PHY_SELECT		0x22
#define AX88772_CMD_READ_NODE_ID	0x13

#define AX_MONITOR_MODE 			0x01
#define AX_MONITOR_LINK 			0x02
#define AX_MONITOR_MAGIC			0x04
#define AX_MONITOR_HSFS 			0x10

#define AX_MCAST_FILTER_SIZE		8
#define AX_MAX_MCAST				64
#define AX_INTERRUPT_BUFSIZE		8

#define AX88772_IPG0_DEFAULT		0x15
#define AX88772_IPG1_DEFAULT		0x0c
#define AX88772_IPG2_DEFAULT		0x12

struct usbnet {
	// housekeeping
	struct usb_device	*udev;

	struct semaphore	sem;

	struct timer_list	event_timer;
	struct tq_struct	kevent;

	// i/o info: pipes etc
	unsigned int		in, out;
	unsigned int		maxpacket;

	struct urb			*rx_urb,
						*tx_urb,
						*ctrl_urb;

	u8					rx_buffer[2048];
	u8					tx_buffer[2048];

	// protocol/interface state
	struct net_device	*net;
	struct net_device_stats stats;
	struct mii_if_info	mii;

	volatile unsigned char	CheckLink;

	unsigned char		Status;
#define STATUS_STOPPED				0
#define STATUS_RUNNING				1

	unsigned char		Link;
#define LINK_OFF					0
#define LINK_ON						1
#define LINK_UNKNOW					2

	unsigned char		Speed;
	unsigned char		Duplex;
#define DUPLEX_HALF					0
#define DUPLEX_FULL					1
};

// mask definition to make the build fail.
static const char driver_name[] = "usbnet";

static int
ax8817x_read_cmd(struct usbnet *dev, u8 cmd, u16 value, u16 index,
			    u16 size, void *data)
{
	return usb_control_msg(
		dev->udev,
		usb_rcvctrlpipe(dev->udev, 0),
		cmd,
		USB_DIR_IN | USB_TYPE_VENDOR | USB_RECIP_DEVICE,
		value,
		index,
		data,
		size,
		CONTROL_TIMEOUT_JIFFIES);
}

static int
ax8817x_write_cmd(struct usbnet *dev, u8 cmd, u16 value, u16 index,
			     u16 size, void *data)
{
	return usb_control_msg(
		dev->udev,
		usb_sndctrlpipe(dev->udev, 0),
		cmd,
		USB_DIR_OUT | USB_TYPE_VENDOR | USB_RECIP_DEVICE,
		value,
		index,
		data,
		size,
		CONTROL_TIMEOUT_JIFFIES);
}

static void
ax8817x_async_cmd_callback(struct urb *urb)
{
	struct usb_ctrlrequest *req = (struct usb_ctrlrequest *)urb->context;

	kfree(req);
	usb_free_urb(urb);
}

static void 
ax8817x_write_cmd_async(struct usbnet *dev, u8 cmd, u16 value,
						u16 index, u16 size, void *data)
{
	struct usb_ctrlrequest *req;
	int status;
	struct urb *urb;

	if ((urb = usb_alloc_urb(0)) == NULL) {
		deverr(dev, "Error allocating URB in write_cmd_async!\n");
		return;
	}

	if ((req = kmalloc(sizeof(struct usb_ctrlrequest), GFP_ATOMIC)) == NULL) {
		deverr(dev, "Failed to allocate memory for control request\n");
		usb_free_urb(urb);
		return;
	}

	req->bRequestType = USB_DIR_OUT | USB_TYPE_VENDOR | USB_RECIP_DEVICE;
	req->bRequest = cmd;
	req->wValue = cpu_to_le16(value);
	req->wIndex = cpu_to_le16(index);
	req->wLength = cpu_to_le16(size);

	usb_fill_control_urb(urb, dev->udev,
			     usb_sndctrlpipe(dev->udev, 0),
			     (void *)req, data, size,
			     ax8817x_async_cmd_callback, req);

	if((status = usb_submit_urb(urb)) < 0)
		deverr(dev, "Error submitting the control message: status=%d", status);
}

static int
ax8817x_mdio_read(struct net_device *netdev, int phy_id, int loc)
{
	struct usbnet *dev = netdev->priv;
	u16 res;
	u8 buf[4];

	ax8817x_write_cmd(dev, AX_CMD_SET_SW_MII, 0, 0, 0, &buf);
	ax8817x_read_cmd(dev, AX_CMD_READ_MII_REG, phy_id,
			(__u16)loc, 2, (u16 *)&res);
	ax8817x_write_cmd(dev, AX_CMD_SET_HW_MII, 0, 0, 0, &buf);

	return res & 0xffff;
}

static void
ax8817x_mdio_write(struct net_device *netdev, int phy_id, int loc, int val)
{
	struct usbnet *dev = netdev->priv;
	u16 res = val;
	u8 buf[4];

	ax8817x_write_cmd(dev, AX_CMD_SET_SW_MII, 0, 0, 0, &buf);
	ax8817x_write_cmd(dev, AX_CMD_WRITE_MII_REG, phy_id,
			(__u16)loc, 2, (u16 *)&res);
	ax8817x_write_cmd(dev, AX_CMD_SET_HW_MII, 0, 0, 0, &buf);
}

static void
ax8817x_set_multicast(struct net_device *net)
{

	struct usbnet *dev = (struct usbnet *) net->priv;
	u8 rx_ctl = 0x88;

	if (net->flags & IFF_PROMISC) {
		rx_ctl |= 0x01;
	} else if (net->flags & IFF_ALLMULTI
		   || net->mc_count > AX_MAX_MCAST) {
		rx_ctl |= 0x02;
	} else if (net->mc_count == 0) {
		/* just broadcast and directed */
	} else {
		struct dev_mc_list *mc_list = net->mc_list;
		u8 *multi_filter;
		u32 crc_bits;
		int i;

		multi_filter = kmalloc(AX_MCAST_FILTER_SIZE, GFP_ATOMIC);
		if (multi_filter == NULL) {
			/* Oops, couldn't allocate a buffer for setting the multicast
			   filter. Try all multi mode. */
			rx_ctl |= 0x02;
		} else {
			memset(multi_filter, 0, AX_MCAST_FILTER_SIZE);

			/* Build the multicast hash filter. */
			for (i = 0; i < net->mc_count; i++) {
				crc_bits =
				    ether_crc(ETH_ALEN,
					      mc_list->dmi_addr) >> 26;
				multi_filter[crc_bits >> 3] |=
				    1 << (crc_bits & 7);
				mc_list = mc_list->next;
			}

			ax8817x_write_cmd_async(dev, AX_CMD_WRITE_MULTI_FILTER, 0, 0,
					   AX_MCAST_FILTER_SIZE, multi_filter);

			rx_ctl |= 0x10;
		}
	}

	ax8817x_write_cmd_async(dev, AX_CMD_WRITE_RX_CTL, rx_ctl, 0, 0, NULL);

}

static int
ax88772_bind(struct usbnet *dev, struct usb_device *intf)
{
	int ret;
	u8 buf[6];
	u16 buf16_1[1];//,actual_spm,medium_mode;
	u16 *buf16 = (u16 *) buf;
	int i;

	dev->in = usb_rcvbulkpipe(dev->udev, 2);
	dev->out = usb_sndbulkpipe(dev->udev, 3);

	if ((ret = ax8817x_write_cmd(
				dev,
				AX_CMD_WRITE_GPIOS,
				0x00B0,0, 0,
				buf)) < 0) {
		deverr(dev, "Failed to write gpio %d", ret);
		return ret;
	}
	wait_ms(5);

	if ((ret = ax8817x_write_cmd(
				dev, 0x22,
				0x0001,  0,
				0, buf)) < 0) {
		deverr(dev, "select PHy register failed: %d", ret);
		return ret;
	}

	if ((ret = ax8817x_write_cmd(dev, 0x20, 0x0040, 0, 0, buf)) < 0) {
		deverr(dev, "generate wave form failed: %d", ret);
		return ret;
	}
		wait_ms(150);
	if ((ret = ax8817x_write_cmd(dev, 0x20, 0x0000, 0, 0, buf)) < 0) {
		deverr(dev, "generate wave form failed: %d", ret);
		return ret;
	}
		wait_ms(150);
	if ((ret = ax8817x_write_cmd(dev, 0x20, 0x0028, 0, 0, buf)) < 0) {
		deverr(dev, "generate wave form failed: %d", ret);
		return ret;
	}

		wait_ms(150);

	if ((ret = ax8817x_write_cmd(dev, AX_CMD_WRITE_RX_CTL,
					0x0000, 0, 0, buf)) < 0) {
		deverr(dev, "send AX_CMD_WRITE_RX_CTL failed: %d", ret);
		return ret;	//CACTUS stop
	}

	/* Get the MAC address */
	memset(buf, 0, ETH_ALEN);
	if ((ret = ax8817x_read_cmd(dev, 0x13, 0, 0, 6, buf)) < 0) {
		deverr(dev, "read AX_CMD_READ_NODE_ID failed: %d", ret);
		return ret;
	}
	memcpy(dev->net->dev_addr, buf, ETH_ALEN);


	if ((ret = ax8817x_write_cmd(dev, 0x06, 0x0000, 0, 0, buf)) < 0) {
		deverr(dev, "enable PHY reg. reading capability: %d", ret);
		return ret;	//enable Phy register reading capability
	}
	if (((ret = ax8817x_read_cmd(dev, 0x07, 0x0010, 0x0002, 0x0002, buf16_1)) < 0)
	    ||(buf16_1[0]!=0x003b)  ) {
		deverr(dev, "read PHY register 2 must be 0x3b00: %d", ret);
		return ret;	//read Phy register 2 must be 0x3b00
	}

	/* Get the PHY id */
	if ((ret = ax8817x_read_cmd(dev, AX_CMD_READ_PHY_ID, 0, 0, 2, buf)) < 0) {
		deverr(dev, "error on read AX_CMD_READ_PHY_ID: %02x", ret);
		return ret;
	} else if (ret < 2) {
		/* this should always return 2 bytes */
		deverr(dev,"read MAC address less than 2 bytes: ret=%02x", ret);
		return -EIO;
	}

	if ((ret = ax8817x_write_cmd(dev, 0x20, 0x0008, 0, 0, buf)) < 0) {
		deverr(dev, "generate wave form failed: %d", ret);
		return ret;
	}
		wait_ms(150);
	if ((ret = ax8817x_write_cmd(dev, 0x20, 0x0028, 0, 0, buf)) < 0) {
		deverr(dev, "generate wave form failed: %d", ret);
		return ret;
	}
		wait_ms(150);
	/* Initialize MII structure */
	dev->mii.dev = dev->net;
	dev->mii.mdio_read = ax8817x_mdio_read;
	dev->mii.mdio_write = ax8817x_mdio_write;
	dev->mii.phy_id_mask = 0x3f;
	dev->mii.reg_num_mask = 0x1f;
	dev->mii.phy_id = buf[1];


	*buf16 = cpu_to_le16(BMCR_RESET);
	if ((ret = ax8817x_write_cmd(dev, AX_CMD_WRITE_MII_REG,
				     dev->mii.phy_id, MII_BMCR, 2, buf16)) < 0) {
		deverr(dev, "Failed to write MII reg - MII_BMCR: %02x", ret);
		return ret;
	}

	/* Advertise that we can do full-duplex pause */
	*buf16 = cpu_to_le16(ADVERTISE_ALL | ADVERTISE_CSMA | 0x0400);
	if ((ret = ax8817x_write_cmd(dev, AX_CMD_WRITE_MII_REG,
			dev->mii.phy_id, MII_ADVERTISE, 2, buf16)) < 0) {
		deverr(dev, "Failed to write MII_REG advertisement: %02x", ret);
		return ret;   //could be 0xe105
	}

	*buf16 = cpu_to_le16(BMCR_ANRESTART | BMCR_ANENABLE);
	if ((ret = ax8817x_write_cmd(dev, AX_CMD_WRITE_MII_REG,
				dev->mii.phy_id, MII_BMCR, 2, buf16)) < 0) {
		deverr(dev, "Failed to write MII reg autonegotiate: %02x", ret);
		return ret;
	}

	if ((ret = ax8817x_write_cmd(dev, 0x12, 0x0c15, 0x000e, 0, buf)) < 0) {
		deverr(dev, "write IPG,IPG1,IPG2 failed: %d", ret);
		return ret;
	}
	if ((ret = ax8817x_write_cmd(dev, AX_CMD_SET_HW_MII, 0, 0, 0, &buf)) < 0) {
		deverr(dev, "Failed to set hardware MII: %02x", ret);
		return ret;
	}

	if ((ret = ax8817x_write_cmd(dev, AX_CMD_WRITE_RX_CTL,
					0x0088, 0, 0, buf)) < 0) {
		deverr(dev, "send AX_CMD_WRITE_RX_CTL failed: %d", ret);
		return ret;				//CACTUS stop
	}

	dev->net->set_multicast_list = ax8817x_set_multicast;

	wait_ms(100);
	return 0;

}

static struct net_device_stats *
usbnet_get_stats (struct net_device *net)
{
	return &((struct usbnet *) net->priv)->stats;
}

static void
tx_complete(struct urb *urb)
{
	struct usbnet	*dev = (struct usbnet *)urb->context;

	if (!dev)
		return;

	if (!netif_device_present(dev->net))
		return;

	dev->net->trans_start = jiffies;
	netif_wake_queue(dev->net);
}

static int
usbnet_start_xmit(struct sk_buff *skb, struct net_device *net)
{
	struct usbnet	*dev = (struct usbnet *) net->priv;
	u32				*header = (u32 *)dev->tx_buffer;
	u8				*packet = (u8 *)(header + 1);
	int				send_len;
	int				ret;

	if (!netif_device_present(dev->net)) {
		printk("netdev does not present!\n");
		dev->stats.tx_errors++;
		goto free;
	}

	netif_stop_queue(net);

#if 0
{
int i;
printk("Dump transmit packet : %d", skb->len);
for(i = 0; i < skb->len; i++)
{
	if((i % 16) == 0)
		printk("\n%4.4x", i);
	printk(" %2.2x", skb->data[i]);
}
printk("\n");
}
#endif

	*header = ((skb->len ^ 0x0000ffff) << 16) + skb->len;
	memcpy(packet, skb->data, skb->len);

	send_len = skb->len + 4;
	if((send_len % dev->maxpacket) == 0 ||
		(send_len % dev->maxpacket) == 2)
		send_len += 4;

	usb_fill_bulk_urb(dev->tx_urb, dev->udev, dev->out,
		dev->tx_buffer, send_len, tx_complete, dev);

	if ((ret = usb_submit_urb(dev->tx_urb))) {
		deverr(dev, "can't submit urb\n");
		dev->stats.tx_errors++;
		netif_wake_queue(net);
	} else {
		dev->stats.tx_packets++;
		dev->stats.tx_bytes += skb->len;
		net->trans_start = jiffies;
	}

free:
	dev_kfree_skb(skb);

	return 0;
}

static void
ax88772_rx_submit(struct usbnet *dev, int total_len)
{
	u8				tmp[4];
	u32				*header = (u32 *)tmp;
	u8				*packet;
	u16				size;
	struct sk_buff	*skb;

	packet = (u8 *)dev->rx_buffer;

	while(1) {

		// minimum packet length 60 bytes + 4 bytes header
		if(total_len < 64)
			break;

		// the address of the array tmp is always
		// aligned with 32-biit address
		memcpy(tmp, packet, 4);

		// get the packet count of the received skb
		le32_to_cpus (header);

		// set the current packet pointer to the first packet
		packet += 4;  //move on 4 bytes header then reach the data ptr.

		if((short)(*header&0x0000ffff)!=~( (short)((*header&0xffff0000)>>16) ))
		{
			deverr(dev, "header length data is error");
			break;
		}

		// get the packet length
		size = (u16)(*header & 0x0000ffff);

		// this may be a broken packet
		if (size > 1514) {  //ask ?? if 1514 is correct
			dbg ("ax88772: invalid rx length %d", size);
			break;
		}

		// allocate the skb for the individual packet
		skb = dev_alloc_skb(size + 2);
		if (skb) {
			skb->dev = dev->net;
			skb_reserve(skb, 2);
			memcpy(skb_put(skb, size), packet, size);
#if 0
{
	int i;
	printk("Dump receive packet, length = %d", size);
	for(i = 0; i < size; i++)
	{
		if((i % 16) == 0)
			printk("\n%4.4x", i);
		printk(" %2.2x", skb->data[i]);
	}
	printk("\n");
}
#endif
			skb->protocol = eth_type_trans(skb, dev->net);

			netif_rx(skb);

			dev->stats.rx_packets++;
			dev->stats.rx_bytes += size;
		}else{
			break;
		}

		// shift the data pointer to the next header
		packet += ((size + 1) & 0xFFFE);
		total_len -= ((size + 4 + 1) & 0xFFFE);
	}
}

static void
rx_complete(struct urb *urb)
{
	struct usbnet	*dev;	
	int				length = urb->actual_length;
	int				ret;

	dev = (struct usbnet *)urb->context;
	if(!dev)
		return;
	if(!dev->Status)
		return;

	if (!netif_device_present(dev->net)) {
		deverr(dev, "netdev is not present\n");
		return;
	}

	switch (urb->status) {
	case 0:
		if(length != 0) {
			ax88772_rx_submit(dev, length);
		}
		break;
	case -ENOENT:
		return;
	case -ETIMEDOUT:
		deverr(dev, "bulk timed-out\n");
		break;
	default:
		deverr(dev, "Rx status %d\n", urb->status);
		break;
	}

	usb_fill_bulk_urb(dev->rx_urb, dev->udev, dev->in,
		dev->rx_buffer, 2048, rx_complete, dev);
	dev->rx_urb->transfer_flags |= USB_QUEUE_BULK;
	dev->rx_urb->timeout = 1 * HZ;

	if((ret = usb_submit_urb(dev->rx_urb)))
		deverr(dev, "Submitting receive urb failed %d\n", ret);
}

static void
kevent (void *data)
{
	struct usbnet	*dev = (struct usbnet *)data;
	u8				Link, Speed, Duplex;
	u16				bmcr, bmsr;
	u16				mode = 0x0134;

	if(!dev)
		return;

	if(!dev->Status)
		return;

	if (!netif_device_present(dev->net)) {
		return;
	}

	dev->CheckLink = 1;

	bmsr = ax8817x_mdio_read(dev->net, dev->mii.phy_id, MII_BMSR);
	bmcr = ax8817x_mdio_read(dev->net, dev->mii.phy_id, MII_BMCR);

	Link = (bmsr & BMSR_LSTATUS) == 0 ? LINK_OFF : LINK_ON;
	Speed = (bmcr & BMCR_SPEED100) == 0 ? 10 : 100;
	Duplex = (bmcr & BMCR_FULLDPLX) == 0 ? DUPLEX_HALF : DUPLEX_FULL;

	if((Link != dev->Link) || (Speed != dev->Speed) || (Duplex != dev->Duplex)) {
		if(Link) {
			if(Speed == 100)
				mode |= 0x0200;
			if(Duplex)
				mode |= 0x0002;

			ax8817x_write_cmd(dev, 0x1b, mode , 0, 0, NULL);

			netif_carrier_on(dev->net);
			printk("AX88772 : Link up\n");

		} else {
			netif_carrier_off(dev->net);
			printk("AX88772 : Link down\n");
		}
		dev->Link = Link;
		dev->Speed = Speed;
		dev->Duplex = Duplex;
	}

	dev->CheckLink = 0;
}

static void
timer_func(unsigned long arg)
{
	struct usbnet	*dev = (struct usbnet *)arg;

	if(!dev)
		return;

	if(!dev->Status)
		return;

	if (!netif_device_present(dev->net))
		return;

	schedule_task(&dev->kevent);

	mod_timer(&dev->event_timer, jiffies + 2 * HZ);
}

static int
usbnet_open(struct net_device *net)
{
	struct usbnet	*dev = (struct usbnet *) net->priv;
	int				ret;

	down(&dev->sem);

	if (dev == NULL) {
		printk("usbnet_open end... (dev = NULL)\n");
		return -ENODEV;
	}

	dev->rx_urb = usb_alloc_urb(0);
	if (!dev->rx_urb) {
		deverr(dev, "can't alloc urb\n");
		return -1;
	}

	dev->tx_urb = usb_alloc_urb(0);
	if (!dev->tx_urb) {
		deverr(dev, "can't alloc urb\n");
		usb_free_urb(dev->rx_urb);
		return -1;
	}

	usb_fill_bulk_urb(dev->rx_urb, dev->udev, dev->in,
		dev->rx_buffer, 2048, rx_complete, dev);

	dev->rx_urb->transfer_flags |= USB_QUEUE_BULK;

	if ((ret = usb_submit_urb(dev->rx_urb)))
		deverr(dev, "can't submit urb\n");

	netif_start_queue(net);

	dev->Status = STATUS_RUNNING;

	dev->Link = LINK_UNKNOW;
	init_timer(&dev->event_timer);
	dev->event_timer.function = timer_func;
	dev->event_timer.expires = jiffies + 1 * HZ;
	dev->event_timer.data = dev;
	add_timer(&dev->event_timer);

	up(&dev->sem);

	return 0;
}

static int
usbnet_stop (struct net_device *net)
{
	struct usbnet	*dev = (struct usbnet *) net->priv;

	if (!dev)
		return -ENODEV;

	down(&dev->sem);

	del_timer_sync(&dev->event_timer);

	usb_unlink_urb(dev->rx_urb);
	usb_unlink_urb(dev->tx_urb);
	usb_free_urb(dev->rx_urb);
	usb_free_urb(dev->tx_urb);

	netif_stop_queue (net);

	dev->Status = STATUS_STOPPED;

	while(dev->CheckLink == 1) {
		msleep(1);
	}

	up(&dev->sem);
	return 0;
}

static void *
usbnet_probe (struct usb_device *udev, unsigned ifnum,
			const struct usb_device_id *prod)
{
	struct usbnet		*dev;
	struct net_device	*netdev;
	int					ret;

	udev->config[0].bConfigurationValue = 1;
	if (usb_set_configuration(udev, udev->config[0].bConfigurationValue)) {
		printk("usb_set_configuration() failed");
		return NULL;
	}

	// set up our own records
	if (!(dev = kmalloc (sizeof *dev, GFP_KERNEL))) {
		deverr(dev, "can't kmalloc dev\n");
		return NULL;
	}
	memset(dev, 0, sizeof *dev);

	netdev = init_etherdev(NULL, 0);
	if (!netdev) {
		deverr(dev, "can't kmalloc netdev\n");
		kfree(dev);
		return NULL;
	}

	init_MUTEX(&dev->sem);
	down(&dev->sem);

	dev->udev = udev;
	dev->net = netdev;
	netdev->priv = dev;

	// set up network interface records
	SET_MODULE_OWNER(netdev);

	netdev->get_stats = usbnet_get_stats;
	netdev->hard_start_xmit = usbnet_start_xmit;
	netdev->open = usbnet_open;
	netdev->stop = usbnet_stop;

	ret = ax88772_bind(dev, udev);
	if(ret != 0) {
		printk("Init ax88772 error... %d\n", ret);
		kfree(netdev);
		kfree(dev);
		return NULL;
	}

	dev->maxpacket = usb_maxpacket(dev->udev, dev->out, 1);

	dev->Status = STATUS_STOPPED;

	INIT_TQUEUE(&dev->kevent, kevent, dev);

	printk("%s: ax88772 is detected\n", netdev->name);

	up(&dev->sem);
	return dev;
}

static void
usbnet_disconnect (struct usb_device *udev, void *ptr)
{
	struct usbnet	*dev = (struct usbnet *) ptr;

	unregister_netdev(dev->net);

	kfree(dev->net);
	dev->net = NULL;
	kfree(dev);
	dev = NULL;

}

/* table of devices that work with this driver */
static const struct usb_device_id	products [] = {
	{USB_DEVICE (0x0b95, 0x7720),},
	{USB_DEVICE (0x0b95, 0x772A),},
	{ },
};
MODULE_DEVICE_TABLE (usb, products);

static struct usb_driver usbnet_driver = {
	name:		driver_name,
	probe:		usbnet_probe,
	disconnect:	usbnet_disconnect,
	id_table:	products,
};

static int __init
usbnet_init (void)
{
	return usb_register(&usbnet_driver);
}

static void __exit
usbnet_exit (void)
{
	usb_deregister(&usbnet_driver);
}

module_init (usbnet_init);
module_exit (usbnet_exit);

EXPORT_NO_SYMBOLS;
MODULE_AUTHOR ("ASIX <Supportl@asix.com.tw>");
MODULE_DESCRIPTION ("AX88772/772A USB Ethernet Drivers");
MODULE_LICENSE ("GPL");
