
/*
 *			bvci_profiler.h - BVCI profiler
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 *	
 *	(c) Copyright 2006		ARC International plc
 *
 */


#ifndef ARC_BVCI_PROFILER_H
#define ARC_BVCI_PROFILER_H

#define BVCI_PROFILER_MINOR 253 /* Device minor number FIXME Allocate properly */
#define PROFILER_BASE_ADDRESS	 0xc0fed000

#define PROCFILENAME "bvci_profiler"

typedef struct {
	volatile unsigned long id;
	volatile unsigned long bpr_ctrl;
	volatile unsigned long bpr_setup1;
	volatile unsigned long bpr_setup2;
	volatile unsigned long bpr_cyccnt;
	volatile unsigned long bpr_index;
	volatile unsigned long bpr_vacyccnt;
	volatile unsigned long bpr_idlcyccnt;
	volatile unsigned long bpr_pktcnt;
	volatile unsigned long bpr_sumlat;
	volatile unsigned long bpr_lat;
	volatile unsigned long bpr_cychi;
	volatile unsigned long bpr_vacychi;
	volatile unsigned long bpr_idlcychi;
	volatile unsigned long bpr_pkthi;
	volatile unsigned long bpr_sumlathi;
	volatile unsigned long bpr_vacyccap;
	volatile unsigned long bpr_idlcyccap;
	volatile unsigned long bpr_pktcap;
	volatile unsigned long bpr_sumlatcap;
	volatile unsigned long bpr_latcap;
} bvci_profiler;



/* ioctls */
#define BVCI_GET_CTRL			0xe0000001
#define BVCI_SET_CTRL			0xe0000002
#define BVCI_GET_PROFILER 0xe0000003
#define BVCI_SET_PROFILER 0xe0000004

/* Max no of monitoring nodes */
#define MAX_NODE 5
/* Number of nodes per setup reg */
#define NODES_PER_REG 4

/* BPR_CTRL defns */
#define BPR_CTRL_HALT				 0x01
#define BPR_CTRL_CLEAR_CNT	 0x02
#define BPR_CTRL_CLEAR_LAT	 0x04
#define BPR_CTRL_SCAD				 0x08
#define BPR_CTRL_NO_DEBUG		 0x10
#define BPR_CTRL_CAP_EN			 0x20
#define BPR_CTRL_CAP_FLAG		 0x40

#define BPR_CTRL_STOP_TRIG_MASK 0xffffff00
#define TO_BPR_CTRL_STOP_TRIG(trigval) ((trigval << 8) & BPR_CTRL_STOP_TRIG_MASK)


/* BPR_SETUPx defns */
#define SETUP1_FIELD_LEN 5
#define SETUP1_FIELD_MASK 0x1f
#define SETUP1_FIELD_OFFSET 8

#define SETUP_INSTR_FETCH		 0x00							 /* i0, Instruction fetch initiator interface. */
#define SETUP_DATA_FETCH		 0x01							 /* i1, Data fetch initiator interface */
#define SETUP_PERIPH_BRDGE	 0x02							 /* i2, Peripheral Bus Bridge initiator Interface */
#define SETUP_XY_DMA				 0x03							 /* i3, XY DMA initiator Interface */
#define SETUP_PGU						 0x04							 /* i4, PGU initiator interface */
#define SETUP_MEDIA_OUT			 0x05							 /* i5, Media DMA Out initiator Interface. */
#define SETUP_MEDIA_IN			 0x06							 /* i6, Media DMA In initiator Interface. */
#define SETUP_HW_PROF				 0x08							 /* i8, Hardware Profilier initiator Interface. */
#define SETUP_DEFAULT				 0x20							 /* t0, Default target interface */
#define SETUP_MEM_CTLR			 0x21							 /* t1, Memory Controller target interface */
#define SETUP_BRIDGE				 0x22							 /* t2, Bridge target interface */
#define SETUP_BOOT_ROM			 0x23							 /* t3, Boot_rom target Interface */

/* BPR_LAT and BPRLATCAP */

#define MAX_LAT_MASK 0xffffff
#define MAX_LAT(regval) (regval & MAX_LAT_MASK)
#define MIN_LAT_MASK 0xff000000
#define MIN_LAT(regval) ((regval & MIN_LAT_MASK) >> 24)


#endif
