/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  linux/include/asm-arcnommu/entry.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authors: Amit Bhor
 */

#ifndef __ASM_ARC_ENTRY_H
#define __ASM_ARC_ENTRY_H

#ifdef __ASSEMBLY__
#include <linux/sys.h> /* For NR_syscalls defination */
/* Context saving macros -
 */

/* Save caller saved registers (scratch registers) ( r0 - r15, r24, r25 )
 * Registers are pushed / popped in the order defined in struct ptregs
 * in asm/ptrace.h
 */
.macro	SAVE_CALLER_SAVED
	st.a	r0, [sp, -4]
	st.a	r1, [sp, -4]
	st.a	r2, [sp, -4]
	st.a	r3, [sp, -4]
	st.a	r4, [sp, -4]
	st.a	r5, [sp, -4]
	st.a	r6, [sp, -4]
	st.a	r7, [sp, -4]
	st.a	r8, [sp, -4]
	st.a	r9, [sp, -4]
	st.a	r10, [sp, -4]
	st.a	r11, [sp, -4]
	st.a	r12, [sp, -4]
.endm

/* Restore caller saved registers (scratch registers)
 * Registers are pushed / popped in the order defined in struct callee_regs
 * in asm/ptrace.h
 */
.macro RESTORE_CALLER_SAVED
	ld.ab	r12, [sp, 4]
	ld.ab	r11, [sp, 4]	
	ld.ab	r10, [sp, 4]	
	ld.ab	r9, [sp, 4]	
	ld.ab	r8, [sp, 4]	
	ld.ab	r7, [sp, 4]	
	ld.ab	r6, [sp, 4]	
	ld.ab	r5, [sp, 4]	
	ld.ab	r4, [sp, 4]	
	ld.ab	r3, [sp, 4]	
	ld.ab	r2, [sp, 4]	
	ld.ab	r1, [sp, 4]	
	ld.ab	r0, [sp, 4]
.endm	

/* Save callee saved registers (non scratch registers) ( r16 - r23 )
 * This is the actual context saved when a task gets scheduled out.
 * sp points to the next free element on the stack at exit of this macro.
 * Registers are pushed / popped in the order defined in struct callee_regs
 * in asm/ptrace.h
 */
.macro SAVE_CALLEE_SAVED
	st.a	r13, [sp, -4]
	st.a	r14, [sp, -4]
	st.a	r15, [sp, -4]
	st.a	r16, [sp, -4]
	st.a	r17, [sp, -4]
	st.a	r18, [sp, -4]
	st.a	r19, [sp, -4]
	st.a	r20, [sp, -4]
	st.a	r21, [sp, -4]
	st.a	r22, [sp, -4]
	st.a	r23, [sp, -4]
	st.a	r24, [sp, -4]
	st.a	r25, [sp, -4]
	/* move sp to next free entry */
	sub	sp, sp, 4
.endm

/* Restore callee saved registers (non scratch registers)
 * This is the actual context saved when a task gets scheduled out.
 * sp points to the next free element on the stack at exit of this macro.
 * Registers are pushed / popped in the order defined in struct callee_regs
 * in asm/ptrace.h
 */
.macro RESTORE_CALLEE_SAVED
	add	sp, sp, 4	/* restoring sp to original value */
	ld.ab	r25, [sp, 4]	
	ld.ab	r24, [sp, 4]	
	ld.ab	r23, [sp, 4]	
	ld.ab	r22, [sp, 4]	
	ld.ab	r21, [sp, 4]	
	ld.ab	r20, [sp, 4]	
	ld.ab	r19, [sp, 4]	
	ld.ab	r18, [sp, 4]	
	ld.ab	r17, [sp, 4]	
	ld.ab	r16, [sp, 4]	
	ld.ab	r15, [sp, 4]	
	ld.ab	r14, [sp, 4]	
	ld.ab	r13, [sp, 4]	
.endm

/* Save all registers used by non-system-exception calls.
 * sp points to the next free element on the stack at exit of this macro.
 * Registers are pushed / popped in the order defined in struct ptregs
 * in asm/ptrace.h
 */
.macro SAVE_ALL_SYS
	/* user stack pointer( or kernel sp) , put in r9 by sys call handler */ 
	st.a	r9, [sp, -4]	
	/* retsore original r9 , saved in ex_saved_reg1 */
	ld	r9, [SYMBOL_NAME(ex_saved_reg1)]
	/* now we are ready to save the remaining context */
	st.a	NR_syscalls + 1, [sp, -4]	/* store a positive value in orig_r8 */
	st.a	r0, [sp, -4]	/* orig_r0 */
	SAVE_CALLER_SAVED
	st.a	r26, [sp, -4]	/* gp */
	st.a	fp, [sp, - 4]
	st.a	blink, [sp, -4]
	lr	r9, [eret]
	st.a	r9, [sp, -4]
	lr	r9, [erstatus]
	st.a	r9, [sp, -4]
	st.a	lp_count, [sp, -4]
	lr	r9, [lp_end]
	st.a	r9, [sp, -4]
	lr	r9, [lp_start]
	st.a	r9, [sp, -4]
	lr	r9, [erbta]
	st.a	r9, [sp, -4]
        /* move sp to next free entry */
	sub	sp, sp, 4
.endm

/* Save all registers used by system calls.
 * sp points to the next free element on the stack at exit of this macro.
 * Registers are pushed / popped in the order defined in struct ptregs
 * in asm/ptrace.h
 */
.macro SAVE_ALL_SYS_TRAP
	/* user stack pointer( or kernel sp) , put in r9 by sys call handler */ 
	st.a	r9, [sp, -4]	
	/* retsore original r9 , saved in ex_saved_reg1 */
	ld	r9, [SYMBOL_NAME(ex_saved_reg1)]
	/* now we are ready to save the remaining context */
	st.a	r8, [sp, -4]	/* orig_r8 */	
	st.a	r0, [sp, -4]	/* orig_r0 */
	SAVE_CALLER_SAVED
	st.a	r26, [sp, -4]	/* gp */
	st.a	fp, [sp, - 4]
	st.a	blink, [sp, -4]
	lr	r9, [eret]
	st.a	r9, [sp, -4]
	lr	r9, [erstatus]
	st.a	r9, [sp, -4]
	st.a	lp_count, [sp, -4]
	lr	r9, [lp_end]
	st.a	r9, [sp, -4]
	lr	r9, [lp_start]
	st.a	r9, [sp, -4]
	lr	r9, [erbta]
	st.a	r9, [sp, -4]
        /* move sp to next free entry */
	sub	sp, sp, 4
.endm

/* Restore all registers used by system call.
 * Stack pointer sp should always be pointing to the next free stack element
 * when entering this macro. 
 * sp points to the next free element on the stack at exit of this macro.
 * Registers are pushed / popped in the order defined in struct ptregs
 * in asm/ptrace.h
 * It is recommended that lp_count/ilink1/ilink2 not be used as a destination
 * for memory load operations. If used in that way interrupts are defered by
 * hardware and that is not good. 
 */
.macro RESTORE_ALL_SYS
	/* ignore free entry */
	add	sp, sp, 4
	ld.ab	r9, [sp, 4]
	sr	r9, [erbta] 
        ld.ab	r9, [sp, 4]
	sr	r9, [lp_start]
	ld.ab	r9, [sp, 4]
	sr	r9, [lp_end]
	ld.ab	r9, [sp, 4]
	mov	lp_count, r9
	ld.ab	r9, [sp, 4]
	sr	r9, [erstatus]
	ld.ab	r9, [sp, 4]
	sr	r9, [eret]
	ld.ab	blink, [sp, 4]
	ld.ab	fp, [sp, 4]	
	ld.ab	r26, [sp, 4]	/* gp */
	RESTORE_CALLER_SAVED
	add	sp, sp, 4	/* ignoring orig_r0 */
	add 	sp, sp, 4	/* ignoring orig_r8 */
	ld	sp, [sp]	/* restore original sp */
.endm



/* Save all registers used by interrupt handlers.
 * sp points to the next free element on the stack at exit of this macro.
 * Registers are pushed / popped in the order defined in struct ptregs
 * in asm/ptrace.h
 */
.macro SAVE_ALL_INT1
	/* user stack pointer( or kernel sp) , put in r9 by sys call handler */
	st.a	r9, [sp, -4]	
	/* retsore original r9 , saved in sys_saved_r9 */
	ld	r9, [SYMBOL_NAME(int1_saved_reg)]
	/* now we are ready to save the remaining context :) */
	st.a	-1, [sp, -4]    /* orig_r8, -1 for interuppt level one */	
	st.a	-1, [sp, -4]	/* orig_r0 , -1 for interrpts */
	SAVE_CALLER_SAVED
	st.a	r26, [sp, -4]	/* gp */
	st.a	fp, [sp, - 4]
	st.a	blink, [sp, -4]
	st.a	ilink1, [sp, -4]
	lr	r9, [status32_l1]
	st.a	r9, [sp, -4]
	st.a	lp_count, [sp, -4]
	lr	r9, [lp_end]
	st.a	r9, [sp, -4]
	lr	r9, [lp_start]
	st.a	r9, [sp, -4]
	lr	r9, [bta_l1]
	st.a	r9, [sp, -4]
        /* move sp to next free entry */
	sub	sp, sp, 4
#if 0 //wschen 2008-07-16
        lr      r9,[icause1]
	st	r9, [SYMBOL_NAME(icse1)]
        lr      r9,[status32]
	st	r9, [SYMBOL_NAME(stat32)]
 	st	ilink1, [SYMBOL_NAME(ilnk1)]
 	st	ilink2, [SYMBOL_NAME(ilnk2)]
 	st	blink, [SYMBOL_NAME(blnk)] 
#endif
.endm

.macro SAVE_ALL_INT2
	/* user stack pointer( or kernel sp) , put in r9 by interrupt handler stub*/ 
	st.a	r9, [sp, -4]	
	/* retsore original r9 , saved in sys_saved_r9 */
	ld	r9, [SYMBOL_NAME(int2_saved_reg)]
	st.a	-2, [sp, -4]	/* orig_r8, -2 for interrupt level 2*/ 
	/* now we are ready to save the remaining context :) */
	st.a	-1, [sp, -4]	/* orig_r0 , -1 for interrpts */
	SAVE_CALLER_SAVED
	st.a	r26, [sp, -4]	/* gp */
	st.a	fp, [sp, - 4]
	st.a	blink, [sp, -4]
	st.a	ilink2, [sp, -4]
	lr	r9, [status32_l2]
	st.a	r9, [sp, -4]
	st.a	lp_count, [sp, -4]
	lr	r9, [lp_end]
	st.a	r9, [sp, -4]
	lr	r9, [lp_start]
	st.a	r9, [sp, -4]
	lr	r9, [bta_l2]
	st.a	r9, [sp, -4]
        /* move sp to next free entry */
	sub	sp, sp, 4
#if 0 //wschen 2008-07-16
        lr      r9,[icause2]
	st	r9, [SYMBOL_NAME(icse2)]
        lr      r9,[status32]
	st	r9, [SYMBOL_NAME(stat32)]
   	st	ilink1, [SYMBOL_NAME(ilnk1)]
 	st	ilink2, [SYMBOL_NAME(ilnk2)]
 	st	blink, [SYMBOL_NAME(blnk)] 
#endif
.endm

/* Restore all registers used by interrupt handlers.
 * Stack pointer sp should always be pointing to the next free stack element
 * when entering this macro. 
 * sp points to the next free element on the stack at exit of this macro.
 * Registers are pushed / popped in the order defined in struct ptregs
 * in asm/ptrace.h
 * It is recommended that lp_count/ilink1/ilink2 not be used as a destination for memory
 * load operations. If used in that way interrupts are deffered by hardware and that is
 * not good. 
 */
.macro RESTORE_ALL_INT1
	/* ignore free entry */
	add	sp, sp, 4
	ld	r9, [sp]
	add	sp, sp, 4
	sr	r9, [bta_l1]
	ld	r9, [sp]
	add	sp, sp, 4
	sr	r9, [lp_start]
	ld	r9, [sp]
	add	sp, sp, 4
	sr	r9, [lp_end]
	ld	r9, [sp]
	add	sp, sp, 4
	mov	lp_count, r9
	ld	r9, [sp]
	add	sp, sp, 4
	sr	r9, [status32_l1]
	ld	r9, [sp]
	add	sp, sp, 4
	mov	ilink1, r9
	ld	blink, [sp]
	add	sp, sp, 4
	ld	fp, [sp]	
	add	sp, sp, 4
	ld	r26, [sp]	/* gp */
	add	sp, sp, 4
	RESTORE_CALLER_SAVED
	add	sp, sp, 4	/* ignoring orig_r0 */
	add	sp, sp, 4	/* ignoring orig_r8 */
	ld	sp, [sp]	/* restore original sp */
.endm

.macro RESTORE_ALL_INT2
	/* ignore free entry */
	add	sp, sp, 4
	ld.ab	r9, [sp, 4]
	sr	r9, [bta_l2]
	ld.ab	r9, [sp, 4]
	sr	r9, [lp_start]
	ld.ab	r9, [sp, 4]
	sr	r9, [lp_end]
	ld.ab	r9, [sp, 4]
	mov	lp_count, r9
	ld.ab	r9, [sp, 4]
	sr	r9, [status32_l2]
	ld.ab	r9, [sp, 4]
	mov	ilink2, r9
	ld.ab	blink, [sp, 4]
	ld.ab	fp, [sp, 4]	
	ld.ab	r26, [sp, 4]	/* gp */
	RESTORE_CALLER_SAVED
	add	sp, sp, 4 	/* ignoring orig_r0 */	
	add 	sp, sp, 4	/* ignoring orig_r8 */	
	ld	sp, [sp]	/* restore original sp */
	
.endm

#endif	/* __ASSEMBLY__ */

#endif	/* __ASM_ARC_ENTRY_H */
