/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  linux/include/asm-arc/mmu_context.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authors: Amit Bhor
 */

#ifndef _ASM_ARC_MMU_CONTEXT_H
#define _ASM_ARC_MMU_CONTEXT_H

#include <linux/config.h>
#include <linux/slab.h>
#include <asm/pgalloc.h>
#include <asm/pgtable.h>
#include <asm/arcregs.h>

#define FIRST_ASID	0
#define MAX_ASID	255	/* ARC 700 8 bit PID field in PID Aux reg */
#define NUM_ASID	((MAX_ASID - FIRST_ASID) + 1 )
#define ASID_MASK	MAX_ASID
/* We use this to indicate that no ASID has been allocated to a mmu context */
#define NO_ASID		(MAX_ASID + 1) 

#if 1 //wschen 2008-07-18
#define MMU_ENABLE  0x80000000
#endif

/* ASID to mm struct mapping */
extern struct mm_struct *asid_mm_map[ NUM_ASID + 1 ];

#if 0 //wschen 2008-07-18
extern int asid_cache;
#else
extern volatile int asid_cache;
#endif


static inline void enter_lazy_tlb(struct mm_struct *mm, 
					struct task_struct *tsk, unsigned cpu)
{
}

/* Get a new mmu context or a hardware PID/ASID to work with.
 * If PID rolls over flush cache and tlb.
 */ 
#if 0 //wschen 2008-07-18


static inline void
get_new_mmu_context(struct mm_struct *mm)
{
	int asid, old_asid = mm->context.asid;
	struct mm_struct *oldmm;

	asid = asid_cache;
	if ( ++asid > MAX_ASID) {
#ifdef	CONFIG_ARC_TLB_DBG
		printk("ARC_TLB_DBG: Starting new ASID allocation cycle\n");
#endif		
	/* Start a new asid allocation cycle */	
		asid = FIRST_ASID;
		flush_icache_all();
		flush_tlb_all();
	}
	/* Are we re using a asid that is currently allocated ? */
	if ( (oldmm = asid_mm_map[asid]) ) {
		oldmm->context.asid = NO_ASID;
	}
	asid_mm_map[asid] = mm;
	asid_mm_map[old_asid] = (struct mm_struct *) NULL;
	asid_cache = asid;
	mm->context.asid = asid;
#ifdef	CONFIG_ARC_TLB_DBG
	printk ("ARC_TLB_DBG: Task: %s, pid:%u, assigned asid:%lu\n", (mm->context.tsk)->comm, \
						(mm->context.tsk)->pid, mm->context.asid);
#endif
}
#else
static inline void
get_new_mmu_context(struct mm_struct *mm, int retiring_mm)
{
	int new_asid, mm_asid;
	struct mm_struct *prev_owner;
    unsigned long flags;

    /* TODO-vineet:
     *  1. SMP
     *  2. Can we reduce the duration of Interrupts lockout
     */ 
    local_irq_save(flags);
    
    mm_asid = mm->context.asid;

    /* Is this call from a flush_tlb_xxx () type function
     *  which wants to get rid of old address space
     */
    if ( retiring_mm ) {
        if (mm_asid == NO_ASID ) goto finish_up;
        //if ( current->active_mm->context.asid != mm_asid ) {
        if ( current->mm != mm ) {
	        asid_mm_map[mm_asid] = (struct mm_struct *) NULL;
            mm->context.asid = NO_ASID;
            goto finish_up;
        }
    }

    /* If Task already has an ASID, it should give-up
     * After all he can have only one ASID.
     * As someone said, "to get something you have to forgoe something"
     */
    if ( mm_asid != NO_ASID )
	    asid_mm_map[mm_asid] = (struct mm_struct *) NULL;

	new_asid = asid_cache;
	if ( ++new_asid > MAX_ASID) {
        /* Start a new asid allocation cycle */	
		new_asid = FIRST_ASID;
		flush_icache_all();
		flush_tlb_all();
	}

	/* Are we sealing someone else's ASID. If yes, set that task's ASID to 
     * invalid so when it runs it asks for a new ASID
     */ 
	if ( (prev_owner = asid_mm_map[new_asid]) ) {
		prev_owner->context.asid = NO_ASID;
	}

    /* Actual assignment of ASID to task */
	asid_mm_map[new_asid] = mm;
	mm->context.asid = new_asid;

	asid_cache = new_asid;

#ifdef	CONFIG_ARC_TLB_DBG
	printk ("ARC_TLB_DBG: NewMM=0x%x OldMM=0x%x task_struct=0x%x Task: %s,"
            " pid:%u, assigned asid:%lu\n", 
            mm, prev_owner, (mm->context.tsk), (mm->context.tsk)->comm, 
            (mm->context.tsk)->pid, mm->context.asid);

    /* This is to double check TLB entries already exist for 
     *    this newly allocated ASID
     */
    tlb_find_asid(new_asid);

#endif

    write_new_aux_reg(ARC_REG_PID, (new_asid | MMU_ENABLE));

finish_up:
    local_irq_restore(flags);
}
#endif

/*
 * Initialize the context related info for a new mm_struct
 * instance.
 */
static inline int
init_new_context(struct task_struct *tsk, struct mm_struct *mm)
{
	mm->context.asid = NO_ASID;
#ifdef CONFIG_ARC_TLB_DBG
	mm->context.tsk = tsk;
#endif	/* CONFIG_ARC_TLB_DBG */
	return 0;
}

/* Switch the mm context */
static inline void switch_mm(struct mm_struct *prev, struct mm_struct *next,
                             struct task_struct *tsk, unsigned cpu)
{
#if 0 //wschen 2008-07-18
	unsigned long pid;

	
	if ( next->context.asid == NO_ASID) {
		get_new_mmu_context(next);
	}
	/* Should we assume that unused bits will be unused ? We can do without 
	 * the following bit masking if we do.
	 */
	pid = read_new_aux_reg(ARC_REG_PID);
	pid = (pid & (~ASID_MASK)) | next->context.asid;
	write_new_aux_reg(ARC_REG_PID, pid); 

	/* We keep the current processes PGD base ptr in the SCRATCH_DATA0 
	 * auxilliary reg for quick access in exception handlers. 
	 */
	write_new_aux_reg(ARC_REG_SCRATCH_DATA0, next->pgd);
#else
	if ( next->context.asid > asid_cache) {
		get_new_mmu_context(next, 0);
	}
    else {
        write_new_aux_reg(ARC_REG_PID, (next->context.asid & 0xff) | MMU_ENABLE);
    }

	/* We keep the current processes PGD base ptr in the SCRATCH_DATA0 
	 * auxilliary reg for quick access in exception handlers. 
	 */
	write_new_aux_reg(ARC_REG_SCRATCH_DATA0, next->pgd);
#endif
}

static inline void destroy_context(struct mm_struct *mm)
{
  /* do nothing here */
#if 1 //wschen 2008-07-18
    int asid = mm->context.asid;

    asid_mm_map[asid] = NULL;
    mm->context.asid = NO_ASID;

#endif

}

static inline void 
activate_mm (struct mm_struct *prev, struct mm_struct *next)
{
#if 0 //wschen 2008-07-18
  unsigned long pid;


  /* Unconditionally get a new ASID */
  get_new_mmu_context(next); 

  pid = read_new_aux_reg(ARC_REG_PID);
  pid = (pid & (~ASID_MASK)) | next->context.asid;
  write_new_aux_reg(ARC_REG_PID, pid); 

  write_new_aux_reg(ARC_REG_SCRATCH_DATA0, next->pgd);
#else
  /* Unconditionally get a new ASID */
  get_new_mmu_context(next, 0); 

  write_new_aux_reg(ARC_REG_SCRATCH_DATA0, next->pgd);
#endif
} 
#endif	/* __ASM_ARC_MMU_CONTEXT_H */
