/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  linux/include/asm-arc/page.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authors: Amit Bhor
 */
#ifndef __ASM_ARC_PAGE_H
#define __ASM_ARC_PAGE_H

/* PAGE_SHIFT determines the page size */
#define PAGE_SHIFT	13
#ifdef __ASSEMBLY__
#define PAGE_SIZE       (1 << PAGE_SHIFT)
#else
#define PAGE_SIZE       (1UL << PAGE_SHIFT)
#endif	/* __ASSEMBLY */
#define PAGE_MASK       (~(PAGE_SIZE-1))

#ifdef __KERNEL__

#include <linux/config.h>

#ifdef CONFIG_SET_MEM_PARAM
#define CONFIG_SRAM_BASE 0x00000000UL
#define CONFIG_SRAM_SIZE 0x00400000UL
#define CONFIG_FLASH_MEM_BASE 0x20000000UL
#define CONFIG_FLASH_SIZE     0x00200000UL
#endif /* CONFIG_SET_MEM_PARAM */

/* Kernel runs in upper 2GB untranslated memory */
#ifndef __ASSEMBLY__
#define PAGE_OFFSET  (0x80000000UL)
#else
#define PAGE_OFFSET  (0x80000000)
#endif /* __ASSEMBLY */

#define PHYS_SRAM_OFFSET 0x80000000

#ifndef __ASSEMBLY__

extern void clear_page(void *page);
extern void copy_page(void *to, void *from);
extern unsigned int dc_has_aliasing;

#define get_user_page(vaddr)		__get_free_page(GFP_KERNEL)
#define free_user_page(page, addr)	free_page(addr)


/*
 * #define clear_user_page(page, vaddr)	clear_page(page)
 * #define copy_user_page(to, from, vaddr)	copy_page(to, from)
 */

#ifdef CONFIG_ARC700_USE_DCACHE 
extern void flush_dcache_page_virt(unsigned long *page);
#endif

static inline void clear_user_page(void *page,unsigned long vaddr)
{
	clear_page(page);
#ifdef CONFIG_ARC700_USE_DCACHE 
	if(dc_has_aliasing)
    		flush_dcache_page_virt((unsigned long *)page);
#endif
}

static inline void copy_user_page(void * to, void * from, unsigned long vaddr)
{
	copy_page(to,from);
#ifdef CONFIG_ARC700_USE_DCACHE 
  	if(dc_has_aliasing)
		flush_dcache_page_virt((unsigned long*)to);
#endif
}

/*
 * These are used to make use of C type-checking..
 */
typedef struct { unsigned long pte_lo; } pte_t;
typedef struct { unsigned long pmd; } pmd_t;
typedef struct { unsigned long pgd; } pgd_t;
typedef struct { unsigned long pgprot; } pgprot_t;

#define pte_val(x)      ((x).pte_lo)
#define pmd_val(x)      ((x).pmd)
#define pgd_val(x)      ((x).pgd)
#define pgprot_val(x)   ((x).pgprot)

#define __pte(x)        ((pte_t) { (x) } )
#define __pmd(x)        ((pmd_t) { (x) } )
#define __pgd(x)        ((pgd_t) { (x) } )
#define __pgprot(x)     ((pgprot_t) { (x) } )

/* Pure 2^n version of get_order */
extern __inline__ int get_order(unsigned long size)
{
	int order;

	size = (size-1) >> (PAGE_SHIFT-1);
	order = -1;
	do {
		size >>= 1;
		order++;
	} while (size);
	return order;
}

/* to align the pointer to the (next) page boundary */
#define PAGE_ALIGN(addr)	(((addr)+PAGE_SIZE-1)&PAGE_MASK)

#define __pa(vaddr)  (((unsigned long)vaddr - PAGE_OFFSET) + PHYS_SRAM_OFFSET)
#define __va(paddr)  ((void *)(((unsigned long)(paddr) - PHYS_SRAM_OFFSET) + PAGE_OFFSET))

#define virt_to_page(kaddr)	(mem_map + (__pa(kaddr) >> PAGE_SHIFT) - \
(PHYS_SRAM_OFFSET >> PAGE_SHIFT))
#define VALID_PAGE(page)	((page - mem_map) < max_mapnr)

#define VM_DATA_DEFAULT_FLAGS	(VM_READ | VM_WRITE | VM_EXEC | \
					VM_MAYREAD | VM_MAYWRITE | VM_MAYEXEC)

#ifdef CONFIG_NO_KERNEL_MSG
#define BUG_PRINT()
#else
#define BUG_PRINT() printk("Kernel BUG in file %s: function %s: line %d!\n", __FILE__, __FUNCTION__,  __LINE__)
#endif

#define BUG() do { \
        BUG_PRINT(); \
} while(0)

#define PAGE_BUG(page) do { \
	BUG(); \
  } while (0)

#endif /* !__ASSEMBLY__ */

#ifdef	CONFIG_ARC700_SSRAM
#ifdef	CONFIG_ARC700_SSRAMB2
#define END_MEM            CONFIG_SSRAMB2_BASE + CONFIG_SSRAMB2_SIZE + PHYS_SRAM_OFFSET
#elif	CONFIG_ARC700_SSRAMB1
#define END_MEM            CONFIG_SSRAMB1_BASE + CONFIG_SSRAMB1_SIZE + PHYS_SRAM_OFFSET
#elif	CONFIG_ARC700_SSRAMB0
#define END_MEM            CONFIG_SSRAMB0_BASE + CONFIG_SSRAMB0_SIZE + PHYS_SRAM_OFFSET
#endif
#else
#define END_MEM            CONFIG_SRAM_BASE + CONFIG_SRAM_SIZE + PHYS_SRAM_OFFSET
#endif

/* rtee: check if not using the page_address  macro helps */
#define WANT_PAGE_VIRTUAL   1
#endif	/* __KERNEL__ */

#endif /* __ASM_ARC_PAGE_H */
