/******************************************************************************
 * Copyright Codito Technologies (www.codito.com) Oct 01, 2004
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *****************************************************************************/

/*
 *  linux/include/asm-arc/pgtable.h
 *
 *  Copyright (C) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Authors: Amit Bhor
 */

#ifndef _ASM_ARC_PGTABLE_H
#define _ASM_ARC_PGTABLE_H

#include <linux/config.h>
#include <asm/setup.h>
#include <asm/page.h>
#include <asm/mmu.h>
	
/* The linux MM assumes a 3 level page table mechanism. The ARC 700 MMU has
 * a software managed TLB so it can support any levels of paging. We use a 3
 * level page table which folds into a 2 level page table (similar to i386)
 * since it seems to be more memory efficient this way.
 */

/* PMD_SHIFT determines the size of the area that can be mapped by a second
 * level page table. In memory ARC700 linux PTE's are 4 bytes wide as follows :
 * 
 *	31            13|12      8        5      3      0
 * 	 -------------------------------------------------
 * 	|       PFN     |M|V|L|G|Rk|Wk|Ek|Ru|Wu|Eu|Fc|A|P |
 *	 -------------------------------------------------
 *	
 *	Rk, Ek and Wk are not used by linux. The kernel has all permissions 
 *	hence they are each set to 1. 
 *	M -> Modified(dirty) , A -> Accessed , P -> Present are implemented
 *	in software. 
 *	V->Valid, L->Locked, G->Global are kept here and copied into TLBPD0
 *	when updating the TLB. 
 */
#define PMD_SHIFT	(PAGE_SHIFT + (PAGE_SHIFT -2 )) /* 24 , PAGE_SHIFT 13 */
#define PMD_SIZE	(1UL << PMD_SHIFT)
#define PMD_MASK	(~(PMD_SIZE-1))

/* PGDIR_SHIFT determines the size of the area mapped by a third level page
 * table entry. Since we fold into a 2 level structure this is the same as 
 * PMD_SHIFT.
 */
#define PGDIR_SHIFT	PMD_SHIFT
#define PGDIR_SIZE	(1UL << PGDIR_SHIFT)
#define PGDIR_MASK	(~(PGDIR_SIZE-1))

/* Entries per directory level. We use a 2 level page table structure so we
 * dont have a physical PMD. On A700 pointers are 4 bytes wide so divide
 * page size by 4
 */
#define	PTRS_PER_PTE	2048
#define	PTRS_PER_PMD	1 
#define	PTRS_PER_PGD	256

/* Number of entries a user land program use .
 * TASK_SIZE is the maximum Virtual address that can be used by a userland
 * program.  
 */
#define	USER_PTRS_PER_PGD	(TASK_SIZE / PGDIR_SIZE)
#define FIRST_USER_PGD_NR	0


/* Kernels Virtual memory area.
 * Unlike other architectures(MIPS, sh, cris ) ARC 700 does not have a 
 * "kernel translated" region (like KSEG2 in MIPS). So we use a upper part 
 * of the translated bottom 2GB for kernel virtual memory and protect
 * these pages from user accesses by disabling Ru, Eu and Wu.
 */
#define VMALLOC_SIZE	(CONFIG_VMALLOC_SIZE * 1024 * 1024)
#define VMALLOC_START	(PAGE_OFFSET - VMALLOC_SIZE)
#define VMALLOC_END	(PAGE_OFFSET)
#define VMALLOC_VMADDR(x) ((unsigned long)(x))  

/* ARC700 can do exclusive execute/write protection but most of the other
 * architectures implement it such that execute implies read permission
 * and write imples read permission. So to be compatible we do the same
 */
#define _PAGE_TABLE	(_PAGE_PRESENT \
			| _PAGE_ACCESSED | _PAGE_MODIFIED \
			| _KERNEL_PAGE_PERMS)
#define _PAGE_CHG_MASK	(PAGE_MASK | _PAGE_ACCESSED | _PAGE_MODIFIED) 
#define PAGE_NONE	__pgprot(_PAGE_PRESENT | _PAGE_DEFAULT_CACHEABLE \
			| _KERNEL_PAGE_PERMS)
#define PAGE_SHARED	__pgprot(_PAGE_PRESENT | _PAGE_READ | _PAGE_WRITE \
			| _PAGE_DEFAULT_CACHEABLE | _KERNEL_PAGE_PERMS)
#define PAGE_SHARED_EXECUTE	__pgprot(_PAGE_PRESENT | _PAGE_READ | _PAGE_WRITE \
			        | _PAGE_EXECUTE | _PAGE_DEFAULT_CACHEABLE\
                                | _KERNEL_PAGE_PERMS)
#define	PAGE_COPY	__pgprot(_PAGE_PRESENT | _PAGE_READ \
			| _PAGE_DEFAULT_CACHEABLE | _KERNEL_PAGE_PERMS)
#define	PAGE_COPY_EXECUTE	__pgprot(_PAGE_PRESENT | _PAGE_READ \
				|_PAGE_EXECUTE | _PAGE_DEFAULT_CACHEABLE \
				| _KERNEL_PAGE_PERMS)
#define PAGE_READONLY	__pgprot(_PAGE_PRESENT | _PAGE_READ \
			| _PAGE_DEFAULT_CACHEABLE | _KERNEL_PAGE_PERMS)
#define PAGE_READONLY_EXECUTE	__pgprot(_PAGE_PRESENT | _PAGE_READ \
				| _PAGE_EXECUTE | _PAGE_DEFAULT_CACHEABLE \
				| _KERNEL_PAGE_PERMS )
/* Kernel only page (vmalloc) : global page with all user mode perms 
 * denied. kernel mode perms are set for all pages */ 
/* TODO: rtee: verify this with bhor */
#define _PAGE_KERNEL	(_PAGE_PRESENT | _PAGE_GLOBAL \
			| _PAGE_DEFAULT_CACHEABLE | _KERNEL_PAGE_PERMS)
#define PAGE_KERNEL	__pgprot(_PAGE_KERNEL)

#define _KERNPG_TABLE   (_PAGE_TABLE | _PAGE_KERNEL)


        /* xwr */
#define __P000	PAGE_NONE
#define __P001	PAGE_READONLY
#define __P010	PAGE_COPY
#define __P011	PAGE_COPY
#define __P100	PAGE_READONLY_EXECUTE
#define __P101	PAGE_READONLY_EXECUTE
#define __P110	PAGE_COPY_EXECUTE
#define __P111	PAGE_COPY_EXECUTE

#define __S000	PAGE_NONE
#define __S001	PAGE_READONLY
#define __S010	PAGE_SHARED
#define __S011	PAGE_SHARED
#define __S100	PAGE_READONLY_EXECUTE
#define __S101	PAGE_READONLY_EXECUTE   
#define __S110	PAGE_SHARED_EXECUTE
#define __S111	PAGE_SHARED_EXECUTE

#ifndef __ASSEMBLY__

#define pte_ERROR(e) \
	printk("%s:%d: bad pte %08lx.\n", __FILE__, __LINE__, pte_val(e))
#define pmd_ERROR(e) \
	printk("%s:%d: bad pmd %08lx.\n", __FILE__, __LINE__, pmd_val(e))
#define pgd_ERROR(e) \
	printk("%s:%d: bad pgd %08lx.\n", __FILE__, __LINE__, pgd_val(e))

/* the zero page used for uninitialized and anonymous pages */
#define ZERO_PAGE(vaddr)	(virt_to_page(empty_zero_page))

static inline void flush_tlb_pgtables(struct mm_struct *mm, 
				      unsigned long start, unsigned long end)
{
  /* ARC doesn't keep page table caches in TLB */
}

/* Certain architectures need to do special things when pte's
 * within a page table are directly modified.  Thus, the following
 * hook is made available.
 */
#define set_pte(pteptr, pteval) ((*(pteptr)) = (pteval))
/*
 * (pmds are folded into pgds so this doesn't get actually called,
 * but the define is needed for a generic inline function.)
 */
#define set_pmd(pmdptr, pmdval) (*(pmdptr) = pmdval)
#define set_pgd(pgdptr, pgdval) (*(pgdptr) = pgdval)

/*
 * Conversion functions: convert a page and protection to a page entry,
 * and a page entry and page directory to the page they refer to.
 */
static inline unsigned long pmd_page(pmd_t pmd) 
	{ return (pmd_val(pmd) & PAGE_MASK); }
static inline void pmd_set(pmd_t * pmdp, pte_t * ptep) 
	{ pmd_val(*pmdp) = (_PAGE_TABLE | (unsigned long) ptep); }

#define pte_none(x)	(!pte_val(x))
#define pte_present(x)	(pte_val(x) & _PAGE_PRESENT)
#define pte_clear(xp)	do { pte_val(*(xp)) = 0; } while (0)

#define pmd_none(x)	(!pmd_val(x))
/* by removing the _PAGE__KERNEL bit from the comparision, the same pmd_bad
 * works for both _PAGE_TABLE and _KERNPG_TABLE pmd entries.
 */
#define	pmd_bad(x)	((pmd_val(x) & ~PAGE_MASK) != _PAGE_TABLE)
#define pmd_present(x)	(pmd_val(x) & _PAGE_PRESENT)
#define pmd_clear(xp)	do { pmd_val(*(xp)) = 0; } while (0)

/*
 * The "pgd_xxx()" functions here are trivial for a folded two-level
 * setup: the pgd is never bad, and a pmd always exists (as it's folded
 * into the pgd entry)
 */
static inline int pgd_none(pgd_t pgd)		{ return 0; }
static inline int pgd_bad(pgd_t pgd)		{ return 0; }
static inline int pgd_present(pgd_t pgd)	{ return 1; }
static inline void pgd_clear(pgd_t * pgdp)	{ }

#define pte_page(x) (mem_map + (unsigned long)(((pte_val(x)-PAGE_OFFSET) >> PAGE_SHIFT)))
/*
 * The following only work if pte_present() is true.
 * Undefined behaviour if not..
 */
static inline int pte_read(pte_t pte)	{ return pte_val(pte) & _PAGE_READ; }
static inline int pte_write(pte_t pte)	{ return pte_val(pte) & _PAGE_WRITE; }
static inline int pte_dirty(pte_t pte)	{ return pte_val(pte) & _PAGE_MODIFIED; }
static inline int pte_young(pte_t pte)	{ return pte_val(pte) & _PAGE_ACCESSED; }

static inline pte_t pte_wrprotect(pte_t pte)
{
	pte_val(pte) &= ~(_PAGE_WRITE );
	return pte;
}

static inline pte_t pte_rdprotect(pte_t pte)
{
	pte_val(pte) &= ~(_PAGE_READ );
	return pte;
}

static inline pte_t pte_exprotect(pte_t pte)
{
	pte_val(pte) &= ~(_PAGE_EXECUTE);
	return pte;
}

static inline pte_t pte_mkclean(pte_t pte)
{
	pte_val(pte) &= ~(_PAGE_MODIFIED); 
	return pte; 
}

static inline pte_t pte_mkold(pte_t pte)
{
	pte_val(pte) &= ~(_PAGE_ACCESSED);
	return pte;
}

static inline pte_t pte_mkwrite(pte_t pte)
{
	pte_val(pte) |= _PAGE_WRITE;
	return pte;
}

static inline pte_t pte_mkread(pte_t pte)
{
	pte_val(pte) |= _PAGE_READ;
	return pte;
}

static inline pte_t pte_mkexec(pte_t pte)
{
	pte_val(pte) |= _PAGE_EXECUTE;
	return pte;
}

static inline pte_t pte_mkdirty(pte_t pte)
{
	pte_val(pte) |= _PAGE_MODIFIED;
	return pte;
}

static inline pte_t pte_mkyoung(pte_t pte)
{
	pte_val(pte) |= _PAGE_ACCESSED;
	return pte;
}

/* Macro to mark a page protection as uncacheable */
#define pgprot_noncached pgprot_noncached

static inline pgprot_t pgprot_noncached(pgprot_t _prot)
{
	unsigned long prot = pgprot_val(_prot);

	prot = (prot & ~_PAGE_CACHEABLE);
	
	return __pgprot(prot);
}

#if 0
#define __mk_pte(page_nr, pgprot) __pte(((page_nr) << PAGE_SHIFT) | 	\
						pgprot_val(pgprot))
#define mk_pte(page, pgprot) __mk_pte((page) - mem_map, (pgprot))
#else

#define mk_pte(page, pgprot)  \
 ({ \
      pte_t pte;  \
      pte_val(pte) = __pa(page_address(page)) + pgprot_val(pgprot); \
      pte; \
 })
#endif

static inline pte_t mk_pte_phys(unsigned long physpage, pgprot_t pgprot)
{
	return __pte(physpage | pgprot_val(pgprot));
}

static inline pte_t pte_modify(pte_t pte, pgprot_t newprot)
{
	return __pte((pte_val(pte) & _PAGE_CHG_MASK) | pgprot_val(newprot));
}

/* to find an entry in a kernel page-table-directory.
 * All kernel related VM pages are in init's mm.
 */
#define pgd_offset_k(address) pgd_offset(&init_mm, address)

#define pgd_index(address)	((address) >> PGDIR_SHIFT)

/* Find an entry in a page-table-directory */
static inline pgd_t *pgd_offset(struct mm_struct *mm, unsigned long address)
{
	return mm->pgd + pgd_index(address);
}

/* Find an entry in the second-level page table */
static inline pmd_t *pmd_offset(pgd_t *dir, unsigned long address)
{
	return (pmd_t *) dir;
}

/* Find an entry in the third-level page table.. */
static inline pte_t *pte_offset(pmd_t * dir, unsigned long address)
{
	return (pte_t *) (pmd_page(*dir)) +
	       ((address >> PAGE_SHIFT) & (PTRS_PER_PTE - 1));
}

extern int do_check_pgt_cache(int, int);
extern void paging_init(void);

extern char empty_zero_page[PAGE_SIZE]; 
extern pgd_t swapper_pg_dir[]__attribute__((aligned(PAGE_SIZE)));

void update_mmu_cache(struct vm_area_struct *vma,
				 unsigned long address, pte_t pte);

/* Linux reserves _PAGE_PRESENT of the pte for swap implementation.
 * Our _PAGE_PRESENT is in bit 0 so we can use 5 bits 1-6 for swap type
 * and top 24 bits for swap offset ( Max 64 GB  swap area )
 */
#define SWP_TYPE(x)		(((x).val >> 1) & 0x1f)
#define SWP_OFFSET(x)		((x).val >> 8)
#define SWP_ENTRY(type,offset)	((swp_entry_t) { ((type) << 1) | ((offset) << 8) })

#define pte_to_swp_entry(pte)	((swp_entry_t) { pte_val(pte) })
#define swp_entry_to_pte(x)	((pte_t) { (x).val })

/* Needs to be defined here and not in linux/mm.h, as it is arch dependent */
#define PageSkip(page)		(0)
#define kern_addr_valid(addr)	(1)

#include <asm-generic/pgtable.h>

#define io_remap_page_range remap_page_range
	
/*
 * No page table caches to initialise
 */
#define pgtable_cache_init()   do { } while (0)

#endif /* __ASSEMBLY__ */
#endif /* _ARM_ARC_PGTABLE_H */
