/*
 * arc_pgu.h - Register definitions for the ARC pixel graphics unit.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * THIS	 SOFTWARE  IS PROVIDED	  ``AS	IS'' AND   ANY	EXPRESS OR IMPLIED
 * WARRANTIES,	  INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.	 IN
 * NO EVENT  SHALL   THE AUTHOR	 BE  LIABLE FOR ANY   DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED	  TO, PROCUREMENT OF  SUBSTITUTE GOODS	OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN	 CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright 2005 (C) ARC International (www.arc.com). 
 * 
 */

#ifndef _ASM_ARC_PGU_H
#define _ASM_ARC_PGU_H

#include <linux/fb.h>

/*--------------------------------------------------------------------------------*/
/* Flowbee@960524. */
#define USE_TERAWINS_TCON			1

////bk @ 070821
//#ifdef USE_DESC_TV
//  #undef USE_TERAWINS_TCON
//#endif

/*Flowbee@960125*/
#ifndef FB_TRUE
#define FB_TRUE						(1==1)
#define FB_FALSE					(1==0)
#endif

#ifndef FB_SUCCESS
#define FB_SUCCESS					(1==0)
#define FB_ERROR					(1==1)
#endif

#ifndef FB_YES
#define FB_YES						(1==1)
#define FB_NO						(1==0)
#endif

//-- Flowbee 2008-05-20 mplayer status
enum _mpalyer_status
{
	MPLAYER_INIT		= -100,
	MPLAYER_PLAY		= 0,
	MPLAYER_FORWARD2 	= 1,
	MPLAYER_FORWARD4 	= 2,
	MPLAYER_FORWARD8 	= 3,
	MPLAYER_FORWARD16 	= 4,
	MPLAYER_BACKWARD2 	= -1,
	MPLAYER_BACKWARD4 	= -2,
	MPLAYER_BACKWARD8 	= -3,
	MPLAYER_BACKWARD16 	= -4,	
	MPLAYER_PAUSE		= 100,
};

/*
 *	OSD info:
 *		In FV_ACTIVE, the start_line is encode in the format:
 *		start_line: (bit0~15: x_start, bit16~31: y_start)
 *
 *		and the total_line is encode in the format:
 *		total_line: (bit0~15: x_size, bit16~31: y_size)
 *
 *		blend_all: the active_index
 */
 
struct _osd_info
{
	unsigned char rgb_src;
	unsigned char yuv_src;
	unsigned char yuv_dest;
	unsigned char yuv_bkg;
	unsigned char yuv_osd;	
	unsigned long start_line;
	unsigned long total_line;	
	unsigned char blend_all;	
	unsigned char current_mode;
};

struct _osd_line
{
	unsigned short ram_addr;
	unsigned char str[64];
};

typedef enum
{
	YUV_0 = 0,
	YUV_1,
	RGB_0,
	RGB_1,
	RGB_2,
	B_TMP,
}OSD_BUF;

enum OSD_MUTE_VOL{
	OSD_VOL0,
	OSD_VOL1,
	OSD_VOL2,
	OSD_VOL3,
	OSD_VOL4,
	OSD_VOL5,
	OSD_VOL6,
	OSD_VOL7,
	OSD_VOL8,
	OSD_VOL9,
	OSD_VOL10,
	OSD_MUTE,	
};

enum OSD_ICON{
	OSD_UP,
	OSD_DOWN,
	OSD_LEFT,
	OSD_RIGHT,
	OSD_ENTER,
	OSD_ESC,
	OSD_INVALID,
};

//-- Display index.
enum DisplayIndex
{
	DISP_NTSC = 0,			// 0
	DISP_PAL,				// 1
	DISP_HDTV,				// 2
	DISP_1024x600_LCD,		// 3
	DISP_800x480_LCD,		// 4
	DISP_800x600_LCD,		// 5
	DISP_1366x768_LCD,		// 6
	DISP_640x480_LCD,		// 7
	DISP_800x480_LCD_25MHZ,	// 8
	DISP_HIMAX_480240_TFT,	// 9
	DISP_HIMAX_NTSC,			// a
	DISP_HIMAX_PAL,			// b
	DISP_MAX,					// c
};

enum TvOutputInterface{
	SKY_TV_CVBS_SVIDEO,
	SKY_TV_YPBPR,
	SKY_TV_COMPONENT,
	SKY_TV_MAX,
};


/*--------------------------------------------------------------------------------*/

struct known_displays
{
	unsigned long xres;
	unsigned long yres;
	unsigned long bpp;
	unsigned char display_name[256];
	unsigned long control;
	unsigned long clkcontrol;
	unsigned long hsync;
	unsigned long vsync;
	unsigned char bl_active_high;
	unsigned long max_freq;
};


/*--------------------------------------------------------------------------------*/
//== Flowbee 2007-11-09 Advertisment
struct area_info{
	unsigned short co_x;
	unsigned short co_y;
	unsigned short res_w;
	unsigned short res_h;
};

struct update_info{
	unsigned short src_co_x;
	unsigned short src_co_y;
	unsigned short dst_co_x;
	unsigned short dst_co_y;
	unsigned short res_w;
	unsigned short res_h;
};

//== Flowbee 2007-11-09 Net Config
struct net_config
{
	unsigned char flag;
	unsigned long addr;
	unsigned long len;
};

/*--------------------------------------------------------------------------------------------*/

/* Ioctl selectors	      FIXME - make sure these are unique within the fb subsystem */

#define ARCPGUFB_GETYUV					0xffff0001	// Send the fb_info to the user space.
#define ARCPGUFB_SETYUV					0xffff0002	// Set fb_info, not used.
#define ARCPGUFB_FILLYUV				0xffff0003	// not used
#define ARCPGUFB_SWITCH_YUV				0xffff0004
#define ARCPGUFB_BKLGHT_OFF				0xffff0005
#define ARCPGUFB_BKLGHT_ON				0xffff0006
#define ARCPGUFB_START_DISPLAY			0xffff0007
#define ARCPGUFB_STOP_DISPLAY			0xffff0008
#define ARCPGUFB_GET_PGU_STATE			0xffff0009
#define ARCPGUFB_SET_PGU_STATE			0xffff000a
#define ARCPGUFB_GET_MAIN_MODE			0xffff000b
#define ARCPGUFB_SET_MAIN_MODE			0xffff000c
#define ARCPGUFB_SET_RGB_BUF			0xffff000d
#define ARCPGUFB_GET_RGB_BUF			0xffff000e
#define ARCPGUFB_GET_FB_MAIN			0xffff000f
#define ARCPGUFB_CHECKPOINT				0xffff0010
#define ARCPGUFB_RESTORE				0xffff0011
#define ARCPGUFB_RESET					0xffff0012
#define ARCPGUFB_SET_YUV_RES			0xffff0013
#define ARCPGUFB_GET_FB_INFO			0xffff0014
#define ARCPGUFB_SET_OL_MODE			0xffff0015
#define ARCPGUFB_GET_OL_MODE			0xffff0016
#define ARCPGUFB_GET_OL_COLKEY			0xffff0017
#define ARCPGUFB_SET_OL_COLKEY			0xffff0018
#define ARCPGUFB_GET_OL_START			0xffff0019
#define ARCPGUFB_SET_OL_START			0xffff001a
#define ARCPGUFB_GET_OL_END				0xffff001b
#define ARCPGUFB_SET_OL_END				0xffff001c
#define ARCPGUFB_SET_OL_OFFSET			0xffff001d
#define ARCPGUFB_GET_YUV_BASE			0xffff002c /* return address of yuv buffer n */

#define ARCPGUFB_GET_ORIGIN				0xffff001e
#define ARCPGUFB_SET_ORIGIN				0xffff001f

#define ARCPGUFB_ALLOC					0xffff0020
#define ARCPGUFB_FREE					0xffff0021
#define ARCPGUFB_SEND					0xffff0022
#define ARCPGUFB_FRAMERATE_SET			0xffff0023
#define ARCPGUFB_QUEUE_LEN_GET			0xffff0024
#define ARCPGUFB_FLUSH					0xffff0025
#define ARCPGUFB_FRAME_DROP				0xffff0026
#define ARCPGUFB_PAUSE					0xffff0027
#define ARCPGUFB_RESUME					0xffff0028
#define ARCPGUFB_RESET_SYNC				0xffff0029
#define ARCPGUFB_NUM_FREE_BUFS			0xffff002a
#define ARCPGUFB_PAUSE_MS				0xffff002b

//Flowbee
#define ARCPGUFB_GET_IRVALUE			0xffff0030
#define ARCPGUFB_SET_IRVALUE			0xffff0031
#define ARCPGUFB_GET_DISP_INDEX 		0xffff0032	// get the display mode (NTSC, PAL, LCD)
#define ARCPGUFB_SET_DISP_INDEX 		0xffff0033	// set the display mode (NTSC, PAL, LCD)
#define ARCPGUFB_GET_TV_INTERFACE		0xffff0034	// get tv output interface
#define ARCPGUFB_SET_TV_INTERFACE		0xffff0035	// set tv output interface
#define ARCPGUFB_GET_MPLAYER_PLAY_TIME	0xffff0036	// get play time 
#define ARCPGUFB_SET_MPLAYER_PLAY_TIME	0xffff0037	// set play time
#define ARCPGUFB_GET_MPLAYER_DURATION   0xffff0038      // get play duration
#define ARCPGUFB_SET_MPLAYER_DURATION   0xffff0039      // set play duration
#define ARCPGUFB_GET_DEFAULT_DISP_INDEX	0xffff003a	// get display index of startup stage
#define ARCPGUFB_GET_MPLAYER_STATUS		0xffff003b	// get mplayer status
#define ARCPGUFB_SET_MPLAYER_STATUS		0xffff003c	// set mplayer status

//Flowbee
#define ARCPGUFB_GET_DEFAULT_YUV		0xffff0040	// get the default yuv width and height.
#define ARCPGUFB_SET_DEC_YUV_RES		0xffff0041	// set decoder buffer resolution.
#define ARCPGUFB_GET_DEC_YUV			0xffff0042	// get decoder buffer info.
#define ARCPGUFB_GET_DEC_YUV_BASE		0xffff0043	// get decoder buffer base address.
#define ARCPGUFB_DEC_ALLOC				0xffff0044	// allocate decoder buffer.
#define ARCPGUFB_DEC_FREE				0xffff0045	// free decoder buffer.
#define ARCPGUFB_SCALE_ALLOC			0xffff0046	// allocate display buffer.
#define ARCPGUFB_SCALE_FREE				0xffff0047	// free display buffer.
#define ARCPGUFB_SCALE_SEND				0xffff0048	// send display buffer.
#define ARCPGUFB_NUM_FREE_DEC_BUFS		0xffff0049	// get the num of free decoder buffers.
#define ARCPGUFB_NUM_FREE_SCALE_BUFS	0xffff004A	// get the num of free display buffers.
#define ARCPGUFB_SWITCH_USER_DEFINED_YUV	0xffff004B	// set user defined YUV address
#define ARCPGUFB_GET_FB_SIZE			0xffff004C	// get frame buffer size.
#define ARCPGUFB_SET_SCALE_YUV_RES		0xffff004D

// Robert
#define ARCPGUFB_WAIT_BLANK				0xffff004E
#define ARCPGUFB_CHECK_BLANK			0xffff004F
#define ARCPGUFB_SWITCH_YUV_WITH_INTERRUPT		0xffff0050

//== Flowbee 2007-11-09 Advertisment
#define ARCPGUFB_MPLAYER_ADVERTISMENT	0xffff0060
#define ARCPGUFB_ADVERTISMENT_UPDATE	0xffff0061
#define ARCPGUFB_GET_ADVER_VIDEO_INFO	0xffff0062
#define ARCPGUFB_SET_ADVER_VIDEO_INFO	0xffff0063
#define ARCPGUFB_GET_ADVER_JPEG_INFO	0xffff0064
#define ARCPGUFB_SET_ADVER_JPEG_INFO	0xffff0065

//== Flowbee 2007-11-12 Net config
#define ARCPGUFB_GET_NET_CONFIG_INFO	0xffff0070
#define ARCPGUFB_SET_NET_CONFIG_INFO	0xffff0071

//== Flowbee 2008-02-21 Terawins scale table
#define ARCPGUFB_SET_TCON_SCALE_TABLE	0xffff00c0

//== Flowbee 2008-02-15 OSD function
#define ARCPGUFB_TCON_SET_COMMAND		0xffff00d0
#define ARCPGUFB_TCON_SET_OSD_CMD		0xffff00d1
#define ARCPGUFB_TCON_SET_OSD_RAMADDR	0xffff00d2
#define ARCPGUFB_TCON_SET_OSD_RAMDATA	0xffff00d3
#define ARCPGUFB_TCON_SET_OSD_LINE_DATA	0xffff00d4

#define ARCPGUFB_SET_OSD_SUBTITLE_INFO	0xffff00dc
#define ARCPGUFB_SET_OSD_SPEED_INFO		0xffff00dd

//== Flowbee 2007-12-20 OSD function
#define ARCPGUFB_SHOW_MUTE_VOL			0xffff00e0
#define ARCPGUFB_SHOW_ICON				0xffff00e1
#define ARCPGUFB_SHOW_VIDEO_SUBTITLE	0xffff00e2
#define ARCPGUFB_SHOW_VIDEO_TIME		0xffff00e3
#define ARCPGUFB_SHOW_VIDEO_SPEED		0xffff00e4
#define ARCPGUFB_CLEAR_MUTE_VOL			0xffff00e8
#define ARCPGUFB_CLEAR_ICON				0xffff00e9
#define ARCPGUFB_CLEAR_VIDEO_SUBTITLE	0xffff00ea
#define ARCPGUFB_CLEAR_VIDEO_TIME		0xffff00eb
#define ARCPGUFB_CLEAR_VIDEO_SPEED		0xffff00ec
#define ARCPGUFB_GET_VIDEO_TIME_ONOFF 	0xffff00ed
#define ARCPGUFB_SET_VIDEO_TIME_ONOFF 	0xffff00ee
#define ARCPGUFB_CLEAR_LCD_OSD			0xffff00ef

// Flowbee
#define ARCPGUFB_GET_SCALAR_STATE		0xffff00f0
#define ARCPGUFB_START_SCALAR			0xffff00f1
#define ARCPGUFB_STOP_SCALAR			0xffff00f2
#define ARCPGUFB_SET_SCALAR_INFO		0xffff00f3
#define ARCPGUFB_SET_CONTRAST			0xffff00f4
#define ARCPGUFB_SET_PWM				0xffff00f5
#define ARCPGUFB_DISABLE_MUTE_VOL_OSD	0xffff00f6
#define ARCPGUFB_ENABLE_TV_OSD			0xffff00f7
#define ARCPGUFB_DISABLE_TV_OSD			0xffff00f8
#define ARCPGUFB_UPDATE_OSD				0xffff00f9
#define ARCPGUFB_ENABLE_OL				0xffff00fa
#define ARCPGUFB_DISABLE_OL				0xffff00fb
#define ARCPGUFB_GET_OL_ALPHA			0xffff00fc
#define ARCPGUFB_SET_OL_ALPHA			0xffff00fd
#define ARCPGUFB_GET_STATCTRL			0xffff00fe
#define ARCPGUFB_SET_STATCTRL			0xffff00ff

#define ARCPGUFB_TEST_MODE				0xffff0fff	// test mode for user specified func.

/*--------------------------------------------------------------------------------------------*/

/* YUV buffer layout - all sizes and offsets in bytes. Note that base addresses tend to be 
   passed around as unsigned longs, in which case you'll need to typecast before applying 
   any offsets. */

struct yuv_info {
  unsigned int displayed_buffer;
  unsigned long phys_base;
  unsigned long virt_base;
  unsigned long yuv_size;
  unsigned long alignment;
  unsigned long num_buffers;
  unsigned long start_offset;
  unsigned long width;
  unsigned long height;
  unsigned long u_offset;
  unsigned long v_offset;
  unsigned long y_stride;
  unsigned long u_stride;
  unsigned long v_stride;
#ifdef CONFIG_ARC_PGU_CROPPING
  unsigned long cropped_width;
  unsigned long cropped_height;
  unsigned long x_offset;
  unsigned long y_offset;
#endif
};

/*----------------------------------------------------------------------------*/

/* Register definitions */

/*	Base+0x00    PGU_MODULE_ID   Register containing read only module ID and
 *				     Revision numbers */
#define PGU_MODULE_ID_OFFSET 0x0
#define PGU_DISPLAY_TYPE_MASK 0x1000000
#define PGU_VERSION_MASK 0xff0000
#define PGU_VERSION_SHIFT 16

/*	 Base+0x08    CLK_CFG_REG     Clock Configuration register. It is used to
 *				      Define the clock divider value, and the
 *				      pulse width (high cycle) in clock cycles:
 *				      [ 7: 0] CLK_DIV  : clock divider value - 1
 *				      [23:16] CLK_HIGH : High pulse width - 1 */
#define CLK_CFG_REG_OFFSET 0x8
#define CLK_CFG_REG_CLK_DIV_MASK 0xff
#define CLK_CFG_REG_CLK_HIGH_MASK 0xff0000
#define GET_CLK_CFG_REG_CLK_HIGH_VAL(x) ((x) & CLK_CFG_REG_CLK_HIGH_MASK) >> 16
#define SET_CLK_CFG_REG_CLK_HIGH_VAL(x, y) ((x) & !CLK_CFG_REG_CLK_HIGH_MASK) | (((y) << 16) & CLK_CFG_REG_CLK_HIGH_MASK)

/*	Base+0x10    DISP_DIM_REG    Display Dimensions register. This specify
 *				     the dimensions of the actual display.
 *				     [10: 0] PGU_X_RES : X dimension - 1
 *				     [26:16] PGU_Y_RES : Y dimension - 1 */

#define DISP_DIM_REG_OFFSET 0x10
#define DISP_DIM_REG_PGU_X_RES_MASK 0x7ff
#define DISP_DIM_REG_PGU_Y_RES_MASK 0x7ff0000
#define DISP_DIM_Y_SHIFT 16
#define ENCODE_PGU_DIMS(x,y) (((x) & DISP_DIM_REG_PGU_X_RES_MASK) | (((y) & DISP_DIM_REG_PGU_X_RES_MASK) * 0x10000))

/*	Base+0x18    HSYNC_CFG_REG   Hsync Configuration register. This is used
 *				     to specify various hsync related timings in
 *				     clock cycles:
 *				     [11: 0] HSYNC_CYC	 : Hsync period - 1
 *				     [23:16] H_BCK_PORTH : Back porth width - 1
 *				     [31:23] HSYNC_WIDTH : Hsync low width - 1 */

#define PGU_SYNC_REGS_CYC_MASK 0xfff
#define PGU_SYNC_REGS_PORCH_MASK 0xff0000
#define PGU_SYNC_REGS_WIDTH_MASK 0xff000000
#define PGU_SYNC_REGS_PORCH_OFFSET 16
#define PGU_SYNC_REGS_WIDTH_OFFSET 24

#define HSYNC_CFG_REG_OFFSET 0x18

/*	Base+0x20    VSYNC_CFG_REG   Vsync Configuration register. This is used
 *				     to specify various Vsync related timings in
 *				     Hsync cycles:
 *				     [11: 0] VSYNC_CYC	 : Vsync period - 1
 *				     [23:16] V_BCK_PORTH : Back porth width - 1
 *				     [31:23] VSYNC_WIDTH : Hsync low width */

#define VSYNC_CFG_REG_OFFSET 0x20

/*	Base+0x28   FRMSTART_REG     Frame Start Address. This 32 bit address
 *				     specifies the start of the display frame
 *				     in memory. Must be at least 2 byte aligned
 *				     for 16 bit per pixel format. */

#define FRMSTART_REG_OFFSET 0x28

/*	Base+0x30   STRIDE_REG	     Line Stride Address. This 32 bit address
 *				     specifies the different in address between
 *				     the first pixels of adjecent display lines
 *				     in memory. Must be at least 2 byte aligned
 *				     for 16 bit per pixel format. */

#define STRIDE_REG_OFFSET 0x30

/*	Base+0x38   FRM_DIM_REG	     Frame Dimension register. This register
 *				     defines the frame size of the display in
 *				     memory and can be smaller than the display.
 *				     [10:0] FRM_WTH  : frame line width in
 *						       pixels. Must be <=
 *						       PGU_X_RES without x2 mode
 *						       or <= PGU_X_RES/2 in x2 
 *						       mode.
 *				     [26:16] FRM_LNS : frame number of lines
 *						       Must be <= PGU_X_RES 
 *						       without x2 mode or <= 
 *						       PGU_X_RES/2 in x2 mode. */

#define FRM_DIM_REG_OFFSET 0x38
// flowbee
#define FRM_DIM_REG_FRM_WTH_MASK 0x7ff
#define FRM_DIM_REG_FRM_LNS_MASK 0x7ff
//#define FRM_DIM_REG_FRM_WTH_MASK 0x000007ff
//#define FRM_DIM_REG_FRM_LNS_MASK 0x07ff0000
						   
/*
 *	Base+0x40   STATCTRL_REG     Status and Control register. This register
 *				     is used to control the display module and
 *				     monitor its status: */
#define STATCTRL_REG_OFFSET 0x40
/*				     [0] DISP_EN   : Display enable bit. When
 *						     set to high, starts the
 *						     display. It then clears 
 *						     after a few cycles, if
 *						     the CONT bit is not set,
 *						     hence allowing the display
 *						     to do the frame once. */
#define STATCTRL_REG_DISP_EN_MASK 1
/*				     [1] INTR_EN   : Interrupt Enable. Active
 *						     high. */
#define STATCTRL_INTR_EN_MASK 2
/*				     [2] INTR_CLR  : Interrupt Clear. Set to 
 *						     high to clear the interrupt
 *						     flag/request. Always reads
 *						     as zeros */
#define STATCTRL_INTR_CLR_MASK 4
/*				     [3] CONT	   : Continuous mode. When set,
 *						     will not allow the DISP_EN
 *						     bit to be cleared by itself
 *						     and as long as DISP_EN is
 *						     not cleared manually, the
 *						     Dispaly will drive the PGU
 *						     continuously, frame after
 *						     frame. */
#define STATCTRL_CONT_B_MASK 8
/*				     [5:4] PIX_FMT   Pixel Format. Select the
 *						     pixel format:
 *						     "00" - RGB555
 *						     "01" - YUV420 (aka YV12)
 *						     "10" - reserved
 *						     "11" - reserved */
#define STATCTRL_PIX_FMT_MASK 0x30
#define STATCTRL_PIX_FMT_OFFSET 4
#define STATCTRL_PIX_FMT_RGB555 0
#define STATCTRL_PIX_FMT_YUV420 0x10
/*				     [6] BACKLIGHT   Active high backlight 
 *						     enable. When set to high,
 *						     sets the back light control
 *						     signal to high to turn on
 *						     the backlight of the PGU. */
#define STATCTRL_BACKLIGHT_MASK 0x40
/*				     [7] X2	     Zoom 2 mode. when set, 
 *						     every pixel in the frame is
 *						     display twice in a line &
 *						     the line is displayed twice
 *						     therefore enlarging the
 *						     picture. */
#define STATCTRL_X2_MASK 0x80
/*				     [8] REV_LN_SCAN Reverse line scan. When set
 *						     the display displays the
 *						     last line first, back 
 *						     towards the first */
#define STATCTRL_REV_LN_SCAN_MASK 0x100
/*				     [9] HPOL	     HSync signal polarity */
#define STATCTRL_HPOL_MASK 0x200
/*				     [10] VPOL	     VSync signal polarity */
#define STATCTRL_VPOL_MASK 0x400
/*				     [11] DPOL	     Data enable signal polarity */
#define STATCTRL_DPOL_MASK 0x800
/*				     [12] OL_EN	     Overlay enable */
#define STATCTRL_OL_EN_MASK 0x1000
#define STATCTRL_OL_VAL_MASK 	0xF0000000
#define STATCTRL_OL_VAL_SHIFT 	28
/*				     [13-15] OL_FMT  Overlay format:
 *						     "00" - RGBA4444
 *						     "01" - RGBA5551
 *						     "10" - RGB555 with colour keying */
#define STATCTRL_OL_FMT_MASK 0x6000
#define STATCTRL_OL_FMT_SHIFT 13
#define STATCTRL_OL_FMT_RGBA4444 0
#define STATCTRL_OL_FMT_RGBA5551 0x2000
#define STATCTRL_OL_FMT_RGB555 0x4000
/*				     [16] DISP_BUSY  Display Busy status. Active
 *						     High. When set indicates 
 *						     that the display is 
 *						     currently scanning out 
 *						     pixel data, or generating 
 *						     syncs for the current 
 *						     display */
#define STATCTRL_DISP_BUSY_MASK 0x10000
/*				     [17] BU_BUSY    Burst Unit busy. Active 
 *						     high. When set indicates
 *						     that the burst unit is
 *						     busy fetching pixel data. */
#define STATCTRL_BU_BUSY_MASK 0x20000
/*				     [18] INTR_FLG   Interrupt flag. Refelects
 *						     the interrupt signal.
 *						     Active low level interrupt. */
#define STATCTRL_INTR_FLAG 0x40000
/*				     [19-20] IL_MD   Interface mode select
 *						     "00" - Non-interlaced
 *						     "01" - Frame mode interlace
 *						     "10" - Even field interlace
 *						     "11" - Odd field interlace */
#define STATCTRL_IL_MODE_MASK 0x180000
#define STATCTRL_IL_NONE 0
#define STATCTRL_IL_FRAME 0x80000
#define STATCTRL_IL_EVEN 0x100000
#define STATCTRL_IL_ODD 0x180000

/*				     [28-31] OL_VAL  Overlay value:
*/
#define STATCTRL_OL_VAL_MASK 			0xF0000000
#define STATCTRL_OL_VAL_SHIFT 			28
#define STATCTRL_OL_VAL					0xC0000000


/*----------------------------------------------------------------------------*/

/*	Pixel Graphic Unit Registers  */

typedef volatile struct	 {
  volatile unsigned long pgu_id;	 /* Identity register */
  volatile unsigned long padding1;
  volatile unsigned long clk_cfg;	 /* Clock control */
  volatile unsigned long padding2;
  volatile unsigned long disp_dim;	 /* Display dimension */
  volatile unsigned long padding3;
  volatile unsigned long hsync_cfg;	 /* HSYNC */
  volatile unsigned long padding4;
  volatile unsigned long vsync_cfg;	 /* VSYNC */
  volatile unsigned long padding5;
  volatile unsigned long frm_start;	 /* Frame start */
  volatile unsigned long padding6;
  volatile unsigned long stride;	 /* Frame stride */
  volatile unsigned long padding7;
  volatile unsigned long frm_dim;	 /* Frame dimension */
  volatile unsigned long padding8;
  volatile unsigned long statctrl;	 /* Status and control */
  volatile unsigned long padding9;
  volatile unsigned long cru_frm_st;	 /* Chroma U frame start */
  volatile unsigned long padding10;
  volatile unsigned long cru_stride;	 /* Chroma U stride */
  volatile unsigned long padding11;
  volatile unsigned long crv_frm_st;	 /* Chroma V frame start */
  volatile unsigned long padding12;
  volatile unsigned long crv_stride;	 /* Chroma V stride */

  volatile unsigned long padding13;
  volatile unsigned long ol_frm_st;	 /* Overlay frame start */
  volatile unsigned long padding14;
  volatile unsigned long ol_stride;	 /* Overlay stride */
  volatile unsigned long padding15;
  volatile unsigned long ol_scn_line;	 /* Overlay start and end line numbers */
  volatile unsigned long padding16;
  volatile unsigned long ol_col_key;	 /* Overlay transparency colour key - RGB 555 */

  volatile unsigned long padding17;
  volatile unsigned long centreoffsets;	 /* X, Y offsets for centering */
} arc_pgu;

/*----------------------------------------------------------------------------*/

/*	YPbPr Display Control  Registers  */

/*
 *	Base+0x00	YPBPR_CTRL_REG:
 *					Status and Control register. This register
 *				    is used to control the display module and
 *				    monitor its status: 
 */
#define YPBPR_CTRL_REG_OFFSET			0x00
/*		
 *			[0] INV_CLK:		Invert clock for video DAC. 
 *				0: non-invert
 *				1: invert
 */
#define YPBPR_CTRL_REG_INV_CLK_MASK		0x00000001

/*		
 *			[1] SYNC_ALL:		Indicate the sync signal is only inserting to Y 
 *								or to all YPbPr.
 *				0: only in Y
 *				1: all YPbPr
 */
#define YPBPR_CTRL_REG_SYNC_ALL_MASK	0x00000002

/*		
 *			[2] VIDEO_DAC_PDN:	
 *				0: power down Video DAC
 *				1: power up Video DAC
 */
#define YPBPR_CTRL_REG_VID_DAC_PDN_MASK	0x00000004

/*		
 *			[3] CBCR_INS:		4:2:2 to 4:4:4 insert point mode
 *				0: average two connection point's value
 *				1: copy previous point's value
 */
#define YPBPR_CTRL_REG_CBCR_INS_MASK	0x00000008

/*		
 *			[4-5] ACTIVE_MODE:	Indicate the mode of output support. Include 
 *								resolution, frame rate, and clock frequency.
 *				00: 1080i, 30fps, 74.25MHz
 *				01: 1080i, 25fps, 74.25MHz
 *				10: 576i, 25fps, 13.5MHz
 *				11: 480i, 29.97fps, 13.5MHz
 */
#define YPBPR_CTRL_REG_ACTIVE_MODE_MASK	0x00000030
#define ACTIVE_MODE_1080I_30FPS			0x00000000
#define ACTIVE_MODE_1080I_25FPS			0x00000010
#define ACTIVE_MODE_PAL					0x00000020
#define ACTIVE_MODE_NTSC				0x00000030

/*		
 *			[08] FRAME_BODY Locked Status:	
 *				0: Frame body does'nt used by hardware.
 *				1: Frame body used by hardware, firmware can't
 *				   update pointer now.
 */
#define YPBPR_CTRL_REG_FRAME_BODY_MASK	0x00000100

/*		
 *			[09] FRAME_PTR Locked Status:	
 *				0: Frame Pointer does'nt used by hardware.
 *				1: Frame Pointer used by hardware, firmware can't
 *				   update pointer now.
 */
#define YPBPR_CTRL_REG_FRAME_PTR_MASK	0x00000200

/*		
 *			[24-26] OL_FMT:		Overlay Pix Format.
 *				000: ARGB4444
 *				001: ARGB1555
 *				010~111: Reserved
 *				(A: overlay frame weight)
 *				Result = OL_pix * A + Original_pix * (1-A)
 */
#define YPBPR_CTRL_REG_OL_FMT_MASK		0x07000000
#define YPBPR_CTRL_REG_OL_FMT_SHIFT		24
#define OL_FMT_ARGB4444					0x00000000
#define OL_FMT_ARGB1555					0x01000000

/*		
 *			[27] OL_EN:			Overlay enable.
 *				0: diable
 *				1: enable
 */
#define YPBPR_CTRL_REG_OL_EN_MASK		0x08000000

/*		
 *			[28] TVE_EN:		TV encoder output enable.
 *				0: diable
 *				1: enable
 */
#define YPBPR_CTRL_REG_TVE_EN_MASK		0x10000000

/*		
 *			[29] INTERLACE:		Indicate the Decoder output data format.
 *				0: progressive mode
 *				1: interlace mode
 */
#define YPBPR_CTRL_REG_INTERLACE_MASK	0x20000000

/*		
 *			[30] CONTINUOUS:	Continuous mode. When set, will not allow the 
 *								module-en bit to be cleared by itself and 
 *								as long as Module_en is not cleared manually, 
 *								the display will driven continuously, 
 *								frame after frame.
 *				0: not continuous
 *				1: continuous
 */
#define YPBPR_CTRL_REG_CONTINUOUS_MASK	0x40000000

/*		
 *			[31] MODULE_EN:		Turn on this module.
 *				0: turn off
 *				1: turn on
 */
#define YPBPR_CTRL_REG_MODULE_EN_MASK	0x80000000

#define TV_YPBPR_MASK					0xF0000000
#define TV_EN							0xF0000000
#define YPBPR_EN						0xE0000000


/*
 *	Base+0x18	YPBPR_OL_LINE_REG:
 *					Overlay pix line stride address and 
 *					the Alpha value for ARGB1555mode.
 */
#define YPBPR_OL_LINE_REG_OFFSET		0x18

#define YPBPR_OL_LINE_REG_LINE_MASK		0x00000FFF

#define YPBPR_OL_LINE_REG_ALPHA_MASK	0x000F0000
#define YPBPR_ALPHA_SHIFT						16
#define YPBPR_ALPHA_VALUE						(4 << ALPHA_SHIFT)


/*
 *	Base+0x1C	YPBPR_OL_RANGE_REG:
 *					Overlay start and stop line number.
 */
#define YPBPR_OL_RANGE_REG_OFFSET		0x18

#define YPBPR_OL_RANGE_REG_START_MASK	0x000007FF
#define YPBPR_OL_RANGE_REG_STOP_MASK	0x07FF0000


/*----------------------------------------------------------------------------*/

/*	TV encoder Control Registers  */

/*
 *	Base+0x00	TVE_FORMAT_REG:
 *					TV encoder output format control register.
 *					Controlling the output format of TV signal.
 *					(NTSC, PAL etc...)
 */
#define TVE_FORMAT_REG_OFFSET			0x00000000					

/*		
 *			[0-1] TVF:			TV standard format select(should select correct 
 *								Black0IRE)
 *				00: NTSC-M,J (M:Black level=7.5 IRE, J:Black level=0 IRE)
 *				01: PAL-M (BL=7.5 IRE)(Mono, 480I/60Hz)
 *				10: PAL-B,D,G,H,I,N (N:BL=7.5 IRE, others:0 IRE)
 *				11: PAL-Nc (BL=0 IRE)(No color, 576I/50Hz)
 */
#define TVE_FORMAT_REG_TVF_MASK			0x00000007
#define TVE_NTSC_M						0x00000000
#define TVE_NTSC_J						0x00000004
#define TVE_PAL_M						0x00000001
#define TVE_PAL_N						0x00000002
#define TVE_PAL_B_D_G_H_I				0x00000006
#define TVE_PAL_NC						0x00000007

/*		
 *			[2] BLACK0IRE:		Black level select
 *				0: BL is 7.5 IRE above blanking level.
 *				1: BL is 0 IRE (Black level is the same as blanking level)
 */
#define TVE_FORMAT_REG_BLACK0IRE_MASK	0x00000004
#define BLACK0IRE_7_5_IRE				0x00000000
#define BLACK0IRE_0_IRE					0x00000004

/*		
 *			[3] VIN_RANGE:		The range of the video pixel input range.
 *				0: 16~235 for Y, 16~240 for Cb and Cr
 *				1: 1~254 for Y, Cb and Cr
 */
#define TVE_FORMAT_REG_VIN_RANGE_MASK	0x00000008
#define VIN_RANGE_16_235				0x00000000
#define VIN_RANGE_1_254					0x00000008

/*
 *	Base+0x04	TVE_INPUT_REG:
 *					TV encoder input control register.
 *					Controlling the input format of signal.
 *					(BT601, BT656, Color Bar etc...)
 */
#define TVE_INPUT_REG_OFFSET			0x00000004		

/*		
 *			[1-2] VINF:			Video input mode selection.
 *				00: BT601 slave mode
 *				01: BT656 mode
 *				10: Reserved
 *				11: internal 100% color bar pattern (Field-mode should be = 0)
 */
#define TVE_INPUT_REG_VINF_MASK			0x00000006
#define VINF_BT601						0x00000000
#define VINF_BT656						0x00000002
#define VINF_COLOR_BAR					0x00000006

/*
 *	Base+0x08	TVE_OUTPUT_REG:
 *					TV encoder output control register.
 *					Controlling the output format of signal.
 *					(CVBS and S-video, component YUV etc...)
 */
#define TVE_OUTPUT_REG_OFFSET			0x00000008

/*		
 *			[3] NO_C:			Disable color signal
 *				0: Normal video output
 *				1: Color signal is disable on CVBS output. S-video or component
 *				   output is normal.
 */
#define TVE_OUTPUT_REG_NO_C_MASK		0x00000008
#define NO_C_NORMAL						0x00000000
#define NO_C_DISABLE					0x00000008

/*		
 *			[4] BLACK:			Enable black output
 *				0: Normal video output
 *				1: Black output in all output modes
 */
#define TVE_OUTPUT_REG_BLACK_MASK		0x00000010
#define BLACK_NORMAL					0x00000000
#define BLACK_OUTPUT					0x00000010

/*		
 *			[5] BLUE:			Enable blue output.	It is used to display blue
 *								screen while there is no video input signal.
 *				0: Normal video output
 *				1: Blue output in all output modes
 */
#define TVE_OUTPUT_REG_BLUE_MASK		0x00000020
#define BLUE_NORMAL						0x00000000
#define BLUE_OUTPUT						0x00000020

/*		
 *			[6] VOF:			Output format
 *				0: CVBS and S-video output simultaneously
 *				1: Component YUV output
 */
#define TVE_OUTPUT_REG_VOF_MASK			0x00000040
#define VOF_CVBS_SVIDEO					0x00000000
#define VOF_COMPONENT_YUV				0x00000040

/*
 *	Base+0x10	TVE_SWRST_REG:
 *					Software reset. It will reset all registers to default 
 *					reset values.
 */
#define TVE_SWRST_REG_OFFSET			0x00000010

/*		
 *			[0] SRESET:			Software reset. It will reset all registers 
 *								to default reset values.
 *				0: No reset
 *				1: Software reset. Writing '1' to this bit enables the software
 *				   reset function and this bit will automatically reset to '0'. 
 *				   It is not necessary to write '0' to disable "sreset".
 */
#define TVE_SWRST_REG_SRESET_MASK		0x00000001
#define SRESET_DISABLE					0x00000000
#define SRESET_ENABLE					0x00000001


typedef volatile struct	 {
  volatile unsigned long disp_ctrl;
  volatile unsigned long y_addr;
  volatile unsigned long cb_addr;
  volatile unsigned long cr_addr;
  volatile unsigned long ycbcr_stride;	// Y Stride[27:19] CbCr Stride [10:3]
  volatile unsigned long rgb_addr;
  volatile unsigned long rgb_stride;	// Stride [11:3] Alpha value [19:16]
  volatile unsigned long rgb_range;	// Start Line[10:0] End Line[26:16]
  volatile unsigned long VBlank;	// Bottom Blank [10:0] Top Blank [26:16]
  volatile unsigned long HBlank;	// Right Blank [10:0] Left Blank [26:16]
} sky_ypbpr;

/*----------------------------------------------------------------------------*/

/*	TV Encoder Control Registers  */

typedef volatile struct	 {
  volatile unsigned long fmt_ctrl;
  volatile unsigned long input_ctrl;
  volatile unsigned long output_ctrl;
  volatile unsigned long power;
  volatile unsigned long reset;
} sky_tv;
/*--------------------------------------------------------------------------------------------*/

#endif	/* _ASM_ARC_PGU_H */
