/************************************************************************
 *		This file include primitive functions which can control the
 *		TeraWins T-CON.
 ************************************************************************/
#include "arc_pgu.h"
#include "arc_tcon.h"
#include "pgu_lib.h"

#if USE_TERAWINS_TCON

#ifndef NULL
#define NULL						0
#endif
void terawins_set_osd_line_info(struct _terawins_osd_info* _osd);
extern int *TerawinsCmdTbl[TERAWINS_END*2];
extern unsigned char *AllFonts[];
extern unsigned short terawins_osd_color_table[16];

extern unsigned char font_index;	// index of font type.
extern struct _font_info font_info[FONT_END];

extern struct _terawins_osd_info video_speed;
extern struct _terawins_osd_info icon;
extern struct _terawins_osd_info subtitle;
extern struct _terawins_osd_info mute_vol;
extern struct _terawins_osd_info audio_channels;
extern struct _terawins_osd_info video_time;
extern struct _terawins_osd_info* terawins_osd[OI_ALL];
extern struct _terawins_osd_info blank;

extern char valid_icon[ICON_NUM * ICON_NUM];
extern char invalid_icon[ICON_NUM * ICON_NUM];
extern char mute_icon[MUTE_NUM];
extern char audio_channels_icon[3][2];
extern char vol_icon[VOL_NUM];

//== Global variable
static unsigned long val = 0;
static volatile unsigned long vtval; 	
unsigned long tval;
struct _osd_line osd_line_data;
uint16 sort_tbl[OI_ALL][2];

/* -------------------------------------------------------------------------- */
int terawins_set_cmd(uint8 dev_addr, uint8 reg_addr, uint8 reg_val)
{
	unsigned long _val;
	int i;

	_val = (((unsigned long)dev_addr) << 16) 
		+ (((unsigned long)reg_addr) << 8) 
		+ (((unsigned long)reg_val));

	i = pgu_tcon_set_cmd(_val);
	
	if(i != 0){
		//TW_MSG("Terawins I2C write error !!\n");
	}
	return i;
}

/* -------------------------------------------------------------------------- */
int terawins_set_page(unsigned char _page_num, int *_page)
{
	int ret_val = 0;
	while(*(_page) != END_CMD){
		ret_val = terawins_set_cmd(_page_num, (unsigned char)(*(_page)), (unsigned char)(*(_page + 1)));
		if((ret_val == 2) || (ret_val == 1)){
			TW_MSG("Write to TeraWins Error!! page:%x %x %x\n", _page_num, (unsigned char)(*(_page)), (unsigned char)(*(_page + 1)));
			return ret_val;
		}
		_page += 2;
	}
	return ret_val;
}

/*******************************************************************************
			Terawins functions
*******************************************************************************/
/* -------------------------------------------------------------------------- */
void terawins_on_off(uint8 _on)
{
	if(_on == 1){
		//-- turn on
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xC2, 0x00);
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x91, 0x07);
		usleep(0x200);
	}
	else{
		//-- turn off
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xC2, 0x12);
		terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x91, 0x87);
	}
}

/* -------------------------------------------------------------------------- */
int terawins_set_resolution(uint8 idx)
{
	int ret_val = 0;

	/* Set command to terawins T-CON. */
	//-- Setting page0 registers.
	ret_val = terawins_set_page(TERAWINS_PAGE0_ADDR, TerawinsCmdTbl[idx * 2]);
	//-- Setting page1 registers.
	ret_val |= terawins_set_page(TERAWINS_PAGE1_ADDR, TerawinsCmdTbl[idx * 2 + 1]);
	return ret_val;
}

/* -------------------------------------------------------------------------- */
void terawins_pwm_on_off(uint8 _on)
{
	int i = 0;
	if(_on == 1){
		//-- Turn on PWM
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x10);
	}
	else{
		//-- Turn off PWM
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe8, 0x00);			
	}
	if(i != 0){
		TW_MSG("Set PWM on/off fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
void terawins_backlight_on_off(uint8 _on)
{
	int i = 0;
	if(_on == 1){
		//-- Turn on terawins backlight
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xec, 0xe2);
	}
	else{
		//-- Turn off terawins backlight
		i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xec, 0xc0);			
	}
	if(i != 0){
		TW_MSG("Terawins backlight on/off fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
void terawins_set_pwm(uint8 _val)
{
	int i = 0;
	//TW_MSG("\nPWM !!\n");
	//-- Set pwm duty cycle.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0xe9, _val);	
	if(i != 0){
		TW_MSG("Set PWM fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
void terawins_set_brightness(uint8 _val)
{
	int i = 0;
	//TW_MSG("\nBrightness !!\n");
	//-- Set pwm freq.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x69, _val);
	if(i != 0){
		TW_MSG("Set Brightness fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
void terawins_set_contrast(uint8 _val)
{
	int i = 0;
	//TW_MSG("\nContrast !!\n");
	//-- Set pwm freq.
	i = terawins_set_cmd(TERAWINS_PAGE0_ADDR, 0x68, _val);
	if(i != 0){
		TW_MSG("Set Contrast fail !\n");
	}
}

/* -------------------------------------------------------------------------- */
int init_terawins_tcon(void)
{
	int *p0, *p1;

	TW_MSG("\nInit Terawins T-CON !!\n");
	/* Set default commands to terawins T-CON. */
	//-- Setting page0 registers.
	p0 = TerawinsCmdTbl[TERAWINS_INIT * 2];
	terawins_set_page(TERAWINS_PAGE0_ADDR, p0);
	//-- Setting page1 registers.
	p1 = TerawinsCmdTbl[TERAWINS_INIT * 2 + 1];
	terawins_set_page(TERAWINS_PAGE1_ADDR, p1);
	return 0;
}


/*******************************************************************************
			Terawins OSD functions.
*******************************************************************************/
/* -------------------------------------------------------------------------- */
int terawins_set_osd_cmd(unsigned char cfg_idx, unsigned char cfg_data)
{
	int ret_val = 0;
	//-- set osd control register address.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_INDEX, cfg_idx);
	
	//-- set osd control register data.
	ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_DATA, cfg_data);

	if(ret_val != 0){
		TW_MSG("\nTeraWins set osd cmd error!\n");
	}
	
	return ret_val;
}

/* -------------------------------------------------------------------------- */
int terawins_set_osd_ram_addr(unsigned short ram_addr)
{
	int ret_val = 0;
	
	//-- set osd ram address.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_LO, (unsigned char)(ram_addr & 0x00FF));
	ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_HI, (unsigned char)((ram_addr & 0xFF00) >> 8));

	if(ret_val != 0){
		//ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_LO, (unsigned char)(ram_addr & 0x00FF));
		//ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_ADDR_HI, (unsigned char)((ram_addr & 0xFF00) >> 8));
	}
	return ret_val;
}

/* -------------------------------------------------------------------------- */
int terawins_set_osd_ram_data(unsigned char ram_data)
{
	int ret_val = 0;

	//-- set osd ram data.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_DATA, ram_data);

	if(ret_val != 0){
		//ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_RAM_DATA, ram_data);
	}
	
	return ret_val;
}

/* -------------------------------------------------------------------------- */
static int terawins_load_osd_color(unsigned short *color_table)
{
	int ret_val = 0;
	unsigned long i;

	/* Load OSD color LUT. */	
	//-- Configure OSD color LUT.
	ret_val = terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_INDEX, OSD_COLOR_LUT_PORT);	

	//-- Load data.
	for(i = 0; i < 0x10; i++){
		ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_DATA, (unsigned char)(color_table[i] & 0x00FF));
		ret_val |= terawins_set_cmd(TERAWINS_PAGE0_ADDR, OSD_CFG_DATA, (unsigned char)((color_table[i] & 0xFF00) >> 8));
	}

	if(ret_val != 0){
		TW_MSG("\nTeraWins load osd color error!\n");
	}

	return ret_val;	
}

/* -------------------------------------------------------------------------- */
void terawins_clear_osd_ram(void)
{
	unsigned long i;
	unsigned long word_nums;

	word_nums = (TERAWINS_OSD_RAM_SIZE / 2);

//	TW_MSG("\nword_nums = %d, %x", word_nums, word_nums);
	
	//-- Clear OSD ram.(clear word.)
	terawins_set_osd_ram_addr(0x0000);
	for(i = 0; i < word_nums; i++){
		//low byte
		terawins_set_osd_ram_data(0x00);
		//high byte
		terawins_set_osd_ram_data(0x00);
	}
}

/* -------------------------------------------------------------------------- */
unsigned char terawins_load_osd_fonts(unsigned char** _font_array)
{
	unsigned char i = 0;
	unsigned char j = 0;	
	unsigned char bytes_per_font;
	unsigned char *p;
	int _ack = 0;

	bytes_per_font = FONT_WIDTH_BYTES * OSD_FONT_HEIGHT;
	terawins_set_osd_ram_addr(0x0000);
	
	while(_font_array[i] != NULL){
		p = _font_array[i];
		for(j = 0; j < (bytes_per_font); j++){
			_ack |= terawins_set_osd_ram_data(p[j]);
		}
		i ++;
	}
	if(_ack != 0){
		TW_MSG("\nLoad Fonts error, font_num = %d\n", i);
	}
	
	return i;
}

/* -------------------------------------------------------------------------- */
void terawins_set_osd_menu(unsigned short h_start, 
									unsigned short v_start,
									unsigned char row_index,
									unsigned char n_rows	)
{
	unsigned char h_lbyte, h_hbyte;
	unsigned char v_lbyte, v_hbyte;	
	unsigned char start_addr, end_addr;
	
	/* Set terawins osd menu info. */
	//-- Set start address.
	h_lbyte = (unsigned char)(h_start & 0x00FF);
	h_hbyte = (unsigned char)((h_start & 0x0700) >> 8);
	v_lbyte = (unsigned char)(v_start & 0x00FF);
	v_hbyte = (unsigned char)((v_start & 0x0700) >> 8);
	terawins_set_osd_cmd(CHAR_DLY1, (h_hbyte | (v_hbyte << 4)));
	terawins_set_osd_cmd(CHAR_DLY2, h_lbyte);
	terawins_set_osd_cmd(CHAR_DLY3, v_lbyte);

//	TW_MSG("\nCHAR_DLY1: %02x", (h_hbyte | (v_hbyte << 4)));
//	TW_MSG("\nCHAR_DLY2: %02x", h_lbyte);
//	TW_MSG("\nCHAR_DLY3: %02x", v_lbyte);	

	//-- Set end address of menu info.
	if(n_rows < 1){
		n_rows = 0;
		TW_MSG("\nOSD input row error !!");
	}

	start_addr = CHAR_START_ADDR(font_info[font_index].font_nums) + row_index;
	terawins_set_osd_cmd(CHAR_RAM_START_ADDR, start_addr);
	
	end_addr = start_addr + (n_rows - 1);
	terawins_set_osd_cmd(CHAR_RAM_STOP_ADDR, end_addr);
//	TW_MSG("\nCHAR_RAM_START_ADDR: %02x", CHAR_START_ADDR(font_info[font_index].font_nums));
//	TW_MSG("\nCHAR_RAM_STOP_ADDR: %02x", end_addr);	
}

/* -------------------------------------------------------------------------- */
void terawins_set_osd_alpha_blending(unsigned char mode, unsigned char _val)
{
	terawins_set_osd_cmd(ALPHA_BLENDING_CTRL, (((mode&0x03) << 6) | (_val&0x0f)));
}

/* -------------------------------------------------------------------------- */
void terawins_set_osd_blinking(unsigned char blk_freq, unsigned char duty)
{
	terawins_set_osd_cmd(BLINKING_CTRL, (((blk_freq&0x03) << 2) | (duty&0x03)));
}

/* -------------------------------------------------------------------------- */
void terawins_enable_osd(unsigned char _en)
{
	if(_en == 1){
		terawins_set_osd_cmd(OSD_CTRL, 0x80);
	}
	else{
		terawins_set_osd_cmd(OSD_CTRL, 0x00);
	}
}

/* -------------------------------------------------------------------------- */
void terawins_set_osd_line_data(unsigned short ram_addr, unsigned short *str)
{
	uint8 i = 0;
	uint8 c_idx, c_attr;

	//-- Set start address of osd sram.
	terawins_set_osd_ram_addr(ram_addr);

	//-- Set osd data
	for(i = 0; i < WORDS_PER_ROW; i ++){
		c_idx = (uint8)((str[i] & 0xFF00) >> 8);
		c_attr = (uint8)(str[i] & 0x00FF);
		//-- low byte first.(char index.)
		terawins_set_osd_ram_data(c_idx);
		//-- high byte.(char attribute.)
		terawins_set_osd_ram_data(c_attr);
	}
}

/* -------------------------------------------------------------------------- */
void clear_osd_line(struct _terawins_osd_info* _osd)
{
	uint8 i = 0, j = 0;

#if 1
	osd_line_data.ram_addr = _osd->ram_addr;
	for(i = 0; i < ((2*WORDS_PER_ROW) - 1); i ++){
		osd_line_data.str[i] = 0x00;
	}
	osd_line_data.str[i] = _osd->row_attr;

	for(j = 0; j < _osd->n_row; j ++){
		pgu_tcon_set_osd_line_data(&osd_line_data);
		osd_line_data.ram_addr += WORDS_PER_ROW;
	}
	//TW_MSG("clear: %x, %d", _osd->ram_addr, _osd->n_row);

#else
	
	//-- Set start ram address in terawins sram.
	terawins_set_osd_ram_addr(_osd->ram_addr);

	//-- Clear osd data.
	for(j = 0; j < _osd->n_row; j ++){
		for(i = 0; i < (WORDS_PER_ROW - 1); i++){
			//-- low byte first.(char index.)
			terawins_set_osd_ram_data(0x00);
			//-- high byte.(char attribute.)
			terawins_set_osd_ram_data(0x00);
		}
		terawins_set_osd_ram_data(0x00);
		terawins_set_osd_ram_data(_osd->info->row_attr);
	}
#endif

}

/* -------------------------------------------------------------------------- */
void clear_osd(int _osd_idx)
{
	unsigned int i;

//	TW_MSG("\n-- clear_osd :%d --\n", _osd_idx);
	if(_osd_idx >= OI_ALL){
		for(i = 0; i < OI_ALL; i++){
			clear_osd_line(terawins_osd[i]);
		}
	}
	else{
		clear_osd_line(terawins_osd[_osd_idx]);
	}
}

/* -------------------------------------------------------------------------- */
void terawins_set_osd_line_info(struct _terawins_osd_info* _osd)
{
	char rgap_bg = 0;
	char rgap = 0;
	char r_cws = 0;
	
	_osd->char_attr = ((_osd->b_color & 0x07) << 5) 
								| ((_osd->blink & 0x01) << 4) 
								| (_osd->f_color & 0x0F);
	_osd->row_attr 	= ((rgap_bg & 0x01) << 7) | ((rgap & 0x1F) << 2) 
							| ((_osd->d_line & 0x01) << 1) | (r_cws & 0x01);
}

/* -------------------------------------------------------------------------- */
void terawins_set_osd_line(char *_str, uint16 r_addr, struct _terawins_osd_info *_info)
{
	uint8 i = 0, j = 0;
	int _ack = 0;

#if 1
	osd_line_data.ram_addr = r_addr;

	for(i = 0; i < FONTS_PER_ROW; i ++){
		if(_str[i] != 0){
			//-- low byte first.(char index.)
			osd_line_data.str[j ++] = _str[i];
			//-- high byte.(char attribute.)
			osd_line_data.str[j ++] = _info->char_attr;
		}
		else{
			//-- low byte first.(char index.)
			osd_line_data.str[j ++] = 0x00;
			//-- high byte.(char attribute.)
			osd_line_data.str[j ++] = 0x00;
		}
	}
	osd_line_data.str[j ++] = 0x00;
	osd_line_data.str[j ++] = 0x00;
	osd_line_data.str[j ++] = 0x00;
	osd_line_data.str[j ++] = _info->row_attr;

	pgu_tcon_set_osd_line_data(&osd_line_data);

#else

	for(i = 0; i < 30; i ++){
		if(_str[i] != 0){
			//-- low byte first.(char index.)
			_ack |= terawins_set_osd_ram_data(_str[i]);
			//-- high byte.(char attribute.)
			_ack |= terawins_set_osd_ram_data(_info->char_attr);
		}
		else{
			//-- low byte first.(char index.)
			_ack |= terawins_set_osd_ram_data(0x00);
			//-- high byte.(char attribute.)
			_ack |= terawins_set_osd_ram_data(0x00);
		}
	}

	//-- It must be the blank char at the 31th word of the line info.
	_ack |= terawins_set_osd_ram_data(00);
	_ack |= terawins_set_osd_ram_data(00);

	//-- Row attr.
	_ack |= terawins_set_osd_ram_data(00);
	_ack |= terawins_set_osd_ram_data(_info->row_attr);
	
#endif

	if(_ack != 0){
		TW_MSG("\nSet osd line error !!\n");
	}
}

/* -------------------------------------------------------------------------- */
void terawins_show_speed(char *_t)
{
	char _osd[30];
	uint8 i = 0, j = 0;
	struct _terawins_osd_info *_info = terawins_osd[OI_VIDEO_SPEED];		

	if(terawins_osd[OI_VIDEO_SPEED]->on == 0){
		return;
	}

	//-- Clear data array.
	for(j = 0; j < FONTS_PER_ROW; j ++){
		_osd[j] = 0;
	}	

	while( (_t[i] != '\0') && (i < 30) ){
		_osd[i] = _t[i];
		i++;
	}
	
	terawins_set_osd_line(_osd, _info->ram_addr, _info);
}


/* -------------------------------------------------------------------------- */
void terawins_show_time(char *_t)
{
	char _osd[30];
	uint8 i = 0, j = 0;
	struct _terawins_osd_info *_info = terawins_osd[OI_VIDEO_TIME];	

	if(_info->on == 0){
		return;
	}

	//-- Clear data array.
	for(j = 0; j < FONTS_PER_ROW; j ++){
		_osd[j] = 0;
	}	

	while( (_t[i] != '\0') && (i < 30) ){
		_osd[i] = _t[i];
		i++;
	}
	
	terawins_set_osd_line(_osd, _info->ram_addr, _info);	

}


/* -------------------------------------------------------------------------- */
void terawins_show_subtitle(char _s[][31])
{
	char _osd[30];
	uint8 i = 0, j = 0;
	uint16 r_addr;
	struct _terawins_osd_info *_info = terawins_osd[OI_SUBTITLE];	

	if(_info->on == 0){
		return;
	}

	//-- Clear data array.
	for(i = 0; i < FONTS_PER_ROW; i ++){
		_osd[i] = 0;
	}	

	r_addr = _info->ram_addr;
	for(i = 0; i < _info->n_row; i++){
		for( ; j > 0; j--){
			_osd[j] = 0;
		}
		j = 0;
		while( (_s[i][j] != '\0') && (j < 30) ){
			_osd[j] = _s[i][j];
			j++;
		}
		terawins_set_osd_line(_osd, r_addr, _info);
		r_addr += WORDS_PER_ROW;
	}
}


/* -------------------------------------------------------------------------- */
void terawins_show_mute_vol(uint32 _idx)
{
	char _osd[30];
	unsigned int i, start, end;
	struct _terawins_osd_info *_info = terawins_osd[OI_MUTE_VOL];	

	if(_info->on == 0){
		return;
	}

	//-- Clear data array.
	for(i = 0; i < FONTS_PER_ROW; i ++){
		_osd[i] = 0;
	}	
	
	if(_idx == OSD_MUTE){
		start = MUTE_START;
		for(i = 0; i < MUTE_NUM; i++){
			_osd[start ++] = mute_icon[i];
		}
	}
	else{
		start = VOL_START;
		for(i = 0; i < VOL_NUM; i ++){
			_osd[start ++] = vol_icon[i];
		}
		for(i = 0; i < _idx; i ++){
			_osd[start ++] = FONT_VOL_BAR;
		}
		for( ; i < TOTAL_VOL_BAR; i ++){
			_osd[start ++] = FONT_VOL_EMPTY_BAR;
		}
	}
	terawins_set_osd_line(_osd, _info->ram_addr, _info);	
}

void terawins_show_audio_channels(uint32 idx)
{
	char _osd[30];
	unsigned int i, start, end;
	struct _terawins_osd_info *_info = terawins_osd[OI_AUDIO_CHANNELS];	

	if(_info->on == 0){
		return;
	}

	//-- Clear data array.
	for(i = 0; i < FONTS_PER_ROW; i ++){
		_osd[i] = 0;
	}		

	start = AUDIO_CHANNELS_START;
	for(i = 0; i < AUDIO_CHANNELS_NUM; i++){
		_osd[start ++] = audio_channels_icon[idx][i];
	}
	
	terawins_set_osd_line(_osd, _info->ram_addr, _info);	
}


/* -------------------------------------------------------------------------- */
void terawins_show_icon(uint32 _idx)
{
	char _osd[30];
	unsigned int i, j, k, h_end, v_end, h_start;
	char *_p;
	uint16 r_addr;
	struct _terawins_osd_info *_info = terawins_osd[OI_ICON];

	if(_info->on == 0){
		return;
	}

	//-- Clear data array.
	for(i = 0; i < FONTS_PER_ROW; i ++){
		_osd[i] = 0;
	}	

	if(_idx == 0x00){
		_p = valid_icon;
		_info->f_color = FG_COLOR12;
	}
	else if(_idx == 0x01){
		_p = invalid_icon;
		_info->f_color = FG_COLOR10;
	}
	else{
		TW_MSG("\nInput icon num error !\n");
		return;
	}
	//-- Set ICON color
	terawins_set_osd_line_info(_info);	
	
	k = 0;
	r_addr = _info->ram_addr;
	for(j = 0; j < _info->n_row; j ++){
		for(i = 0 ; i < ICON_NUM; i ++){
			_osd[(ICON_START + i)] = _p[k ++];
		}
		terawins_set_osd_line(_osd, r_addr, _info);
		r_addr += WORDS_PER_ROW;
	}
}

/* -------------------------------------------------------------------------- */
void terawins_show_battery(uint32 _idx)
{
	char _osd[30];
	unsigned int i, _start, _end;
	struct _terawins_osd_info *_info = terawins_osd[OI_BATTERY];

	if((_info->on == 0) || (_idx >= BATTERY_MAX)){
		return;
	}

	//-- Clear data array.
	for(i = 0; i < FONTS_PER_ROW; i ++){
		_osd[i] = 0;
	}

	_start = BATTERY_START;
	//_end = FONTS_PER_ROW - 1;
	_end = BATTERY_END;
	_osd[_start ++] = BATTERY_ICON_HEAD;
	if(_idx == BATTERY_0){
		_info->f_color = FG_COLOR4;
		_info->blink = 1;
		for( ; _start < _end; _start ++){
			_osd[_start] = BATTERY_ICON_EMPTY;
		}
	}
	else if(_idx < BATTERY_MAX){
		_info->f_color = FG_COLOR6;
		_info->blink = 0;
		// empty
		for( ; _start < (_end - _idx); _start ++){
			_osd[_start] = BATTERY_ICON_EMPTY;
		}
		// battery
		for( ; _start < _end; _start ++){
			_osd[_start] = BATTERY_ICON_FULL;
		}		
	}
	_osd[_end] = BATTERY_ICON_END;	

	//-- Set ICON color
	terawins_set_osd_line_info(_info);	
	terawins_set_osd_line(_osd, _info->ram_addr, _info);
}


/* -------------------------------------------------------------------------- */
void terawins_clear_icon(void)
{
	clear_osd(OI_ICON);	
}

/* -------------------------------------------------------------------------- */
void terawins_clear_subtitle(void)
{
	clear_osd(OI_SUBTITLE);
}

/* -------------------------------------------------------------------------- */
void terawins_clear_mute_vol(void)
{
	clear_osd(OI_MUTE_VOL);
	//Flowbee test
	//terawins_clear_battery();
}

/* -------------------------------------------------------------------------- */
void terawins_clear_audio_channels(void)
{
	clear_osd(OI_AUDIO_CHANNELS);
}

/* -------------------------------------------------------------------------- */
void terawins_clear_time(void)
{
	clear_osd(OI_VIDEO_TIME);
}

/* -------------------------------------------------------------------------- */
void terawins_clear_speed(void)
{
	clear_osd(OI_VIDEO_SPEED);
}

/* -------------------------------------------------------------------------- */
void terawins_clear_battery(void)
{
	clear_osd(OI_BATTERY);
}

/* -------------------------------------------------------------------------- */
int terawins_clear_all_osd_item(void)
{
	clear_osd(OI_ALL);
	return 0;
}

/* -------------------------------------------------------------------------- */
void calculate_start_line(void)
{
#if 1
	unsigned long line_num;
	uint16 i, j;
	uint16 t_val, t_idx;
	struct _terawins_osd_info *p;

	//-- ICON in the center of display area.
	line_num = ((terawins_osd[OI_ICON]->n_row * OSD_FONT_HEIGHT) 
				<< terawins_osd[OI_ICON]->d_line);
	terawins_osd[OI_ICON]->s_line = (LCD_H - line_num) / 2;

	//-- Get the start line info from osd info.
	for(i = 0; i < OI_ALL; i ++){
		// sort_tbl[][0]: index, sort_tbl[][1]: start line
		p = terawins_osd[i];
		sort_tbl[i][0] = i;
		sort_tbl[i][1] = (p->on == 1) ? terawins_osd[i]->s_line : 0xFFFF;
		TW_MSG("%d, %d\n", sort_tbl[i][0], sort_tbl[i][1]);
	}
	
	//-- Sorting
	for(j = 0; j < OI_ALL; j ++){
		for(i = j; i < OI_ALL; i ++){
			if(sort_tbl[i][1] < sort_tbl[j][1]){
				t_val = sort_tbl[i][1];
				t_idx = sort_tbl[i][0];
				sort_tbl[i][1] = sort_tbl[j][1];
				sort_tbl[i][0] = sort_tbl[j][0];
				sort_tbl[j][1] = t_val;
				sort_tbl[j][0] = t_idx;
			}
		}
		TW_MSG("%d, %d\n", sort_tbl[j][0], sort_tbl[j][1]);
	}
	
#else
	unsigned long line_num;
	
	//-- Video speed
	terawins_osd[OI_VIDEO_SPEED]->s_line = 18;

	//-- ICON
	line_num = ((terawins_osd[OI_ICON]->n_row * OSD_FONT_HEIGHT) 
				<< terawins_osd[OI_ICON]->d_line);
	terawins_osd[OI_ICON]->s_line = (LCD_H - line_num) / 2;

	//-- Video time (the last row)
	line_num = ((terawins_osd[OI_VIDEO_TIME]->n_row * OSD_FONT_HEIGHT) 
				<< terawins_osd[OI_VIDEO_TIME]->d_line);
	terawins_osd[OI_VIDEO_TIME]->s_line = LCD_H - line_num;

	//-- Mute and vol (the row above video time)
	line_num = ((terawins_osd[OI_MUTE_VOL]->n_row * OSD_FONT_HEIGHT) 
				<< terawins_osd[OI_MUTE_VOL]->d_line);
	terawins_osd[OI_MUTE_VOL]->s_line = terawins_osd[OI_VIDEO_TIME]->s_line - line_num;

	//-- Video subtitle (the row above mute and vol)
	line_num = ((terawins_osd[OI_SUBTITLE]->n_row * OSD_FONT_HEIGHT) 
				<< terawins_osd[OI_SUBTITLE]->d_line);
	terawins_osd[OI_SUBTITLE]->s_line = terawins_osd[OI_MUTE_VOL]->s_line - line_num;
#endif
}

/* -------------------------------------------------------------------------- */
uint8 calculate_start_row(void)
{
#if 1
	uint8 i = 0, j = 0;
	uint8 row_num = 0;
	uint8 num_row = 0;
	uint16 line_num = 0;		// gap line between osd item.
	uint16 start_line = 0;		// start line of blank.
	struct _terawins_osd_info *p;

	start_line = terawins_osd[sort_tbl[0][0]]->s_line;
	//-- Start
	//for(i = 1; i < OI_ALL; i ++){
	for(i = 0; i < OI_ALL; i ++){
		p = terawins_osd[sort_tbl[i][0]];
		if(p->on == 0){
			continue;
		}
		// Transfer start line to start row.
		// Find the blank line and clear the blank line.
		TW_MSG("\n%d, %d, %d", sort_tbl[i][0], p->s_line, start_line);
		if(p->s_line < start_line){
			p->s_line = start_line;
		}
		line_num = p->s_line - start_line;
		num_row = line_num / (OSD_FONT_HEIGHT << blank.d_line);
		line_num = num_row * (OSD_FONT_HEIGHT << blank.d_line);
		p->s_line = start_line + line_num;

		//-- Set ram addr of blank osd
		blank.n_row = num_row;
		blank.r_num = row_num;
		blank.ram_addr = CHAR_START_RAM_ADDR(font_info[font_index].font_nums) 
								+ (WORDS_PER_ROW * row_num);
		clear_osd_line(&blank);
		TW_MSG("\n");
		TW_MSG("\nrow_num  : %d", row_num);		
		TW_MSG("\nline_num : %d", line_num);
		TW_MSG("\nnum_row  : %d", num_row);
		TW_MSG("\ns_line   : %d", p->s_line);
		
		// Calculate the start row of osd item.
		row_num = row_num + num_row;
		p->r_num = row_num;

		// Update the row num of blank line behind previous osd item.
		row_num = row_num + p->n_row;
		start_line = p->s_line + ((p->n_row * OSD_FONT_HEIGHT) << p->d_line);
		
		// Clear the osd item.
		clear_osd_line(p);
		TW_MSG("\nr_num   : %d", p->r_num);
		TW_MSG("\nn_row   : %d", p->n_row);
		TW_MSG("\nrow_num : %d", row_num);
		TW_MSG("\ns_line  : %d", start_line);		
		
	}
	TW_MSG("\n");	
	return row_num;
#else
	uint8 i = 0, j = 0;
	uint8 row_num = 0;
	uint8 num_row = 0;
	uint16 line_num = 0;		// gap line between osd item.
	uint16 start_line = 0;		// start line of blank.

	start_line = terawins_osd[i]->s_line;
	//-- Start
	for(i = 0; ; i ++){
		if(terawins_osd[i] == NULL){
			break;
		}
		if(terawins_osd[i]->on == 0){
			continue;
		}
		// Transfer start line to start row.
		// Find the blank line and clear the blank line.
		line_num = terawins_osd[i]->s_line - start_line;
		num_row = line_num / (OSD_FONT_HEIGHT << blank.d_line);
		line_num = num_row * (OSD_FONT_HEIGHT << blank.d_line);
		terawins_osd[i]->s_line = start_line + line_num;

		//-- Set ram addr of blank osd
		blank.n_row = num_row;
		blank.r_num = row_num;
		blank.ram_addr = CHAR_START_RAM_ADDR(font_info[font_index].font_nums) 
								+ (WORDS_PER_ROW * row_num);
		clear_osd_line(&blank);
//		TW_MSG("\n");
//		TW_MSG("\nrow_num  : %d", row_num);		
//		TW_MSG("\nline_num : %d", line_num);
//		TW_MSG("\nnum_row  : %d", num_row);
//		TW_MSG("\ns_line   : %d", terawins_osd[i]->s_line);
		
		// Calculate the start row of osd item.
		row_num = row_num + num_row;
		terawins_osd[i]->r_num = row_num;

		// Update the row num of blank line behind previous osd item.
		row_num = row_num + terawins_osd[i]->n_row;
		start_line = terawins_osd[i]->s_line + ((terawins_osd[i]->n_row * OSD_FONT_HEIGHT) << terawins_osd[i]->d_line);
		
		// Clear the osd item.
		clear_osd_line(terawins_osd[i]);
//		TW_MSG("\nr_num   : %d", terawins_osd[i]->r_num);
//		TW_MSG("\nn_row   : %d", terawins_osd[i]->n_row);
//		TW_MSG("\nrow_num : %d", row_num);
//		TW_MSG("\ns_line  : %d", start_line);		
		
	}
//	TW_MSG("\n");	
	return row_num;
#endif
}

/* -------------------------------------------------------------------------- */
uint8 calculate_osd_ram_address(void)
{
	uint8 i, j;
	uint16 r_addr;					// sram address
	uint16 f_base_addr;				// font base address in sram

	f_base_addr = CHAR_START_RAM_ADDR(font_info[font_index].font_nums);
//	TW_MSG("\nbase_addr: %x\n", f_base_addr);
		
	for(i = 0; ; i ++){
		if(terawins_osd[i] == NULL){
			break;
		}
		//-- Find ram address.
		r_addr = f_base_addr + (WORDS_PER_ROW * terawins_osd[i]->r_num);
		terawins_osd[i]->ram_addr = r_addr;
//		TW_MSG("\nram_addr[%d]: %x\n", i, terawins_osd[i]->ram_addr);
	}
}

/* -------------------------------------------------------------------------- */
void terawins_set_time_on_off(uint32 _on)
{
#if 1
	uint16 num_line;
	uint16 i, idx_icon, idx_time;
	struct _terawins_osd_info *p;

	TW_MSG("\ntime on/off\n");
	num_line = ((terawins_osd[OI_VIDEO_TIME]->n_row * OSD_FONT_HEIGHT) 
				<< (terawins_osd[OI_VIDEO_TIME]->d_line));	

	//-- Find the osd item which under the video time.
	for(i = 0; i < OI_ALL; i ++){
		if(sort_tbl[i][0] == OI_ICON){
			idx_icon = i;
		}
		if(sort_tbl[i][0] == OI_VIDEO_TIME){
			idx_time = i;
		}		
	}
	
	if(_on == 1){	// enable display video time.
		if(terawins_osd[OI_VIDEO_TIME]->on == 1){
			TW_MSG("\nAlready enable video time !!");
			return;
		}
		terawins_osd[OI_VIDEO_TIME]->on = 1;

		//-- The item which under video time
		if(terawins_osd[OI_VIDEO_TIME]->s_line > terawins_osd[OI_ICON]->s_line){
			for(i = (idx_icon + 1); i < idx_time; i ++){
				terawins_osd[sort_tbl[i][0]]->s_line -= num_line;
			}
		}
		else{
			for(i = (idx_time + 1); i < idx_icon; i ++){
				terawins_osd[sort_tbl[i][0]]->s_line -= num_line;
			}
		}
	}
	else{			// disable display video time.
		if(terawins_osd[OI_VIDEO_TIME]->on == 0){
			TW_MSG("\nAlready disable video time !!");
			return;
		}
		terawins_osd[OI_VIDEO_TIME]->on = 0;

		//-- The item which under video time
		if(terawins_osd[OI_VIDEO_TIME]->s_line > terawins_osd[OI_ICON]->s_line){
			for(i = (idx_icon + 1); i < idx_time; i ++){
				terawins_osd[sort_tbl[i][0]]->s_line += num_line;
			}
		}
		else{
			for(i = (idx_time + 1); i < idx_icon; i ++){
				terawins_osd[sort_tbl[i][0]]->s_line += num_line;
			}
		}
	}
//	TW_MSG("num line: %d", num_line);	
//	TW_MSG("mute: %d", terawins_osd[OI_MUTE_VOL]->s_line);
//	TW_MSG("subtitle: %d", terawins_osd[OI_SUBTITLE]->s_line);	
	calculate_start_line();
	calculate_start_row();
	calculate_osd_ram_address();
	//-- update osd info to kernel
	update_osd_info_to_kernel();	

#else
	unsigned short num_line;

	num_line = ((terawins_osd[OI_VIDEO_TIME]->n_row * OSD_FONT_HEIGHT) 
				<< (terawins_osd[OI_VIDEO_TIME]->d_line));	
	
	if(_on == 1){	// enable display video time.
		if(terawins_osd[OI_VIDEO_TIME]->on == 1){
			TW_MSG("\nAlready enable video time !!");
			return;
		}
		terawins_osd[OI_VIDEO_TIME]->on = 1;
		terawins_osd[OI_MUTE_VOL]->s_line -= num_line;
		terawins_osd[OI_SUBTITLE]->s_line -= num_line;
	}
	else{			// disable display video time.
		if(terawins_osd[OI_VIDEO_TIME]->on == 0){
			TW_MSG("\nAlready disable video time !!");
			return;
		}
		terawins_osd[OI_VIDEO_TIME]->on = 0;
		terawins_osd[OI_MUTE_VOL]->s_line += num_line;
		terawins_osd[OI_SUBTITLE]->s_line += num_line;
	}
//	TW_MSG("num line: %d", num_line);	
//	TW_MSG("mute: %d", terawins_osd[OI_MUTE_VOL]->s_line);
//	TW_MSG("subtitle: %d", terawins_osd[OI_SUBTITLE]->s_line);	
	calculate_start_row();
	calculate_osd_ram_address();
	//-- update osd info to kernel
	update_osd_info_to_kernel();	
#endif
}

/* -------------------------------------------------------------------------- */
char terawins_get_time_on_off(void)
{
	return terawins_osd[OI_VIDEO_TIME]->on;
}

/* -------------------------------------------------------------------------- */
int update_osd_info_to_kernel(void)
{
	int i;

//	TW_MSG("\nupdate_osd_info: %x, %x\n", video_speed.ram_addr, subtitle.ram_addr);
	//-- Update the following osd info for Mplayer to display subtitle and video speed.
	i = pgu_set_osd_speed_info(&video_speed);
	i = pgu_set_osd_subtitle_info(&subtitle);
	
	return i;
}

/* -------------------------------------------------------------------------- */
int update_scale_table_to_kernel(void)
{
	int i;
	int _tbl[(SCALE_TABLE_NUM*2) + 1];

//	TW_MSG("\nupdate_scale_table: %x, %x\n", *(TerawinsCmdTbl[i]), *(TerawinsCmdTbl[i]+1));
	for(i = 2; i < (TERAWINS_END* 2); i ++){
		memcpy(_tbl, TerawinsCmdTbl[i], (sizeof(int) * (SCALE_TABLE_NUM*2)));
		_tbl[(SCALE_TABLE_NUM*2)] = i;
		if(pgu_set_tcon_scale_table(_tbl) == -1){
			return -1;
		}
	}
	return 0;
}

/* -------------------------------------------------------------------------- */
void init_osd_info(void)
{
	int i;
	uint8 _total_row = 0;

	TW_MSG("\ninit_osd_info\n");
	//-- init osd line info.
	for(i = 0; ; i ++){
		if(terawins_osd[i] == NULL){
			break;
		}
		terawins_set_osd_line_info(terawins_osd[i]);
	}
	//-- init blank line info.
	terawins_set_osd_line_info(&blank);

	//-- Calaulate the start line of each osd item.
	calculate_start_line();
	
	//-- Calculate start row of each osd item and init osd line data.
	_total_row = calculate_start_row();
	calculate_osd_ram_address();

	//-- update osd info to kernel
	update_osd_info_to_kernel();

	//-- Set osd display range.
	terawins_set_osd_menu(TERAWINS_START_X, terawins_osd[0]->s_line, 0, _total_row);

}

/* -------------------------------------------------------------------------- */
void init_terawins_osd(void)
{
	TW_MSG("\ninit_terawins_osd\n");
	/* Init osd. */ 
	terawins_set_osd_cmd(OSD_CTRL, 				0x00);	// turn off osd, set to char mode.
	terawins_set_osd_cmd(CHAR_DLY1, 			0x00);
	terawins_set_osd_cmd(CHAR_DLY2, 			0x00);	
	terawins_set_osd_cmd(CHAR_DLY3, 			0x00);
	terawins_set_osd_cmd(CHAR_FONT_SIZE, 		0x09);
	terawins_set_osd_cmd(CHAR_RAM_START_ADDR,	0x00);	
	terawins_set_osd_cmd(CHAR_BS_CTRL, 			0x00);
	terawins_set_osd_cmd(CHAR_BS_COLOR, 		0x00);
	terawins_set_osd_cmd(CHAR_HEIGHT_SCALING, 	0x20);	
	terawins_set_osd_cmd(BLINKING_CTRL, 		0x05);
	terawins_set_osd_cmd(BMAP_WIN_SIZE_UPPER, 	0x00);
	terawins_set_osd_cmd(BMAP_WIN_SIZE_WIDTH, 	0x80);	
	terawins_set_osd_cmd(BMAP_WIN_SIZE_HEIGHT, 	0x60);
	terawins_set_osd_cmd(BMAP_DOT_ENLARGE, 		0x11);
//	terawins_set_osd_cmd(OSD_COLOR_LUT_PORT, 	0x84);	
	terawins_set_osd_cmd(CHAR_RAM_ACCESS_CTRL, 	0x00);
	terawins_set_osd_cmd(W1_START_ROW_NUM, 		0x00);//00
	terawins_set_osd_cmd(W1_END_ROW_NUM, 		0x00);	
	terawins_set_osd_cmd(W1_START_COL_NUM, 		0x00);
	terawins_set_osd_cmd(W1_END_COL_NUM, 		0x00);
	terawins_set_osd_cmd(W1_SHADOW_SIZE, 		0x00);	
	terawins_set_osd_cmd(CHAR2BP_IDX_BASE, 		0xf0);
	terawins_set_osd_cmd(CHAR2BP_BASEADDR_LO, 	0x00);
	terawins_set_osd_cmd(CHAR2BP_BASEADDR_HI, 	0x08);	
	terawins_set_osd_cmd(ALPHA_BLENDING_CTRL, 	0x42);
	terawins_set_osd_cmd(REVISION_ID, 			0x12);
	terawins_set_osd_cmd(CHAR_RAM_STOP_ADDR, 	0x60);	

	//-- Clear osd ram.
	terawins_clear_osd_ram();

	//-- Load BMP LUT.
	terawins_load_osd_color(terawins_osd_color_table);

	//-- Load fonts.
	font_info[font_index].font_nums = terawins_load_osd_fonts(AllFonts);

	//-- Set line info start address and font size.
	terawins_set_osd_cmd(CHAR_FONT_SIZE, font_info[font_index].font_size);
	terawins_set_osd_cmd(CHAR_RAM_START_ADDR, CHAR_START_ADDR(font_info[font_index].font_nums));

	//-- Setup menu info.
	init_osd_info();

	//-- Enable char mode OSD function.
	terawins_enable_osd(1);

	//-- Init scale table
	update_scale_table_to_kernel();
}

#endif	//	~#if USE_TERAWINS_TCON
