/*
 *	terawins_tcon.h - Register and variable definitions for TeraWins T-CON.
 */

#ifndef _TERAWINS_TCON_H_
#define _TERAWINS_TCON_H_

//** Debug
//#define TERAWINS_DBG
#ifdef TERAWINS_DBG
#define	TW_MSG(format, arg...)		printf(format, ## arg)
#else
#define TW_MSG
#endif

#define END_CMD						(-1)
#define uint8 						unsigned char
#define uint16						unsigned short
#define uint32 						unsigned long

//** Terawins T-CON device address.
#define TERAWINS_PAGE0_ADDR			0x40
#define TERAWINS_PAGE1_ADDR			0x42

enum TerawinsDim
{
	TERAWINS_INIT,
	TERAWINS_640x480,
	TERAWINS_720x480,
	TERAWINS_720x576,
	TERAWINS_800x600,
	TERAWINS_END,
};

#define SCALE_TABLE_NUM				32

/* Register definitions for TeraWins OSD control. */
enum{
	OSD_CFG_INDEX 		= 0xA0,
	OSD_CFG_DATA 		= 0xA1,
	OSD_RAM_ADDR_LO 	= 0xA2,
	OSD_RAM_ADDR_HI 	= 0xA3,
	OSD_RAM_DATA 		= 0xA4,
};

enum TERAWINS_OSD_CMD{
	OSD_CTRL			= 0x00,
	CHAR_DLY1			= 0x01,
	CHAR_DLY2			= 0x02,
	CHAR_DLY3			= 0x03,
	CHAR_FONT_SIZE		= 0x04,
	CHAR_RAM_START_ADDR	= 0x05,
	CHAR_BS_CTRL		= 0x06,
	CHAR_BS_COLOR		= 0x07,
	CHAR_HEIGHT_SCALING	= 0x08,
	BLINKING_CTRL		= 0x09,
	BMAP_WIN_SIZE_UPPER	= 0x0A,
	BMAP_WIN_SIZE_WIDTH	= 0x0B,
	BMAP_WIN_SIZE_HEIGHT= 0x0C,	
	BMAP_DOT_ENLARGE	= 0x0D,
	OSD_COLOR_LUT_PORT	= 0x0E,
	CHAR_RAM_ACCESS_CTRL= 0x0F,
	W1_START_ROW_NUM	= 0x10,
	W1_END_ROW_NUM		= 0x11,
	W1_START_COL_NUM	= 0x12,
	W1_END_COL_NUM		= 0x13,	
	W1_SHADOW_SIZE		= 0x14,
	CHAR2BP_IDX_BASE	= 0x19,
	CHAR2BP_BASEADDR_LO	= 0x1A,	
	CHAR2BP_BASEADDR_HI	= 0x1B,	
	ALPHA_BLENDING_CTRL	= 0x1C,	
	REVISION_ID			= 0x1D,
	CHAR_RAM_STOP_ADDR	= 0x1E,
};

//-- Flowbee 2007-12-13
struct _terawins_osd_info
{
	char on;							// osd on
	char n_row;							// num of row
	unsigned short s_line;				// start line of the frame (multiples of 16)
	char r_num;							// row num of the osd area
	unsigned short ram_addr;			// ram address
	char blink;							// blink
	char f_color;						// foreground color
	char b_color;						// background color
	char d_line;						// double size of line	
	unsigned char char_attr;
	unsigned char row_attr;
};


enum{
	FONT_10x16 = 0,
	FONT_12x18,
	FONT_16x24,
	FONT_END,
};

struct _font_info
{
	unsigned long font_size;	// register val
	unsigned char width;		// font width
	unsigned char height;		// font height
	unsigned char font_nums;	// total font nums
};

#define TERAWINS_OSD_RAM_SIZE				(6*1024)	// byte
#define CHAR_PER_LINE						30
#define BYTES_PER_LINE_INFO					64

#define OSD_FONT_WIDTH						font_info[font_index].width
#define OSD_FONT_HEIGHT						font_info[font_index].height
#define FONT_WIDTH_BYTES					((OSD_FONT_WIDTH + 7) / 8)

#define CHAR_START_ADDR(num_of_font)		( (((((FONT_WIDTH_BYTES*OSD_FONT_HEIGHT*num_of_font)/32)+1)*32)+63)/64 )
#define CHAR_START_RAM_ADDR(num_of_font)	( CHAR_START_ADDR(num_of_font) * 32 )


/* Define the Foreground color and Background color. */
enum FG_color{
	FG_COLOR0 = 0,
	FG_COLOR1,
	FG_COLOR2,
	FG_COLOR3,
	FG_COLOR4,
	FG_COLOR5,
	FG_COLOR6,
	FG_COLOR7,
	FG_COLOR8,
	FG_COLOR9,
	FG_COLOR10,
	FG_COLOR11,
	FG_COLOR12,
	FG_COLOR13,
	FG_COLOR14,
	FG_COLOR15,	
	FG_MAX,
};

enum BG_color{
	BG_COLOR0 = 0,
	BG_COLOR1,
	BG_COLOR2,
	BG_COLOR3,
	BG_COLOR4,
	BG_COLOR5,
	BG_COLOR6,
	BG_COLOR7,
	BG_MAX,
};

//-- OSD define
/* -------------------------------------------------------------------------- */
#define WORDS_PER_ROW				32
#define FONTS_PER_ROW				30
#define WIDTH_PER_ROW				(FONTS_PER_ROW * OSD_FONT_WIDTH)

#define LCD_W						800
#define LCD_H						600

#define TERAWINS_W					1500

#define LCD_START_X					((LCD_W - WIDTH_PER_ROW) / 2)
#define TERAWINS_START_X			((TERAWINS_W * LCD_START_X) / LCD_W)

enum OSDIndex 
{
	OI_VIDEO_SPEED,
	OI_BATTERY,
	OI_ICON,
	OI_SUBTITLE,
	OI_MUTE_VOL,
	OI_VIDEO_TIME,
	OI_AUDIO_CHANNELS,
	OI_ALL,
	//OI_BATTERY,	
};

enum BatteryIndex 
{
	BATTERY_0,
	BATTERY_1,
	BATTERY_2,
	BATTERY_3,
	BATTERY_4,
	BATTERY_5,
	BATTERY_MAX,
};


//-- Define ICON
#define ICON_START					13
#define ICON_NUM					3

//-- Define MUTE and VOL
#define MUTE_START					13
#define MUTE_NUM					4

#define AUDIO_CHANNELS_START					13
#define AUDIO_CHANNELS_NUM					2

#define VOL_START					8
#define VOL_NUM						4

#define FONT_VOL_BAR				0x13	// font index
#define FONT_VOL_EMPTY_BAR			0x14	// font index
#define TOTAL_VOL_BAR				10

//-- Define BATTERY
#define BATTERY_NUM					7
#define BATTERY_START				23
#define BATTERY_END					(BATTERY_START + BATTERY_NUM - 1)
#define BATTERY_ICON				0x15
#define BATTERY_ICON_HEAD			0x18
#define BATTERY_ICON_END			0x19
#define BATTERY_ICON_FULL			0x17
#define BATTERY_ICON_EMPTY			0x16

/* -------------------------------------------------------------------------- */
/* Function declarations. */
int terawins_set_cmd(uint8 dev_addr, uint8 reg_addr, uint8 reg_val);
int terawins_set_osd_cmd(unsigned char cfg_idx, unsigned char cfg_data);
int terawins_set_osd_ram_addr(unsigned short ram_addr);
int terawins_set_osd_ram_data(unsigned char ram_data);
void terawins_set_osd_line_data(unsigned short ram_addr, unsigned short *str);
int init_terawins_tcon(void);
void init_terawins_osd(void);
void terawins_on_off(unsigned char _on);
int terawins_set_resolution(unsigned char idx);
void terawins_pwm_on_off(uint8 _on);
void terawins_backlight_on_off(uint8 _on);
void terawins_set_pwm(uint8 _val);
void terawins_set_brightness(uint8 _val);
void terawins_set_contrast(uint8 _val);
void terawins_set_osd_alpha_blending(unsigned char mode, unsigned char val);
void terawins_set_osd_blinking(unsigned char blk_freq, unsigned char duty);
void terawins_enable_osd(unsigned char _en);
void terawins_parse_resolution(unsigned long _w, unsigned long _h);


void terawins_show_mute_vol(uint32 idx);
void terawins_show_audio_channels(uint32 _idx);
void terawins_clear_audio_channels(void);
void terawins_clear_mute_vol(void);
void terawins_show_icon(uint32 _idx);
void terawins_clear_icon(void);
void terawins_show_subtitle(char _s[][31]);
void terawins_clear_subtitle(void);
int terawins_clear_all_osd_item(void);
void terawins_show_time(char *_t);
void terawins_clear_time(void);
void terawins_show_speed(char *_t);
void terawins_clear_speed(void);
void terawins_show_battery(uint32 _idx);
void terawins_clear_battery(void);
void terawins_set_time_on_off(uint32 _on);
char terawins_get_time_on_off(void);

void test_terawins(unsigned int _val);

/* -------------------------------------------------------------------------- */
//-- Definition
#if 1
#define tcon_show_mute_vol(_idx)	terawins_show_mute_vol(_idx)
#define tcon_clear_mute_vol()		terawins_clear_mute_vol()

#define tcon_show_audio_channels(_idx)	terawins_show_audio_channels(_idx)
#define tcon_clear_audio_channels()		terawins_clear_audio_channels()

#define tcon_show_icon(_idx)		terawins_show_icon(_idx)
#define tcon_clear_icon()			terawins_clear_icon()
#define tcon_clear_subtitle()		terawins_clear_subtitle()
#define tcon_clear_all_osd_item()	terawins_clear_all_osd_item()
#define tcon_show_time(_t)			terawins_show_time(_t)
#define tcon_clear_time()			terawins_clear_time()
#define tcon_clear_speed()			terawins_clear_speed()
#define tcon_set_time_on_off(_on)	terawins_set_time_on_off(_on)
#define tcon_get_time_on_off()		terawins_get_time_on_off()

#else
#define tcon_show_mute_vol(_idx)
#define tcon_clear_mute_vol()
#define tcon_show_icon(_idx)
#define tcon_clear_icon()
#define tcon_clear_subtitle()
#define tcon_clear_all_osd_item()
#define tcon_show_time(_t)
#define tcon_clear_time()
#define tcon_clear_speed()
#define tcon_set_time_on_off(_on)
#define tcon_get_time_on_off()
#endif	// ~#if USE_TERAWINS_TCON

#endif

