#include "skydemo.h"
#include "bmp.h"
//------------------------------------------------------------------------------------------------
static void *bmp_init(char *filename, Uint32 *width, Uint32 *height)
{
	struct bmp_state *h;
    	h = malloc(sizeof(*h));
    	memset(h, 0, sizeof(*h));

    	h->fp = fopen(filename, "r");
	if (h->fp == NULL)
	{
		DPRINTF("Can not open file %s!\n", filename);
		//return;
		goto oopss;
	}

    	fseek(h->fp, 10, SEEK_SET);
    	fread(&h->hdr, sizeof(struct bmp_hdr), 1, h->fp);

#if BYTE_ORDER == BIG_ENDIAN
    	h->hdr.foobar = le32_to_cpu(h->hdr.foobar);
    	h->hdr.size = le32_to_cpu(h->hdr.size);
    	h->hdr.width = le32_to_cpu(h->hdr.width);
    	h->hdr.height = le32_to_cpu(h->hdr.height);
    
    	h->hdr.planes = le16_to_cpu(h->hdr.planes);
    	h->hdr.bit_cnt = le16_to_cpu(h->hdr.bit_cnt);
    
    	h->hdr.image_size = le32_to_cpu(h->hdr.image_size);
    	h->hdr.xpels_meter = le32_to_cpu(h->hdr.xpels_meter);
    	h->hdr.ypels_meter = le32_to_cpu(h->hdr.ypels_meter);
    	h->hdr.num_colors = le32_to_cpu(h->hdr.num_colors);
    	h->hdr.imp_colors = le32_to_cpu(h->hdr.imp_colors);
#endif
#if 0
    	if(debug)
		DPRINTF("bmp: hdr=%d size=%dx%d planes=%d"
		" bits=%d size=%d res=%dx%d colors=%d/%d | %d\n",
		h->hdr.size,h->hdr.width,h->hdr.height,
		h->hdr.planes,h->hdr.bit_cnt,h->hdr.image_size,
		h->hdr.xpels_meter,h->hdr.ypels_meter,
		h->hdr.num_colors,h->hdr.imp_colors,h->hdr.foobar);
#endif
    	if(h->hdr.bit_cnt != 1  && h->hdr.bit_cnt != 4  && 	h->hdr.bit_cnt != 8  && h->hdr.bit_cnt != 24)
	{
		DPRINTF("bmp: can't handle depth [%d]:[%s]\n", h->hdr.bit_cnt, filename);
		goto oops;
    	}
    	if(h->hdr.compression[0] || h->hdr.compression[1] || h->hdr.compression[2] || h->hdr.compression[3])
	{
		DPRINTF("bmp: can't handle compressed bitmaps [%c%c%c%c]\n",
				h->hdr.compression[0],
				h->hdr.compression[1],
				h->hdr.compression[2],
				h->hdr.compression[3]);
		goto oops;
    	}
    	if(0 == h->hdr.num_colors && h->hdr.bit_cnt <= 8)
		h->hdr.num_colors = (1 << h->hdr.bit_cnt);
    	if(h->hdr.num_colors > 256)
		h->hdr.num_colors = 256;
    	if(h->hdr.num_colors)
	{
		fseek(h->fp,14+h->hdr.size,SEEK_SET);
		fread(&h->cmap,sizeof(struct bmp_cmap),h->hdr.num_colors,h->fp);
    	}
#if 0
    	i->width  = h->hdr.width;
    	i->height = h->hdr.height;
    	if(h->hdr.xpels_meter)
		i->dpi = res_m_to_inch(h->hdr.xpels_meter);
    	i->npages = 1;
#else
    	*width  = h->hdr.width;
    	*height = h->hdr.height;
#endif
    	return h;

oops:
    	fclose(h->fp);
oopss:
    	free(h);
    	return NULL;
}
//------------------------------------------------------------------------------------------------
static void bmp_read(Uint16 *dst, Uint32 line, void *data)
{
    	struct bmp_state *h = data;
    	Uint32 ll, y, x, pixel, byte = 0;
    
    	ll = (((h->hdr.width * h->hdr.bit_cnt + 31) & ~0x1f) >> 3);
    	y = h->hdr.height - line - 1;
    	fseek(h->fp, h->hdr.foobar + y * ll, SEEK_SET);

    	switch (h->hdr.bit_cnt)
	{
    		case 1:
			for(x = 0; x < h->hdr.width; x++)
			{
	    			if(0 == (x & 0x07))
					byte = fgetc(h->fp);
		    		pixel = byte & (0x80 >> (x & 0x07)) ? 1 : 0;
      		      		*(dst++) = ENCODE_RGB555(h->cmap[pixel].red, h->cmap[pixel].green, h->cmap[pixel].blue);
	    			//*(dst++) = h->cmap[pixel].red;
	    			//*(dst++) = h->cmap[pixel].green;
		    		//*(dst++) = h->cmap[pixel].blue;
			}
			break;
    		case 4:
			for(x = 0; x < h->hdr.width; x++)
			{
		    		if(x & 1)
				{
					pixel = byte & 0xf;
	    			}
				else
				{
					byte = fgetc(h->fp);
					pixel = byte >> 4;
	    			}
            			*(dst++) = ENCODE_RGB555(h->cmap[pixel].red, h->cmap[pixel].green, h->cmap[pixel].blue);
	    			//*(dst++) = h->cmap[pixel].red;
	    			//*(dst++) = h->cmap[pixel].green;
	    			//*(dst++) = h->cmap[pixel].blue;
			}
			break;
    		case 8:
			for(x = 0; x < h->hdr.width; x++)
			{
		    		pixel = fgetc(h->fp);
            			*(dst++) = ENCODE_RGB555(h->cmap[pixel].red, h->cmap[pixel].green, h->cmap[pixel].blue);
	    			//*(dst++) = h->cmap[pixel].red;
	    			//*(dst++) = h->cmap[pixel].green;
	    			//*(dst++) = h->cmap[pixel].blue;
			}
			break;
    		case 24:
			for(x = 0; x < h->hdr.width; x++)
			{
            			unsigned char r, g, b;
	    			//dst[2] = fgetc(h->fp);
	    			//dst[1] = fgetc(h->fp);
	    			//dst[0] = fgetc(h->fp);
	    			//dst += 3;
	    			b = fgetc(h->fp);
	    			g = fgetc(h->fp);
	    			r = fgetc(h->fp);
            			*(dst++) = ENCODE_RGB555(r, g, b);
			}
			break;
    		default:
			memset(dst, 0, h->hdr.width * 2);
			break;
    	}
}
//------------------------------------------------------------------------------------------------
static void bmp_done(void *data)
{
    	struct bmp_state *h = data;

    	fclose(h->fp);
    	free(h);
}
//------------------------------------------------------------------------------------------------
//Uint32 LoadBmpFile(char* filename)
Uint8 *LoadBmpFile(char* filename)
{
	void *data;
	Uint32 width, height, i;
	Uint16 *BmpBuf;

	data = bmp_init(filename, &width, &height);
	if (data == NULL) return NULL;
	
	BmpBuf = (Uint16 *)malloc(width * height * 2);
	if(!BmpBuf)
	{
		DMSG("Failed to allocate memory!\n");
		return NULL;
		//exit(1);
	}
	for(i = 0; i < height; i++)
		bmp_read(&BmpBuf[i * width], i, data);

	bmp_done(data);
	//return (Uint32)BmpBuf;
	return (Uint8 *)BmpBuf;
}
//------------------------------------------------------------------------------------------------
//void CloseBmpFile(Uint32 hBmp)
void CloseBmpFile(Uint8 *hBmp)
{
	//free((char*)hBmp);
	free(hBmp);
}
//------------------------------------------------------------------------------------------------
void *GetBmpHdr(char* filename)
{
	void *data;
	Uint32 width, height;

	data = bmp_init(filename, &width, &height);
	return (Uint8 *)data;
}
//------------------------------------------------------------------------------------------------
void FreeBmpHdr(void* data)
{
	bmp_done(data);
}
