#ifndef _BMP_HEADER_
#define _BMP_HEADER_
//------------------------------------------------------------------------------------------------
#include "types.h"
#include "pgu_lib.h"
//------------------------------------------------------------------------------------------------
// bitmap files are little endian
#if BYTE_ORDER == LITTLE_ENDIAN
	#define le16_to_cpu(x) (x)
	#define le32_to_cpu(x) (x)
#elif BYTE_ORDER == BIG_ENDIAN
	#define le16_to_cpu(x) (((x>>8) & 0x00ff) | ((x<<8) & 0xff00))
	#define le32_to_cpu(x) (((x>>24) & 0x000000ff) | ((x>>8)  & 0x0000ff00) | ((x<<8)  & 0x00ff0000) | ((x<<24) & 0xff000000))
#else
	#error "Oops: unknown byte order"
#endif
//------------------------------------------------------------------------------------------------
// Load
struct bmp_hdr
{
    Uint32 foobar;
    Uint32 size;//== BitMapInfoHeader
    Uint32 width;
    Uint32 height;
    Uint16 planes;
    Uint16 bit_cnt;
    char   compression[4];
    Uint32 image_size;
    Uint32 xpels_meter;
    Uint32 ypels_meter;
    Uint32 num_colors;// used colors
    Uint32 imp_colors;// important colors
    /* may be more for some codecs */
};
//------------------------------------------------------------------------------------------------
struct bmp_cmap
{
    Uint8 blue;
    Uint8 green;
    Uint8 red;
    Uint8 unused;
};
//------------------------------------------------------------------------------------------------
struct bmp_state
{
    struct bmp_hdr  hdr;
    struct bmp_cmap cmap[256];
    FILE *fp;
};
//------------------------------------------------------------------------------------------------
static void *bmp_init(char *filename, Uint32 *width, Uint32 *height);
static void bmp_read(Uint16 *dst, Uint32 line, void *data);
static void bmp_done(void *data);
//Uint32 LoadBmpFile(char *filename);
Uint8 *LoadBmpFile(char *filename);
//void CloseBmpFile(Uint32 hBmp);
void CloseBmpFile(Uint8 *hBmp);
#endif
