#include <stdio.h>
#include <string.h>
#include "skyview.h"
#include "fbutil.h"

/*
 * Check image format (jpg, gif, bmp.....)
 * call img_identify(input_file), return image index
 */
int img_identify(FILE * input_file)
{
    unsigned char header[8];
    char *cmp_hdr;
    int filesz;
    int img_type = IMG_FMT_UNKNOWN;

    fseek(input_file, 0L, SEEK_END);
    filesz = ftell(input_file);
    fseek(input_file, 0L, SEEK_SET);

    if (filesz < 4)
    {
        return IMG_FMT_UNKNOWN;
    }

    memset(header, 0x00, 8);
    /* get 4 bytes header */
    fread(header, 4, 1, input_file);
    cmp_hdr = (char *)header;
    printf("header: %x %x %x %x\n", header[0], header[1], header[2], header[3]);
    /* start to compare header */
    /*
     * Offset  Format          Content         Name
     * =============================================================================
     * 0       string          BM              image/bmp
     * 0       beshort         0xffd8          image/jpeg 
     * 0       string          GIF             image/gif
     * 0       string          \x89PNG                 image/png   
     *
     * # NIFF (Navy Interchange File Format, a modification of TIFF)
     * # [GRR:  this *must* go before TIFF]  
     * 0       string          IIN1            image/x-niff
     *
     * # TIFF and friends
     * #                                       TIFF file, big-endian
     * 0       string          MM              image/tiff
     * #                                       TIFF file, little-endian
     * 0       string          II              image/tiff     
     *
     * # PBMPLUS
     * #                                       PBM file
     * 0       string          P1              image/x-portable-bitmap 7bit
     * #                                       PGM file
     * 0       string          P2              image/x-portable-greymap        7bit
     * #                                       PPM file
     * 0       string          P3              image/x-portable-pixmap 7bit
     * #                                       PBM "rawbits" file          
     * 0       string          P4              image/x-portable-bitmap
     * #                                       PGM "rawbits" file
     * 0       string          P5              image/x-portable-greymap
     * #                                       PPM "rawbits" file
     * 0       string          P6              image/x-portable-pixmap
     *
     */

    if ( strncmp(cmp_hdr, "BM", 2) == 0)
    {
        img_type = IMG_FMT_BMP;
    }
    else if ((header[0] == 0xff) && (header[1] == 0xd8))
    {
        img_type = IMG_FMT_JPEG;
    }
    else if (strncmp(cmp_hdr, "GIF", 3) == 0)
    {
        img_type = IMG_FMT_GIF;
    }
    else if (header[0] == 0x89 && strncmp(cmp_hdr+1, "PNG", 3) == 0)
    {
        img_type = IMG_FMT_PNG;
    }
    else if (strncmp(cmp_hdr, "IIN1", 4) == 0)
    {
        img_type = IMG_FMT_NIFF;
    }
    else if (strncmp(cmp_hdr, "MM", 2) == 0)
    {
        img_type = IMG_FMT_TIFF_BE;
    }
    else if (strncmp(cmp_hdr, "II", 2) == 0)
    {
        img_type = IMG_FMT_TIFF_LE;
    }
    else if (strncmp(cmp_hdr, "P1", 2) == 0)
    {
        img_type = IMG_FMT_PBM_P1;
    }
    else if (strncmp(cmp_hdr, "P2", 2) == 0)
    {
        img_type = IMG_FMT_PGM_P2;
    }
    else if (strncmp(cmp_hdr, "P3", 2) == 0)
    {
        img_type = IMG_FMT_PPM_P3;
    }
    else if (strncmp(cmp_hdr, "P4", 2) == 0)
    {
        img_type = IMG_FMT_PBM_P4;
    }
    else if (strncmp(cmp_hdr, "P5", 2) == 0)
    {
        img_type = IMG_FMT_PGM_P5;
    }
    else if (strncmp(cmp_hdr, "P6", 2) == 0)
    {
        img_type = IMG_FMT_PPM_P6;
    }

    fseek(input_file, 0L, SEEK_SET);
    return img_type;
}
