
#include "parser.h"

#ifndef mmioFOURCC
#define mmioFOURCC( ch0, ch1, ch2, ch3 )				\
		( (Uint32_t)(uint8_t)(ch0) | ( (Uint32_t)(uint8_t)(ch1) << 8 ) |	\
		( (Uint32_t)(uint8_t)(ch2) << 16 ) | ( (Uint32_t)(uint8_t)(ch3) << 24 ) )
#endif

// Raymond 2007/12/21

int file_error = 0;

unsigned int read_nbytes(void *ptr, unsigned int rsize, unsigned int rlen, FILE	*fp)
{	
	unsigned int nRet = fread(ptr, rsize, rlen, fp);
	if( nRet != rlen && !feof(fp) )	// Raymond 2007/12/24
	{
		printf("\n------ file_error = 1 --------\n");
		file_error = 1;
	}

	return nRet;
}

// AVI 

typedef struct
{
    Uint32_t		dwMicroSecPerFrame;		// frame display rate (or 0L)
    Uint32_t		dwMaxBytesPerSec;		// max. transfer rate
    Uint32_t		dwPaddingGranularity;	// pad to multiples of this size; normally 2K.
    Uint32_t		dwFlags;				// the ever-present flags
    Uint32_t		dwTotalFrames;			// # frames in file
    Uint32_t		dwInitialFrames;
    Uint32_t		dwStreams;
    Uint32_t		dwSuggestedBufferSize;   
    Uint32_t		dwWidth;
    Uint32_t		dwHeight;   
    Uint32_t		dwReserved[4];
} MainAVIHeader;

typedef struct 
{
    Uint32_t		fccType;
    Uint32_t		fccHandler;
    Uint32_t		dwFlags;				/* Contains AVITF_* flags */
    uint16_t		wPriority;
    uint16_t		wLanguage;
    Uint32_t		dwInitialFrames;
    Uint32_t		dwScale;	
    Uint32_t		dwRate;					/* dwRate / dwScale == samples/second */
    Uint32_t		dwStart;
    Uint32_t		dwLength;				/* In units above... */
    Uint32_t		dwSuggestedBufferSize;
    Uint32_t		dwQuality;
    Uint32_t		dwSampleSize;
	// rectangle_t		rcFrame;
    uint16_t		left;	
    uint16_t		top;
    uint16_t		right;
    uint16_t		bottom;
} AVIStreamHeader;

typedef struct mpeglayer3waveformat_tag 
{
	WAVEFORMATEX wf;
	unsigned short wID;
	unsigned int   fdwFlags;
	unsigned short nBlockSize;
	unsigned short nFramesPerBlock;
	unsigned short nCodecDelay;
} MPEGLAYER3WAVEFORMAT;
/*
void print_wave_header(WAVEFORMATEX *h)
{
	printf("======= WAVE Format =======\n");
	printf("Format Tag: %d (0x%X)\n",h->wFormatTag,h->wFormatTag);
	printf("Channels: %d\n",h->nChannels);
	printf("Samplerate: %ld\n",h->nSamplesPerSec);
	printf("avg byte/sec: %ld\n",h->nAvgBytesPerSec);
	printf("Block align: %d\n",h->nBlockAlign);
	printf("bits/sample: %d\n",h->wBitsPerSample);
	printf("cbSize: %d\n",h->cbSize);

	if( h->wFormatTag == 0x55 && h->cbSize >= 12 )
	{
		MPEGLAYER3WAVEFORMAT* h2 = (MPEGLAYER3WAVEFORMAT *)h;
		printf("mp3.wID=%d\n",h2->wID);
		printf("mp3.fdwFlags=0x%lX\n",h2->fdwFlags);
		printf("mp3.nBlockSize=%d\n",h2->nBlockSize);
		printf("mp3.nFramesPerBlock=%d\n",h2->nFramesPerBlock);
		printf("mp3.nCodecDelay=%d\n",h2->nCodecDelay);
	}
	else if ( h->cbSize > 0 )
	{
		int i;
		uint8_t* p = ((uint8_t*)h) + sizeof(WAVEFORMATEX);
		printf("Unknown extra header dump: ");
		for (i = 0; i < h->cbSize; i++)
			printf("[%x] ", p[i]);
		printf("\n");
	}
	printf("===========================\n");
}

void print_video_header(BITMAPINFOHEADER *h)
{
	Uint32_t size = sizeof(BITMAPINFOHEADER);
	
	printf("======= VIDEO Format ======\n");
	printf("biSize %d\n", h->biSize);
	printf("biWidth %d\n", h->biWidth);
	printf("biHeight %d\n", h->biHeight);
	printf("biPlanes %d\n", h->biPlanes);
	printf("biBitCount %d\n", h->biBitCount);
	printf("biCompression %d ='%.4s'\n", h->biCompression, (char *)&h->biCompression);
	printf("biSizeImage %d\n", h->biSizeImage);

	if ( h->biSize > size )
	{
		Uint32_t i;
		uint8_t* p = ((uint8_t*)h) + size;
		printf("Unknown extra header dump: ");
		for ( i = 0 ; i < h->biSize - size ; i++ )
			printf("[%x] ", *(p+i));
		printf("\n");
	}
	printf("===========================\n");
}
*/
/* form types, list types, and chunk types */
#define formtypeAVI             mmioFOURCC('A', 'V', 'I', ' ')
#define listtypeAVIHEADER       mmioFOURCC('h', 'd', 'r', 'l')
#define ckidAVIMAINHDR          mmioFOURCC('a', 'v', 'i', 'h')
#define listtypeSTREAMHEADER    mmioFOURCC('s', 't', 'r', 'l')
#define ckidSTREAMHEADER        mmioFOURCC('s', 't', 'r', 'h')
#define ckidSTREAMFORMAT        mmioFOURCC('s', 't', 'r', 'f')
#define listtypeAVIMOVIE        mmioFOURCC('m', 'o', 'v', 'i')

// Stream types for the <fccType> field of the stream header.
#define streamtypeVIDEO         mmioFOURCC('v', 'i', 'd', 's')
#define streamtypeAUDIO         mmioFOURCC('a', 'u', 'd', 's')

/* Chunk id to use for extra chunks for padding. */
#define ckidAVIPADDING          mmioFOURCC('J', 'U', 'N', 'K')

#define MIN(a,b) (((a)<(b))?(a):(b))

int avi_check_file(FILE	*fp)
{
	int id;	
	unsigned int size2;

	read_nbytes(&id, 1, 4, fp);
	if(id == mmioFOURCC('R','I','F','F'))
	{
		int riff_type;			
		
		read_nbytes(&size2, 1, 4, fp);
		read_nbytes((char *)&riff_type, 1, 4, fp);	// "AVI "

		if( riff_type == formtypeAVI )
			return 1;
//			printf("RIFF type : %.4s\n", (char *)&riff_type);
		else	// for DAT = CDXA
			return 0;
//			printf("Unknown RIFF type %.4s\n", (char *)&riff_type);
	}
	else
	{
//		printf("Not RIFF header\n");
		return 0;
	}

	return 1;
}

int AVI_Parser(FILE	*fp, FileInfo *finfo)
{
	int id;	
	int last_fccType = 0;
	unsigned int chunksize, size2;
	unsigned int asize = 0, vsize = 0;

//	try
	{
		//---- AVI header:

		if(!avi_check_file(fp))
			return 0;
		
		while(1)
		{		
			read_nbytes(&id, 1, 4, fp);
			if( feof(fp) || file_error == 1 )
				break;
			
			if( id == mmioFOURCC('L','I','S','T') )
			{
				unsigned len;
				read_nbytes(&len, 1, 4, fp);	// list size
				read_nbytes(&id, 1, 4, fp);	// list type
				//			printf("LIST : %.4s  len = %u\n",(char *) &id,len);
				
				if( len >= 4 ) 
				{
					len -= 4;
				}
				else 
				{
					printf("** empty list?!\n");
				}
				
				if( id == listtypeAVIMOVIE )
				{
					// found MOVI header
					fseek(fp, len, SEEK_CUR);	// skip movi
				}
				continue;
			}
			
			read_nbytes(&size2, 1, 4, fp);
			//		printf("CHUNK %.4s  len=%u\n",(char *) &id,size2);
			
			chunksize = ( size2 + 1 ) & (~1);
			
			switch(id)
			{				
			case ckidAVIMAINHDR:          // read 'avih'
				{
					MainAVIHeader avih;
					int size = MIN(size2,sizeof(avih));
					read_nbytes((char*)&avih,1, size, fp);
					finfo->dwWidth  = avih.dwWidth;
					finfo->dwHeight = avih.dwHeight;
					break;
				}
			case ckidSTREAMHEADER:       // read 'strh'
				{
					AVIStreamHeader h;
					int size = MIN(size2,sizeof(h));			
					read_nbytes((char*) &h,1, size, fp);
					
					if( size2 > sizeof(h) )
						fseek(fp, size2 - sizeof(h), SEEK_CUR);
					
					if( h.fccType == streamtypeVIDEO )
					{
						finfo->FPS = (float)h.dwRate / h.dwScale;
						finfo->vFrame = h.dwLength;
					}
					else if( h.fccType == streamtypeAUDIO )
					{
						asize = h.dwLength;
					}
					
					last_fccType = h.fccType;
					break; 
				}
			case ckidSTREAMFORMAT:      // read 'strf'
				{ 	
					if( last_fccType == streamtypeVIDEO )
					{								
						BITMAP_INFO_HEADER *bih = &(finfo->bih);
						int size = sizeof(BITMAP_INFO_HEADER);
						read_nbytes((char*) bih,1, size, fp);	
						
						finfo->bVideo = 1;
						
						if( chunksize > (unsigned int)size )
							fseek(fp, chunksize - size, SEEK_CUR);
					} 
					else if( last_fccType == streamtypeAUDIO )
					{
						WAVEFORMATEX *wf = &(finfo->wf);
						int size = sizeof(WAVEFORMATEX);
						read_nbytes((char*)wf,1, size, fp);
						
						finfo->bAudio = 1;
						finfo->aBitrate = (wf->nAvgBytesPerSec * 8 + 500) / 1000;	// * 8 / 1000
						
						if( chunksize > (unsigned int)size )
							fseek(fp, chunksize - size, SEEK_CUR);
					}
					else
						fseek(fp, chunksize, SEEK_CUR);
					break;
				}		
				
			case ckidAVIPADDING:	// JUNK
			default:				// others skip
#if 1	// Raymond 2007/10/16
				if( chunksize == 0 || ftell(fp) + chunksize > finfo->FileSize)	
				{
					// printf("CHUNK size error : %d\n", chunksize);
					if ( finfo->bAudio || finfo->bVideo )
						return 1;
					else
						return 0;
				}
#else
				// check chunksize
				if (chunksize == 0)	//Barry 2007/07/03
					return 0;
				if( ftell(fp) + chunksize > finfo->FileSize)	
				{
					// printf("CHUNK size error : %d\n", chunksize);
					if ( finfo->bAudio || finfo->bVideo )
						return 1;
					else
						return 0;
				}
#endif
				fseek(fp, chunksize, SEEK_CUR);
				chunksize = 0;
				break;
			}
		}
		
		// estimate video bitrate
		if( asize > finfo->FileSize )
			asize = 0;
		
		vsize = finfo->FileSize - asize - 8 * finfo->vFrame;
		
		finfo->vBitrate = 0.008f * ( (float)vsize / (float)finfo->vFrame ) * finfo->FPS;

		// Raymond 2007/10/24
		if( finfo->FPS != 0 )
			finfo->FileDuration = (unsigned long)(finfo->vFrame / finfo->FPS);

		return 1;
	}
//	catch (...) 
//	{
//		return 0;
//	}
}

// ASF

// ASF Object Header 
typedef struct 
{
	uint8_t guid[16];
	uint64_t size;
} ASF_obj_header_t;

// ASF Header 
typedef struct
{
	ASF_obj_header_t objh;
	Uint32_t cno;		// number of subchunks
	uint8_t v1;			// unknown (0x01)
	uint8_t v2;			// unknown (0x02)
} ASF_header_t;

// ASF File Header 
typedef struct 
{
	uint8_t stream_id[16];		// stream GUID
	uint64_t file_size;
	uint64_t creation_time;		// File creation time FILETIME 8
	uint64_t num_packets;		// Number of packets UINT64 8
	uint64_t play_duration;		// Timestamp of the end position UINT64 8
	uint64_t send_duration;		// Duration of the playback UINT64 8
	uint64_t preroll;			// Time to bufferize before playing UINT32 4
	Uint32_t flags;				// Unknown, maybe flags ( usually contains 2 ) UINT32 4
	Uint32_t min_packet_size;	// Min size of the packet, in bytes UINT32 4
	Uint32_t max_packet_size;	// Max size of the packet  UINT32 4
	Uint32_t max_bitrate;		// Maximum bitrate of the media (sum of all the stream)
} ASF_file_header_t;

// ASF Stream Header
typedef struct 
{
	uint8_t type[16];			// Stream type (audio/video) GUID 16
	uint8_t concealment[16];	// Audio error concealment type GUID 16
	uint64_t unk1;				// Unknown, maybe reserved ( usually contains 0 ) UINT64 8
	Uint32_t type_size;			// Total size of type-specific data UINT32 4
	Uint32_t stream_size;		// Size of stream-specific data UINT32 4
	uint16_t stream_no;			// Stream number UINT16 2
	Uint32_t unk2;				// Unknown UINT32 4
} ASF_stream_header_t;

unsigned char asfhdrguid[16] = {0x30,0x26,0xB2,0x75,0x8E,0x66,0xCF,0x11,0xA6,0xD9,0x00,0xAA,0x00,0x62,0xCE,0x6C};

unsigned char _asf_stream_header_guid[16] = 
	{0x91, 0x07, 0xdc, 0xb7,  0xb7, 0xa9, 0xcf, 0x11, 0x8e, 0xe6, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65};
unsigned char asf_file_header_guid[16] = {0xa1, 0xdc, 0xab, 0x8c,
  0x47, 0xa9, 0xcf, 0x11, 0x8e, 0xe4, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65};

// Raymond 2007/10/26
unsigned char asf_ext_stream_header[16] = {0xCB, 0xA5, 0xE6, 0x14,
  0x72, 0xC6, 0x32, 0x43, 0x83, 0x99, 0xA9, 0x69, 0x52, 0x06, 0x5B, 0x5A};

//#ifdef ARCH_X86
//#define	ASF_LOAD_GUID_PREFIX(guid)	(*(Uint32_t *)(guid))
//#else
#define	ASF_LOAD_GUID_PREFIX(guid)	((guid)[3] << 24 | (guid)[2] << 16 | (guid)[1] << 8 | (guid)[0])
//#endif

#define ASF_GUID_PREFIX_audio_stream				0xF8699E40
#define ASF_GUID_PREFIX_video_stream				0xBC19EFC0
#define ASF_GUID_PREFIX_audio_conceal_none			0x49f1a440
#define ASF_GUID_PREFIX_audio_conceal_interleave	0xbfc3cd50

ASF_header_t asfh;

int asf_check_file(FILE	*fp)
{
	// asf_check_header
	read_nbytes((char*) &asfh,1, 30, fp); // header obj : sizeof(asfh)

	if( memcmp(asfhdrguid, asfh.objh.guid,16) )
	{
//		printf("ASF_check : not ASF guid!\n");
		return 0; // not ASF guid
	}
	if( asfh.cno > 256 )
	{
//		printf("ASF_check : invalid subchunks_no %d\n",(int)asfh.cno);
		return 0; // invalid header???
	}

	return 1;
}

int find_asf_guid(unsigned char *buf, const unsigned char *guid, int cur_pos, int buf_len)
{
	int i;
	for ( i = cur_pos ; i < buf_len - 19; i++ ) 
	{
		if (memcmp(&buf[i], guid, 16) == 0)
			return i + 16 + 8; // point after guid + length
	}
	return -1;
}

// Raymond 2007/10/26
int get_ext_stream_properties(unsigned char *buf, int buf_len, int stream_num, float* fps)
{
	// this function currently only gets the average frame time if available

	int pos=0;
	unsigned char *buffer = &buf[0];
	Uint32_t avg_ft;
	
	while ((pos = find_asf_guid(buf, asf_ext_stream_header, pos, buf_len)) >= 0) 
	{
		int this_stream_num;
		buffer = &buf[pos];
		
		buffer +=48;
		this_stream_num = buffer[0] + (buffer[1]<<8);
		
		//	printf("buffer = %d %d\n", buffer[0], buffer[1]);
		//	printf("this_stream_num = %d\n", this_stream_num);
		
		if (this_stream_num == stream_num) 
		{
			buffer += 4; //skip stream-language-id-index
			avg_ft = buffer[0] + (buffer[1]<<8) + (buffer[2]<<16) + (buffer[3]<<24);	// provided in 100ns units
			*fps = 10000000.0f / avg_ft;
			
			return 1;
		} 
	}
	return 0;
}

int ASF_Parser(FILE	*fp, FileInfo *finfo)
{
	int hdr_len;
	int pos = 0;
	unsigned char *hdr = NULL;
	unsigned char *buffer = NULL;

//	try
	{
		if(!asf_check_file(fp))
			return 0;
		
		fseek(fp, 0, SEEK_SET);		// reset to the begin
		
		hdr_len = (int)(asfh.objh.size); 
		if (hdr_len > 64 * 1024) 
		{
			//		printf("ASF_check : header size bigger than 64 kB (%d)!\n", hdr_len);
			return 0;
		}
		
		hdr = (unsigned char *)malloc(hdr_len);
		if ( hdr == NULL )
		{
			//		printf("ASF_check : hdr NULL\n");
			return 0;
		}
				
		read_nbytes(hdr, 1, hdr_len, fp); // header obj
		
		// find stream headers
		pos = 30;	// sizeof(asfh)
		
		while ( ( pos = find_asf_guid(hdr, _asf_stream_header_guid, pos, hdr_len) ) >= 0 )
		{
			ASF_stream_header_t *streamh = (ASF_stream_header_t *)&hdr[pos];
			Uint32_t type_size, stream_num;

			// Get type_size
			buffer = &hdr[pos+40];
			type_size = (buffer[3] << 24) + (buffer[2] << 16) + (buffer[1] << 8) + buffer[0];

			// Get stream video number
			buffer = &hdr[pos+48];
			stream_num = (buffer[1] << 8) + buffer[0];
			
			pos += 54; // sizeof(ASF_stream_header_t)
			if (pos > hdr_len) 
				goto len_err_out;
			
			// type-specific data:
			buffer = &hdr[pos];
			pos += type_size;
			
			if (pos > hdr_len) 
				goto len_err_out;
			
			switch(ASF_LOAD_GUID_PREFIX(streamh->type))
			{
			case ASF_GUID_PREFIX_audio_stream: 
				{
//			int len = ( streamh->type_size < sizeof(WAVEFORMATEX) ) ? sizeof(WAVEFORMATEX) : streamh->type_size;
					
					int len = sizeof(WAVEFORMATEX);
					WAVEFORMATEX *wf = &(finfo->wf);
					memcpy((char *)wf, buffer, len);
					finfo->bAudio = 1;

					finfo->aBitrate = (wf->nAvgBytesPerSec * 8 + 500) / 1000;	// * 8 / 1000
					break;
				}
			case ASF_GUID_PREFIX_video_stream: 
				{
					//			int len = streamh->type_size - (4+4+1+2);
					//			len ( len < sizeof(BITMAPINFOHEADER) ) ? sizeof(BITMAPINFOHEADER) : len;
					
					int len = sizeof(BITMAP_INFO_HEADER);
					BITMAP_INFO_HEADER *bih = &(finfo->bih);
					memcpy((char *)bih, &buffer[4+4+1+2], len);
					
					finfo->bVideo = 1;
					
					finfo->dwWidth  = bih->biWidth;
					finfo->dwHeight = bih->biHeight;
					
					// Raymond 2007/10/26
					get_ext_stream_properties(hdr, hdr_len, stream_num, &finfo->FPS);
					
					break;
				}
			default:
				break;
			}
		}
		
		// find file header
		pos = find_asf_guid(hdr, asf_file_header_guid, 0, hdr_len);
		if ( pos >= 0 ) 
		{
			Uint32_t send_duration1, send_duration2, max_bitrate;
			buffer = &hdr[pos+48];			
			
			send_duration1 = (buffer[3] << 24) + (buffer[2] << 16) + (buffer[1] << 8) + buffer[0];
			buffer += 4;
			send_duration2 = (buffer[3] << 24) + (buffer[2] << 16) + (buffer[1] << 8) + buffer[0];
			
			finfo->FileDuration = (int)(send_duration1/10000000);

			if( send_duration2 > 0 )
				finfo->FileDuration += (int)(send_duration2 * 429);
				
			// Raymond 2007/10/26
			buffer = &hdr[pos+76];
			max_bitrate = (buffer[3] << 24) + (buffer[2] << 16) + (buffer[1] << 8) + buffer[0];
			finfo->vBitrate = (float)(max_bitrate / 1000);	
		}

		free(hdr);
		return 1;
		
len_err_out:
		free(hdr);
		return 0;
	}
//	catch (...) 
//	{
//		free(hdr);
//		return 0;
//	}
}

// MPG

#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif

#define MAX_PS_PACKETSIZE (224*1024)

#define UNKNOWN         0
#define VIDEO_MPEG1     0x10000001
#define VIDEO_MPEG2     0x10000002
#define VIDEO_MPEG4     0x10000004
#define VIDEO_H264      0x10000005
#define AUDIO_MP2       0x50
#define AUDIO_A52       0x2000
#define AUDIO_LPCM_BE   0x10001
#define AUDIO_AAC       mmioFOURCC('M', 'P', '4', 'A')

int bElementary = 0;	// Raymond 2007/11/13

inline static int read_char(FILE *fp)
{
//	return getc(fp);
	int c = 0;
	read_nbytes(&c, 1, 1, fp);
	return c;
}

inline static unsigned int read_word(FILE *fp)
{
	int x,y;
	x = read_char(fp);
	y = read_char(fp);
	return (x<<8)|y;
}

inline static unsigned int read_dword(FILE *fp)
{
	unsigned int y;
	y = read_char(fp);
	y = (y<<8)|read_char(fp);
	y = (y<<8)|read_char(fp);
	y = (y<<8)|read_char(fp);
	return y;
}

inline static unsigned int read_dword_le(FILE *fp)
{
	unsigned int y;
	y =  read_char(fp);
	y |= read_char(fp)<<8;
	y |= read_char(fp)<<16;
	y |= read_char(fp)<<24;
	return y;
}

inline static uint64_t read_qword(FILE *fp)
{
	uint64_t y;
	y = read_char(fp);
	y = (y<<8)|read_char(fp);
	y = (y<<8)|read_char(fp);
	y = (y<<8)|read_char(fp);
	y = (y<<8)|read_char(fp);
	y = (y<<8)|read_char(fp);
	y = (y<<8)|read_char(fp);
	y = (y<<8)|read_char(fp);
	return y;
}

static int frameratecode2framerate[16] =
{
	0,
	// Official mpeg1/2 framerates: (1-8)
	24000*10000/1001, 24*10000,25*10000,
	30000*10000/1001, 30*10000,50*10000,
	60000*10000/1001, 60*10000,
	// Xing's 15fps: (9)
	15*10000,
	// libmpeg3's "Unofficial economy rates": (10-13)
	5*10000,10*10000,12*10000,15*10000,
	// some invalid ones: (14-15)
	0,0
};

// Raymond 2007/10/26
static unsigned int read_mpeg_timestamp(FILE *fp, int c)
{
	int d,e;
	unsigned int pts;
	d = read_word(fp);
	e = read_word(fp);
	if( ((c&1)!=1) || ((d&1)!=1) || ((e&1)!=1) )
	{
//		++mpeg_pts_error;
		return 0; // invalid pts
	}
	pts = ( ( ( c >> 1 ) & 7 ) << 30 ) | ( ( d >> 1 ) << 15 ) | ( e >> 1 ) ;
	
	return pts;
}

static int parse_psm(FILE *fp, int len, FileInfo *finfo) 
{
//	try
	{
		unsigned char c, id, type;
		unsigned int plen, prog_len, es_map_len;
		
		if(! len)
			return 0;
		
		c = read_char(fp);
		if(! (c & 0x80)) 
		{
			//		stream_skip(demux->stream, len - 1);  //not yet valid, discard
			fseek(fp, len - 1, SEEK_CUR);
			return 0;
		}
		
		fseek(fp, 1, SEEK_CUR);
		
		prog_len = read_word(fp);			// length of program descriptors
		//	stream_skip(demux->stream, prog_len);				// .. that we ignore
		fseek(fp, prog_len, SEEK_CUR);
		
		es_map_len = read_word(fp);		// length of elementary streams map
		es_map_len = min(es_map_len, len - prog_len - 8);	// sanity check
		
		while(es_map_len > 0) 
		{
			type = read_char(fp);
			id = read_char(fp);
			if( id >= 0xB0 && id <= 0xEF ) 
			{
				int idoffset = id - 0xB0;
				unsigned int v_format = 0, a_format = 0;
				switch(type) 
				{
				case 0x1:
					finfo->VideoType = VIDEO_MPEG1;
					finfo->bih.biCompression = mmioFOURCC('M','P','G','1');
					break;
				case 0x2:
					finfo->VideoType = VIDEO_MPEG2;
					finfo->bih.biCompression = mmioFOURCC('M','P','G','2');
					break;
				case 0x3:
				case 0x4:
					finfo->wf.wFormatTag = AUDIO_MP2;
					break;
				case 0x0f:
				case 0x11:
					finfo->AudioType = AUDIO_AAC;
					break;
				case 0x10:
					finfo->VideoType = VIDEO_MPEG4;
					break;
				case 0x1b:
					finfo->VideoType = VIDEO_H264;
					break;
				case 0x81:
					finfo->wf.wFormatTag = AUDIO_A52;
					break;
				}
				
				// mp_dbg(MSGT_DEMUX,MSGL_V, "PSM ES, id=0x%x, type=%x, stype: %x\n", id, type, priv->es_map[idoffset]);
			}
			plen = read_word(fp);		//length of elementary stream descriptors
			plen = min(plen, es_map_len);			//sanity check
			fseek(fp, plen, SEEK_CUR);			//skip descriptors for now
			es_map_len -= 4 + plen;
		}

		//	stream_skip(demux->stream, 4);			//skip crc32
		fseek(fp, 4, SEEK_CUR);					//skip crc32
		return 1;
	}
//	catch (...) 
//	{
//		return 0;
//	}
}

static int mpg_read_packet( FILE *fp, int id, int *plen, FileInfo *finfo )
{
//	try
	{
		int len;
		unsigned char c=0;
		unsigned int pts = 0;
		unsigned int dts = 0;
		
		if( id < 0x1BC || id >= 0x1F0 )		return -1;
		if( id == 0x1BE )					return -1; // padding stream
		if( id == 0x1BF )					return -1; // private2
		
		len = read_word(fp);
		//	mp_dbg(MSGT_DEMUX,MSGL_DBG3,"PACKET len=%d",len);
		//  if(len==62480){ demux->synced=0;return -1;} 
		if( len == 0 || len > MAX_PS_PACKETSIZE )
		{
			// mp_dbg(MSGT_DEMUX,MSGL_DBG2,"Invalid PS packet len: %d\n",len);
			return -2;  // invalid packet !!!!!!
		}
		
		//	mpeg_pts_error=0;
		
		if( id == 0x1BC ) 
		{
			parse_psm(fp, len, finfo);
			return 0;
		}
		
		while( len > 0 )
		{   // Skip stuFFing bytes
			c = read_char(fp);
			--len;
			if( c != 0xFF )
				break;
		}
#if 1	// Raymond 2007/10/26
		if((c>>6)==1)  // Read (skip) STD scale & size value
		{	
			//    printf("  STD_scale=%d",(c>>5)&1);
			read_char(fp);
			len -= 2;
			//    printf("  STD_size=%d",d);
			c = read_char(fp); 
		}
		// Read System-1 stream timestamps:
		if((c>>4)==2)
		{
			pts = read_mpeg_timestamp(fp, c);
			len-=4;
		} 
		else if((c>>4)==3)
		{
			pts = read_mpeg_timestamp(fp, c);
			c = read_char(fp); 
			if((c>>4)!=1) pts=0; //printf("{ERROR4}");
			dts = read_mpeg_timestamp(fp, c);
			len-=4+1+4;
		} 
		else
#endif
		if( ( c >> 6 ) == 2 )
		{
			// System-2 (.VOB) stream:
			int pts_flags;
			int hdrlen;
			
			c = read_char(fp); 
			pts_flags = c >> 6;
			c = read_char(fp); 
			hdrlen=c;
			len -= 2;
			
			if( hdrlen > len )
			{ 
				return -1;
			}
			if( pts_flags == 2 && hdrlen >= 5 )
			{
//				fseek(fp, 5, SEEK_CUR);
				c = read_char(fp); 
				pts = read_mpeg_timestamp(fp, c);
				len -= 5;
				hdrlen -= 5;
			}
			else if( pts_flags == 3 && hdrlen >= 10 )
			{
//				fseek(fp, 10, SEEK_CUR);
				c = read_char(fp); 
				pts = read_mpeg_timestamp(fp, c);
				c = read_char(fp); 
				dts = read_mpeg_timestamp(fp, c);
				len -= 10; 
				hdrlen -= 10;
			}
			
			len -= hdrlen;
			
			if( hdrlen > 0 )  
				fseek(fp, hdrlen, SEEK_CUR);	// skip header bytes
			
			//============== DVD Audio sub-stream ======================
			if( id == 0x1BD )
			{
				int aid = read_char(fp);
				--len;
				if( len < 3 ) 
					return -1; // invalid audio packet
				
				// AID:
				// 0x20..0x3F  subtitle
				// 0x80..0x9F  AC3 audio
				// 0xA0..0xBF  PCM audio
				
				finfo->bAudio = 1;
								
				if( aid >= 0x80 && aid <= 0x9F )
				{
					if((aid & 0xF8) == 0x88)	
						finfo->wf.wFormatTag = 0x2001;		// dts
					else	
						finfo->wf.wFormatTag = 0x2000;		// ac3
				}
				else if( ( aid & 0xC0 ) == 0x80 || (aid & 0xE0) == 0x00 ) 
				{					
					// aid = 128 + ( aid & 0x7F );
					// aid = 0x80..0xBF
					if( ( aid & 0xE0 ) == 0xA0 && len >= 3 )
						finfo->wf.wFormatTag = 0x1001;			// dvd pcm				
				} 				

			} //if(id==0x1BD)			
		}
#if 0
		else 
		{
			if( c != 0x0f )
			{
				return -1;  // invalid packet !!!!!!
			}
		}
#endif				
		if( len <= 0 || len > MAX_PS_PACKETSIZE )
		{
			// Invalid PS data len: %d\n"
			return -1;  // invalid packet !!!!!!
		}
		
		if( id >= 0x1C0 && id <= 0x1DF )
		{
			// mpeg audio
			int aid = id - 0x1C0;
			unsigned int format = 0;
			
			finfo->bAudio = 1;
			
			if( finfo->wf.wFormatTag == 0 )
			{				
				switch(aid & 0xE0)  // 1110 0000 b  (high 3 bit: type  low 5: id)
				{
				case 0x00:	format = 0x50;				// mpeg
					break;				
				case 0xA0:	format = 0x10001;			// dvd pcm
					break;
				case 0x80:	
					if((aid & 0xF8) == 0x88)	
						format = 0x2001;		// dts
					else	
						format = 0x2000;		// ac3
					break;
				}
				
				finfo->wf.wFormatTag = format;
			}
		} 
		else if( id >= 0x1E0 && id <= 0x1EF )
		{
			// mpeg video
//			int aid = id - 0x1E0;
			
			finfo->bVideo = 1;
			
			// Raymond 2007/10/26
			if( pts != 0 )
				finfo->FileDuration = pts / 90000;
		}
		
		*plen = len;
		return 0;
	}
//	catch (...) 
//	{
//		return -1;
//	}
}

// Raymond 2007/10/26
int GetMPGDuration(FILE	*fp, FileInfo *finfo)
{
	unsigned int head = 0;
	int pes = 1;
	int skipped = 0;
	int ret = 0;	
	int packet_len = 0;
	int seekpos = 0;

	if( finfo->FileSize > 600000 )
//		seekpos = finfo->FileSize - 500000;
		seekpos = finfo->FileSize - 250000;	// Raymond 2007/12/11
	else
		seekpos = finfo->FileSize - 50000;
	
	fseek(fp, seekpos, SEEK_SET);		// reset to the begin
	
	// System stream
	do
	{			
		head = read_dword(fp);
		
		if( ( head & 0xFFFFFF00 ) != 0x100 )
		{
			// sync...
			while( 1 )
			{
				int c = read_char(fp);
				if( feof(fp) || file_error == 1 ) 
					break;
				head <<= 8;
				if( head != 0x100 )
				{
					head |= c;
					++skipped; 
					continue;
				}
				head |= c;
				break;
			}
		}
		
		if( feof(fp) || skipped > 10000 || file_error == 1 ) // Raymond 2007/12/11
			break;
				
		packet_len = 0;
		ret = mpg_read_packet(fp, head, &packet_len, finfo);
		if(!ret)
		{
			if( finfo->bih.biCompression != 0 )
				fseek(fp, packet_len, SEEK_CUR);
		}
		
	} while( !feof(fp) && file_error == 0 );

	return 1;
}

#define mmio_FOURCC( ch0, ch1, ch2, ch3 )				\
		( (Uint32_t)(uint8_t)(ch3) | ( (Uint32_t)(uint8_t)(ch2) << 8 ) |	\
		( (Uint32_t)(uint8_t)(ch1) << 16 ) | ( (Uint32_t)(uint8_t)(ch0) << 24 ) )

#define formtypeCDXA             mmio_FOURCC('C', 'D', 'X', 'A')

int mpg_check_file(FILE	*fp)
{
	unsigned int size2;

	unsigned int head = read_dword(fp);

	if(head == mmio_FOURCC('R','I','F','F'))
	{
		unsigned int riff_type;			
		
		read_nbytes(&size2, 1, 4, fp);
		riff_type = read_dword(fp);

		if( riff_type == formtypeCDXA )			// "CDXA"	
			return 1;
//			printf("RIFF type : %.4s\n", (char *)&riff_type);
		else	// for DAT = CDXA
			return 0;
//			printf("Unknown RIFF type %.4s\n", (char *)&riff_type);
	}
	// Raymond 2007/08/31
	else if ( head == mmio_FOURCC('C','o','n','t') )	// Content-type
	{
		fseek(fp, 50, SEEK_SET);
		head = read_dword(fp);
		if( head == 0x1BA )
			return 1;
		else 
			return 0;
	}
	//
	else
	{
		// Raymond 2007/09/17
		if( ( head & 0xFFFFFF00 ) == 0x100 )
			return 1;
		else
		{
			int skipped = 0, c = 0;

			while( skipped < 200 )
			{
				if( head == 0x1BA )
				{
					return 1;
				}
				else
				{
					c = read_char(fp);
					
					head <<= 8;
					head |= c;
					++skipped; 
				}
			}

			return 0;
		}
	}

	return 1;
}

int MPG_Parser(FILE	*fp, FileInfo *finfo)
{
	unsigned int head = 0;
	int pes = 1;
	int skipped = 0;
//	int max_packs = 128; // 256kbyte
//	int max_packs = 256; // 512kbyte
	int max_packs = 512; // 1024 kbyte	// Raymond 2007/09/17
	int ret = 0;	
	int eof = 0;
	int synced = 0;
	int packet_len = 0;
	// Raymond 2007/06/15 - fix bug if file too small
	int packet_num = finfo->FileSize / 2048;
	if( packet_num < max_packs )
		max_packs = packet_num;
		
	bElementary = 0;	// Raymond 2007/11/13
		
//	try
	{
		// try to pre-detect PES:
		head = read_dword(fp);
		
		if( head == 0x1E0 || head == 0x1C0 )
		{
			head = read_word(fp);
			if( head > 1 && head <= 2048 )
				pes = 0; // synced = 3; // PES...
		}
		
		if( !pes ) 
			synced = 3; // hack!
				
		if( head == mmio_FOURCC('R','I','F','F') )	// for DAT = CDXA
		{
			unsigned int riff_type, size2;			
			
			read_nbytes(&size2, 1, 4, fp);
			riff_type = read_dword(fp);// Raymond 2007/12/11	
			
			if( riff_type == formtypeCDXA )		// "CDXA"	
				fseek(fp, 60000, SEEK_SET);
		}
		// Raymond 2007/08/31
		else if ( head == mmio_FOURCC('C','o','n','t') )	// Content-type
		{
			fseek(fp, 50, SEEK_SET);
		}
		else
			fseek(fp, 0, SEEK_SET);		// reset to the begin
				
		// System stream
		do
		{
//			skipped = 0;
			
			head = read_dword(fp);
			
			if( ( head & 0xFFFFFF00 ) != 0x100 )
			{
				// sync...
				while( 1 )
				{
					int c = read_char(fp);
					if( feof(fp) || file_error == 1 ) 
						break;
					
					head <<= 8;
					if( head != 0x100 )
					{
						head |= c;
						++skipped; 
						continue;
					}
					head |= c;
					break;
				}
			}
			
			if( feof(fp) || file_error == 1 ) 
				break;
			
			// VIDEO_MPEG12
			if( head == 0x1B3 )		// sequence_header	
			{
				int temp, width, height;
				int frame_rate_code;
				
				temp = read_dword(fp);
				height = temp >> 8;
				
				width = (height >> 12);
				height = (height & 0xfff);
				
				finfo->bih.biWidth  = width;
				finfo->bih.biHeight = height;
				
				frame_rate_code = temp & 15;
				finfo->FPS = (float)frameratecode2framerate[frame_rate_code]/10000;
				
				temp = read_dword(fp);
				finfo->vBitrate = (float)( (temp >> 14) * 400 / 1000 );	// 18 bit
				
				finfo->bih.biCompression = mmioFOURCC('M','P','G','1');
				
				if( skipped == 0 || synced == 0 )
				{
					// MPEG Elementary Video
					finfo->bVideo = 1;
					bElementary = 1;
				}
			}
			
			if( head == 0x1B5 )		// sequence_extension	
			{
				finfo->bih.biCompression = mmioFOURCC('M','P','G','2');
				if( bElementary == 1 )
					return 1;
			}
			
			// VIDEO_H264
			
			if( ( head  & ~0x60 ) == 0x107 && head != 0x107 ) 
			{
				finfo->bih.biCompression = mmioFOURCC('H','2','6','4');
				finfo->bVideo = 1;
				
				// width / height ?
			}
			
			// Raymond 2007/01/30
			if( bElementary == 1 && skipped > 4096 )
				return 1;	// if not find 0xB5

			if( synced == 0 )
			{
				if( head == 0x1BA ) 
					synced = 1; //else
				if( head == 0x1BD || ( head >= 0x1C0 && head <= 0x1EF ) ) 
					synced = 3; // PES?
			}
			else
			{
				if( synced == 1 )
				{
					if( head == 0x1BB || head == 0x1BD || ( head >= 0x1C0 && head <= 0x1EF ) )
					{
						synced = 2;
					}
					else 
						synced = 0;
				} 
				
				if( synced >= 2 )
				{
					packet_len = 0;
					ret = mpg_read_packet(fp, head, &packet_len, finfo);
					if(!ret)
					{
						if( finfo->bih.biCompression != 0 )
							fseek(fp, packet_len, SEEK_CUR);

						if( --max_packs == 0 )
							eof = 1;
						
						if( max_packs < (skipped >> 11) )	// Raymond 2007/09/21
							eof = 1;
						
//						if( finfo->bih.biCompression != 0 && finfo->bAudio == 1 && max_packs < 230 )
						if( finfo->bih.biCompression != 0 && finfo->bAudio == 1 && max_packs < 480 )	// Raymond 2007/09/17
							eof = 1;					
					}
					
					if( synced == 3 ) 
						synced = (ret == 1 ) ? 2 : 0; // PES detect
				} 
			}
			
		} while( !feof(fp) && !eof );
	}
//	catch (...) 
//	{
//		return 0;
//	}
	
	return 1;
}

// MOV - MP4 , 3GP , 3G2

#define MOV_FOURCC(a,b,c,d) ( ( a << 24 ) | ( b << 16 ) | ( c << 8 ) | (d) )

int mov_check_file(FILE	*fp)
{
    int flags = 0;
    int no = 0;

//	try
	{
		while(1)
		{
			int skipped = 8;
			//		unsigned int len = read_dword(fp);
			uint64_t len  = read_dword(fp);
			unsigned int id  = read_dword(fp);
			
			if( feof(fp) || file_error == 1 ) 
				break;	// EOF
			
			if (len == 1) /* real size is 64bits - cjb */
			{
#if 0	// #ifndef _LARGEFILE_SOURCE
				if (read_dword(fp) != 0)
				{
					// mp_msg(MSGT_DEMUX, MSGL_WARN, "64bit file, but you've compiled MPlayer without LARGEFILE support!\n");
				}
				len = read_dword(fp);
#else
				len = read_qword(fp);
#endif
				skipped += 8;
			}
			else if( len < 8 ) 
				break; // invalid chunk
			
			//	printf("ID is 0x%x, len is %d\n",id,len);
			
			switch(id)
			{
			case MOV_FOURCC('f','t','y','p'):		
				break;
				
			case MOV_FOURCC('m','o','o','v'):			
				flags |= 1;
				break;
			case MOV_FOURCC('w','i','d','e'):	// 'WIDE' chunk
				if( flags & 2 ) 
					break;
			case MOV_FOURCC('m','d','a','t'):			
				flags |= 2;
				if( flags == 3 )
					return 1;	// if we're over the headers, then we can stop parsing here!			
				break;
			case MOV_FOURCC('f','r','e','e'):
			case MOV_FOURCC('s','k','i','p'):
			case MOV_FOURCC('j','u','n','k'):	// free space
				break;
			case MOV_FOURCC('p','n','o','t'):
			case MOV_FOURCC('P','I','C','T'):
				/* dunno what, but we shoudl ignore it */
				break;
			default:
				if( no == 0 )				
					return 0;	// first chunk is bad!
			}
			
			if( fseek(fp, (unsigned int)len - skipped, SEEK_CUR) )
				break;
			++no;
		}
		
		if( flags == 3 )
		{	
			return 1;	// got 'mdat' and 'moov'
		}

		return 0;
	}
//	catch (...) 
//	{
//		return 0;
//	}    
}

enum TRACK_TYPE_3GP
{
	UNKNOWN_TRACK = 0,		
	VIDEO_TRACK,		// "vide"
	AUDIO_TRACK,		// "soun"
	ODSM_TRACK,			// "odsm"
	SDSM_TRACK,			// "sdsm"
	HINT_TRACK,			// "hint"
};

enum CODEC_TYPE_3GP
{
	NO_CODEC = 0,		
	MPEG4_VIDEO,		// "mp4v"	- not support , patent issue !
	MPEG4_AUDIO,		// "mp4a"	- AAC, MP3, if 3g2, it might be QCELP !
	H263_VIDEO,			// "s263"
	AMR_SPEECH,			// "samr"
	UNKNOWN_CODEC,		// other not support codec ..
};

typedef struct  
{
	int		AtomSize;	        // [0-3]
	char	AtomType[4];        // [4-7]
	
} ATOM_3GP;

unsigned long BElong_32(unsigned long long32)
{
	unsigned long result,temp;

	temp = long32;
	result = (temp & 0xFF) << 24 ;
	temp = long32;
	result += (temp & 0xFF00) << 8;
	temp = long32;
	result += (temp & 0xFF0000) >> 8;
	temp = long32;
	result += (temp & 0xFF000000) >> 24;

	return result;
}

/* one byte tag identifiers */
#define MP4ODescrTag				0x01 
#define MP4IODescrTag				0x02 
#define MP4ESDescrTag				0x03 
#define MP4DecConfigDescrTag		0x04 
#define MP4DecSpecificDescrTag		0x05 
#define MP4SLConfigDescrTag			0x06 
#define MP4ContentIdDescrTag		0x07 
#define MP4SupplContentIdDescrTag	0x08 
#define MP4IPIPtrDescrTag			0x09 
#define MP4IPMPPtrDescrTag			0x0A 
#define MP4IPMPDescrTag				0x0B 
#define MP4RegistrationDescrTag		0x0D 
#define MP4ESIDIncDescrTag			0x0E 
#define MP4ESIDRefDescrTag			0x0F 
#define MP4FileIODescrTag			0x10 
#define MP4FileODescrTag			0x11 
#define MP4ExtProfileLevelDescrTag	0x13 
#define MP4ExtDescrTagsStart		0x80 
#define MP4ExtDescrTagsEnd			0xFE 

/* object type identifiers in the ESDS */
/* See http://gpac.sourceforge.net/tutorial/mediatypes.htm */
/* BIFS stream version 1 */
#define MP4OTI_MPEG4Systems1                                0x01
/* BIFS stream version 2 */
#define MP4OTI_MPEG4Systems2                                0x02
/* MPEG-4 visual stream */
#define MP4OTI_MPEG4Visual                                  0x20
/* MPEG-4 audio stream */
#define MP4OTI_MPEG4Audio                                   0x40
/* MPEG-2 visual streams with various profiles */
#define MP4OTI_MPEG2VisualSimple                            0x60
#define MP4OTI_MPEG2VisualMain                              0x61
#define MP4OTI_MPEG2VisualSNR                               0x62
#define MP4OTI_MPEG2VisualSpatial                           0x63
#define MP4OTI_MPEG2VisualHigh                              0x64
#define MP4OTI_MPEG2Visual422                               0x65
/* MPEG-2 audio stream part 7 ("AAC") with various profiles */
#define MP4OTI_MPEG2AudioMain                               0x66
#define MP4OTI_MPEG2AudioLowComplexity                      0x67
#define MP4OTI_MPEG2AudioScaleableSamplingRate              0x68
/* MPEG-2 audio part 3 ("MP3") */
#define MP4OTI_MPEG2AudioPart3                              0x69
/* MPEG-1 visual visual stream */
#define MP4OTI_MPEG1Visual                                  0x6A
/* MPEG-1 audio stream part 3 ("MP3") */
#define MP4OTI_MPEG1Audio                                   0x6B
/* JPEG visual stream */
#define MP4OTI_JPEG                                         0x6C

int mp4_read_descr_len(FILE *fp) 
{
	uint8_t b;
	uint8_t numBytes = 0;
	Uint32_t length = 0;
	
	do {
		b = read_char(fp);
		numBytes++;
		length = (length << 7) | (b & 0x7F);
	} while ((b & 0x80) && numBytes < 4);
	
	//printf("MP4 read desc len: %d\n", length);
	return length;
}

/* parse the data part of MP4 esds atoms */
//int mp4_parse_esds(TRACK_TYPE_3GP	TrackType, FILE *fp, FileInfo *finfo) 
int mp4_parse_esds(int TrackType, FILE *fp, FileInfo *finfo) 
{
//	try
//	{
		uint8_t len;
		
		uint8_t  version;
		
		/* 0x03 ESDescrTag */
		uint16_t ESId;
		uint8_t  streamPriority;
		
		/* 0x04 DecConfigDescrTag */
		uint8_t  objectTypeId;
		uint8_t  streamType;
		
		Uint32_t maxBitrate;
		Uint32_t avgBitrate;
		
		/* 0x05 DecSpecificDescrTag */
		uint8_t  decoderConfigLen;
		
		version = read_char(fp);
		//	flags = stream_read_int24(s);
		fseek(fp, 3, SEEK_CUR);
		
		/* get and verify ES_DescrTag */
		if ( read_char(fp) == MP4ESDescrTag )	// 0x03
		{
			/* read length */
			len = mp4_read_descr_len(fp);
			
			ESId = read_word(fp);
			streamPriority = read_char(fp);
			
			if (len < (5 + 15)) 
			{
				return 1;
			}
		}
		else
		{
			ESId = read_word(fp);
		}
		
		/* get and verify DecoderConfigDescrTab */
		if (read_char(fp) != MP4DecConfigDescrTag)	// 0x04
		{
			return 1;
		}
		
		/* read length */
		len = mp4_read_descr_len(fp);
		
		objectTypeId	= read_char(fp);
		streamType		= read_char(fp);
		//	bufferSizeDB	= read_int24(fp);
		fseek(fp, 3, SEEK_CUR);
		maxBitrate		= read_dword(fp);
		avgBitrate		= read_dword(fp);	// esds->avgBitrate/1000.0
		
		if ( TrackType == AUDIO_TRACK )
//			finfo->wf.nAvgBytesPerSec = avgBitrate / 8;
			finfo->aBitrate = (avgBitrate + 500) / 1000;
		else if ( TrackType == VIDEO_TRACK )
			finfo->vBitrate = (float)avgBitrate / 1024;
		
		if( objectTypeId == MP4OTI_MPEG1Audio || objectTypeId == MP4OTI_MPEG2AudioPart3 )
			finfo->AudioType = mmioFOURCC('M', 'P', '3', ' ');
//			finfo->wf.wFormatTag = 0x55;	// MP3
		else if ( objectTypeId == 0xDD )	// Vorbis ?
			finfo->AudioType = mmioFOURCC('v', 'r', 'b', 's');
		else if ( objectTypeId == 0xE1 )	// QCELP
			finfo->AudioType = mmioFOURCC('Q', 'C', 'L', 'P');
		
		decoderConfigLen = 0;
		
		if (len < 15) 
		{
			return 0;
		}
		
		/* get and verify DecSpecificInfoTag */
		if (read_char(fp) != MP4DecSpecificDescrTag)	// 0x05
		{
			return 0;
		}
		
		/* read length */
		decoderConfigLen = len = mp4_read_descr_len(fp); 
		
		if( decoderConfigLen )
		{
			uint8_t temp = read_char(fp);
			if( ( temp >> 3 ) == 29 )
				finfo->AudioType = 0x1d61346d; // "m4a " : request multi-channel mp3 decoder
		}
		
		/* will skip the remainder of the atom */	
		return 0;
//	}
//	catch (...) 
//	{
//		return 0;
//	}
}

void ParseSTSD(int TrackType, FILE *fp, FileInfo *finfo )
{
	// get (mp4a, mp4v, s263, samr, sawb, avc1) from "stsd" atom
	ATOM_3GP	tAtom;	
	char temp[8];
	read_nbytes(temp, 1, 8, fp);	// [4-7] : EntryCount
				
	read_nbytes((void *)&tAtom, 1, 8, fp);
				
	if ( TrackType == VIDEO_TRACK )
	{
		unsigned short width , height;
		
		fseek(fp, 24, SEEK_CUR);
		
		read_nbytes(&width, 1, 2, fp);
		read_nbytes(&height, 1, 2, fp);
		
		finfo->bih.biWidth  = ( (width  & 0xFF) << 8 ) | (width  >> 8);
		finfo->bih.biHeight = ( (height & 0xFF) << 8 ) | (height >> 8);
		
		finfo->bih.biCompression = mmioFOURCC(tAtom.AtomType[0], tAtom.AtomType[1], tAtom.AtomType[2], tAtom.AtomType[3]);
		
		if( !strncmp(tAtom.AtomType, "avc1", 4) )
		{
#if 1	// Raymond 2007/12/14
			// check if baesline profile
			unsigned int avc1_len = BElong_32(tAtom.AtomSize);
			if( avc1_len > 58 )	// 50 + 8
			{
				fseek(fp, 50, SEEK_CUR);
				avc1_len -= 58;
								
				while( avc1_len > 0 )
				{
					read_nbytes((void *)&tAtom, 1, 8, fp);
					tAtom.AtomSize = BElong_32(tAtom.AtomSize);
					
					// bbc-africa_m720p.mov - has additional "colr" atom
					if ( !strncmp(tAtom.AtomType, "avcC", 4) ) 
					{
						read_nbytes(temp, 1, 2, fp);
						finfo->H264Profile = (unsigned int)temp[1];	// 66 : baseline profile
						break;
					}
					else
					{
						avc1_len -= tAtom.AtomSize;
						fseek(fp, tAtom.AtomSize - 8, SEEK_CUR);
					}
				}
			}
#endif
		}
		else
		{
			// parse esds atom
			fseek(fp, 50, SEEK_CUR);
			read_nbytes((void *)&tAtom, 1, 8, fp);
			if ( !strncmp(tAtom.AtomType, "esds", 4) ) 
			{
				mp4_parse_esds(TrackType, fp, finfo);
			}
		}
	}
	else if ( TrackType == AUDIO_TRACK )
	{
		unsigned short sr, ch;
		
		fseek(fp, 16, SEEK_CUR);
		read_nbytes(&ch, 1, 2, fp);
		fseek(fp, 6, SEEK_CUR);
		read_nbytes(&sr, 1, 2, fp);
		
		if( !strncmp(tAtom.AtomType, "samr", 4) || !strncmp(tAtom.AtomType, "sawb", 4) ) 
			finfo->wf.nChannels = 1;
		else
			finfo->wf.nChannels			= ( (ch & 0xFF) << 8 ) | (ch >> 8);

		if( !strncmp(tAtom.AtomType, "samr", 4) )
			finfo->wf.nSamplesPerSec	= 8000;
		else if(	!strncmp(tAtom.AtomType, "sawb", 4) ) 
			finfo->wf.nSamplesPerSec	= 16000;
		else
			finfo->wf.nSamplesPerSec	= ( (sr & 0xFF) << 8 ) | (sr >> 8);
		
		finfo->AudioType = mmioFOURCC(tAtom.AtomType[0], tAtom.AtomType[1], tAtom.AtomType[2], tAtom.AtomType[3]);					

		// parse esds atom
		fseek(fp, 2, SEEK_CUR);
		read_nbytes((void *)&tAtom, 1, 8, fp);
		if ( !strncmp(tAtom.AtomType, "esds", 4) ) 
		{
			mp4_parse_esds(TrackType, fp, finfo);
		}
	}
}

int GetTrackType(int nPos, int AtomSize, FILE *fp, FileInfo *finfo)
{
	int TrackType = UNKNOWN_TRACK;  

//	try
	{
		int nRet = 0;
		int CurrPos = nPos + 8;
		int StopPos = nPos + AtomSize;
		int Layer = 0;	
		
		char* p3GPAtomType[2] = { "mdia","hdlr" };
		
		ATOM_3GP	tAtom;
				
		// trak -> mdia -> (hdlr, minf) -> stbl -> stsd -> (mp4a, mp4v, s263, samr) 
		while ( ( CurrPos < StopPos ) && ( CurrPos >= 0 ) )
		{		
			while ( ( CurrPos < StopPos ) && ( CurrPos >= 0 ) )
			{
				nRet = fseek(fp, CurrPos, SEEK_SET);
				
				read_nbytes((void *)&tAtom, 1, 8, fp);
				
				tAtom.AtomSize = BElong_32(tAtom.AtomSize);
				
				if ( ( tAtom.AtomSize <= 0 ) || ( tAtom.AtomSize + CurrPos > StopPos ) ) 
				{
					return UNKNOWN_TRACK;	// AtomSize error!	
				}

				// get track type 
				if ( !strncmp(tAtom.AtomType, "hdlr", 4) ) 
				{	
					char temp[8];
					char HandleType[4];
					read_nbytes(temp, 1, 8, fp);
					read_nbytes(HandleType, 1, 4, fp);

					if( strncmp(&temp[4], "dhlr", 4) )	// "mhlr" or "dhlr" in MOV , "" in MP4
					{
						if ( !strncmp(HandleType, "vide", 4) ) 
						{									
							TrackType = VIDEO_TRACK;		
							finfo->bVideo = 1;
							return TrackType;
						}
						else if ( !strncmp(HandleType, "soun", 4) ) 
						{									
							TrackType = AUDIO_TRACK;
							finfo->bAudio = 1;
							return TrackType;
						}
					}
				}

				CurrPos += tAtom.AtomSize;
				
				if ( !strncmp(tAtom.AtomType, p3GPAtomType[Layer], 4) ) 
				{		
					Layer++;
					break;				
				}
			}
			
			StopPos = CurrPos;

			CurrPos = StopPos - tAtom.AtomSize + 8;	// + next layer offset
		}		

		return TrackType;
	}
//	catch (...) 
//	{
//		return TrackType;
//	}	
}

int GetTrackInfo( int TrackType, int nPos, int AtomSize, FILE *fp, FileInfo *finfo )
{
//	try
	{
		int nRet = 0;
		int CurrPos = nPos + 8;
		int StopPos = nPos + AtomSize;
		int Layer = 0;	
				
		char* p3GPAtomType[4] = { "mdia","minf","stbl","stsd" };
		
		ATOM_3GP	tAtom;
				
		// trak -> mdia -> (hdlr, minf) -> stbl -> stsd -> (mp4a, mp4v, s263, samr) 
		while ( ( CurrPos < StopPos ) && ( CurrPos >= 0 ) )
		{		
			while ( ( CurrPos < StopPos ) && ( CurrPos >= 0 ) )
			{
				nRet = fseek(fp, CurrPos, SEEK_SET);
				
				read_nbytes((void *)&tAtom, 1, 8, fp);
				
				tAtom.AtomSize = BElong_32(tAtom.AtomSize);
				
				if ( ( tAtom.AtomSize <= 0 ) || ( tAtom.AtomSize + CurrPos > StopPos ) ) 
				{
					return 0;	// AtomSize error!	
				}

				CurrPos += tAtom.AtomSize;

				if ( TrackType == VIDEO_TRACK )	// get video FPS
				{
#if 1				// Raymond 2007/09/13
					if ( !strncmp(tAtom.AtomType, "mdhd", 4) ) 
					{
						// Get TimeScale
						unsigned int TimeScale = 0, dur = 0;
						fseek(fp, 12, SEEK_CUR);
						read_nbytes(&TimeScale, 1, 4, fp);
						TimeScale = BElong_32(TimeScale);
						read_nbytes(&dur, 1, 4, fp);
						dur = BElong_32(dur);
						finfo->FPS = (float)dur / TimeScale;
					}
					
					if( !strncmp(tAtom.AtomType, "stsz", 4) )	// "stsz" atom
					{
						// get Duration from "stsz" atom
						unsigned int len = 0, num = 0, dur = 0;
						fseek(fp, 8, SEEK_CUR);
						read_nbytes(&num, 1, 4, fp);
						num = BElong_32(num);

						finfo->FPS = (float)num / finfo->FPS;
					}
#else					
					if ( !strncmp(tAtom.AtomType, "mdhd", 4) ) 
					{
						// Get TimeScale
						unsigned int TimeScale = 0;
						fseek(fp, 12, SEEK_CUR);
						read_nbytes(&TimeScale, 1, 4, fp);
						TimeScale = BElong_32(TimeScale);
						finfo->FPS = (float)TimeScale;
					}
					
					if( !strncmp(tAtom.AtomType, "stts", 4) )	// "stts" atom
					{
						// get Duration from "stts" atom
						unsigned int len = 0, num = 0, dur = 0;
						fseek(fp, 4, SEEK_CUR);
						read_nbytes(&len, 1, 4, fp);
						len = BElong_32(len);
						
						if( len >= 1 )
						{
							read_nbytes(&num, 1, 4, fp);
							read_nbytes(&dur, 1, 4, fp);
							dur = BElong_32(dur);	
							
							finfo->FPS = finfo->FPS / (float)dur;
						}									
					}
#endif					
				}

				if( !strncmp(tAtom.AtomType, "stsd", 4) )	// "stsd" atom
				{
					// get (mp4a, mp4v, s263, samr) from "stsd" atom
					ParseSTSD(TrackType, fp, finfo);
				}

				if( Layer < 4 )
				{
					if ( !strncmp(tAtom.AtomType, p3GPAtomType[Layer], 4) ) 
					{	
						Layer++;
						if( Layer < 4 )
						break;				
					}
				}
			}
		
			if( Layer == 4 )
				break;

			StopPos = CurrPos;

			CurrPos = StopPos - tAtom.AtomSize + 8;	// + next layer offset
		}		
	}
//	catch (...) 
//	{
//		return false;
//	}
	
	return 1;
}

int MOV_Parser(FILE	*fp, FileInfo *finfo)
{
//	try
	{	
		int nRet = 0;
		
		nRet = mov_check_file(fp);
		if ( nRet == 0 )
			return nRet;
		
		unsigned int nFileSize = 0;
		unsigned int CurrPos = 0;
		unsigned int StopPos = 0;
		int Layer = 0;			
		
		ATOM_3GP	tAtom;
		
		uint8_t	*ptr = NULL;
		
		StopPos = finfo->FileSize;

		// moov -> trak
		while ( ( CurrPos < StopPos ) && ( CurrPos >= 0 ) )
		{		
			while ( ( CurrPos < StopPos ) && ( CurrPos >= 0 ) )
			{
				fseek(fp, CurrPos, SEEK_SET);
				
				read_nbytes((void *)&tAtom, 1, 8, fp);
				
				tAtom.AtomSize = BElong_32(tAtom.AtomSize);
				
				if ( ( tAtom.AtomSize < 8 ) || ( tAtom.AtomSize + CurrPos > StopPos ) ) 
				{
					return 0;	// AtomSize error!	
				}

				if ( !strncmp(tAtom.AtomType, "mvhd", 4) ) 
				{									
					// Get TimeScale and Duration
					unsigned int TimeScale = 0, Duration = 0;
					fseek(fp, 12, SEEK_CUR);

					read_nbytes(&TimeScale, 1, 4, fp);
					TimeScale = BElong_32(TimeScale);
					read_nbytes(&Duration, 1, 4, fp);
					Duration = BElong_32(Duration);
					finfo->FileDuration = (int)(Duration / TimeScale);
				}

				if ( !strncmp(tAtom.AtomType, "trak", 4) ) 
				{
					// Get Track infomation
					int TrackType = GetTrackType(CurrPos, tAtom.AtomSize, fp, finfo);

					fseek(fp, CurrPos + 8, SEEK_SET);

					int bRet = GetTrackInfo(TrackType, CurrPos, tAtom.AtomSize, fp, finfo);
					if( bRet == 0)
						return 0;
				}

				if ( !strncmp(tAtom.AtomType, "cmov", 4) ) 
				{
					return 0;	// Compressed header is not supported!
				}

				CurrPos += tAtom.AtomSize;
				
				if ( !strncmp(tAtom.AtomType, "moov", 4) ) 
				{									
					break;				
				}
			}
			
			Layer++;	
			if( Layer == 2 )
				break;
			
			StopPos = CurrPos;
			
			CurrPos = StopPos - tAtom.AtomSize + 8;	// + next layer offset
		}		

		return nRet;
	}
//	catch (...) 
//	{
//		return 0;
//	}	
}

// AAC Parser

enum{ AAC_ADIF, AAC_ADTS };

typedef struct 
{
	unsigned char* pSource;
	int		nSourceCount;
//	int		nMaxNumSourceBytes;

	int		nbits;
	long	cword;
	
} BitData;

typedef struct 
{
    int num_ele;
    int ele_is_cpe[16];
    int ele_tag[16];
} EleList;

typedef struct 
{
    int present;
    int ele_tag;
    int pseudo_enab;
} MIXdown;

typedef struct 
{
    int profile;
    int sampling_rate_idx;
    int nChannels;
//	char comments[(1<<8)+1];
    long buffer_fullness;	// put this transport level info here 
	EleList front;
    EleList side;
    EleList back;
    EleList lfe;
    EleList data;
    EleList coupling;
    MIXdown mono_mix;
    MIXdown stereo_mix;
    MIXdown matrix_mix;
    
} ProgConfig;

// audio data transport stream frame format header
typedef struct
{
	int	syncword;
	int	id;
	int	layer;
	int	protection_abs;
	int profile;
	int	sampling_freq_idx;
	int	private_bit;
	int	channel_config;
	int	original_copy;
	int	home;
	int	copyright_id_bit;
	int	copyright_id_start;
	int	frame_length;
	int	adts_buffer_fullness;
	int	num_of_rdb;
	int	crc_check;
} ADTSHeader;

// audio data interchange format header
typedef struct 
{
    char    adif_id[5];
    int	    copy_id_present;
    char    copy_id[10];
    int	    original_copy;
    int	    home;
    int	    bitstream_type;
    long    bitrate;		// bps
    int	    num_pce;
//    int	    prog_tags[16];	//(1<<4)

	ProgConfig prog_config;

} ADIFHeader;

typedef struct
{
	ADTSHeader adtsHeader;
	ADIFHeader adifHeader;
	
	int nAACFormat;
	int	nADIFHeaderLength;
	int nChannels;
	int nSamplingFreq;
	int nBitRate;			// kbps
	
} AAC_INFO;

AAC_INFO AacInfo = {0};

// AAC sampling rate
static int SampleRate[16] = 
{
	96000, 88200, 64000, 48000, 44100, 32000, 24000, 
	22050, 16000, 12000, 11025, 8000, 0, 0, 0, 0
};

void InitGetBits(BitData *bf, unsigned char* pSource,int N)
{
	bf->pSource=(unsigned char*)pSource;	
//	bf->nMaxNumSourceBytes=N;
	bf->nSourceCount=0;
	bf->cword=0;
	bf->nbits=0;
}

static int GetByte(BitData* bf)
{
	return (int)(bf->pSource[bf->nSourceCount++]);
}

long GetBits(BitData* bf, int n)
{
	while ( bf->nbits < n )
	{
		bf->cword = (bf->cword << 8) | GetByte(bf);
		bf->nbits += 8;
	}

	bf->nbits -= n;

	return (long)((bf->cword >> bf->nbits) & (((int64_t)1 << n) - 1));
}

void FlushBits(BitData* bf, int n)
{
	while ( bf->nbits < n )
	{
		bf->cword = bf->pSource[bf->nSourceCount++];
		bf->nbits += 8;
	}

	bf->nbits -= n;
}

// program configuration element 
int get_ele_list(BitData *bf, EleList *p, int enable_cpe)
{  
    int i, j, nChannels = 0;
    for (i = 0, j = p->num_ele; i<j; i++) 
	{
		if (enable_cpe)		
		{
			if (p->ele_is_cpe[i] = GetBits(bf, 1))
				nChannels++;
		}		
		else
			p->ele_is_cpe[i] = 0; // sdb 

		p->ele_tag[i] = GetBits(bf, 4);
		nChannels++;
    }

	return nChannels;
}

int get_prog_config(BitData *bf, ProgConfig *p)
{
    int tag;
	int current_program = 0;
		
	p->nChannels = 0;

    tag = GetBits(bf, 4);

    p->profile				= GetBits(bf, 2);
    p->sampling_rate_idx	= GetBits(bf, 4);
    p->front.num_ele		= GetBits(bf, 4);
    p->side.num_ele			= GetBits(bf, 4);
    p->back.num_ele			= GetBits(bf, 4);
    p->lfe.num_ele			= GetBits(bf, 2);
    p->data.num_ele			= GetBits(bf, 3);
    p->coupling.num_ele		= GetBits(bf, 4);

    if ( ( p->mono_mix.present = GetBits(bf, 1) ) == 1 )
		p->mono_mix.ele_tag = GetBits(bf, 4);
    if ( ( p->stereo_mix.present = GetBits(bf, 1) ) == 1 )
		p->stereo_mix.ele_tag = GetBits(bf, 4);
	
    if ( ( p->matrix_mix.present = GetBits(bf, 1) ) == 1 ) 
	{
		p->matrix_mix.ele_tag		= GetBits(bf, 2);
		p->matrix_mix.pseudo_enab	= GetBits(bf, 1);
    }

    p->nChannels += get_ele_list(bf, &p->front, 1);
    p->nChannels += get_ele_list(bf, &p->side, 1);
    p->nChannels += get_ele_list(bf, &p->back, 1);
    p->nChannels += get_ele_list(bf, &p->lfe, 0);
    
    return 1;
}

int get_adif_header(BitData* bf, ADIFHeader *p)
{
    int i, n;
    // adif header 
    for (i = 0; i < 4; i++)
		p->adif_id[i] = (char)GetBits(bf, 8); 
    p->adif_id[i] = 0;	    // null terminated string 
    
	// test for id 
    if (strncmp(p->adif_id, "ADIF", 4) != 0)
		return 0;	    // bad id 
    
	// copyright string 
    if ((p->copy_id_present = GetBits(bf, 1)) == 1) 
	{
		for (i = 0; i < 9; i++)
			p->copy_id[i] = (char)GetBits(bf, 8); 
		p->copy_id[i] = 0;  // null terminated string 
    }

    p->original_copy	= GetBits(bf, 1);
    p->home				= GetBits(bf, 1);
    p->bitstream_type	= GetBits(bf, 1);
    p->bitrate			= GetBits(bf, 23);

    // program config elements 
    n = GetBits(bf, 4) + 1;
    for ( i = 0 ; i < n ; i++ ) 
	{
		if( p->bitstream_type == 0 )
			p->prog_config.buffer_fullness = GetBits(bf, 20);

		if( !get_prog_config(bf, &(p->prog_config)) )
		{
			return 0;
		}
    }
 
    return 1;
}

int get_adts_header(BitData* bf, ADTSHeader *p)
{
	p->syncword = GetBits(bf, 12);
	if ( p->syncword != 0xfff) 
	{
		return 0;
	}

	p->id					= GetBits(bf, 1);
	p->layer				= GetBits(bf, 2);
	p->protection_abs		= GetBits(bf, 1);
	p->profile				= GetBits(bf, 2);
	p->sampling_freq_idx	= GetBits(bf, 4);
	p->private_bit			= GetBits(bf, 1);
	p->channel_config		= GetBits(bf, 3);

	if( p->layer != 0 )		
		return 0;	// ADTS layer data error 
	
	if( p->profile != 1 )		
		return 0;	// AAC profile not supported!

	return 1;
}

// return ID3 Length if "ID3"
int	GetID3Tag(FILE	*fp, FileInfo *finfo)
{
	ID3_TAG *pTag = &(finfo->ID3Tag);

	// check "TAG" in final 128 bytes first
	if ( finfo->FileSize > 128 ) 
	{
		char tag[4];
		fseek(fp, finfo->FileSize - 128, SEEK_SET);
		read_nbytes(tag, 1, 3, fp);
		tag[3] = '\0';
		
		if(strcmp(tag,"TAG"))
		{
//			printf("No TAG\n");
		}
		else 
		{	
//			printf("Got TAG\n");
			finfo->bTag = 1;
			
			unsigned char buf[125];
			read_nbytes(buf, 1, 125, fp);
			
			// Title
			memcpy(pTag->Title,		buf,		30);
			memcpy(pTag->Artist,	buf + 30,	30);
			memcpy(pTag->Album,		buf + 60,	30);
			memcpy(pTag->Year,		buf + 90,	4);
			memcpy(pTag->Comment,	buf + 94,	30);
			
			if(buf[122] == 0 && buf[123] != 0) 
			{
				pTag->Track = (unsigned int)buf[123];
			}
			
			pTag->Genre = (unsigned int)(buf[124]);
			pTag->GenreString = (char *)(ID3_V1GENRE2DESCRIPTION(pTag->Genre));	// Raymond 2008/05/30
			
			finfo->nID3Length = 0;
		}
	}

	// check if "ID3" and get length
	char hdr[10];
		
	fseek(fp, 0, SEEK_SET);
	read_nbytes(hdr, 1, 10, fp);
		
	if( hdr[0] == 'I' && hdr[1] == 'D' && hdr[2] == '3' && (hdr[3] >= 2 ) ) 
	{			
		int ID3_len = (int)(( hdr[6] << 21 ) + ( hdr[7] << 14 ) + ( hdr[8] << 7 ) +  hdr[9]);
//		printf("ID3 tag length = %d\n", ID3_len);
		
		finfo->nID3Length = ID3_len + 10;
		
		if ( finfo->bTag == 0 )
		{
			int pos = 0;
			char *ptr = NULL;
			int len = 0;
			char type[5];
			char *ID3_buf = NULL;
			
			ID3_buf = (char *)malloc(ID3_len);
			if(ID3_buf == NULL)
			{
//				printf("ID3_buf NULL\n");
				return 0;
			}

//			printf("Got ID3\n");
			finfo->bTag = 1;
						
			read_nbytes(ID3_buf, 1, ID3_len, fp);
			
			ptr = ID3_buf;
			type[4] = '\0';
			
			// parse ID3_buf
			
			while( pos < ID3_len )
			{
				len = (int)(ptr[7]);			
				memcpy(type, ptr, 4);
				
				if ( len == 0 )
					break;
				
//				printf("[%s] : Frame length = %d\n", type, len);
				
				if( strncmp(type, "TIT2", 4) == 0 )			/* TIT2 : Title/songname/content description */
				{
					memcpy(pTag->Title,		ptr+11, len-1);	
				}
				else if( strncmp(type, "TPE1", 4) == 0 )	/* TPE1 : Lead performer(s)/Soloist(s) */
				{
					memcpy(pTag->Artist,	ptr+11, len-1);
				}
				else if( strncmp(type, "TALB", 4) == 0 )	/* TALB : Album/Movie/Show title */
				{
					memcpy(pTag->Album,		ptr+11, len-1);
				}
				else if( strncmp(type, "COMM", 4) == 0 )	/* COMM : Comments */
				{
					memcpy(pTag->Comment,	ptr+11, len-1);
				}
				else if( strncmp(type, "TYER", 4) == 0 )	/* TYER : Year */
				{
//					memcpy(pTag->Year,	ptr+11, len-1);
				}
				else if( strncmp(type, "TRCK", 4) == 0 )	/* TRCK : Track number/Position in set */
				{
//					pTag->Track = atoi(ptr+11);
				}
				else if( strncmp(type, "TCON", 4) == 0 )	/* TCON : Content type, Genre */ 
				{
//					pTag->Genre = atoi(ptr+11);
				}
				
				ptr += 10 + len;
				pos += 10 + len;
			}
			
			if(ID3_buf != NULL)
				free(ID3_buf);
			
		}	// end if ( finfo->bTag == 0 )
	}
		
	if ( finfo->bTag == 0 )
		return 0;	// No ID3 tag

	return finfo->nID3Length;
}

int aac_check_file(FILE	*fp, FileInfo *finfo)
{
	int nRet;
	BitData bf;
	unsigned char hdr[100] = {0};

	memset((void *)&AacInfo, 0, sizeof(AAC_INFO));
	
	int nID3Length = GetID3Tag(fp, finfo);

	fseek(fp, nID3Length, SEEK_SET);
	
	read_nbytes(hdr, 1, 100, fp);

	InitGetBits(&bf, hdr, 100);

	// AAC Header
	if(!strncmp((char *)hdr, "ADIF", 4)) 
	{
		// ADIF 
		nRet = get_adif_header(&bf, &(AacInfo.adifHeader));
		if ( nRet == 0 ) 
			return 0;

		AacInfo.nAACFormat = AAC_ADIF;

		AacInfo.nChannels		= AacInfo.adifHeader.prog_config.nChannels;
		AacInfo.nSamplingFreq	= SampleRate[AacInfo.adifHeader.prog_config.sampling_rate_idx];

		AacInfo.nBitRate		= (AacInfo.adifHeader.bitrate + 512) / 1024;
	}
	else if( hdr[0] == 0xFF && ( (unsigned char)(hdr[1] & 0xF6) == (unsigned char)0xF0 ) )
	{
		// ADTS 		
		nRet = get_adts_header(&bf, &(AacInfo.adtsHeader));
		if ( nRet == 0 ) 
			return 0;
		
		AacInfo.nAACFormat = AAC_ADTS;

		AacInfo.nChannels = AacInfo.adtsHeader.channel_config;
		AacInfo.nSamplingFreq = SampleRate[AacInfo.adtsHeader.sampling_freq_idx];		
	}
	else
	{
		return 0;
	}

	return 1;
}

int AAC_Parser(FILE	*fp, FileInfo *finfo)
{
	if ( AacInfo.nAACFormat == AAC_ADTS )
	{
		// get frame number and duration
		
		unsigned char pBuf[6];
		unsigned int pos = finfo->nID3Length;
		unsigned int nAACFrameLen = 0;
		unsigned int nFrame = 0;
		
		// skip if has ID3 tag
		if ( finfo->nID3Length > 0 )
			fseek(fp, finfo->nID3Length, SEEK_SET);

		while ( pos < finfo->FileSize ) 
		{
			read_nbytes(pBuf, 1, 6, fp);

			if( pBuf[0] == 0xFF && ( (unsigned char)(pBuf[1] & 0xF6) == (unsigned char)0xF0 ) )
			{		
				nAACFrameLen = (( ( int ) (pBuf[3] & 3) ) << 11 ) +	( ( int )pBuf[4] << 3 ) + ( pBuf[5] >> 5 );
				
				if( (pos + nAACFrameLen > finfo->FileSize) || (nAACFrameLen == 0) )
				{
					return 0;
				}
				
				pos += nAACFrameLen;
				nFrame++;
				fseek(fp, pos, SEEK_SET);
			}
			else if( !strncmp((char *)pBuf, "TAG", 3) )	// Skip the final TAG data if "TAG"
			{
				break;	// Skip the final TAG data
			}
			else 
				break;
		}
				
		finfo->AudioDuration = (int)((nFrame * 1024) / AacInfo.nSamplingFreq);
		AacInfo.nBitRate = (int) ( ( finfo->FileSize - finfo->nID3Length ) / finfo->AudioDuration );

		AacInfo.nBitRate = AacInfo.nBitRate * 8 / 1024;
	}
	else if ( AacInfo.nAACFormat == AAC_ADIF )
	{
		finfo->AudioDuration = (int)( finfo->FileSize * 8 / (float)(AacInfo.adifHeader.bitrate) );
	}
	else
		return 0;
	
	finfo->bAudio = 1;
	finfo->AudioType = mmioFOURCC('A', 'A', 'C', ' ');

	finfo->FileDuration = finfo->AudioDuration;

	finfo->wf.nSamplesPerSec	= AacInfo.nSamplingFreq;
	finfo->wf.nChannels			= AacInfo.nChannels;

	finfo->aBitrate = AacInfo.nBitRate;

	return 1;
}

// MP3 Parser

#define MP3_PARSE_PART

#define MP3_PARSE_SIZE	500000

static int tabsel_123[2][3][16] = 
{
   { 
		{0,32,64,96,128,160,192,224,256,288,320,352,384,416,448,0},
		{0,32,48,56, 64, 80, 96,112,128,160,192,224,256,320,384,0},
		{0,32,40,48, 56, 64, 80, 96,112,128,160,192,224,256,320,0} 
   },

   {
		{0,32,48,56,64,80,96,112,128,144,160,176,192,224,256,0},
		{0,8,16,24,32,40,48,56,64,80,96,112,128,144,160,0},
		{0,8,16,24,32,40,48,56,64,80,96,112,128,144,160,0} 
   }
};

static int freqs[9] = 
{ 
	44100, 48000, 32000,	// MPEG 1.0
	22050, 24000, 16000,	// MPEG 2.0
	11025, 12000,  8000		// MPEG 2.5
};

static int mult[3] = { 12000, 144000, 144000 };

// return frame size or -1 (bad frame)

int mp_get_mp3_header(unsigned char* hbuf, int* chans, int* srate, int* bitrate)
{
    int lsf,framesize, padding, bitrate_index, sampling_frequency, layer;
	unsigned long newhead = hbuf[0] << 8 | hbuf[1];
		
    // head_check:
	if( ( newhead & 0xffe0 ) != 0xffe0 )
	{
//		printf("head_check failed\n");
		return -1;
    }
	
    layer = 4 - ( ( hbuf[1] >> 1 ) & 3 );
    if( layer == 4 )
	{ 
//		printf("not layer-1/2/3\n"); 
		return -1;
    }
	
    sampling_frequency = ( ( hbuf[2] >> 2 ) & 0x3 );  // valid: 0..2
    if( sampling_frequency == 3 )
	{
//		printf("invalid sampling_frequency\n");
		return -1;
    }
	
    if( hbuf[1] & 0x10 ) 
	{
		// MPEG 1.0 (lsf==0) or MPEG 2.0 (lsf==1)
		lsf = ( hbuf[1] & 0x8 ) ? 0x0 : 0x1;
		sampling_frequency += ( lsf * 3 );
    }
	else 
	{
		// MPEG 2.5
		lsf = 1;
		sampling_frequency += 6;
    }
	
    bitrate_index = hbuf[2] >> 4 ;  // valid: 1..14
    padding   = ( ( hbuf[2] >> 1 ) & 0x1 );
    *chans    = ( ( ( hbuf[3] >> 6 ) ) == 3 ) ? 1 : 2;

	*srate = freqs[sampling_frequency];
	*bitrate = tabsel_123[lsf][layer-1][bitrate_index];
	framesize = (*bitrate) * mult[layer-1];

    if(!framesize)
	{
//		printf("invalid framesize/bitrate_index\n");
		return -1;
    }
	
    framesize /= (*srate << lsf);
    if( layer == 1 )
		framesize = ( framesize + padding ) << 2;	// * 4
    else
		framesize += padding;
	 	
    return framesize;
}

int mp3_check_file(FILE	*fp, FileInfo *finfo)
{
	uint8_t hdr[4];
	int mp3_freq = 0, mp3_chans = 0, mp3_bitrate = 0, mp3_flen = 0;

	int nID3Length = GetID3Tag(fp, finfo);

	fseek(fp, nID3Length, SEEK_SET);
	read_nbytes(hdr, 1, 4, fp);

	mp3_flen = mp_get_mp3_header(hdr, &mp3_chans, &mp3_freq, &mp3_bitrate );

	if( mp3_flen > 0 )
	{
		finfo->wf.nSamplesPerSec	= mp3_freq;
		finfo->wf.nChannels			= mp3_chans;		
		finfo->aBitrate				= mp3_bitrate; 

		return 1;
	}
//	else	// Raymond 2007/06/13
//	else if( nID3Length > 0 )	// Raymond 2007/08/02	
	else if( nID3Length > 0 || extensions_table[finfo->SubFileNameIndex].demuxer_type == FILE_TYPE_MP3 )	// Raymond 2007/09/20		
	{
		unsigned int pos = 0, parse_size = 5000;	// Raymond 2008/03/20
		uint8_t *buf = (uint8_t *)malloc(parse_size+3);
		if( buf == NULL )
			return 0;

#if 1	// Raymond 2008/08/01
		fseek(fp, nID3Length, SEEK_SET);

		if( hdr[0] == 'I' && hdr[1] == 'D' && hdr[2] == '3' && (hdr[3] >= 2 ) )	// check if 2 ID3
		{
			read_nbytes(buf, 1, 10, fp);
			nID3Length = (int)(( buf[6] << 21 ) + ( buf[7] << 14 ) + ( buf[8] << 7 ) +  buf[9]);
			finfo->nID3Length += nID3Length + 10;
			if( (unsigned)finfo->nID3Length >= finfo->FileSize )
				return 0;
			fseek(fp, finfo->nID3Length, SEEK_SET);
		}

		parse_size = read_nbytes(buf, 1, parse_size, fp);
#else
		buf[0] = hdr[1];
		buf[1] = hdr[2];
		buf[2] = hdr[3];
		parse_size = read_nbytes(buf+3, 1, parse_size, fp);
#endif		
		while( pos < parse_size )
		{
			if( buf[pos] == 0xFF )
			{
				mp3_flen = mp_get_mp3_header(&buf[pos], &mp3_chans, &mp3_freq, &mp3_bitrate );
				
				if( mp3_flen > 0 )
				{
					finfo->wf.nSamplesPerSec	= mp3_freq;
					finfo->wf.nChannels			= mp3_chans;		
					finfo->aBitrate				= mp3_bitrate; 
					
					finfo->nID3Length += pos + 1;
					free(buf);
					return 1;
				}		
			}
			
			pos++;
		}

		free(buf);
	}
	
	return 0;
}

int MP3_Parser(FILE	*fp, FileInfo *finfo)
{		
	uint8_t hdr[4];
	int mp3_freq = 0, mp3_chans = 0, mp3_bitrate = 0, mp3_flen = 0;

	unsigned long pos = finfo->nID3Length;

	unsigned int nFrame = 0, nVBRFrame = 0;
	int isVBR = 0;	
	unsigned int FileSize = finfo->FileSize - 128;
	unsigned int parse_size = FileSize;

#ifdef MP3_PARSE_PART
	if( FileSize > MP3_PARSE_SIZE )
		parse_size = MP3_PARSE_SIZE;
#endif

	while( pos < parse_size - 4 )
	{
		fseek(fp, pos, SEEK_SET);
		read_nbytes(hdr, 1, 4, fp);
		
		if( ( mp3_flen = mp_get_mp3_header(hdr, &mp3_chans, &mp3_freq, &mp3_bitrate ) ) > 0 )
		{
			finfo->wf.nSamplesPerSec	= mp3_freq;
			finfo->wf.nChannels			= mp3_chans;
//			finfo->aBitrate				= mp3_bitrate; 

			if( finfo->aBitrate	!= (unsigned int)mp3_bitrate)
			{
				nVBRFrame++;
				isVBR = 1;
			}
			else if ( isVBR == 0 && nFrame > 30 )
				break;

			nFrame++;

			pos += mp3_flen;			
		}
		else
			pos++;
	}
	
	finfo->bAudio = 1;
	finfo->AudioType = mmioFOURCC('M', 'P', '3', ' ');
	
//	if( isVBR == 1 )
	if( isVBR == 1 && ( nVBRFrame > 0.2 * nFrame ) )
	{
		if( finfo->wf.nSamplesPerSec != 0 )
		{
#if 1	// Raymond 2007/09/19
			float audio_dur = ((float)(1152 * nFrame) / finfo->wf.nSamplesPerSec);

#ifdef MP3_PARSE_PART
			if( FileSize > MP3_PARSE_SIZE )
				audio_dur = (int)( (audio_dur * FileSize ) / MP3_PARSE_SIZE );			
#endif
			finfo->AudioDuration = (int)audio_dur;

#else 	// end Raymond 2007/09/19			
			finfo->AudioDuration = (int)((1152 * nFrame) / finfo->wf.nSamplesPerSec);

#ifdef MP3_PARSE_PART
			if( FileSize > MP3_PARSE_SIZE )
				finfo->AudioDuration = (int)( (finfo->AudioDuration * FileSize ) / MP3_PARSE_SIZE );
#endif
#endif			
			if( finfo->AudioDuration != 0 )
				finfo->aBitrate = (Uint32_t) ( (finfo->FileSize - finfo->nID3Length) * 8 / (finfo->AudioDuration * 1000) );
		}
	}
	else if( finfo->aBitrate != 0 )
		finfo->AudioDuration = (int)((finfo->FileSize - finfo->nID3Length) * 8 / (finfo->aBitrate * 1000));

	finfo->FileDuration = finfo->AudioDuration;

	return 1;
}

// WAV Parser

#define formtypeWAVE             mmioFOURCC('W', 'A', 'V', 'E')

int wav_check_file(FILE	*fp)
{
	int id;	
	
	read_nbytes(&id, 1, 4, fp);
	if(id == mmioFOURCC('R','I','F','F'))
	{
		int riff_type;	
		unsigned int size2;
		
		read_nbytes(&size2, 1, 4, fp);
		read_nbytes((char *)&riff_type, 1, 4, fp);	

		if( riff_type == formtypeWAVE )	// "AVI "
			return 1;
	}
		
	return 0;
}

int WAV_Parser(FILE	*fp, FileInfo *finfo)
{
	unsigned int len = 0;
	WAVEFORMATEX *wf = &(finfo->wf);

	fseek(fp, 16, SEEK_CUR);
	read_nbytes(&len, 1, 4, fp);
		
	read_nbytes((char*)wf,1, len, fp);
						
	finfo->bAudio = 1;

	finfo->aBitrate = wf->nAvgBytesPerSec * 8 / 1000;

	if( wf->nAvgBytesPerSec != 0 )
		finfo->AudioDuration = (int)(finfo->FileSize / wf->nAvgBytesPerSec);

	finfo->FileDuration = finfo->AudioDuration;

	return 1;
}

// OGG Parser

#define oggpack_read_dword(ptr)	( (int)( ptr[3] << 24 | ptr[2] << 16 | ptr[1] << 8 | ptr[0] ) )

uint64_t ogg_page_granulepos(unsigned char *page)
{
	uint64_t granulepos = page[13]&(0xff);
	granulepos = (granulepos<<8)|(page[12]&0xff);
	granulepos = (granulepos<<8)|(page[11]&0xff);
	granulepos = (granulepos<<8)|(page[10]&0xff);
	granulepos = (granulepos<<8)|(page[9]&0xff);
	granulepos = (granulepos<<8)|(page[8]&0xff);
	granulepos = (granulepos<<8)|(page[7]&0xff);
	granulepos = (granulepos<<8)|(page[6]&0xff);
	return(granulepos);
}

int vorbis_comment(unsigned char *hdr, FileInfo *finfo)
{
	unsigned char *ptr = hdr;
	int num_comment, i;
	
	num_comment = oggpack_read_dword(ptr);	
	ptr += 4;
	
	if( num_comment < 0 )
		return 0;

	finfo->bTag = 1;

	for( i = 0 ; i < num_comment ; i++ )
	{
		int len = oggpack_read_dword(ptr);	
		ptr += 4;
		
		if( len < 0 )	
			break;

		if( !strncmp((char *)ptr, "TITLE", 5) )					// TITLE
		{
			memcpy(&(finfo->ID3Tag.Title), ptr + 6, len - 6);	
		}
		else if( !strncmp((char *)ptr, "ARTIST", 6) )			// ARTIST
		{
			memcpy(&(finfo->ID3Tag.Artist), ptr + 7, len - 7);
		}
		else if( !strncmp((char *)ptr, "ALBUM", 5) )			// ALBUM
		{
			memcpy(&(finfo->ID3Tag.Album), ptr + 6, len - 6);	
		}
		else if( !strncmp((char *)ptr, "COMMENT", 7) )			// COMMENT
		{
			memcpy(&(finfo->ID3Tag.Comment), ptr + 8, len - 8);	
		}
		else if( !strncmp((char *)ptr, "DATE", 4) )				// DATE
		{
			memcpy(&(finfo->ID3Tag.Year), ptr + 5, len - 5);		
		}
		else if( !strncmp((char *)ptr, "TRACKNUMBER", 11) )		// TRACKNUMBER
		{
//			memcpy(&(finfo->ID3Tag.Track), ptr + 12, len - 12);		
		}
		else if( !strncmp((char *)ptr, "GENRE", 5) )			// GENRE
		{
//			memcpy(&(finfo->ID3Tag.Genre), ptr + 6, len - 6);		
		}
		
		ptr += len;
	}	  
	
	return 1;
}

int ogg_check_file(FILE	*fp, FileInfo *finfo)
{
	unsigned char hdr[500] = {0};	// Raymond 2008/05/27

	read_nbytes(hdr, 1, 58, fp);

	if(!strncmp((char *)hdr, "OggS", 4)) 
	{
		int headerbytes = 0, bodybytes = 0, i, pos = 0;
		unsigned char *ptr;
		
		headerbytes = hdr[26] + 27;
				
		/* count up body length in the segment table */
		
		for( i = 0 ; i < hdr[26] ; i++ )
			bodybytes += hdr[27+i];

		ptr = &hdr[headerbytes];

		if( bodybytes >= 7 && !strncmp((char *)(ptr+1),"vorbis", 6) ) 
		{
			int version;
			long bitrate_upper;
			long bitrate_nominal;
			long bitrate_lower;

			ptr += 7;

			version		= oggpack_read_dword(ptr);		ptr += 4;
			if( version != 0 )
				return 0;
			
			finfo->wf.nChannels	= *ptr;					ptr++;

			finfo->wf.nSamplesPerSec	= oggpack_read_dword(ptr);	ptr += 4;
			
			bitrate_upper	= oggpack_read_dword(ptr);	ptr += 4;
			bitrate_nominal	= oggpack_read_dword(ptr);	ptr += 4;
			bitrate_lower	= oggpack_read_dword(ptr);	ptr += 4;
			
			finfo->aBitrate = bitrate_nominal / 1000;

			// parse comment
			pos = ( headerbytes + bodybytes );

			fseek(fp, pos, SEEK_SET);
			read_nbytes(hdr, 1, 200, fp);

			if(!strncmp((char *)hdr, "OggS", 4)) 
			{
				i = hdr[26];
				headerbytes = i + 27;
				
				// parse comment
				if ( hdr[27+i] == 0x3 && (!strncmp((char *)&hdr[27+i+1], "vorbis", 6)) )
				{
					ptr = &hdr[ headerbytes + 7 ];
					int vendorlen  = oggpack_read_dword(ptr);
					
					if( vendorlen > 0 )
					{	
						// skip vendor information
						fseek(fp, pos + headerbytes + 7 + 4 + vendorlen, SEEK_SET);
						read_nbytes(hdr, 1, 500, fp);	// Raymond 2008/05/27
						
						vorbis_comment(hdr, finfo);
					}
				}				
			}
			// end parse comment

			return 1;
		}
		else
			return 0;
	}
	else
		return 0;
}

int OGG_Parser(FILE	*fp, FileInfo *finfo)
{
	unsigned char hdr[4000] = {0};
	unsigned char *ptr = (unsigned char *)hdr;
	unsigned int pos = finfo->FileSize - 4000;
	int i = 0;
	int64_t granulepos = 0;

	fseek(fp, pos, SEEK_SET);
	read_nbytes(hdr, 1, 4000, fp);

	for( i = 0 ; i < 3996 ; i++, ptr++ )
	{
		if(!strncmp((char *)ptr, "OggS", 4)) 
		{
			pos += i;
			
			// find the final "OggS" packet then get granulepos
			granulepos = ogg_page_granulepos(ptr);
			break;
		}
	}
	
	if( finfo->wf.nSamplesPerSec != 0 )
		finfo->AudioDuration = (int)( ( granulepos + (finfo->wf.nSamplesPerSec >> 1) ) / finfo->wf.nSamplesPerSec );
	
	finfo->bAudio = 1;
	finfo->AudioType = mmioFOURCC('v', 'r', 'b', 's');
	
	finfo->FileDuration = finfo->AudioDuration;

	return 1;
}

// FLV - Raymond 2007/11/05

int flv_video_pts = 0;
unsigned int flv_video_size = 0;

int flv_check_file(FILE	*fp)
{
	char dbuf[4];	
	
	read_nbytes(dbuf, 1, 3, fp);

	if (dbuf[0] == 'F' && dbuf[1] == 'L' && dbuf[2] == 'V') 
	{
        return 1;
    }
		
	return 0;
}

inline static unsigned int read_be24(FILE *fp)
{
    unsigned int val;
    val = read_char(fp) << 16;
    val |= read_char(fp) << 8;
    val |= read_char(fp);
    return val;
}

static int flv_read_packet(FILE	*fp, FileInfo *finfo)
{
    int type, size, pts, flags, is_audio = 0;
    int h263_hdr = 0, w = 0, h = 0, read_tmp = 0, format = 0;

	for(;;)
	{
		fseek(fp, 4, SEEK_CUR); // size of previous packet 
		type = read_char(fp);
		size = read_be24(fp);
		pts = read_be24(fp);

		if( feof(fp) || file_error == 1 ) 
			return 1;

		fseek(fp, 4, SEEK_CUR); /* reserved */
		flags = 0;
		
		if( size == 0 )
			continue;
		
		if (type == 8) 
		{
			is_audio = 1;
			flags = read_char(fp);
			size--;
		}
		else if (type == 9) 
		{				
			is_audio = 0;
			flags = read_char(fp);
			size--;
			finfo->vFrame++;
			flv_video_size += size;
		}
		else 
		{
			// skip packet 
			fseek(fp, size, SEEK_CUR);
			continue;
		}

		break;
	}
	
    if( is_audio )
	{
        if(finfo->wf.nSamplesPerSec == 0)
		{
            finfo->bAudio = 1;

            finfo->wf.nChannels = ( flags & 1 ) + 1;

            if((flags >> 4) == 5)
                finfo->wf.nSamplesPerSec = 8000;
            else
                finfo->wf.nSamplesPerSec = (44100<<((flags>>2)&3))>>3;

            switch(flags >> 4)	// 0: uncompressed 1: ADPCM 2: mp3 5: Nellymoser 8kHz mono 6: Nellymoser
			{				
			case 0: 
				finfo->wf.wFormatTag = 0x1;	// PCM
				break;
			case 1: 
				finfo->wf.wFormatTag = 0x5346;	// ADPCM SWF - 'SF', pseudo id
				break;
            case 2: 
				finfo->wf.wFormatTag = 0x55;	// MP3
				break;
				// this is not listed at FLV but at SWF, strange...
			case 3: 
				finfo->wf.wFormatTag = 0x1;	// PCM
				break;
            default:	// 5: Nellymoser 8kHz mono 6: Nellymoser
				finfo->wf.wFormatTag = (flags >> 4);
				break;
            }

			finfo->wf.wBitsPerSample = (flags & 2) ? 16 : 8;
		}
		
		fseek(fp, size, SEEK_CUR);	// skip packet data
    }
	else
	{
		finfo->bVideo = 1;
		//flv_video_pts = pts;
		
		switch( flags & 0xF )
		{
		case 2: 
			finfo->bih.biCompression = mmioFOURCC('F','L','V','1'); 
			flv_video_pts = pts;

			// Get WxH
			if( finfo->bih.biWidth == 0 )
			{
				h263_hdr = read_be24(fp);
				size -= 3;
				
				if ( h263_hdr == 0x84 )
				{
					read_tmp = read_word(fp);
					size -= 2;
					format = (read_tmp>>7) & 0x7;
					
					switch (format) 
					{
					case 0:
						read_tmp = (read_tmp<<8) | read_char(fp);
						w = (read_tmp >> 7) & 0xFF;
//						read_tmp = (read_tmp & 0x7F) | read_char(fp);
						read_tmp = ( (read_tmp & 0x7F) << 8 ) | read_char(fp);	// Raymond 2007/11/15
						h = (read_tmp >> 7) & 0xFF;
						size -= 2;
						break;
					case 1:
						read_tmp = (read_tmp<<16) | read_word(fp);
						w = (read_tmp >> 7) & 0xFFFF;
//						read_tmp = (read_tmp & 0x7F) | read_word(fp);
						read_tmp = ( (read_tmp & 0x7F) << 16 ) | read_word(fp);	// Raymond 2007/11/15
						h = (read_tmp >> 7) & 0xFFFF;
						size -= 4;
						break;
					case 2:
						w = 352;
						h = 288;
						break;
					case 3:
						w = 176;
						h = 144;
						break;
					case 4:
						w = 128;
						h = 96;
						break;
					case 5:
						w = 320;
						h = 240;
						break;
					case 6:
						w = 160;
						h = 120;
						break;
					default:
						w = h = 0;
						break;
					}
					finfo->bih.biWidth = w;
					finfo->bih.biHeight = h;
				}
			}
//			break;
			return 0;	// Raymond 2007/11/05 - speed up, but no fps, and bit rate

		case 3: 
			finfo->bih.biCompression = mmioFOURCC('F','S','V','1');		// Flash/Screen Video	
			return 0;
			//break;

		case 4:		// without alpha
		case 5:		// with alpha
			finfo->bih.biCompression = mmioFOURCC('F','L','V','4');		// VP62
			return 0;
			//break;

		default:
			return 0;
//			break;
		}

		fseek(fp, size, SEEK_CUR);	// skip packet data
    }

	return 1;
}

int FLV_Parser(FILE	*fp, FileInfo *finfo)
{
	unsigned int offset, size;
	int flags, ret;
	
    fseek(fp, 4, SEEK_SET);		// skip "FLV" prefix and version

    flags = read_char(fp);
    offset = read_dword(fp);

	if(flags & 0x1)	finfo->bVideo = 1;
	if(flags & 0x4)	finfo->bAudio = 1;

	// Reset
	flv_video_pts = 0;
	flv_video_size = 0;

	while(!feof(fp))
	{
		if( file_error == 1 ) 
			break;
		
		ret = flv_read_packet(fp, finfo);
		if( ret == 0 )	
			break;		// if video not support
	}
	
	// Get info
	if( flv_video_pts != 0 )
	{
		finfo->FPS = (float)( finfo->vFrame * 1000 ) / (float)flv_video_pts;
		finfo->FileDuration = flv_video_pts / 1000;
		finfo->vBitrate = 0.008f * ( (float)flv_video_size / (float)finfo->vFrame ) * finfo->FPS;
	}
	else
	{
		// Get duration
		fseek(fp, finfo->FileSize - 4, SEEK_SET);	// last packet size
		size = read_dword(fp);
		fseek(fp, finfo->FileSize - 3 - size, SEEK_SET);
		if( size == read_be24(fp) + 11)
			finfo->FileDuration = read_be24(fp) / 1000;
//		printf("FLV duration = %d\n", finfo->FileDuration);
	}

	return 1;
}

//////////////////////////////////////////////////////////////////////////////

// detection based on the extension

//int strcasecmp(const char* s1, const char* s2) { return stricmp(s1,s2); }

int file_type_by_filename(const char* filename)
{
	int i;
	char* extension = strrchr(filename,'.');
//	printf("Filename %s ext: %s\n", filename, extension);

	if(extension)
	{
		++extension;

		// Look for the extension in the extensions table
		for( i=0 ; i<(sizeof(extensions_table)/sizeof(extensions_table[0])) ; i++ ) 
		{
//			if( !stricmp(extension, extensions_table[i].extension) ) 
			if( !strcasecmp(extension, extensions_table[i].extension) ) 
			{
//				printf("File type %d based on filename extension\n",extensions_table[i].demuxer_type);
//				return extensions_table[i].demuxer_type;
				return i;
			}
		}
	}
//	return FILE_TYPE_UNKNOWN_FILE;
	return -1;
}

int file_type_by_demuxer(FILE *fp, FileInfo *finfo)
{
	int i;
	int nRet = 0;
	
	fseek(fp, 0, SEEK_SET);

//	for ( i = 1 ; i < 6 ; i++ ) // Raymond 2007/11/06
	for ( i = FILE_TYPE_AVI ; i <= FILE_TYPE_MOV ; i++ )	// Raymond 2007/11/07
	{
		switch(i)
		{
		case FILE_TYPE_AVI :	
			nRet = avi_check_file(fp);	break;
		case FILE_TYPE_MPEG_PS :
#if 1	// Raymond 2007/07/07
			if( finfo->SubFileNameIndex >= 0 && finfo->SubFileNameIndex <= 5)
			{
				nRet = mpg_check_file(fp);				
			}
#endif
			break;
		// Raymond 2007/11/05
		case FILE_TYPE_FLV :
			nRet = flv_check_file(fp);	break;
		// end Raymond 2007/11/05
		case FILE_TYPE_ASF :
			nRet = asf_check_file(fp);	break;
		case FILE_TYPE_MOV :
			nRet = mov_check_file(fp);	break;		
		}
		
		// Raymond 2007/12/21
		if( file_error == 1 )	
			return FILE_TYPE_UNKNOWN_FILE;

		fseek(fp, 0, SEEK_SET);
		if ( nRet == 1 )
			return i;
	}

	return FILE_TYPE_UNKNOWN_FILE;
}

int file_type_by_audio_demuxer(FILE *fp, FileInfo *finfo)
{
	int i;
	int nRet = 0;
	
	fseek(fp, 0, SEEK_SET);

	for ( i = FILE_TYPE_AAC ; i <= FILE_TYPE_OGG ; i++ )	// Raymond 2007/11/07
	{
		switch(i)
		{
		case FILE_TYPE_AAC :
			nRet = aac_check_file(fp, finfo);
			break;
		case FILE_TYPE_MP3 :
			nRet = mp3_check_file(fp, finfo);
			break;
		case FILE_TYPE_WAV :
			nRet = wav_check_file(fp);
			break;
		case FILE_TYPE_OGG :
			nRet = ogg_check_file(fp, finfo);
			break;		
		}
		
		// Raymond 2007/12/21
		if( file_error == 1 )	
			return FILE_TYPE_UNKNOWN_FILE;

		fseek(fp, 0, SEEK_SET);
		if ( nRet == 1 )
			return i;
	}

	return FILE_TYPE_UNKNOWN_FILE;
}

int MediaParser(const char *in_filename, FileInfo *finfo)
{
	// open file
	FILE	*fp = NULL;
	int index = -1, FileType = 0;
	int DemuxerType = 0;
	int nRet = 0;
		
	// check input file
	if((fp = fopen(in_filename, "rb")) == NULL) 
	{ 
//	    printf("FileParser : Unable to open input file %s\n", in_filename); 
		return 0; 
	}

	memset(finfo, 0, sizeof(FileInfo));	// clear
	
	file_error = 0;	// Raymond 2007/12/21

	// get file length
	fseek(fp, 0, SEEK_END);
    finfo->FileSize = ftell(fp);
    fseek(fp, 0, SEEK_SET);

//	printf("File size of %s = %d\n", in_filename, finfo->FileSize);

	// decide file type based on filename extension
	// Raymond 2007/06/14
	index = file_type_by_filename(in_filename);
	if( index < 0 )
		FileType = FILE_TYPE_UNKNOWN_FILE;
	else
		FileType = extensions_table[index].demuxer_type;

	finfo->SubFileNameIndex = index;

	// decide file type based on simple parsing first
//	if( FileType >= 5 )	
	if( FileType >= FILE_TYPE_AAC )	// Raymond 2007/11/07
		DemuxerType = file_type_by_audio_demuxer(fp, finfo);
	else
		DemuxerType = file_type_by_demuxer(fp, finfo);
	
	// Raymond 2007/12/21
	if( file_error == 1 )	
		return PARSER_FILE_NOT_SUPPORTED;
		
	switch(DemuxerType)
	{
	case FILE_TYPE_AVI :
		nRet = AVI_Parser(fp, finfo);
		break;

	case FILE_TYPE_MPEG_PS :

		nRet = MPG_Parser(fp, finfo);

		// Raymond 2007/10/26
		if( nRet && bElementary == 0 )		// Raymond 2007/11/13
			GetMPGDuration(fp, finfo);
			
		break;
	
	case FILE_TYPE_FLV :				// Raymond 2007/11/05
		nRet = FLV_Parser(fp, finfo);
		break;
		
	case FILE_TYPE_ASF :
		nRet = ASF_Parser(fp, finfo);
		break;
	
	case FILE_TYPE_MOV :
		nRet = MOV_Parser(fp, finfo);
		break;

	case FILE_TYPE_AAC :
		nRet = AAC_Parser(fp, finfo);
		break;

	case FILE_TYPE_MP3 :
		nRet = MP3_Parser(fp, finfo);
		break;

	case FILE_TYPE_WAV :
		nRet = WAV_Parser(fp, finfo);
		break;

	case FILE_TYPE_OGG :
		nRet = OGG_Parser(fp, finfo);
		break;

	case FILE_TYPE_UNKNOWN_FILE :		
		break;

	default :
		break;
	}

	finfo->FileType = DemuxerType;

//	print_video_header(&(finfo->bih));
//	print_wave_header (&(finfo->wf));
	
	// Raymond 2007/12/21
	if( file_error == 1 )	
		return PARSER_FILE_NOT_SUPPORTED;
		
	// close file
	fclose(fp);

	// determine support or not
	if( nRet == 0 )
	{
		return PARSER_FILE_NOT_SUPPORTED;
	}
	
	// nRet = 1 
	if ( finfo->bVideo && index < 13 )	// Raymond 2007/06/14
	{	
		if( finfo->bih.biCompression )
		{
			int i = 0;
			int n = sizeof(VideoFourCC) / sizeof(VideoFourCC[0]);
			char fcc4[5] = {0};
			char *fcc = (char *)&(finfo->bih.biCompression);			
			memcpy(fcc4, fcc, 4);
			fcc4[4] = '\0';
												
			for( i = 0 ; i < n ; i++ )
			{
				if ( finfo->FileType == VideoFourCC[i].file_type )
				{					
					if( !strcasecmp(fcc4, VideoFourCC[i].fourcc) ) 	
					{
						finfo->VideoFormat = VideoFourCC[i].video_format;	// Raymond 2007/10/26
#if 1	// Raymond 2007/12/10
						finfo->bVideoSupported = 1;	

						// Raymond 2007/10/26
						if( !strcasecmp(fcc4, "WMV3") )
						{
							if( finfo->bih.biWidth > 320 || finfo->bih.biHeight > 240 )	
								finfo->bVideoSupported = 0;	// if width x height > 320 x 240							
						}
						else if( !strcasecmp(fcc4, "avc1") )	
						{
							if( finfo->H264Profile != 66 )
								finfo->bVideoSupported = 0;	// if H.264 profile not baseline
							else if( finfo->bih.biWidth > 720 || finfo->bih.biHeight > 576 )	// Raymond 2007/09/17
								finfo->bVideoSupported = 0;	// if width x height > 720 x 576
						}
						// Raymond 2008/01/24 - limit M-JPEG resolution to 720x576
						else if( !strcasecmp(fcc4, "MJPG") ||
							 !strcasecmp(fcc4, "jpeg") ||
						  	 !strcasecmp(fcc4, "AVRn") ||
							 !strcasecmp(fcc4, "AVDJ") 
							)	
						{
							if( finfo->bih.biWidth > 720 || finfo->bih.biHeight > 576 )	
								finfo->bVideoSupported = 0;	// if width x height > 720 x 576
						}
						// Raymond 2008/04/28 - HW scalar limitation
						else
						{
							if( finfo->bih.biWidth > 1440 || finfo->bih.biHeight > 960 )	
								finfo->bVideoSupported = 0;	// if width x height > 1440 x 960
						}
						
						break;	
#endif
								
					}
				}
			}
		}

		if( !finfo->bVideoSupported )
			return PARSER_VIDEO_NOT_SUPPORTED;
		else
			return PARSER_VIDEO_FILE_SUPPORTED;
	}
//	else if ( finfo->bAudio )	// Audio only

	if ( finfo->bAudio && index >= 13 )	// Audio only	// Raymond 2007/06/14
	{
		if( finfo->AudioDuration == 0 )
			finfo->AudioDuration = finfo->FileDuration;

		return PARSER_AUDIO_FILE_SUPPORTED;
	}
	else	// no video and audio
	{
		return PARSER_FILE_NOT_SUPPORTED;
	}
}

#if 0

int main(int argc, char* argv[])
{
	if(argc > 3)
	{
		printf("Usage : ParserApp in_filename\n\n");
		return 0 ;
	}

	FileInfo finfo;

	char *in_filename = argv[1];	// input filename
	int nRet = MediaParser(in_filename, &finfo);

	if ( nRet == 0 )
		printf("File not supported\n");
	else if ( nRet == 1 )
		printf("Video [%.4s] of %s not supported\n", (char *)&(finfo.bih.biCompression), file_format_info[finfo.FileType] );
	else if ( nRet == 2 )
		printf("Video file supported : %s - Video : %.4s\n", file_format_info[finfo.FileType], (char *)&(finfo.bih.biCompression) );
	else if ( nRet == 3 )
		printf("Audio file supported : %s\n", file_format_info[finfo.FileType]);
	
	return 0;
}

#endif
