// Utility library for the ARC Pixel Graphics Unit
// ARC International, 2005.
//----------------------------------------------------------------------------------------------------
#include <pgu_lib.h>
/* Flowbee */
#include "fbutil.h"
#include "arc_tcon.h"
//----------------------------------------------------------------------------------------------------
#define DEFAULT_FB 					"/dev/fb0"
#define DEFAULT_UART2 				"/dev/ttyS1"
#define UART2_BAUD_RATE				9600
//----------------------------------------------------------------------------------------------------
static int reset_on_quit = 0;
static int fbfd = -1;
static int uart2_fd = -1;
static arc_pgu safe_pgu;
static arc_pgu user_pgu;
YUV_planes planes;
static unsigned char* baseptr = (unsigned char*)(-1);
struct yuv_info yuv;
unsigned int fb_size = 0x01000000;	//default 16MB

//----------------------------------------------------------------------------------------------------
int  pgu_open_uart2(void)
{
	int ret = -1;    
 	struct termios tio;
 	int rate = UART2_BAUD_RATE;
 	
 	if((uart2_fd = open(DEFAULT_UART2,O_RDWR)) == -1){
		DPRINTF("Can\'t open %s: %s\n", DEFAULT_UART2, strerror(errno));
		return -1;
	}
	else
	{
		//setting UART
		if ( tcgetattr(uart2_fd, &tio ) == -1 ){
			DPRINTF("\n Can't set UART2 byddev/ttyS1\n");
			close(uart2_fd);
			return ret;
		};
		//tyhuang: set baudrate from ioctl directly
		ioctl(uart2_fd, 1, &rate);
		
		//set to no parity check; stopbit = 8; no flow control
		tio.c_lflag &=~(ECHO|ICANON|ISIG|IEXTEN);
		tio.c_lflag  = tio.c_lflag | (FLUSHO);
		tio.c_iflag &=~ (ICRNL|BRKINT|IXON|IXOFF|ISTRIP|INPCK);
		tio.c_iflag  = tio.c_iflag | (IGNBRK|IMAXBEL|IXANY);
		tio.c_cflag &= ~(CSIZE| PARENB);
		tio.c_cflag  = tio.c_cflag | (CS8 |CREAD| CLOCAL);
		tio.c_oflag &= ~(OPOST);
		tio.c_cc[VMIN]	= 0;
		tio.c_cc[VTIME]	= 0;

		if(tcsetattr(uart2_fd, TCSANOW, &tio) == -1){
			return ret;
		}
		ret = 0;//uart2_fd;	// set ok now
	}
	return ret;

}

//----------------------------------------------------------------------------------------------------
int  pgu_read_battery_val(void)
{
	int rcvflag = 0;
	char rcv;
	int b_val = 0;

	//-- High byte first.(bit 8: unused, bit 14~10: version, bit 9~8: MSB 2 bit)
	rcvflag = read(uart2_fd, &rcv, 1);
	if(rcvflag == 0){
		return -1;
	}
	b_val = (int)((rcv & 0x03) << 8);
	//-- Low byte.
	rcvflag = read(uart2_fd, &rcv, 1);
	if(rcvflag == 0){
		return -1;
	}
	b_val += (int)rcv;

	return b_val;
}

//----------------------------------------------------------------------------------------------------
int  pgu_mask_and_set_statctrl(unsigned long mask, unsigned long newbits)
{
	unsigned long statctrl;

	if(fbfd == -1) return -1;
	if(ioctl(fbfd, ARCPGUFB_GET_STATCTRL, &statctrl) == -1) 
	{
		DPRINTF("Can\'t set PGU control reg: ARCPGUFB_GET_STATCTRL ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	//printf("~pgu_mask_and_set_statctrl~\n");
	//printf("statctrl: %x\n", statctrl);
	statctrl &= ~mask;
	//printf("statctrl &= ~maskl: %x\n", statctrl);
	statctrl |= (newbits & mask);
	//printf("statctrl |= (newbits & mask): %x\n", statctrl);
	//  statctrl &= ~STATCTRL_REG_DISP_EN_MASK;

	if(ioctl(fbfd, ARCPGUFB_SET_STATCTRL, &statctrl) == -1) 
	{
		DPRINTF("Can\'t set PGU control reg: ARCPGUFB_SET_STATCTRL ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
int pgu_get_statctrl(unsigned long *dest)
{
	if(ioctl(fbfd, ARCPGUFB_GET_STATCTRL, dest) == -1) 
	{
		DPRINTF("Can\'t get PGU: ARCPGUFB_GET_STATCTRL ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	DPRINTF("statctrl: %x\n", *dest);
	return 0;
}
//----------------------------------------------------------------------------------------------------
int  pgu_open_fb()
{
	if((fbfd = open(DEFAULT_FB, O_RDWR)) == -1) 
	{
		DPRINTF("Can\'t open %s: %s\n", DEFAULT_FB, strerror(errno));
		return -1;
	}
	if(ioctl(fbfd, ARCPGUFB_GET_PGU_STATE, &safe_pgu) == -1) 
	{
		DPRINTF("Can\'t open framebuffer: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
int pgu_get_fb_size(void)
{
	if (ioctl(fbfd, ARCPGUFB_GET_FB_SIZE, &fb_size) == -1)
	{
		DPRINTF("Can\'t get FB_SIZE: ARCPGUFB_GET_FB_SIZE ioctl failed due to %s\n", strerror(errno));
//		fb_size = 0x01000000;
		return -1;
	}
	//printf("%d\n", fb_size);

	return 0;
}
//----------------------------------------------------------------------------------------------------
void  pgu_close_fb()
{
/*   if (reset_on_quit) */
/*   { */
/*     printf("Resetting PGU\n"); */
/*     if (ioctl(fbfd,  ARCPGUFB_SET_PGU_STATE, &safe_pgu) == -1) { */
/*       DPRINTF("close_fb: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno)); */
/*       return -1; */
/*     } */
/*   } */
	if(fbfd != -1) 
  		close(fbfd);
	fbfd = -1;
	//return 0;
}
//----------------------------------------------------------------------------------------------------
unsigned char *pgu_map_buffer(int length, int offset)
{
	if(fbfd == -1) return (unsigned char *)(-1);

	baseptr = mmap(0, length, PROT_READ | PROT_WRITE, MAP_SHARED, fbfd, offset); 

	if((long)baseptr == -1) 
	{
		DPRINTF("Couldn\'t map memory area: %s\n", strerror(errno));
		return (unsigned char *)(-1);
	}
	return baseptr;
}
//----------------------------------------------------------------------------------------------------
int  pgu_unmap_buffer(unsigned char *ptr, int length)
{
	baseptr = (unsigned char *)(-1);
	if(ptr)
	{
		return munmap(ptr, length);
	}
	return -1;
}
//----------------------------------------------------------------------------------------------------
unsigned int  pgu_get_yuv_base(unsigned char bufno)
{
	return ioctl(fbfd, ARCPGUFB_GET_YUV_BASE, bufno); 
}
//----------------------------------------------------------------------------------------------------
int  pgu_get_yuv_buffers(unsigned char *ptr)
{	
	int i;

	if(ioctl(fbfd, ARCPGUFB_GETYUV, &yuv) == -1) 
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_GETYUV ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	for(i = 0; i < yuv.num_buffers; i++)
	{
		planes.Y[i] = ptr + ioctl(fbfd, ARCPGUFB_GET_YUV_BASE, i); 
		planes.U[i] = planes.Y[i] + (yuv.u_offset);
		planes.V[i] = planes.Y[i] + (yuv.v_offset);
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
int  pgu_set_primary_mode(int newmode)
{
	if((newmode != STATCTRL_PIX_FMT_RGB555) && (newmode != STATCTRL_PIX_FMT_YUV420))
		return -1;

	//return  pgu_mask_and_set_statctrl(STATCTRL_PIX_FMT_MASK,newmode);
	if(ioctl(fbfd, ARCPGUFB_SET_MAIN_MODE, &newmode) == -1) 
	{
		DPRINTF("Can\'t set PGU control reg: ARCPGUFB_SET_MAIN_MODE ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
}
//----------------------------------------------------------------------------------------------------
int  pgu_set_overlay_mode(int newmode)
{
	if((newmode != STATCTRL_OL_FMT_RGBA4444) && (newmode != STATCTRL_OL_FMT_RGBA5551) && (newmode != STATCTRL_OL_FMT_RGB555))
		return -1;

	return  pgu_mask_and_set_statctrl(STATCTRL_OL_FMT_MASK, newmode);
}
//----------------------------------------------------------------------------------------------------
int  pgu_enable_overlay(int yesno)
{
	if (yesno){
		if(ioctl(fbfd, ARCPGUFB_ENABLE_OL) == -1) 
		{
			DPRINTF("Can\'t get PGU: ARCPGUFB_ENABLE_OL ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	else{
		/* Flowbee@960424 */
		clear_osd_info();
		if(ioctl(fbfd, ARCPGUFB_DISABLE_OL) == -1) 
		{
			DPRINTF("Can\'t get PGU: ARCPGUFB_DISABLE_OL ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	return 0;
//	return  pgu_mask_and_set_statctrl(STATCTRL_OL_EN_MASK, (yesno ? STATCTRL_OL_EN_MASK : 0));  
}
//----------------------------------------------------------------------------------------------------
int pgu_enable_tv_osd(int yesno)
{
	if (yesno){
		if(ioctl(fbfd, ARCPGUFB_ENABLE_TV_OSD) == -1) 
		{
			DPRINTF("Can\'t get PGU: ARCPGUFB_ENABLE_TV_OSD ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	else{
		/* Flowbee@960424 */
		clear_osd_info();
		if(ioctl(fbfd, ARCPGUFB_DISABLE_TV_OSD) == -1) 
		{
			DPRINTF("Can\'t get PGU: ARCPGUFB_DISABLE_TV_OSD ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	return 0;
}
#if 0
//----------------------------------------------------------------------------------------------------
int  pgu_set_ol_startend(int start, int end)
{
	if(ioctl(fbfd, ARCPGUFB_SET_OL_START, &start) == -1) 
	{
		DPRINTF("Can\'t get PGU: ARCPGUFB_SET_OL_START ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	if(ioctl(fbfd, ARCPGUFB_SET_OL_END, &end) == -1) 
	{
		DPRINTF("Can\'t get PGU: ARCPGUFB_SET_OL_END ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
int pgu_set_ol_offset(unsigned long offset)
{
	if(ioctl(fbfd, ARCPGUFB_SET_OL_OFFSET, &offset) == -1) 
	{
		DPRINTF("Can\'t get PGU: ARCPGUFB_SET_OL_OFFSET ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//----------------------------------------------------------------------------------------------------
int pgu_set_origin(unsigned int x, unsigned int y)
{
	unsigned long xy_offset;
	xy_offset = ENCODE_PGU_DIMS(x, y);

	if(ioctl(fbfd, ARCPGUFB_SET_ORIGIN, &xy_offset) == -1) 
	{ 
		DPRINTF("arcpgufb: ARCPGUFB_SET_ORIGIN ioctl failed due to %s\n", strerror(errno)); 
		return -1; 
	}
	return 0;
}
#endif
//----------------------------------------------------------------------------------------------------
int  pgu_switch_to_buffer(int bufno)
{
	if(ioctl(fbfd, ARCPGUFB_SWITCH_YUV, &bufno) == -1)
	{
		DPRINTF("ARCPGUFB_SWITCH ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
int  pgu_reset()
{
	if(ioctl(fbfd, ARCPGUFB_RESET) == -1)
	{
		DPRINTF("ARCPGUFB_RESET ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
int  pgu_set_resolution(unsigned int x, unsigned int y)
{
	arc_pgu *pgu_ptr;
	//Flowbee@960213	
//	pgu_stop_pgu();

	if(!(pgu_ptr = pgu_get_pgu())) return -1;

	pgu_ptr->frm_dim = ENCODE_PGU_DIMS((x-1), (y-1));
	pgu_ptr->stride = x*2;
//	pgu_set_pgu(pgu_ptr);
//	return  pgu_start_pgu();
	return  pgu_set_pgu(pgu_ptr);
}
//----------------------------------------------------------------------------------------------------
int  pgu_set_yuv_resolution(unsigned int x, unsigned int y)
{
	arc_pgu *pgu_ptr;
	//Flowbee@960213
	//pgu_stop_pgu();

	if(!(pgu_ptr =  pgu_get_pgu())) return -1;

	pgu_ptr->frm_dim = ENCODE_PGU_DIMS((x-1), (y-1));
	pgu_ptr->stride = x;
	pgu_ptr->cru_frm_st = x*y;
	pgu_ptr->cru_stride = x>>1;
	pgu_ptr->crv_frm_st = pgu_ptr->cru_frm_st + ((x>>1)*(y>>1));
	pgu_ptr->crv_stride = x>>1;
//	pgu_set_pgu(pgu_ptr);
//	return  pgu_start_pgu();
	return  pgu_set_pgu(pgu_ptr);
}

//----------------------------------------------------------------------------------------------------
/* Flowbee. */
int  pgu_get_yuv_resolution(unsigned long *buf_x, unsigned long *x, unsigned long *y)
{
	if(ioctl(fbfd, ARCPGUFB_GETYUV, &yuv) == -1) 
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_GETYUV ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	/* Flowbee@960514. */
	*buf_x = yuv.y_stride;		
	*x = yuv.width;
	*y = yuv.height;
	return 0;
}

//----------------------------------------------------------------------------------------------------
/* Flowbee. 2007-11-07 */
int  pgu_get_default_yuv_resolution(unsigned long *x, unsigned long *y)
{
	struct yuv_info tmp_yuv;
	
	if(ioctl(fbfd, ARCPGUFB_GET_DEFAULT_YUV, &tmp_yuv) == -1) 
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_GET_DEFAULT_YUV ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	/* Flowbee@960514. */
	*x = tmp_yuv.width;
	*y = tmp_yuv.height;
	return 0;
}

//----------------------------------------------------------------------------------------------------
/* Flowbee. 2007-11-07 */
int  pgu_get_original_disp_index(unsigned long *idx, unsigned long *x, unsigned long *y)
{
	struct yuv_info tmp_yuv;
	
	if(ioctl(fbfd, ARCPGUFB_GET_DEFAULT_DISP_INDEX, &tmp_yuv) == -1) 
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_GET_DEFAULT_YUV ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	/* Flowbee@960514. */
	*idx = tmp_yuv.num_buffers;	// the original display index
	*x = tmp_yuv.width;			// original width
	*y = tmp_yuv.height;		// original height
	return 0;
}

#if 0
//----------------------------------------------------------------------------------------------------
int pgu_get_colkey(unsigned long *dest)
{
	if(ioctl(fbfd,  ARCPGUFB_GET_OL_COLKEY, dest) == -1) 
	{
		DPRINTF("Can\'t get PGU: ARCPGUFB_GET_COLKEY ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0; 
}
//----------------------------------------------------------------------------------------------------
int  pgu_set_colkey(unsigned long newkey)
{
	if(ioctl(fbfd,  ARCPGUFB_SET_OL_COLKEY, &newkey) == -1) 
	{
		DPRINTF("Can\'t get PGU: ARCPGUFB_SET_COLKEY ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0; 
}
#endif
//----------------------------------------------------------------------------------------------------
int pgu_get_alpha(unsigned long *dest)
{
	if(ioctl(fbfd,  ARCPGUFB_GET_OL_ALPHA, dest) == -1) 
	{
		DPRINTF("Can\'t get PGU: ARCPGUFB_GET_OL_ALPHA ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0; 
}
//----------------------------------------------------------------------------------------------------
int  pgu_set_alpha(unsigned long newkey)
{
	if(ioctl(fbfd,  ARCPGUFB_SET_OL_ALPHA, &newkey) == -1) 
	{
		DPRINTF("Can\'t get PGU: ARCPGUFB_SET_OL_ALPHA ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0; 
}
//----------------------------------------------------------------------------------------------------
//unsigned long pgu_set_clkcfg(unsigned long pxlclk, unsigned long sysclk)
unsigned long pgu_set_clkcfg(unsigned long clk_div)
{
	arc_pgu tmppgu;

	if(ioctl(fbfd, ARCPGUFB_GET_PGU_STATE, &tmppgu) == -1) 
	{
		DPRINTF("Set CLK_CFG: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
		return 0xffffffff;
	}

	if(clk_div != (tmppgu.clk_cfg & 0x0000ffff)){
		tmppgu.clk_cfg = ((((clk_div+1)/2)-1) << 16) | (clk_div);
		pgu_stop_pgu();
		if(ioctl(fbfd, ARCPGUFB_SET_PGU_STATE, &tmppgu) == -1) 
		{
			DPRINTF("Set CLK_CFG: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
			return 0xffffffff;
		}
		pgu_start_pgu();
	}
	return (tmppgu.clk_cfg & 0x0000ffff);
}
//----------------------------------------------------------------------------------------------------
int  pgu_stop_pgu()
{
	int busy = 0;
	unsigned long statctrl;

	if(fbfd == -1) return -1;
	if(ioctl(fbfd, ARCPGUFB_STOP_DISPLAY) == -1) 
	{
		DPRINTF("Can\'t stop PGU: ARCPGUFB_STOP_DISPLAY ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	while(busy) 
	{
		if(ioctl(fbfd, ARCPGUFB_GET_STATCTRL, &statctrl) == -1) 
		{
			DPRINTF("Can\'t stop PGU: ARCPGUFB_GET_STATCTRL ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
		busy = statctrl & STATCTRL_DISP_BUSY_MASK;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
int  pgu_start_pgu()
{
#ifdef FAKEPOWER
	if(isPowerOn == 0)
		return 0;
#endif
	if(ioctl(fbfd, ARCPGUFB_START_DISPLAY) == -1) 
	{
		DPRINTF("Can\'t get PGU: ARCPGUFB_START_DISPLAY ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
int pgu_set_primary_buffer(unsigned long bufno)
{
	if(ioctl(fbfd, ARCPGUFB_SET_RGB_BUF, &bufno) == -1) 
	{
		DPRINTF("Can\'t get PGU: ARCPGUFB_SET_RGB_BUF ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
arc_pgu *pgu_get_pgu()
{
	if(ioctl(fbfd, ARCPGUFB_GET_PGU_STATE, &user_pgu) == -1) 
	{
		DPRINTF("Can\'t get PGU: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
		return 0;
	}
	return &user_pgu;
}
//----------------------------------------------------------------------------------------------------
int  pgu_set_pgu(arc_pgu *newpgu)
{
	if(ioctl(fbfd, ARCPGUFB_SET_PGU_STATE, newpgu) == -1) 
	{
		DPRINTF("Can\'t set PGU: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}
//----------------------------------------------------------------------------------------------------
void pgu_blank_rgb(unsigned long base_offset, int width, int height)
{
	if(baseptr != (unsigned char *)(-1))
		memset(baseptr+base_offset, 0, (width*height*2));
}
//----------------------------------------------------------------------------------------------------
int get_hsync_cycle(arc_pgu* tmppgu)
{
	if(!tmppgu) return -1;

	return (tmppgu->hsync_cfg & PGU_SYNC_REGS_CYC_MASK);
}
//----------------------------------------------------------------------------------------------------
int get_hsync_backporch(arc_pgu *tmppgu)
{
	if(!tmppgu) return -1;

	return ((tmppgu->hsync_cfg & PGU_SYNC_REGS_PORCH_MASK) >> PGU_SYNC_REGS_PORCH_OFFSET);
}
//----------------------------------------------------------------------------------------------------
int get_hsync_width(arc_pgu *tmppgu)
{
	if(!tmppgu) return -1;

	return ((tmppgu->hsync_cfg & PGU_SYNC_REGS_WIDTH_MASK) >> PGU_SYNC_REGS_WIDTH_OFFSET);
}
//----------------------------------------------------------------------------------------------------
unsigned long set_hsync(int cycle, int backporch, int width)
{
	arc_pgu tmppgu;

	if(ioctl(fbfd, ARCPGUFB_GET_PGU_STATE, &tmppgu) == -1) 
	{
		DPRINTF("Set HSYNC: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
		return 0xffffffff;
	}

	tmppgu.hsync_cfg = (((width << PGU_SYNC_REGS_WIDTH_OFFSET) & PGU_SYNC_REGS_WIDTH_MASK)
						| ((backporch << PGU_SYNC_REGS_PORCH_OFFSET) & PGU_SYNC_REGS_PORCH_MASK)
						| (cycle & PGU_SYNC_REGS_CYC_MASK));

	if(ioctl(fbfd, ARCPGUFB_SET_PGU_STATE, &tmppgu) == -1) 
	{
		DPRINTF("Set HSYNC: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
		return 0xffffffff;
	}
	return tmppgu.hsync_cfg;
}
//----------------------------------------------------------------------------------------------------
int get_vsync_cycle(arc_pgu *tmppgu)
{
	if(!tmppgu) return -1;

	return (tmppgu->vsync_cfg & PGU_SYNC_REGS_CYC_MASK);
}
//----------------------------------------------------------------------------------------------------
int get_vsync_backporch(arc_pgu *tmppgu)
{
	if(!tmppgu) return -1;

	return ((tmppgu->vsync_cfg & PGU_SYNC_REGS_PORCH_MASK) >> PGU_SYNC_REGS_PORCH_OFFSET);
}
//----------------------------------------------------------------------------------------------------
int get_vsync_width(arc_pgu *tmppgu)
{
	if(!tmppgu) return -1;

	return ((tmppgu->vsync_cfg & PGU_SYNC_REGS_WIDTH_MASK) >> PGU_SYNC_REGS_WIDTH_OFFSET);
}
//----------------------------------------------------------------------------------------------------
unsigned long set_vsync(int cycle, int backporch, int width)
{
	arc_pgu tmppgu;

	if(ioctl(fbfd, ARCPGUFB_GET_PGU_STATE, &tmppgu) == -1) 
	{
		DPRINTF("Set VSYNC: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
		return 0xffffffff;
	}
	tmppgu.vsync_cfg = (((width << PGU_SYNC_REGS_WIDTH_OFFSET) & PGU_SYNC_REGS_WIDTH_MASK)
						| ((backporch << PGU_SYNC_REGS_PORCH_OFFSET) & PGU_SYNC_REGS_PORCH_MASK)
						| (cycle & PGU_SYNC_REGS_CYC_MASK));

	if(ioctl(fbfd, ARCPGUFB_SET_PGU_STATE, &tmppgu) == -1) 
	{
		DPRINTF("Set VSYNC: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
		return 0xffffffff;
	}
	return tmppgu.vsync_cfg;
}
//----------------------------------------------------------------------------------------------------
void  pgu_dump_controller_regs(arc_pgu *pguptr)
{
	printf("Offset\tReg ID\t\tContents\tSummary\n---------------------------------------------\n\n");
	printf("0x00\tID\t\t0x%08x\tIdentity\n", pguptr->pgu_id);
	printf("0x08\tCLK_CTRL\t0x%08x\tClock Cfg - Divider %d, Pulse width %d\n", pguptr->clk_cfg,
			(pguptr->clk_cfg & CLK_CFG_REG_CLK_DIV_MASK), GET_CLK_CFG_REG_CLK_HIGH_VAL(pguptr->clk_cfg));
	printf("0x10\tDISP_DIM\t0x%08x\tDisplay dimensions - X %d, Y %d\n", pguptr->disp_dim, 
			pguptr->disp_dim & DISP_DIM_REG_PGU_X_RES_MASK, ((pguptr->disp_dim >> 16) & DISP_DIM_REG_PGU_X_RES_MASK));
	printf("0x18\tHSYNC_CFG\t0x%08x\tHSync - Width %d, Back porch %d, Pulse width %d\n", pguptr->hsync_cfg,
			((pguptr->hsync_cfg & PGU_SYNC_REGS_WIDTH_MASK) >> PGU_SYNC_REGS_WIDTH_OFFSET),
			((pguptr->hsync_cfg & PGU_SYNC_REGS_PORCH_MASK) >> PGU_SYNC_REGS_PORCH_OFFSET),
			(pguptr->hsync_cfg & PGU_SYNC_REGS_CYC_MASK));
	printf("0x20\tVSYNC_CFG\t0x%08x\tVSync - Width %d, Back porch %d, Pulse width %d\n", pguptr->vsync_cfg,
			((pguptr->vsync_cfg & PGU_SYNC_REGS_WIDTH_MASK) >> PGU_SYNC_REGS_WIDTH_OFFSET),
			((pguptr->vsync_cfg & PGU_SYNC_REGS_PORCH_MASK) >> PGU_SYNC_REGS_PORCH_OFFSET),
			(pguptr->vsync_cfg & PGU_SYNC_REGS_CYC_MASK));
	printf("0x28\tFRM_ST\t\t0x%08x\tMain start address\n", pguptr->frm_start);
	printf("0x30\tSTRIDE\t\t0x%08x\tMain stride\n", pguptr->stride);
	printf("0x38\tFRM_DIM\t\t0x%08x\tFrame dimensions - X %d, Y %d\n", pguptr->frm_dim,
			(FRM_DIM_REG_FRM_WTH_MASK & pguptr->frm_dim),
			(FRM_DIM_REG_FRM_LNS_MASK & (pguptr->frm_dim >> 16)));
	printf("0x40\tSTAT_CTRL\t0x%08x\tStatus/Control\n", pguptr->statctrl);

	printf("\t\t\t\t\tDISP_EN\t%s\n", ((pguptr->statctrl & STATCTRL_REG_DISP_EN_MASK) ? "ON" : "OFF"));
	printf("\t\t\t\t\tI_EN\t%s\n", ((pguptr->statctrl & STATCTRL_INTR_EN_MASK) ? "ON" : "OFF"));
	printf("\t\t\t\t\tI_CLR\t%s\n", ((pguptr->statctrl & STATCTRL_INTR_CLR_MASK) ? "ON" : "OFF"));
	printf("\t\t\t\t\tCNST\t%s\n", ((pguptr->statctrl & STATCTRL_CONT_B_MASK) ? "ON" : "OFF"));

	printf("\t\t\t\t\tMode\t");
	if (!(pguptr->statctrl & STATCTRL_PIX_FMT_MASK)) printf("RGB555\n");
	else if (pguptr->statctrl & STATCTRL_PIX_FMT_YUV420) printf("YUV420\n");
	else printf("Unrecognised\n");

	printf("\t\t\t\t\tBL\t%s\n", ((pguptr->statctrl & STATCTRL_BACKLIGHT_MASK) ? "ON" : "OFF"));
	printf("\t\t\t\t\tX2\t%s\n", ((pguptr->statctrl & STATCTRL_X2_MASK) ? "ON" : "OFF"));
	printf("\t\t\t\t\tRS\t%s\n", ((pguptr->statctrl &  STATCTRL_REV_LN_SCAN_MASK) ? "ON" : "OFF"));
	printf("\t\t\t\t\tHPOL\t%s\n", ((pguptr->statctrl & STATCTRL_HPOL_MASK) ? "ON" : "OFF"));
	printf("\t\t\t\t\tVPOL\t%s\n", ((pguptr->statctrl & STATCTRL_VPOL_MASK) ? "ON" : "OFF"));
	printf("\t\t\t\t\tDPOL\t%s\n", ((pguptr->statctrl & STATCTRL_DPOL_MASK) ? "ON" : "OFF"));
	printf("\t\t\t\t\tOL_EN\t%s\n", ((pguptr->statctrl & STATCTRL_OL_EN_MASK) ? "ON" : "OFF"));

	printf("\t\t\t\t\tOL Mode\t");
	if (!(pguptr->statctrl & STATCTRL_OL_FMT_MASK)) printf("RGBA4444\n");
	else if (pguptr->statctrl & STATCTRL_OL_FMT_RGBA5551) printf("RGBA5551\n");
	else if (pguptr->statctrl & STATCTRL_OL_FMT_RGB555) printf("RGB555\n");
	else printf("Unrecognised\n");

	printf("\t\t\t\t\tDB\t%s\n", ((pguptr->statctrl & STATCTRL_DISP_BUSY_MASK) ? "ON" : "OFF"));
	printf("\t\t\t\t\tBB\t%s\n", ((pguptr->statctrl & STATCTRL_BU_BUSY_MASK) ? "ON" : "OFF"));
	printf("\t\t\t\t\tI_FLG\t%s\n", ((pguptr->statctrl & STATCTRL_INTR_FLAG) ? "ON" : "OFF"));

	printf("0x48\tCRU_FRM_ST\t0x%08x\tU offset\n", pguptr->cru_frm_st);
	printf("0x50\tCRU_STRIDE\t0x%08x\tU stride\n", pguptr->cru_stride);
	printf("0x58\tCRV_FRM_ST\t0x%08x\tV offset\n", pguptr->crv_frm_st);
	printf("0x60\tCRV_STRIDE\t0x%08x\tV stride\n", pguptr->crv_stride);

	printf("0x68\tOL_FRM_ST\t0x%08x\tOverlay start address\n", pguptr->ol_frm_st);
	printf("0x70\tOL_STRIDE\t0x%08x\tOverlay stride\n", pguptr->ol_stride);
	printf("0x78\tOL_SCN_LINE\t0x%08x\tOverlay start/end line - Start %d, End %d\n", pguptr->ol_scn_line,
			(FRM_DIM_REG_FRM_WTH_MASK & pguptr->ol_scn_line),
			(FRM_DIM_REG_FRM_LNS_MASK & (pguptr->ol_scn_line >> 16)));
	printf("0x80\tOL_COL_KEY\t0x%08x\tOverlay colour key\n", pguptr->ol_col_key);

	printf("0x88\tDISPLAY_OFFSET\t0x%08x\tDisplay offset - X %d, Y %d\n", pguptr->disp_dim, 
	pguptr->centreoffsets & DISP_DIM_REG_PGU_X_RES_MASK, ((pguptr->centreoffsets >> 16) & DISP_DIM_REG_PGU_X_RES_MASK));
	printf("\n\n");
}
//----------------------------------------------------------------------------------------------------
void  pgu_display_controller()
{
	arc_pgu pgu;

	if(ioctl(fbfd, ARCPGUFB_GET_PGU_STATE, &pgu) == -1) 
	{
		DPRINTF("Can\'t display PGU: ARCPGUFB_GETPGU_STATE ioctl failed due to %s\n", strerror(errno));
		return;
	}
	pgu_dump_controller_regs(&pgu);
}
//----------------------------------------------------------------------------------------------------
void set_ir_value(unsigned long value)
{
	ioctl(fbfd, ARCPGUFB_SET_IRVALUE, &value);
}
//----------------------------------------------------------------------------------------------------
unsigned long get_ir_value()
{
	unsigned long value;
	ioctl(fbfd, ARCPGUFB_GET_IRVALUE, &value);
	return value;
}
//----------------------------------------------------------------------------------------------------
int  pgu_enable_backlight(int yesno)
{
	if (yesno)
	{
		if(ioctl(fbfd, ARCPGUFB_BKLGHT_ON) == -1) 
		{
			DPRINTF("Can\'t get PGU: ARCPGUFB_ENABLE_OL ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	else
	{
		if(ioctl(fbfd, ARCPGUFB_BKLGHT_OFF) == -1) 
		{
			DPRINTF("Can\'t get PGU: ARCPGUFB_DISABLE_OL ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
	}
	return 0;
}

//-----------------------------------------------------------------------------
int  pgu_update_osd(struct _osd_info info)
{
#if 0
	printf("\npgu_update_osd: \n");
	printf("info.rgb_src = %d\n", info.rgb_src);
	printf("info.yuv_src = %d\n", info.yuv_src);	
	printf("info.yuv_dest = %d\n", info.yuv_dest);
	printf("info.yuv_bkg = %d\n", info.yuv_bkg);	
	printf("info.yuv_osd = %d\n", info.yuv_osd);
	printf("info.start_line = %d\n", info.start_line);	
	printf("info.total_line = %d\n", info.rgb_src);
	printf("info.current_mode = %d\n", info.current_mode);
#endif
	
	if(ioctl(fbfd, ARCPGUFB_UPDATE_OSD, &(info)) == -1) 
	{
		DPRINTF("Can\'t update OSD: ARCPGUFB_UPDATE_OSD ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_reset_buf_resolution(unsigned short x, unsigned short y)
{
	unsigned long i;
	/* Get frame buffer resolution. */
	if(ioctl(fbfd, ARCPGUFB_GETYUV, &yuv) == -1) 
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_GETYUV ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
#if 0
	/* Check resolution. */
	if ( (yuv.y_stride != x) || (yuv.width != x) || (yuv.height != y) ){
		yuv.width = x;
		yuv.height = y;
		/* Stop the PGU. */
		pgu_stop_pgu();
		/* Reset frame buffer resolution. */
		if(ioctl(fbfd, ARCPGUFB_SET_YUV_RES, &yuv) == -1){
			DPRINTF("Can\'t set YUV info: ARCPGUFB_SET_YUV_RES ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
		pgu_set_origin(0,0);		
		pgu_start_pgu();		
		return 1;
	}
#else
	arc_pgu *pgu_ptr;

	if(!(pgu_ptr =  pgu_get_pgu())){
		pgu_start_pgu();
		return -1;		
	} 

	if(	(pgu_ptr->frm_dim != ENCODE_PGU_DIMS((x-1), (y-1))) ||
		(pgu_ptr->stride != x) ||
		(pgu_ptr->cru_frm_st != x*y) ||
		(pgu_ptr->cru_stride != x>>1) ||
		(pgu_ptr->crv_frm_st != pgu_ptr->cru_frm_st + ((x>>1)*(y>>1))) ||
		(pgu_ptr->crv_stride != x>>1) ){

		yuv.width = x;
		yuv.height = y;
		/* Stop the PGU. */
		//pgu_stop_pgu();
		/* Reset frame buffer resolution. */
		if(ioctl(fbfd, ARCPGUFB_SET_YUV_RES, &yuv) == -1){
			DPRINTF("Can\'t set YUV info: ARCPGUFB_SET_YUV_RES ioctl failed due to %s\n", strerror(errno));
			return -1;
		}
		//pgu_set_origin(0,0);		
	}
	//pgu_start_pgu();
#endif
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_disable_mute_vol_osd(void)
{
	if(ioctl(fbfd, ARCPGUFB_CLEAR_MUTE_VOL) == -1) 
	{
		DPRINTF("Can\'t update OSD: ARCPGUFB_CLEAR_MUTE_VOL ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_enable_mute_vol_osd(unsigned long _type)
{
	if(ioctl(fbfd, ARCPGUFB_SHOW_MUTE_VOL, &(_type)) == -1) 
	{
		DPRINTF("ARCPGUFB_SHOW_MUTE_VOL ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_show_icon(unsigned long _type)
{
	if(ioctl(fbfd, ARCPGUFB_SHOW_ICON, &(_type)) == -1) 
	{
		DPRINTF("ARCPGUFB_SHOW_ICON ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_clear_icon(void)
{
	if(ioctl(fbfd, ARCPGUFB_CLEAR_ICON) == -1) 
	{
		DPRINTF("ARCPGUFB_CLEAR_ICON ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_show_mplayer_time(char _time[])
{
	if(ioctl(fbfd, ARCPGUFB_SHOW_VIDEO_TIME, _time) == -1) 
	{
		DPRINTF("ARCPGUFB_SHOW_VIDEO_TIME ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_clear_mplayer_time(void)
{
	if(ioctl(fbfd, ARCPGUFB_CLEAR_VIDEO_TIME) == -1) 
	{
		DPRINTF("ARCPGUFB_CLEAR_VIDEO_TIME ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_show_mplayer_speed(char _time[])
{
	if(ioctl(fbfd, ARCPGUFB_SHOW_VIDEO_SPEED, _time) == -1) 
	{
		DPRINTF("ARCPGUFB_SHOW_VIDEO_SPEED ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_clear_mplayer_speed(void)
{
	if(ioctl(fbfd, ARCPGUFB_CLEAR_VIDEO_SPEED) == -1) 
	{
		DPRINTF("ARCPGUFB_CLEAR_VIDEO_SPEED ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_set_mplayer_time_on_off(unsigned long _val)
{
	if(ioctl(fbfd, ARCPGUFB_SET_VIDEO_TIME_ONOFF, &_val) == -1) 
	{
		DPRINTF("ARCPGUFB_SET_VIDEO_TIME_ONOFF ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_get_mplayer_time_on_off(void)
{
 	unsigned long _on;
 	
	if(ioctl(fbfd, ARCPGUFB_GET_VIDEO_TIME_ONOFF, &_on) == -1) 
	{
		DPRINTF("ARCPGUFB_GET_VIDEO_TIME_ONOFF ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return ((int)_on);
}

//-----------------------------------------------------------------------------
int pgu_show_mplayer_subtitle(char _s[][31])
{
	if(ioctl(fbfd, ARCPGUFB_SHOW_VIDEO_SUBTITLE, _s) == -1) 
	{
		DPRINTF("ARCPGUFB_SHOW_VIDEO_SUBTITLE ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_clear_mplayer_subtitle(void)
{
	if(ioctl(fbfd, ARCPGUFB_CLEAR_VIDEO_SUBTITLE) == -1) 
	{
		DPRINTF("ARCPGUFB_CLEAR_VIDEO_SUBTITLE ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_clear_lcd_osd(void)
{
	if(ioctl(fbfd, ARCPGUFB_CLEAR_LCD_OSD) == -1) 
	{
		DPRINTF("ARCPGUFB_CLEAR_LCD_OSD ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_set_backlight(unsigned long _val)
{
	if(ioctl(fbfd, ARCPGUFB_SET_PWM, &(_val)) == -1) 
	{
		DPRINTF("ARCPGUFB_SET_PWM ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_set_contrast(unsigned long _val)
{
	if(ioctl(fbfd, ARCPGUFB_SET_CONTRAST, &(_val)) == -1) 
	{
		DPRINTF("ARCPGUFB_SET_CONTRAST ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_get_display_index(void)
{
	unsigned long _idx = 0xff;

	if(ioctl(fbfd, ARCPGUFB_GET_DISP_INDEX, &(_idx)) == -1) 
	{
		DPRINTF("ARCPGUFB_GET_DISP_INDEX ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return ((int)_idx);
}

//-----------------------------------------------------------------------------
int pgu_set_display_index(unsigned long _idx, unsigned long _on)
{
	unsigned long idx;
	
	idx = ((_on & 0x01) << 16) | (_idx);
	
	if(ioctl(fbfd, ARCPGUFB_SET_DISP_INDEX, &(idx)) == -1)
	{
		DPRINTF("ARCPGUFB_GET_DISP_INDEX ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_get_tv_interface(void)
{
	unsigned long _idx = 0xff;
	
	if(ioctl(fbfd, ARCPGUFB_GET_TV_INTERFACE, &(_idx)) == -1)
	{
		DPRINTF("ARCPGUFB_GET_TV_INTERFACE ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return ((int)_idx);
}

//-----------------------------------------------------------------------------
int pgu_set_tv_interface(unsigned long _idx)
{
	if(ioctl(fbfd, ARCPGUFB_SET_TV_INTERFACE, &(_idx)) == -1)
	{
		DPRINTF("ARCPGUFB_SET_TV_INTERFACE ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//----------------------------------------------------------------------------------------------------
void set_mplayer_play_time(unsigned long _time)
{
	ioctl(fbfd, ARCPGUFB_SET_MPLAYER_PLAY_TIME, &_time);
}

//----------------------------------------------------------------------------------------------------
unsigned long get_mplayer_play_time()
{
	unsigned long _time;

	ioctl(fbfd, ARCPGUFB_GET_MPLAYER_PLAY_TIME, &_time);
	return _time;
}

//----------------------------------------------------------------------------------------------------
void set_mplayer_status(int _status)
{
	ioctl(fbfd, ARCPGUFB_SET_MPLAYER_STATUS, &_status);
}

//----------------------------------------------------------------------------------------------------
int get_mplayer_status()
{
	int _status;

	ioctl(fbfd, ARCPGUFB_GET_MPLAYER_STATUS, &_status);
	return _status;
}

//----------------------------------------------------------------------------------------------------
unsigned long get_mplayer_play_duration()
{
	unsigned long _time;
	ioctl(fbfd, ARCPGUFB_GET_MPLAYER_DURATION, &_time);
	return _time;
}

//----------------------------------------------------------------------------------------------------
int pgu_set_mplayer_advertisment(unsigned long _yesno)
{
	if(ioctl(fbfd, ARCPGUFB_MPLAYER_ADVERTISMENT, &(_yesno)) == -1)
	{
		DPRINTF("ARCPGUFB_MPLAYER_ADVERTISMENT ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//----------------------------------------------------------------------------------------------------
int pgu_set_advertisment_update(unsigned long _yesno)
{
	if(ioctl(fbfd, ARCPGUFB_ADVERTISMENT_UPDATE, &(_yesno)) == -1)
	{
		DPRINTF("ARCPGUFB_ADVERTISMENT_UPDATE ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//----------------------------------------------------------------------------------------------------
int pgu_set_advertisment_video_info(unsigned short _co_x, unsigned short _co_y)
{
	struct area_info _v_info;
	
	_v_info.co_x = _co_x;
	_v_info.co_y = _co_y;
	_v_info.res_w = 0;
	_v_info.res_h = 0;
	
	if(ioctl(fbfd, ARCPGUFB_SET_ADVER_VIDEO_INFO, &_v_info) == -1)
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_SET_ADVER_VIDEO_INFO ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//----------------------------------------------------------------------------------------------------
int pgu_get_advertisment_video_info(unsigned short *_co_x, unsigned short *_co_y)
{
	struct area_info _v_info;
	
	if(ioctl(fbfd, ARCPGUFB_GET_ADVER_VIDEO_INFO, &_v_info) == -1)
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_GET_ADVER_VIDEO_INFO ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	*_co_x = _v_info.co_x;
	*_co_y = _v_info.co_y;
	
	return 0;
}

//----------------------------------------------------------------------------------------------------
int pgu_set_advertisment_jpeg_info(unsigned short _src_co_x, unsigned short _src_co_y,
		unsigned short _dest_co_x, unsigned short _dest_co_y, unsigned short _len_w, unsigned short _len_h)
{
	struct update_info _j_info;
	
	_j_info.src_co_x = _src_co_x;
	_j_info.src_co_y = _src_co_y;
	_j_info.dst_co_x = _dest_co_x;
	_j_info.dst_co_y = _dest_co_y;
	_j_info.res_w = _len_w;
	_j_info.res_h = _len_h;
	
	if(ioctl(fbfd, ARCPGUFB_SET_ADVER_JPEG_INFO, &_j_info) == -1)
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_SET_ADVER_JPEG_INFO ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//----------------------------------------------------------------------------------------------------
int pgu_get_advertisment_jpeg_info(struct update_info *_j_info)
{
	if(ioctl(fbfd, ARCPGUFB_GET_ADVER_JPEG_INFO, _j_info) == -1)
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_GET_ADVER_JPEG_INFO ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//----------------------------------------------------------------------------------------------------
int pgu_set_net_config_info(struct net_config _n_info)
{
	if(ioctl(fbfd, ARCPGUFB_SET_NET_CONFIG_INFO, &_n_info) == -1)
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_SET_ADVER_JPEG_INFO ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//----------------------------------------------------------------------------------------------------
int pgu_get_net_config_info(struct net_config *_n_info)
{
	if(ioctl(fbfd, ARCPGUFB_GET_NET_CONFIG_INFO, _n_info) == -1)
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_GET_ADVER_JPEG_INFO ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//----------------------------------------------------------------------------------------------------
int pgu_tcon_set_cmd(unsigned long _val)
{
	if(ioctl(fbfd, ARCPGUFB_TCON_SET_COMMAND, &_val) == -1)
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_TCON_SET_COMMAND ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//----------------------------------------------------------------------------------------------------
int pgu_tcon_set_osd_line_data(struct _osd_line *_info)
{
	if(ioctl(fbfd, ARCPGUFB_TCON_SET_OSD_LINE_DATA, _info) == -1)
	{
		DPRINTF("Can\'t get YUV info: ARCPGUFB_TCON_SET_OSD_LINE_DATA ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_set_osd_speed_info(struct _terawins_osd_info *_info)
{
	//printf("\npgu_set_osd_speed: %x\n", _info->ram_addr);
	if(ioctl(fbfd, ARCPGUFB_SET_OSD_SPEED_INFO, _info) == -1) 
	{
		DPRINTF("ARCPGUFB_SET_OSD_SPEED_INFO ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_set_osd_subtitle_info(struct _terawins_osd_info *_info)
{
	//printf("\npgu_set_osd_speed: %x\n", _info->ram_addr);
	if(ioctl(fbfd, ARCPGUFB_SET_OSD_SUBTITLE_INFO, _info) == -1) 
	{
		DPRINTF("ARCPGUFB_SET_OSD_SUBTITLE_INFO ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//-----------------------------------------------------------------------------
int pgu_set_tcon_scale_table(int *_tbl)
{
	//printf("\npgu_set_osd_speed: %x\n", _info->ram_addr);
	if(ioctl(fbfd, ARCPGUFB_SET_TCON_SCALE_TABLE, _tbl) == -1) 
	{
		DPRINTF("ARCPGUFB_SET_TCON_SCALE_TABLE ioctl failed due to %s\n", strerror(errno));
		return -1;
	}
	return 0;
}

//----------------------------------------------------------------------------------------------------
void test_func(unsigned int _val)
{
	unsigned int value = _val;
	
	if (ioctl(fbfd,  ARCPGUFB_TEST_MODE, &value) == 1)
		DPRINTF("App test error : %s\n", strerror(errno));
}



