#ifndef PGU_LIB_H
#define PGU_LIB_H

#include "types.h"
#include "arc_pgu.h"
#include "arc_tcon.h"

#define ENCODE_RGBA5551(r,g,b,a) (((a) << 15) | (((r) >> 3) << 10) | (((g) >> 3) << 5) | ((b) >> 3))
#define ENCODE_RGB555(r,g,b) ((((r) >> 3) << 10) | (((g) >> 3) << 5) | ((b) >> 3))
#define ENCODE_RGBA4444(r,g,b,a) ((((a) >> 4) << 12) | (((r) >> 4) << 8) | (((g) >> 4) << 4) | ((b) >> 4))
#define RGB_555_REDCONTENT(pixel) (((pixel >> 7) & 0xf8))
#define RGB_555_GREENCONTENT(pixel) (((pixel >> 2) & 0xf8))
#define RGB_555_BLUECONTENT(pixel) ((pixel & 0x1f) << 3)

#define NUMBUFS 40
typedef struct
{
	volatile void* Y[NUMBUFS];
	volatile void* U[NUMBUFS];
	volatile void* V[NUMBUFS];
} YUV_planes;

extern YUV_planes planes;
extern struct yuv_info yuv;
extern unsigned int fb_size;	
#ifdef FAKEPOWER
extern int isPowerOn;
#endif

int pgu_mask_and_set_statctrl(unsigned long mask, unsigned long newbits);
int pgu_get_statctrl(unsigned long *dest);
int  pgu_open_fb();
void  pgu_close_fb();
unsigned char *pgu_map_buffer(int length, int offset);
int  pgu_unmap_buffer(unsigned char *ptr, int length);
unsigned int  pgu_get_yuv_base(unsigned char bufno);
int  pgu_get_yuv_buffers(unsigned char* ptr);
int  pgu_set_primary_mode(int newmode);
int  pgu_set_overlay_mode(int newmode);
int  pgu_enable_overlay(int yesno);
int  pgu_enable_tv_osd(int yesno);
int  pgu_update_osd(struct _osd_info info);
int  pgu_set_resolution(unsigned int x, unsigned int y);
int  pgu_set_yuv_resolution(unsigned int x, unsigned int y);
int  pgu_get_yuv_resolution(unsigned long *buf_x, unsigned long *x, unsigned long *y);
int  pgu_get_default_yuv_resolution(unsigned long *x, unsigned long *y);
int  pgu_get_original_disp_index(unsigned long *idx, unsigned long *x, unsigned long *y);
void  pgu_display_controller();
int  pgu_switch_to_buffer(int bufno);
arc_pgu *pgu_get_pgu();
int  pgu_set_pgu(arc_pgu* newpgu);
int  pgu_reset();
int  pgu_stop_pgu();
int  pgu_start_pgu();
int  pgu_get_colkey(unsigned long *dest);
int  pgu_set_colkey(unsigned long newkey);
int  pgu_get_alpha(unsigned long *dest);
int  pgu_set_alpha(unsigned long newkey);
int  pgu_set_ol_startend(int start, int end);
unsigned long pgu_set_clkcfg(unsigned long clk_div);
int pgu_set_ol_offset(unsigned long offset);
int pgu_set_origin(unsigned int x, unsigned int y);
int pgu_set_primary_buffer(unsigned long bufno);
void pgu_blank_rgb(unsigned long base_offset, int width, int height);
void  pgu_dump_controller_regs(arc_pgu *pguptr);
int get_hsync_cycle(arc_pgu* tmppgu);
int get_hsync_backporch(arc_pgu* tmppgu);
int get_hsync_width(arc_pgu* tmppgu);
unsigned long set_hsync(int cycle, int backporch, int width);
int get_vsync_cycle(arc_pgu* tmppgu);
int get_vsync_backporch(arc_pgu* tmppgu);
int get_vsync_width(arc_pgu* tmppgu);
unsigned long set_vsync(int cycle, int backporch, int width);

void set_ir_value(unsigned long value);
unsigned long get_ir_value();

int pgu_reset_buf_resolution(unsigned short x, unsigned short y);
int pgu_get_fb_size(void);//add by bk@070508 - adding for get real fb_size
int pgu_set_backlight(unsigned long _val);
int pgu_set_contrast(unsigned long _val);
int pgu_enable_mute_vol_osd(unsigned long _type);
int pgu_disable_mute_vol_osd(void);
int pgu_show_mplayer_time(char _time[]);
int pgu_clear_mplayer_time(void);
int pgu_set_mplayer_time_on_off(unsigned long _val);
int pgu_get_mplayer_time_on_off(void);
int pgu_show_mplayer_speed(char _time[]);
int pgu_clear_mplayer_speed(void);
int pgu_show_mplayer_subtitle(char _s[][31]);
int pgu_clear_mplayer_subtitle(void);
int pgu_clear_lcd_osd(void);
int pgu_get_display_index(void);
//int pgu_set_display_index(unsigned long _idx);
int pgu_set_display_index(unsigned long _idx, unsigned long _on);
int pgu_get_tv_interface(void);
int pgu_set_tv_interface(unsigned long _idx);
void set_mplayer_play_time(unsigned long _time);
unsigned long get_mplayer_play_time(void);
//-- Flowbee 2008-05-20
void set_mplayer_status(int _status);
int get_mplayer_status();
unsigned long get_mplayer_play_duration(void);
//-- Flowbee 2007-11-13	Advertisment
int pgu_set_mplayer_advertisment(unsigned long _yesno);
int pgu_set_advertisment_update(unsigned long _yesno);
int pgu_set_advertisment_video_info(unsigned short _co_x, unsigned short _co_y);
int pgu_set_advertisment_jpeg_info(unsigned short _src_co_x, unsigned short _src_co_y,
		unsigned short _dest_co_x, unsigned short _dest_co_y, unsigned short _len_w, unsigned short _len_h);

int pgu_set_osd_speed_info(struct _terawins_osd_info *_info);
int pgu_set_osd_subtitle_info(struct _terawins_osd_info *_info);
int pgu_set_tcon_scale_table(int *_tbl);

int pgu_tcon_set_cmd(unsigned long _val);

//-- Flowbee 2007-11-08
void test_func(unsigned int _val);

#endif
