#include "libcurl/curl.h"
#include "libexpat/expat.h"
#include "sky_rss.h"
#include "sky_lib.h"

#ifdef XML_LARGE_SIZE
#if defined(XML_USE_MSC_EXTENSIONS) && _MSC_VER < 1400
#define XML_FMT_INT_MOD "I64"
#else
#define XML_FMT_INT_MOD "ll"
#endif
#else
#define XML_FMT_INT_MOD "l"
#endif

unsigned char main_rss_data_buffer[RSS_PLAYER_SIZE];
RSS_PLAYER_ID rss_player = (RSS_PLAYER_ID)&(main_rss_data_buffer);

#define XML_BUFFER_SIZE 2048
char xml_buf[XML_BUFFER_SIZE];

int download_content(char *url, char *file_name, int timeout)
{
    FILE *p_file;
    CURLcode ret = 0;
	char timeout_str[8];
	int i = 0;
	FILE *chk_file;
	pid_t pid;
	char str[80];
	int status;
    
    //DBG_MSG("<sky_rss> target url : %s\n", url);
    DBG_MSG("<sky_rss> saving content to %s, timeout = %d\n", file_name, timeout);
    
    /*rywu20080715
    p_file = fopen(file_name, "w+");
    if(p_file == 0) 
    {
        printf("<sky_rss>    can't open file to download image\n");
        return -1;
    }
    */
    
    ret = -1;
    pid = fork();
    switch(pid)
    {
    	case -1:
    		DBG_MSG("<sky_rss>    fork failed on download_content\n");
    		break;
    		
    	case 0:
    		//child process
    		sprintf(timeout_str, "%d", timeout);
    		execl("/sky/curl", "curl", url, file_name, timeout_str, NULL);
        	_exit(0);
        	
    	default:
    		//parent process
    		i=0;
            while(i++ < timeout)//wait for 2 seconds as most
            {
                sleep(1);
                if(waitpid(pid, &status, WNOHANG))
                {
                    break;
                }
            }
            
            if(WIFEXITED(status))
            {
                chk_file = fopen("/tmp/curl_working", "r");
                if(chk_file != 0) 
                {
                    fgets(str, 80, chk_file);  
                    ret = atoi(str);
                    fclose(chk_file);
                    DBG_MSG("<sky_rss> done(%d)\n", (int)ret);
                }
                unlink("/tmp/curl_working");
                system("sync;sync");
            }
    		break;
    }
    /*rywu20080715 
    fclose(p_file);
    
    //rywu20080709
    if (ret != CURLE_OK)
    {
        unlink(file_name);
        system("sync;sync");
    }*/
    
	DBG_MSG("<sky_rss> download_content done(%d) <--\n", (int)ret);
	return (int)ret;
}	
	
int parse_option(RSS_PLAYER_ID rss_player, int argc, char **argv)
{
    int argn;
    char *arg;
    UINT8 result = 0;
    
    /* Scan command line options, adjust parameters */
    for (argn = 1; argn < argc && !result; argn++)
    {
        arg = argv[argn];
        if(*arg != '-')
        {
            /* Not a switch, must be a file name argument */
            break;			/* else done parsing switches */
        }
        arg++;			/* advance past switch marker character */
        argn++;
        
        switch(*arg)
        {
            case 'd':
                sprintf(rss_player->main_path, "%s", argv[argn]);
                DBG_MSG("<sky_rss> dir_prefix = %s\n", rss_player->main_path);
                break;

            case 's':
                sprintf(rss_player->sub_path, "%s", argv[argn]);
                DBG_MSG("<sky_rss> sub_path = %s\n", rss_player->sub_path);
                break;
                    
            case 'f':
                //set rss config file which contains channel information
                sprintf(rss_player->rss_config_file, "%s", argv[argn]);
                DBG_MSG("<sky_rss> rss_config_file = %s\n", rss_player->rss_config_file);
                
                if (parse_xml_file(rss_player, rss_player->rss_config_file, 0))
            	{
            	    notify_server(rss_player, IPC_CONFIG_FILE_BROKEN, NULL);
            	    result = -1;
                }
                DBG_MSG("<sky_rss> total channels = %d\n", rss_player->channel_num);
                break;
                
            case 'r':
                rss_player->recycle_mode = (UINT8)atoi(argv[argn]);
                DBG_MSG("<sky_rss> file recycle mode = %d\n", rss_player->recycle_mode);
                break;
                    
            default:
                //DBG_MSG("<sky_rss> unknow option = %c.", *arg);
                result = -1;
                notify_server(rss_player, IPC_INVALID_ARGUMENT, NULL);
                break;
        }
    }
    
    //set download folder
    sprintf(rss_player->download_dir, "%s%s", rss_player->main_path, rss_player->sub_path);
    DBG_MSG("<sky_rss> download path = %s\n", rss_player->download_dir);

    return result;
}

int init_IPC(RSS_PLAYER_ID rss_player)
{
    DBG_MSG("<sky_rss> init_IPC -->\n");
	/*rywu20080709
    rss_player->rx_msg_id = msgget((key_t)3014, 0660|IPC_CREAT);
    if(rss_player->rx_msg_id == -1)
	{
        DBG_MSG("<sky_rss> init_IPC <-- rx_msg_id msgget failed.\n");
		return -1;
	}
	*/
	//rywu20080709++
	while(1)
	{
        rss_player->rx_msg_id = msgget((key_t)3014, 0660|IPC_CREAT);
        if(rss_player->rx_msg_id == -1)
    	{
            DBG_MSG("<sky_rss> init_IPC <-- rx_msg_id msgget failed.\n");
    		sleep(1);
    		continue;
    	}
    	else
    	{
    		DBG_MSG("<sky_rss> init_IPC <-- rx_msg_id ok.\n");
    		break;
		}
    }
    //rywu20080709--
    
    pthread_create(&(rss_player->signal_thread), NULL, (void *)(&signal_thread), rss_player);//rywu20080616
    
    while(1)
	{
        rss_player->tx_msg_id = msgget((key_t)3012, 0660|IPC_CREAT);
        if(rss_player->tx_msg_id == -1)
    	{
            DBG_MSG("<sky_rss> init_IPC <-- tx_msg_id msgget failed.\n");
    		sleep(1);
    		continue;
    	}
    	else
    	{
    		DBG_MSG("<sky_rss> init_IPC <-- tx_msg_id ok.\n");
    		break;
		}
    }
    
	return 0;
}

void close_IPC(RSS_PLAYER_ID rss_player)
{
    rss_player->signal_thread_state = 0;
	pthread_join(rss_player->signal_thread, &(rss_player->signal_thread_result));
	
	/*rywu20080709 let skydemo controlling create/delete of all message queues
	if (msgctl(rss_player->rx_msg_id, IPC_RMID, 0) == -1)
    {
        DBG_MSG("<sky_rss> msgctl(IPC_RMID) failed.\n");
	}*/
}

void signal_thread(RSS_PLAYER_ID rss_player)
{
#ifdef _DEBUG
    int i;
#endif

    long int msg_to_receive;
    MSG_DATA msg_data;
    INTERNAL_MESSAGE_ID message_block;

	DBG_MSG("<sky_rss> signal_thread -->(%d)\n", getpid());
	
	rss_player->signal_thread_state = 1;
	while(rss_player->signal_thread_state) 
    {
        msg_to_receive = 2;
        if (msgrcv(rss_player->rx_msg_id, (void *)&(msg_data), INTERNAL_MESSAGE_SIZE, msg_to_receive, IPC_NOWAIT) == -1)
        {
            usleep(10000);
            continue;
        }
        
        message_block = (INTERNAL_MESSAGE_ID)&(msg_data.data);
        if (message_block->magic_number != 0xaa55)
        {
            DBG_MSG("<sky_rss>    wrong magic number\n");
            usleep(10000);
            continue;
        }
        
        DBG_MSG("<sky_rss> get message(%d)\n", message_block->type);
        switch (message_block->type)
        {
    		case IPC_SHUTDOWN:
    			DBG_MSG("<sky_rss> signal from skydemo to terminate RSS function.\n");
    			rss_player->sky_rss_status = 0;
    			break;
    			
            case IPC_NETWORK_UNAVAILABLE:
    			DBG_MSG("<sky_rss> inform link down.\n");
            	
    			rss_player->link_status = LINK_STATUS_DOWN;
    			rss_player->link_status_changed = 1;
    			
    			notify_server(rss_player, IPC_NETWORK_UNAVAILABLE, NULL);

#ifdef _DEBUG
    			DBG_MSG("<sky_rss>    web-site    = %s\n", rss_player->web_site);
    			DBG_MSG("<sky_rss>    channel num = %d\n", rss_player->channel_num);
    			for (i=0; i<rss_player->channel_num; i++)
    			{
    			    DBG_MSG("<sky_rss>    RSS url     = %s\n", (char *)&(rss_player->rss_feed[i][0]));
    			}
                DBG_MSG("<sky_rss>    conf file   = %s\n", rss_player->rss_config_file);
                DBG_MSG("<sky_rss>    main path   = %s\n", rss_player->main_path);
                DBG_MSG("<sky_rss>    sub path    = %s\n", rss_player->sub_path);
                DBG_MSG("<sky_rss>    RSS url     = %s\n", rss_player->download_dir);
                DBG_MSG("<sky_rss>    ttl         = %d\n", rss_player->ttl);
                DBG_MSG("<sky_rss>    flag        = %d\n", rss_player->sky_rss_status);
                DBG_MSG("<sky_rss>    link        = %d\n", rss_player->link_status);
#endif

    			break;
    		
    		case IPC_NETWORK_AVAILABLE:
    			DBG_MSG("<sky_rss> inform link active.\n");
                
    			rss_player->link_status = LINK_STATUS_ACTIVE;
    			rss_player->link_status_changed = 1;
    			
    			notify_server(rss_player, IPC_NETWORK_AVAILABLE, NULL);
    			break;
        }
    }
    DBG_MSG("<sky_rss> signal_thread <--\n");
}

void notify_server(RSS_PLAYER_ID rss_player, UINT8 message_type, FILE_DESC_ID file_desc)
{
    DBG_MSG("<sky_rss> --> send_msg: 0x%2.2x\n", message_type);
    
    MSG_DATA msg_data;
    INTERNAL_MESSAGE_ID rss_content;
    
    rss_content = (INTERNAL_MESSAGE_ID)&(msg_data.data);
    
    msg_data.msg_type = 1;
    rss_content->type = message_type;
    rss_content->web_site = rss_player->web_site_no;
    if (rss_player->session_desc_cnt>0)
    {
        rss_content->rest_frame_count = rss_player->session_desc_cnt - rss_player->update_count;
    }
    else
    {
        rss_content->rest_frame_count = -1;
    }
    sprintf(rss_content->file_dir, "%s", rss_player->download_dir);
    if (file_desc != NULL)
    {
        strcpy(rss_content->file_name, file_desc->file_name);
        rss_content->duration = file_desc->duration;
    }
    rss_content->magic_number = 0x55aa;
    
    if (msgsnd(rss_player->tx_msg_id, (void *)&msg_data, INTERNAL_MESSAGE_SIZE, 0) == -1)
    {
        printf("<sky_rss>    msgsnd failed!!\n");
    }
}


void init_rss_player(RSS_PLAYER_ID rss_player)
{
    DBG_MSG("<sky_rss> init_rss_player -->\n");

	rss_player->sky_rss_status = 1;
	
    memset((char *)(&(rss_player->file_info)), 0x00, FILE_DESC_SIZE*TOTAL_FILE_QUOTA);
    rss_player->used_desc_index = 0;
    rss_player->empty_desc_index = 0;
    rss_player->depth = 0;
    rss_player->channel_num = 0;
    rss_player->link_status = LINK_STATUS_DOWN;
    rss_player->link_status_changed = 0;
    
    //the below parameters may be changed after reading option/configuration
    sprintf(rss_player->main_path, "%s", DIR_PREFIX);
    sprintf(rss_player->download_dir, "%s%s", DIR_PREFIX, DOWNLOAD_DIR);
    //rywu20080705 sprintf(rss_player->rss_config_file, "%s", RSS_CONFIG_FILE);
    rss_player->web_site_no = WEB_UNKNOWN;
    
    unlink("/tmp/conf/rss_time");
    
    DBG_MSG("<sky_rss> init_rss_player <--\n");
}

void reset_session(RSS_PLAYER_ID rss_player)
{
    char str[128];
    /*rywu20080715 for purpose of showing image after system reboot, we can remove cache files here
    //based on recycle mode to recycle disk space
    if(rss_player->recycle_mode == RECYCLE_MODE_ERASE_ALL_FILE)
    {
        //remove previous folder, if it exists
    	sprintf(str, "rm -rf %s", rss_player->download_dir);
    	system(str);

        //create folder to download files
    	sprintf(str, "mkdir %s", rss_player->download_dir);
    	system(str);

        memset((char *)(&(rss_player->file_info)), 0x00, FILE_DESC_SIZE*TOTAL_FILE_QUOTA);
        rss_player->used_desc_index = 0;
        rss_player->empty_desc_index = 0;
        rss_player->depth = 0;
    }*/
    rss_player->session_desc_cnt = 0;
    
    //set default information for this rss session
    rss_player->ttl = DEFAULT_TTL*60;
    //rywu20080618++
    if (rss_player->web_site_no == WEB_FRAMECHANNEL)
    {
        rss_player->ttl = 900; //15 min
    }
    //rywu20080618--
}
	
void notify_server_existing_file(RSS_PLAYER_ID rss_player)
{
    int i;
    FILE_DESC_ID file_desc;
    
    for (i=0; i<rss_player->empty_desc_index; i++)
    {
        if (!rss_player->sky_rss_status)
        {
            break;
        }
        file_desc = (FILE_DESC_ID)&(rss_player->file_info[i]);
        file_desc->status |= FILE_STATUS_DOWNLOADED;
        notify_server(rss_player, IPC_DOWNLOAD_ONE_FILE_OK, file_desc);
        //usleep(500000);
    }
}

int parse_xml_file(RSS_PLAYER_ID rss_player, char *file_name, UINT8 file_type)
{
    FILE *p_file;
    XML_Parser parser;
    //char *buf;
    int len, done;
    //int i;
    
    //parse xml file to get image information
    //
    //printf("parse file %s, BUFSIZ = %d\n", file_name, BUFSIZ);
    p_file = fopen(file_name, "r");
    if(p_file == 0) 
    {
        printf("expat open file error!\n");
        return -1;
    }

    rss_player->xml_file_usage = file_type;
    
    //buf = malloc(BUFSIZ);
    //parse xml
    parser = XML_ParserCreate("UTF-8");
    XML_SetUserData(parser, rss_player);
    XML_SetElementHandler(parser, parse_start, parse_end);
    do {
        len = (int)fread(xml_buf, 1, XML_BUFFER_SIZE, p_file);
        done = len < XML_BUFFER_SIZE;
        
        if(XML_Parse(parser, xml_buf, len, done) == XML_STATUS_ERROR) 
        {
            printf("%s at line %" XML_FMT_INT_MOD "u\n",
                    XML_ErrorString(XML_GetErrorCode(parser)),
                    XML_GetCurrentLineNumber(parser));
            break;
        }
    } while (!done);
    XML_ParserFree(parser);
        
    //free(buf);
    fclose(p_file);
    return 0;
}

int parse_xml_config(RSS_PLAYER_ID rss_player, char *file_name)
{
    FILE *p_file;
    XML_Parser parser;
    //char *buf;
    int len, done;

    DBG_MSG("<sky_rss> parse_xml_config -->\n");
    
    //parse xml file to get image information
    //
    p_file = fopen(file_name, "r");
    if(p_file == 0) 
    {
        printf("expat open file error!\n");
        DBG_MSG("<sky_rss> parse_xml_config(1) <--\n");
        return -1;
    }

    //buf = malloc(BUFSIZ);
    
    //parse xml
    parser = XML_ParserCreate("UTF-8");
    XML_SetUserData(parser, rss_player);
    XML_SetDefaultHandler(parser, config_handler);
    do {
        len = (int)fread(xml_buf, 1, XML_BUFFER_SIZE, p_file);
        done = len < XML_BUFFER_SIZE;
        if(XML_Parse(parser, xml_buf, len, done) == XML_STATUS_ERROR) 
        {
            printf("%s at line %" XML_FMT_INT_MOD "u\n",
                    XML_ErrorString(XML_GetErrorCode(parser)),
                    XML_GetCurrentLineNumber(parser));
            break;
        }
    } while (!done);
    XML_ParserFree(parser);
    
    //free(buf);
    fclose(p_file);
    DBG_MSG("<sky_rss> parse_xml_config <--\n");
    return 0;
}

void insert_existing_file_desc(RSS_PLAYER_ID rss_player)
{
    DIR *dir;
    struct dirent *ptr;
    int i, j, len;
    FILE_DESC_ID file_desc;
    //FILE *fp;
    //char str[128];
    
    DBG_MSG("<sky_rss> search existing file in %s\n", rss_player->download_dir);
    DBG_MSG("<sky_rss> empty desc index = %d\n", rss_player->empty_desc_index);

    dir = opendir(rss_player->download_dir);
    i=0;
    while((ptr=readdir(dir))!=NULL)
    {
        i++;
        if (i>2)
        {
            len = strlen(ptr->d_name);
            for (j=len-1; j>0; j--)
            {
                if(*(ptr->d_name+j) == 0x2e)//compare "."
                {
                    break;
                }
            }
            
            //check if it is .jpg file
            if(!strcmp((ptr->d_name+j+1), "jpg") || !strcmp((ptr->d_name+j+1), "JPG"))
            {
                file_desc = (FILE_DESC_ID)&(rss_player->file_info[rss_player->empty_desc_index]);
                
                file_desc->index = rss_player->empty_desc_index;
                sprintf(file_desc->file_name, "%s", ptr->d_name);//copy file name
                file_desc->duration = DEF_DURATION;
                file_desc->status = 0;
                
                rss_player->empty_desc_index++;
            }
            
        }
        else
        {
            DBG_MSG("<sky_rss> get file = %s\n", ptr->d_name);
        }
    }
    closedir(dir);
    
    /*rywu20080709 due to download to /tmp first and then copy to /mnt/sknand, so it is no need to check file integrity.
    //This is an little tricky.
    //For close thumbnail mode quickly, sky_rss will be killed and the last file should be corrupted.
    sprintf(str, "%s/downloading", rss_player->download_dir);
    fp = fopen(str, "r");
    if(fp != 0) 
    {
        fclose(fp);

        DBG_MSG("<sky_rss> get deltete flag to remove latest file\n");

        //remove file
        if (rss_player->empty_desc_index > 0)
        {
            //decrease 1 to index
            rss_player->empty_desc_index--;
            
            //get the been-deleted file desc
            file_desc = (FILE_DESC_ID)&(rss_player->file_info[rss_player->empty_desc_index]);
            
            //making instruction to remove last file
            sprintf(str, "rm %s/%s", rss_player->download_dir, file_desc->file_name);
            system(str);
            
            DBG_MSG("<sky_rss> remove %s/%s\n", rss_player->download_dir, file_desc->file_name);
        }
        
        //remove the marked file
        sprintf(str, "rm %s/downloading", rss_player->download_dir);
        system(str);
    }
    */

#ifdef _DEBUG
    printf("<sky_rss> existing files --> %d files\n", rss_player->empty_desc_index);
    for (i=0; i<rss_player->empty_desc_index; i++)
    {
        file_desc = (FILE_DESC_ID)&(rss_player->file_info[i]);
        printf("      %s\n", file_desc->file_name);
    }
#endif
}

int insert_file_desc(RSS_PLAYER_ID rss_player, FILE_DESC_ID new_desc)
{
    int i, j;
    int len;
    FILE_DESC_ID    file_desc, chk_file_desc;
    
    if (strcmp(new_desc->data_type, "image/jpeg") && 
        strcmp(new_desc->data_type, "image/jpg"))//rywu20080625 for photozou.jp
    {
        //printf("<sky_rss> not support data type --> %s\n", new_desc->data_type);
        return -1;
    }
    
    //rywu20080618a++
    if (rss_player->empty_desc_index+rss_player->session_desc_cnt > (TOTAL_FILE_QUOTA-1))
    {
        printf("<sky_rss>    exceed max file amount %d\n", TOTAL_FILE_QUOTA);
        return -1;
    }
    //rywu20080618a--
    
    file_desc = (FILE_DESC_ID)&(rss_player->file_info[rss_player->empty_desc_index+rss_player->session_desc_cnt]);
    
    //copy necessary informtion(1)
    //--> url, data type, duration
    strcpy(file_desc->url, new_desc->url);
    strcpy(file_desc->data_type, new_desc->data_type);
    file_desc->duration = new_desc->duration;
    
    len = strlen(new_desc->url);
    for (i=len-1; i>0; i--)
    {
        if(*(new_desc->url+i) == 0x2f ||//)//compare "/"
           *(new_desc->url+i) == 0x2c )    //compare ","
        {
            //*(new_desc->url+i) = 0x2f;
            //*(file_desc->url+i) = 0x2f;
            break;
        }
    }
    
    for(j=rss_player->used_desc_index; j<rss_player->empty_desc_index; j++)
    {
        chk_file_desc = (FILE_DESC_ID)&(rss_player->file_info[j]);
        //printf("%s vs %s\n", (url_str+i+1), (chk_file_desc->file_name+5));
        if(!strcmp((chk_file_desc->file_name+5), (new_desc->url+i+1)))
        {
            DBG_MSG("<sky_rss> duplicated file(%s)\n", (new_desc->url+i+1));
            return -1;
        }
    }
    
    //prepare file prefix index
    file_desc->index = rss_player->empty_desc_index+rss_player->session_desc_cnt;
    
    //copy necessary informtion(2)
    //--> file name
    sprintf(file_desc->file_name, "%4.4d_%s", file_desc->index, (new_desc->url+i+1));
    file_desc->status = 0;
    
    rss_player->session_desc_cnt++;
    
    DBG_MSG("<sky_rss> store to %s, dur = %d sec\n", file_desc->file_name, file_desc->duration);
    return 0;
}

int main(int argc, char** argv) 
{
    int i, ret;
    int sleep_count = 0;
    FILE_DESC_ID file_desc = 0;
    char str[256];
    UINT8 ipc_msg_type = IPC_INIT;
    DIR *dir;
    UINT8 channel;
    FILE *fp_time;
    unsigned char retry;//rywu20080707
    
#ifdef _DEBUG
    int link_down_cnt = 0;
#endif
    
	//
	// show usage
	//
    if(argc < 8)
    {
        printf("Usage : \n");
        printf("    sky_rss -f [file_name] -d [main_path] -s [sub_path] -r [recycle_mode]\n");
        printf("\nParameter : \n");
        printf("    file_name    - indicate the file containing RSS feed information\n");
        printf("    main_path    - indicate the directory to put sub_path for each image provider.\n");
        printf("    sub_path     - indicate the directory to put download files.\n");
        printf("    recycle_mode - indicate taking what action for existing files.\n");
        printf("                   0 : according web site to take action automatically.\n");
        printf("                   1 : keep all existing files.\n");
        printf("                   2 : erase all existing files.\n");
        printf("\nExample : \n");
        printf("    sky_rss -f /tmp/conf/channel.xml -d /mnt/sknand -s /my_download -r 0\n\n");
        printf("    sky_rss will parse '/tmp/conf/channel.xml' to get RSS feed, \n");
        printf("    then download files in '/mnt/sknand/my_download'. If there is any file \n");
        printf("    existing in '/mnt/sknand/my_download', sky_rss will erase/keep them\n"); 
        printf("    according what web site it is.\n");
        return 0;
    }
    else
    {
        printf("\nrunning sky_rss(version %s)\n", VERSION);
    }
	
	//allocate memory for rss_player, and remember to free it 
	//when program is terminated.
    memset((char *)rss_player, 0x00, RSS_PLAYER_SIZE);
    
    //initialize default values
    init_rss_player(rss_player);
    
    //initialize internal socket
	if (init_IPC(rss_player))
	{
	    printf("<sky_rss> failed on initializing IPC.\n");
	    //rywu20080623 close_IPC(rss_player);
	    
	    //wait for skydemo to kill sky_rss
	    while(rss_player->sky_rss_status)
	    {
	        sleep(1);
        }
        
        //rywu20080623
        close_IPC(rss_player);
        return -1;
    }
    
    //get input parameters
    ret = parse_option(rss_player, argc, argv);
    if(ret)
	{
	    printf("<sky_rss> option error!!! please check again...\n");
	    //rywu20080623 close_IPC(rss_player);
	    
	    //wait for skydemo to kill sky_rss
	    while(rss_player->sky_rss_status)
	    {
	        sleep(1);
        }
        
        //rywu20080623
        close_IPC(rss_player);
        return -1;
    }
	
	//wait for signal thread up
	while(rss_player->signal_thread_state == 0)
	{
	    usleep(1000);
    }
    DBG_MSG("<sky_rss> my pid = %d\n", getpid());
    
	//tell skydemo, rss client is initializing.
    notify_server(rss_player, IPC_INIT, NULL);
    
 	//prepare designated directory for storing image
 	//RECYCLE_MODE_ERASE_ALL_FILE will remove whole content in target directory
 	//RECYCLE_MODE_NONE will keep existing file in target directory
 	/*rywu20080715 for purpose of showing image after system reboot, we can remove cache files here
 	if(rss_player->recycle_mode == RECYCLE_MODE_ERASE_ALL_FILE)
 	{
	    //remove previous folder, if it exists
    	sprintf(str, "rm -rf %s", rss_player->download_dir);
    	system(str);
    	
    	//create folder to download files
	    sprintf(str, "mkdir %s", rss_player->download_dir);
	    system(str);
	}
	else if(rss_player->recycle_mode == RECYCLE_MODE_NONE)*/
 	{
        dir = opendir(rss_player->download_dir);
        if (dir==NULL)
        {
            //create folder to download files
    	    sprintf(str, "mkdir %s", rss_player->download_dir);
    	    system(str);
        }
        else
        {
            closedir(dir);
        }
       
	    insert_existing_file_desc(rss_player);
	    notify_server_existing_file(rss_player);
	}
	
	//RSS life cycle
	while(rss_player->sky_rss_status)
	{
	    //detect network status...
	    //if link-down, do nothing
        if(rss_player->link_status == LINK_STATUS_DOWN)
		{
#ifdef _DEBUG
		    if (link_down_cnt != 0 && !(link_down_cnt%64))
            {
                printf("<sky_rss>    link down last %d sec\n", link_down_cnt);
            }
            link_down_cnt++;
#endif
			sleep(1);
	        continue;
		}
		//if link-active and is state-transition, start to get image immediately
		if (rss_player->link_status == LINK_STATUS_ACTIVE &&
		    rss_player->link_status_changed == 1)
        {
            rss_player->link_status_changed = 0;
            sleep_count = 3;//sleep 3 second to wait network been bringing up
        }
		
	    //run ttl 
	    if(rss_player->sky_rss_status && 
	       sleep_count)
	    {
	        sleep_count--;
#ifdef _DEBUG
            if (sleep_count!= 0 && !(sleep_count%64))
            {
	            printf("<sky_rss> sleep = %d\n", sleep_count);
	        }
#endif
	        sleep(1);
	        continue;
        }
		
        if (!rss_player->sky_rss_status)
        {
        	DBG_MSG("<sky_rss> break main loop\n");
            break;
        }
        
#ifdef _DEBUG
        link_down_cnt = 0;
#endif

        //initialize session
        //rywu20080715 notify_server(rss_player, IPC_RESET_RSS_SESSION, NULL);
        reset_session(rss_player);
        	
        //rywu20080617 rss_player->update_count = 0;
	    for (channel=0; channel<rss_player->channel_num && rss_player->sky_rss_status==1; channel++)
	    {
	        rss_player->update_count = 0;//rywu20080617
	        notify_server(rss_player, IPC_RETRIEVE_FEED_INFORMATION, NULL);
	        
	        //set downloading status as running
	        ipc_msg_type = IPC_RSS_RUNNING;
	        
	        //
        	//run http client to get web content
        	//
        	DBG_MSG("<sky_rss> get xml file from %d %s\n", channel, rss_player->rss_feed[channel]);
        	//rywu20080707++
        	//download_content(rss_player->rss_feed[channel], TARGET_RSS_FEED, RSS_FEED_TIMEOUT);
        	retry = 3;
        	while(retry > 0 && rss_player->sky_rss_status)
        	{
        	    ret = download_content(rss_player->rss_feed[channel], TARGET_RSS_FEED, RSS_FEED_TIMEOUT);
        	    if (ret == CURLE_OK)
                {
                    retry = 0;
                }
                else
                {
                    retry--;
                    usleep(100000);
                    DBG_MSG("<sky_rss>    retry %d times for error %d\n", 3-retry, ret);
                }
            }
        	//rywu20080707--
        	//rywu20080709 system("sync");
        	if (ret != CURLE_OK)
            {
                break;
            }
            
            //rywu20080715++
            //1. inform skydemo sky_rss is starring to download image files.
            //2. remove cache files under FrameChannel
            notify_server(rss_player, IPC_RESET_RSS_SESSION, NULL);
	        //based on recycle mode to recycle disk space
            if(rss_player->recycle_mode == RECYCLE_MODE_ERASE_ALL_FILE)
            {
                //remove previous folder, if it exists
                sprintf(str, "rm -rf %s", rss_player->download_dir);
                system(str);
            
                //create folder to download files
                sprintf(str, "mkdir %s", rss_player->download_dir);
                system(str);
            
                memset((char *)(&(rss_player->file_info)), 0x00, FILE_DESC_SIZE*TOTAL_FILE_QUOTA);
                rss_player->used_desc_index = 0;
                rss_player->empty_desc_index = 0;
                rss_player->depth = 0;
            }
            //rywu20080715--
        	
        	//parse xml by rss feed
            parse_xml_file(rss_player, TARGET_RSS_FEED, 1);
            parse_xml_config(rss_player, TARGET_RSS_FEED);
            DBG_MSG("<sky_rss>    ttl = %d\n", rss_player->ttl);
            
            //show download message on screen
            notify_server(rss_player, IPC_DOWNLOAD_START, NULL);
            
            //
            //get image file back
            //
            DBG_MSG("<sky_rss> update items = %4.4d\n", rss_player->session_desc_cnt);
            
            if (rss_player->session_desc_cnt == 0)
            {
                sleep(2);
            }
                
            for (i=0; i<rss_player->session_desc_cnt; i++)
        	{
        	    file_desc = (FILE_DESC_ID)&(rss_player->file_info[rss_player->empty_desc_index]);
                
                //DBG_MSG("%4.4d, %s %s\n", file_desc->index, file_desc->url, file_desc->file_name);

                //rywu20080709 sprintf(str, "touch %s/downloading", rss_player->download_dir);
                //rywu20080709 system(str);
                
        	    memset(str, 0x00, 256);
        	    sprintf(str, "%s/%s", rss_player->download_dir, file_desc->file_name);
        	    //rywu20080707++
            	//ret = download_content(file_desc->url, str, FILE_DOWNLOAD_TIMEOUT);
            	retry = 3;
            	while(retry > 0 && rss_player->sky_rss_status)
            	{
            	    ret = download_content(file_desc->url, str, FILE_DOWNLOAD_TIMEOUT);
            	    if (ret == CURLE_OK)
                    {
                        retry = 0;
                    }
                    else
                    {
                        retry--;
                        sleep(2);//rywu20080715 usleep(100000);
                        DBG_MSG("<sky_rss>    retry %d times for error %d\n", 3-retry, ret);
                    }
                }
            	//rywu20080707--
        	    //DBG_MSG("  ret = %d\n", ret);

                //rywu20080707++
                //sprintf(str, "rm %s/downloading", rss_player->download_dir);
                //system(str);
                //rywu20080709 sprintf(str, "%s/downloading", rss_player->download_dir);
                //rywu20080709 unlink(str);
                //rywu20080707--
                
                system("sync;sync");
                
                //handle all events here while download is running.
        	    if(ret == CURLE_OK)
                {
                    //keep update count
                    rss_player->update_count += 1;
            
                    //shift index to first empty desc
                    rss_player->empty_desc_index += 1;
            
                    file_desc->status |= FILE_STATUS_DOWNLOADED;
                    notify_server(rss_player, IPC_DOWNLOAD_ONE_FILE_OK, file_desc);
                    //usleep(200000);
                }
        	    else 
        	    {
        	        if(ret == CURLE_WRITE_ERROR)
        	        {
                        printf("<sky_rss> disk full!! stop updating!!\n");
                        ipc_msg_type = IPC_MEMORY_NOT_ENOUGH;
        	        }
        	        else if(ret == CURLE_COULDNT_RESOLVE_HOST)
                    {
                        printf("<sky_rss> can't resolve host\n");
                        ipc_msg_type = IPC_CANNOT_RESOLVE_HOST;
                    }
                    else if(ret == CURLE_OPERATION_TIMEDOUT)
                    {
                        printf("<sky_rss> rx timeout\n");
                        ipc_msg_type = IPC_RX_TIMEOUT;
                    }
        	        else
        	        {
        	            printf("<sky_rss> error(%d) on downloading\n", ret);
        	            ipc_msg_type = IPC_FILE_WRITE_FAIL;
        	        }
        	        
        	        //remove this empty file
                    //rywu20080709 sprintf(str, "rm %s/%s", rss_player->download_dir, file_desc->file_name);
                    //rywu20080709 system(str);
                    
                    //stop running by error 
                    break;
                }
                
                //Downloading takes most time of RSS module. 
        	    //In case of RSS being shuting down, we have to react immediately.
        	    if (!rss_player->sky_rss_status)
        	    {
                    DBG_MSG("<sky_rss> terminated by skydemo\n");
                    ipc_msg_type = IPC_SHUTDOWN;
                    break;
        	    }
        	    //stop running by link down 
        	    if (rss_player->link_status == LINK_STATUS_DOWN)
                {
                    DBG_MSG("<sky_rss> stop rx file due to link down\n");
                    ipc_msg_type = IPC_NETWORK_UNAVAILABLE;
                    break;
                }
            }//end for
            
            //reset this count for next channel
            rss_player->session_desc_cnt = 0;
            
            if (ipc_msg_type != IPC_RSS_RUNNING)
            {
                break;
            }
        }//end for
        
        //rywu20080709
        if (rss_player->sky_rss_status == 0)
        {
            ipc_msg_type = IPC_SHUTDOWN;
        }
        
        if(ipc_msg_type == IPC_RSS_RUNNING)
        {
            if(rss_player->update_count == 0)
            {
                DBG_MSG("<sky_rss> no update.\n");
                ipc_msg_type = IPC_NO_UPDATE;
                sleep(2);
            }
            else
            {
                DBG_MSG("<sky_rss> download complete\n");
                ipc_msg_type = IPC_DOWNLOAD_COMPLETE;
            }
        }
        
        //tell skydemo session result, except shutdown rss
        if (ipc_msg_type != IPC_SHUTDOWN && 
            //rywu20080617 ipc_msg_type != IPC_DOWNLOAD_COMPLETE &&
            ipc_msg_type != IPC_INIT)//stop to trigger skydemo sending link status, when no channel existing.
        {
            sleep(1);
            notify_server(rss_player, ipc_msg_type, NULL);
        }
        
        sleep_count = rss_player->ttl;
        DBG_MSG("<sky_rss> last msg = 0x%2.2x\n", ipc_msg_type);
        DBG_MSG("<sky_rss> channel = %d/%d\n", channel, rss_player->channel_num);
        DBG_MSG("<sky_rss> sleep = %d vs %d\n", sleep_count, rss_player->ttl);
        //usleep(500000);
        
        fp_time = fopen("/tmp/conf/rss_time", "w+");
        if(fp_time == 0) 
        {
            printf("<skydemo>    Huston, can't create fp_time !!!\n");
        }
        else
        {
            sprintf(str, "LATEST_TIME=%d\n", (int)time((time_t *)NULL));
            fputs(str, fp_time);
            sprintf(str, "TTL=%d\n", rss_player->ttl);
            fputs(str, fp_time);
            
            fclose(fp_time);
            DBG_MSG("<sky_rss>    rss_time file is done.\n");
        }
    }//end while
    
    close_IPC(rss_player);
	
	DBG_MSG("<sky_rss> exit sky_rss\n");
	
	return 0;
}



