#ifndef _SKY_RSS_H_
#define _SKY_RSS_H_

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <unistd.h>
#include <dirent.h>
#include <signal.h>
#include <sys/wait.h>
#include <pthread.h>
#include <sched.h>
#include <sys/msg.h>

#ifdef _DEBUG
    #define DBG_MSG(str, args...)   	printf(str, ##args)
#else
    #define DBG_MSG(str, args...)
#endif

#define VERSION "1.00.0613.2008"

//#define RSS_CONFIG_FILE "/tmp/conf/channel.xml"
#define TARGET_RSS_FEED "/tmp/sky_rss.xml" //rywu20080704 "/mnt/sknand/sky_rss.xml"
#define DIR_PREFIX      "/mnt/sknand"
#define DOWNLOAD_DIR    "/my_download"

#define CHANNEL_QTY             8
#define WEB_SITE_NAME_LENGTH    32
#define RSS_FEED_LENGTH         256
#define DOWNLOAD_DIR_LENGTH     128
#define TOTAL_FILE_QUOTA        256
#define DEFAULT_TTL             60  //min
#define DEF_DURATION            6 //sec

#define FILE_NAME_LENGTH        64
#define URL_LENGTH              256
#define MAX_TYPE_LENGTH         16//rywu20080709

#define RSS_FEED_TIMEOUT        240
#define FILE_DOWNLOAD_TIMEOUT   480

#define WEB_UNKNOWN             0x00
#define WEB_FLICKR              0x01
#define WEB_PICASA              0x02
#define WEB_FACEBOOK            0x03
#define WEB_FRAMECHANNEL        0x04

#define FILE_STATUS_DOWNLOADED  0x01

#define RECYCLE_MODE_AS_WILL    0x00
#define RECYCLE_MODE_NONE       0x01
#define RECYCLE_MODE_ERASE_ALL_FILE 0x02
//#define RECYCLE_MODE_WRAP       0x03

#define LINK_STATUS_DOWN	0x00
#define LINK_STATUS_ACTIVE	0x01
#define LINK_STATUS_UNKNOWN 0xff

//NOTE : the below definition MUST be same as skydemo
//---------------------------------------------------
#define IPC_INIT                    0x00    //l
#define IPC_DOWNLOAD_START          0x01    //}lU
#define IPC_DOWNLOAD_ONE_FILE_OK    0x02    //U@ɮOK
#define IPC_DOWNLOAD_COMPLETE       0x03    //`/ϥΪ̫}
#define IPC_RESET_RSS_SESSION       0x04    //rss session is reseting //rywu20080528
#define IPC_CONFIG_FILE_BROKEN      0x05    //WD]wɷl
#define IPC_NO_UPDATE               0x06    //SsӤiѤU
#define IPC_MEMORY_NOT_ENOUGH       0x07    //OХdeq
#define IPC_FILE_WRITE_FAIL         0x08    //OХdLkgJ
#define IPC_RETRIEVE_FEED_INFORMATION 0x09  //retrieve rss feed information

#define IPC_NO_SUBSCRIBED_CHANNEL   0x0a    //Sq\WD
#define IPC_NETWORK_UNAVAILABLE     0x0b    //Lks network link off
#define IPC_CHANNEL_UNREADABLE      0x0c    //LkŪYWD==>RSS Reader|۰ʵ
#define IPC_RSS_RUNNING             0x0d    //rss is running
#define IPC_NETWORK_AVAILABLE 	    0x0e	//network link on
#define IPC_SHUTDOWN		 	    0x0f	//shutdown rss client
#define IPC_INVALID_ARGUMENT        0x10    //Ѽƿ~

#define IPC_CANNOT_RESOLVE_HOST     0x20    //can't resolve host
#define IPC_RX_TIMEOUT              0x21    //rx time-out
#define IPC_UNKNOWN_ERROR           0xff

//---------------------------------------------------
//NOTE : the above definition MUST be same as skydemo

#define CONFIG_XML_FILE 0x00
#define RSS_FEED_FILE   0x01

typedef	unsigned char   UINT8;
typedef	unsigned short  UINT16;
typedef	unsigned int    UINT32;

typedef struct _internal_message_ 
{
    UINT8 own;  //used to lock mutual resource
    UINT8 type; //status code
    UINT8 web_site; //taget web site
    UINT8 duration;                     //display time
    int rest_frame_count;             //rest frame of not downloading yet
    char file_dir[DOWNLOAD_DIR_LENGTH]; //downloaded sub folder
    char file_name[FILE_NAME_LENGTH];   //downloaded file name
    UINT32 magic_number;//rywu20080616
} INTERNAL_MESSAGE, *INTERNAL_MESSAGE_ID;
#define INTERNAL_MESSAGE_SIZE sizeof(INTERNAL_MESSAGE)

typedef struct
{
    long int msg_type;
    unsigned data[INTERNAL_MESSAGE_SIZE];
}MSG_DATA, *MSG_DATA_ID;
#define MSG_DATA_SIZE sizeof(MSG_DATA)

typedef struct _file_discrption_ 
{
    char url[URL_LENGTH];               //url of this file
    char file_name[FILE_NAME_LENGTH];   //file name that stores image
    UINT8 index;                        //index witn total files
    UINT8 status;                       //indicate file status. bit 0 : been downloaded
    UINT8 duration;                     //display time
    char data_type[16];                //content type
} FILE_DESC, *FILE_DESC_ID;
#define FILE_DESC_SIZE  sizeof(FILE_DESC)

typedef struct _rss_player_ 
{
    char web_site[WEB_SITE_NAME_LENGTH];
    char rss_feed[CHANNEL_QTY][RSS_FEED_LENGTH];
    char rss_config_file[FILE_NAME_LENGTH];
    char main_path[FILE_NAME_LENGTH];
    char sub_path[FILE_NAME_LENGTH];
    char download_dir[DOWNLOAD_DIR_LENGTH];
    UINT8 depth;
    UINT8 channel_num;
    UINT8 web_site_no;    //indicate designated web-site we are connecting to.
    UINT8 recycle_mode; //indicate the mode to handle disk space full
    UINT8 xml_file_usage;
    int ttl;//seconds
    
    int session_desc_cnt;   //count downloadable files
    int update_count; //already download index
    UINT16  used_desc_index;    //index of used file descriptor
    UINT16  empty_desc_index;   //index of un-used file descriptor
    
    int tx_msg_id;
    int rx_msg_id;
    
    UINT8 sky_rss_status;
    
    UINT8 link_status;
    UINT8 link_status_changed;

    int signal_thread_state;
    pthread_t signal_thread;
    void *signal_thread_result;
    
    FILE_DESC file_info[TOTAL_FILE_QUOTA+2];
    
} RSS_PLAYER, *RSS_PLAYER_ID;
#define RSS_PLAYER_SIZE  sizeof(RSS_PLAYER)


int parse_option(RSS_PLAYER_ID rss_player, int argc, char **argv);
void init_rss_player(RSS_PLAYER_ID rss_player);
int insert_file_desc(RSS_PLAYER_ID rss_player, FILE_DESC_ID new_desc);
int download_content(char *url, char *file_name, int timeout);
void reset_session(RSS_PLAYER_ID rss_player);
void remove_all_file(RSS_PLAYER_ID rss_player);
int init_IPC(RSS_PLAYER_ID rss_player);
void close_IPC(RSS_PLAYER_ID rss_player);
void notify_server(RSS_PLAYER_ID rss_player, UINT8 message_type, FILE_DESC_ID file_desc);
int parse_xml_file(RSS_PLAYER_ID rss_player, char *file_name, UINT8 file_type);
void insert_existing_file_desc(RSS_PLAYER_ID rss_player);
//rywu20080618a int insert_file_desc(RSS_PLAYER_ID rss_player, FILE_DESC_ID new_desc);
void insert_existing_file_desc(RSS_PLAYER_ID rss_player);
void signal_thread(RSS_PLAYER_ID rss_player);

#endif

