#ifndef _TIFF_HEADER_
#define _TIFF_HEADER_
//------------------------------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "types.h"

//#define DEBUG_TIF_PARSER 1

#if DEBUG_TIF_PARSER
#define TIF_DBG(fmt, args...) printf(fmt, ## args)
#else
#define TIF_DBG(...)
#endif

#define MAX_READ_SIZE 4096

#define TIFF_BIG_ENDIAN 1
#define TIFF_LITTLE_ENDIAN 2

#define TIFFTAG_IMAGEWIDTH              256     /* image width in pixels */
#define TIFFTAG_IMAGELENGTH             257     /* image height in pixels */
#define TIFFTAG_COMPRESSION             259     /* data compression technique */
#define TIFFTAG_DATETIME                306     /* creation date and time */ 
#define TIFFTAG_MAKE                    271     /* scanner manufacturer name */
#define TIFFTAG_MODEL                   272     /* scanner model name/number */
#define TIFFTAG_SAMPLESPERPIXEL         277     /* samples per pixel */      
#define TIFFTAG_PLANARCONFIG            284     /* storage organization */
#define     PLANARCONFIG_CONTIG         1       /* single image plane */
#define     PLANARCONFIG_SEPARATE       2       /* separate planes of data */
#define TIFFTAG_YCBCRSUBSAMPLING        530     /* !YCbCr subsampling factors */
#define TIFFTAG_YCBCRPOSITIONING        531     /* !subsample positioning */

#define COMPRESSION_NONE            1       /* dump mode */
#define COMPRESSION_LZW             5       /* Lempel-Ziv  & Welch */    
#define COMPRESSION_JPEG            7       /* %JPEG DCT compression */
#define COMPRESSION_PACKBITS        32773   /* Macintosh RLE */       
#define COMPRESSION_DEFLATE         32946   /* Deflate compression */      
#define COMPRESSION_ADOBE_DEFLATE   8       /* Deflate compression, as recognized by Adobe */

/*
 * Tag data type information.
 *
 * Note: RATIONALs are the ratio of two 32-bit integer values.
 */
typedef enum {
    TIFF_NOTYPE     = 0,    /* placeholder */
    TIFF_BYTE       = 1,    /* 8-bit unsigned integer */
    TIFF_ASCII      = 2,    /* 8-bit bytes w/ last byte null */
    TIFF_SHORT      = 3,    /* 16-bit unsigned integer */
    TIFF_LONG       = 4,    /* 32-bit unsigned integer */      
    TIFF_RATIONAL   = 5,    /* 64-bit unsigned fraction */
    TIFF_SBYTE      = 6,    /* !8-bit signed integer */   
    TIFF_UNDEFINED  = 7,    /* !8-bit untyped data */     
    TIFF_SSHORT     = 8,    /* !16-bit signed integer */  
    TIFF_SLONG      = 9,    /* !32-bit signed integer */  
    TIFF_SRATIONAL  = 10,   /* !64-bit signed fraction */ 
    TIFF_FLOAT      = 11,   /* !32-bit IEEE floating point */
    TIFF_DOUBLE     = 12,   /* !64-bit IEEE floating point */
    TIFF_IFD        = 13    /* %32-bit unsigned integer (offset) */
} TIFFDataType;

//------------------------------------------------------------------------------------------------
struct tiff_hdr
{
    Uint32 compression_type;
    Uint32 width;
    Uint32 height;
    char   CameraMake[32];
    char   CameraModel[40];
    char   DateTime[20];
};
//------------------------------------------------------------------------------------------------
static void tiff_get_ascii(FILE *fn, unsigned char *buf, int offset, int size);
static unsigned short tiff_get_short(unsigned char **buf, char endian);
static unsigned int tiff_get_int(unsigned char **buf, char endian);
void *GetTiffHdr(char* filename);
void FreeTiffHdr(void* data);

#endif
